#include "crystalfields.h"


CrystalFields::CrystalFields(double a, double c, int na, int nc){
    this->a = a;
    this->c = c;
    Na = na;
    Nc = nc;
    mu_0 = 4*M_PI*1e-7;
    gsl_rng_env_setup();
    T = gsl_rng_default;
    r = gsl_rng_alloc (T);
    d_theta = 0;
    d_phi = 0;
}
double CrystalFields::B_dipol_stripe_AFM(double a, double b, double c,
                          double x, double y, double z, double m,
                          int i, int j, int k,
                          double* Bx, double* By, double* Bz,
                          bool AFM_interlayer_coupling){
    double B[3];
    B[0] = B[1] = B[2] = 0;
    double ILc = -1;
    if(!AFM_interlayer_coupling) ILc = +1;
    for(int ii=-i; ii <= i; ii++)
        for(int jj=-j; jj <= j; jj++)
            for(int kk = -k; kk <= k; kk += 2){
                add_dipol(x-ii*a        ,y+jj*b        ,z-kk*c    ,     m,M_PI/2.,0,B);
                add_dipol(x-ii*a + 0.5*a,y+jj*b        ,z-kk*c    ,    -m,M_PI/2.,0,B);
                add_dipol(x-ii*a        ,y+jj*b + 0.5*b,z-kk*c    ,     m,M_PI/2.,0,B);
                add_dipol(x-ii*a + 0.5*a,y+jj*b + 0.5*b,z-kk*c    ,    -m,M_PI/2.,0,B);
                add_dipol(x-ii*a        ,y+jj*b        ,z-kk*c + c,ILc* m,M_PI/2.,0,B);
                add_dipol(x-ii*a + 0.5*a,y+jj*b        ,z-kk*c + c,ILc*-m,M_PI/2.,0,B);
                add_dipol(x-ii*a        ,y+jj*b + 0.5*b,z-kk*c + c,ILc* m,M_PI/2.,0,B);
                add_dipol(x-ii*a + 0.5*a,y+jj*b + 0.5*b,z-kk*c + c,ILc*-m,M_PI/2.,0,B);
            }
    if(Bx != NULL) *Bx = B[0];
    if(By != NULL) *By = B[1];
    if(Bz != NULL) *Bz = B[2];
    return sqrt(B[0]*B[0]+B[1]*B[1]+B[2]*B[2]);
}
double CrystalFields::B_dipol_bicolinear_AFM(double a, double b, double c,
                          double x, double y, double z, double m,
                          int i, int j, int k,
                          double* Bx, double* By, double* Bz,
                          bool AFM_interlayer_coupling){
    double B[3];
    B[0] = B[1] = B[2] = 0;
    double ILc = -1;
    if(!AFM_interlayer_coupling) ILc = +1;
    for(int ii=-i; ii <= i; ii++)
        for(int jj=-j; jj <= j; jj++)
            for(int kk = -k; kk <= k; kk += 2){
                if((ii+jj) % 2 != 0) m = - qAbs(m);else m = + qAbs(m);
                    add_dipol(x-ii*a        ,y+jj*b        ,z-kk*c    ,     m,M_PI/2.,M_PI*3./4.,B);
                    //add_dipol(x-ii*a + 0.5*a,y+jj*b        ,z-kk*c    ,     m,M_PI/2.,M_PI*3./4.,B);
                    //add_dipol(x-ii*a        ,y+jj*b + 0.5*b,z-kk*c    ,     m,M_PI/2.,M_PI*3./4.,B);
                    //add_dipol(x-ii*a + 0.5*a,y+jj*b + 0.5*b,z-kk*c    ,    -m,M_PI/2.,M_PI*3./4.,B);
                    //add_dipol(x-ii*a        ,y+jj*b        ,z-kk*c + c,ILc* m,M_PI/2.,M_PI*3./4.,B);
                    add_dipol(x-ii*a + 0.5*a,y+jj*b        ,z-kk*c + c,ILc* m,M_PI/2.,M_PI*3./4.,B);
                    add_dipol(x-ii*a        ,y+jj*b + 0.5*b,z-kk*c + c,ILc* m,M_PI/2.,M_PI*3./4.,B);
                    add_dipol(x-ii*a + 0.5*a,y+jj*b + 0.5*b,z-kk*c + c,ILc*-m,M_PI/2.,M_PI*3./4.,B);
            }
    if(Bx != NULL) *Bx = B[0];
    if(By != NULL) *By = B[1];
    if(Bz != NULL) *Bz = B[2];
    return sqrt(B[0]*B[0]+B[1]*B[1]+B[2]*B[2]);
}
double CrystalFields::Fe3PO4O3_dipol(double a, double c,
                                     double x, double y, double z, double m,
                                     int i, int j, int k,
                                     double* Bx, double* By, double* Bz,
                                     double* P){
    double mAFM=0;
    double B[3];
    B[0] = B[1] = B[2] = 0;
    double h_Fe = 0.4092*c;
    double d_Fe = 0.2257439553*a;
    //QString cbs;
    double mtheta = P[0]/180.*M_PI;// M_PI/2.;
    double mphi = -M_PI/6.;//M_PI/2.;

    for(int ii=-i; ii <= i; ii++)
        for(int jj=-j; jj <= j; jj++)
            for(int kk = -k; kk <= k; kk++)
                // for each of the three Phosphor along the diagonal of the unit cell
                for(int Ph = 0; Ph < 3; Ph ++){
                    // assure AFM order
                    if((3*kk+Ph) % 2 == 0)  mAFM = -m;
                    else                    mAFM =  m;
                    if(1 || mAFM > 0){
                        double ph = Ph/3.;
                        double x_Ph = a*             (ii-ph-(jj+ph)*cos(M_PI/3.));
                        double y_Ph = a*sin(M_PI/3.)*(jj+ph);
                        double z_Ph = c*             (kk+ph);
                        mphi = (kk+ph+P[1])/P[2]*2*M_PI;
                        mphi = P[1]/P[2]*2*M_PI;
                        //cbs.append(QString("\n%1\t%2\t%3\t").arg(x_Ph).arg(y_Ph).arg(z_Ph));
                        // add three iron moments for each phospor
                        for(int Fei = 0; Fei < 3; Fei++){
                            double phi = M_PI*(-1./6. +Fei*2./3.);
                            add_dipol(x-x_Ph + cos(phi)*d_Fe,
                                      y-y_Ph + sin(phi)*d_Fe,
                                      z-z_Ph + h_Fe,
                                      mAFM,mtheta,mphi,B);
                            /*cbs.append(QString("%1\t%2\t%3\t%4\t%5\t%6\t")
                                    .arg(x_Ph + cos(phi)*d_Fe+sin(mtheta)*cos(mphi)*mAFM)
                                    .arg(y_Ph + sin(phi)*d_Fe+sin(mtheta)*sin(mphi)*mAFM)
                                    .arg(z_Ph + h_Fe         +cos(mtheta)          *mAFM)
                                    .arg(x_Ph + cos(phi)*d_Fe-sin(mtheta)*cos(mphi)*mAFM)
                                    .arg(y_Ph + sin(phi)*d_Fe-sin(mtheta)*sin(mphi)*mAFM)
                                    .arg(z_Ph + h_Fe         -cos(mtheta)          *mAFM));
                            cbs.append(QString("%1\t%2\t%3\t%4\t%5\t%6\t%7\t%8\t%9\n\t\t\t")
                                    .arg(x_Ph)
                                    .arg(y_Ph)
                                    .arg(z_Ph)
                                    .arg(x_Ph + cos(phi)*d_Fe)
                                    .arg(y_Ph + sin(phi)*d_Fe)
                                    .arg(z_Ph + h_Fe         )
                                    .arg(x_Ph + cos(phi)*sqrt(1./3.)*a)
                                    .arg(y_Ph + sin(phi)*sqrt(1./3.)*a)
                                    .arg(z_Ph + c*2/3                                    ));*/
                        }
                    }
            }

    if(Bx != NULL) *Bx = B[0];
    if(By != NULL) *By = B[1];
    if(Bz != NULL) *Bz = B[2];
    return sqrt(B[0]*B[0]+B[1]*B[1]+B[2]*B[2]);
    //return cbs;
}

double CrystalFields::TriangleDipoleFieldchange(double m, double Bhyp,
                                                double d,
                                                double theta, double phi){
    double B[3] = {0,0,0};
    add_dipol(d*(1+cos(M_PI/3)), d*sin(M_PI/3),0,m,theta*DegRad,phi*DegRad,B);
    add_dipol(d*(1+cos(M_PI/3)),-d*sin(M_PI/3),0,m,theta*DegRad,phi*DegRad,B);
    B[0] -= Bhyp * sin(theta*DegRad)*cos(phi*DegRad);
    B[1] -= Bhyp * sin(theta*DegRad)*sin(phi*DegRad);
    B[2] -= Bhyp * cos(theta*DegRad);

    return norm(B,3)-Bhyp;
}

QString CrystalFields::Fe3PO4O3_Vii(int EZ, double* EW, double *EV){
    log.clear();
    // atomic positions
    a = 8.006;
    c = 6.863;
    double h_O1 = 0.2289*c;
    double h_O2 = 0.5153*c;
    double d_O2 = 0.2178585015*a;
    double h_O3 = 0.6033*c;
    double d_O3 = 0.3977846721*a;
    double h_Fe = 0.4092*c;
    double d_Fe = 0.2257439553*a;

    // probe position
    double X = cos(-M_PI/6.)*d_Fe;
    double Y = sin(-M_PI/6.)*d_Fe;
    double Z = h_Fe;

    // add all point charges
    double Vij[6] = {0,0,0,0,0,0};
    for(int ii=-EZ; ii <= EZ; ii++)
        for(int jj=-EZ; jj <= EZ; jj++)
            for(int kk = -EZ; kk <= EZ; kk++)
                // for each of the three Phosphor along the diagonal of the unit cell
                for(int Ph = 0; Ph < 3; Ph ++){
                    log.append("\n");
                    double ph = Ph/3.;
                    double x_Ph = a*             (ii-ph-(jj+ph)*cos(M_PI/3.));
                    double y_Ph = a*sin(M_PI/3.)*(jj+ph);
                    double z_Ph = c*             (kk+ph);

                    add_point_charge(x_Ph-X,y_Ph-Y,z_Ph     -Z,+5,Vij);  // Ph5+
                    add_point_charge(x_Ph-X,y_Ph-Y,z_Ph+h_O1-Z,-2,Vij);  // O 2-

                    // add three iron moments for each phospor
                    for(int Fei = 0; Fei < 3; Fei++){
                        double phi = M_PI*(-1./6. +2./3.*Fei);
                        double phi02 = phi + M_PI/3.;
                        add_point_charge(x_Ph + cos(phi)  *d_Fe - X,       // Fe3+
                                         y_Ph + sin(phi)  *d_Fe - Y,
                                         z_Ph + h_Fe            - Z,+3,Vij);
                        add_point_charge(x_Ph + cos(phi02)*d_O2 - X,       // O 2-
                                         y_Ph + sin(phi02)*d_O2 - Y,
                                         z_Ph + h_O2            - Z,-2,Vij);
                        add_point_charge(x_Ph + cos(phi)  *d_O3 - X,       // O 2-
                                         y_Ph + sin(phi)  *d_O3 - Y,
                                         z_Ph + h_O3            - Z,-2,Vij);
                    }
            }
    //log.append("\n");
    /*add_point_charge(-1.0,0,0,2,Vij);
    add_point_charge( 1.0,0,0,2,Vij);
    add_point_charge(0,-1.1,0,2,Vij);
    add_point_charge(0, 1.1,0,2,Vij);
    add_point_charge(0,0,-1.5,2,Vij);
    add_point_charge(0,0, 1.5,2,Vij);*/

    // calculate principal axes
    eigen_3x3(Vij,EW,EV);

    return log;
}

void CrystalFields::set_canting(double theta_canting, double phi_canting){
    d_theta = theta_canting/180.*M_PI;
    d_phi = phi_canting/180.*M_PI;
}

double CrystalFields::B_dipol_pnictide(double a, double b, double c,
                                       double x, double y, double z, double m,
                                       int i, int j, int k,
                                       double* Bx, double* By, double* Bz,
                                       bool AFM_interlayer_coupling, MagStructure mtype){
    double B[3];
    B[0] = B[1] = B[2] = 0;
    double ILc = -1;
    double theta1,phi1,theta2,phi2,theta3,phi3,theta4,phi4;
    switch(mtype){
        case stripeAFM:
            theta1 = theta3 = M_PI*1/2.+d_theta;
            theta2 = theta4 = M_PI*1/2.-d_theta;
            phi1 = phi3 = d_phi; phi2 = phi4 = M_PI+d_phi;
            break;
        case bicolinearAFM:
            theta1 = theta2 = theta3 = theta4 = M_PI*1./2.;
            phi1 = phi2 = phi3 = M_PI*3./4.; phi4 = -M_PI*1./4.;
            break;
        case checkerboard:
            theta1 = theta2 = theta3 = theta4 = M_PI*1./2.;
            phi1 = phi2 = phi3 = phi4 = 0;
            break;
        case aFM:
            theta1 = theta2 = theta3 = theta4 = M_PI*1./2.;
            phi1 = phi2 = phi3 = phi4 = 0;
            break;
        case cFM:
            theta1 = theta2 = theta3 = theta4 = 0;
            phi1 = phi2 = phi3 = phi4 = 0;
            break;
        case stripeAFM_c_axis:
            theta1 = theta3 = 0; theta2 = theta4 = M_PI;
            phi1 = phi2 = phi3 = phi4 = 0;
            break;
    }

    if(!AFM_interlayer_coupling) ILc = +1;
    for(int ii=-i; ii <= i; ii++)
        for(int jj=-j; jj <= j; jj++)
            for(int kk = -k; kk <= k; kk += 2){
                if(mtype == checkerboard || mtype == bicolinearAFM)
                    if((ii+jj) % 2 != 0) m = - qAbs(m);else m = + qAbs(m);
                add_dipol(x-ii*a        ,y+jj*b        ,z-kk*c    ,     m,theta1,phi1,B);
                add_dipol(x-ii*a + 0.5*a,y+jj*b        ,z-kk*c    ,     m,theta2,phi2,B);
                add_dipol(x-ii*a        ,y+jj*b + 0.5*b,z-kk*c    ,     m,theta3,phi3,B);
                add_dipol(x-ii*a + 0.5*a,y+jj*b + 0.5*b,z-kk*c    ,     m,theta4,phi4,B);
                add_dipol(x-ii*a        ,y+jj*b        ,z-kk*c + c,ILc* m,theta1,phi1,B);
                add_dipol(x-ii*a + 0.5*a,y+jj*b        ,z-kk*c + c,ILc* m,theta2,phi2,B);
                add_dipol(x-ii*a        ,y+jj*b + 0.5*b,z-kk*c + c,ILc* m,theta3,phi3,B);
                add_dipol(x-ii*a + 0.5*a,y+jj*b + 0.5*b,z-kk*c + c,ILc* m,theta4,phi4,B);
            }
    if(Bx != NULL) *Bx = B[0];
    if(By != NULL) *By = B[1];
    if(Bz != NULL) *Bz = B[2];
    return sqrt(B[0]*B[0]+B[1]*B[1]+B[2]*B[2]);
}

void CrystalFields::add_dipol(double x, double y, double z, double m,
                              double mtheta, double mphi, double* B){
    double mx = m * sin(mtheta)*cos(mphi);
    double my = m * sin(mtheta)*sin(mphi);
    double mz = m * cos(mtheta);
    double r = sqrt(x*x+y*y+z*z)*1e-10;
    double ex = x*1e-10/r;
    double ey = y*1e-10/r;
    double ez = z*1e-10/r;
    B[0] += 9.27499994e-31/(r*r*r)*(3*(ex*mx+ey*my+ez*mz)*ex-mx);
    B[1] += 9.27499994e-31/(r*r*r)*(3*(ex*mx+ey*my+ez*mz)*ey-my);
    B[2] += 9.27499994e-31/(r*r*r)*(3*(ex*mx+ey*my+ez*mz)*ez-mz);
}
double CrystalFields::B_FeSeIntercalated_fm(double m, int cycles){
    m *= 9.27400968e-24;         // Bohr magnetons
    double B[3];
    double Bsum = 0;
    for(int i = 0; i < cycles; i++){
        init_array(B,3,0);
        for(double z = -(Nc-0.5)*c; z < Nc*c; z+=c){
            double phi = floor(gsl_rng_uniform(r)*4)/2*M_PI;
            for(double x = -Na*a; x < (Na+0.5)*a; x+=a)
                for(double y = -Na*a; y < (Na+0.5)*a; y+=a){
                    double theta = 0;//gsl_rng_uniform(r)*M_PI;
                    if(gsl_rng_uniform(r) < 0.2)
                        add_dipol(x,y,z,m,theta,phi,B);
                    else
                        if(gsl_rng_uniform(r) < 0.2)
                            add_dipol(x,y,z,m,theta+M_PI,phi,B);
                }
        }
        Bsum += sqrt(B[0]*B[0]+B[1]*B[1]+B[2]*B[2]);
    }
    return Bsum / cycles;
}
double CrystalFields::B_FeSeIntercalated_fm_Li(double m, int cycles){
    m *= 9.27400968e-24;         // Bohr magnetons
    double B[3];
    double Bsum = 0;
    for(int i = 0; i < cycles; i++){
        init_array(B,3,0);
        for(double z = -Nc*c; z < (Nc+0.5)*c; z+=c){
            double phi = floor(gsl_rng_uniform(r)*4)/2*M_PI;
            for(double x = -Na*a; x < (Na+0.5)*a; x+=a)
                for(double y = -Na*a; y < (Na+0.5)*a; y+=a)
                    if(x > 1e-14 || y > 1e-14 || z > 1e-14){
                        double theta = 0;// = gsl_rng_uniform(r)*M_PI;
                        if(gsl_rng_uniform(r) < 0.2)
                            add_dipol(x,y,z,m,theta,phi,B);
                        else
                            if(gsl_rng_uniform(r) < 0.2)
                                add_dipol(x,y,z,m,theta+M_PI,phi,B);
                    }
        }
        Bsum += sqrt(B[0]*B[0]+B[1]*B[1]+B[2]*B[2]);
    }
    return Bsum / cycles;
}
void CrystalFields::Vzz_FeSe(double q, double alpha, double SeVacancies, int cycles, double* Vij){
    double Se_z = a/2./tan(alpha/2./180.*M_PI);
    init_array(Vij,6,0);
    q /= cycles;

    for(int i = 0; i < cycles; i++){
        for(double z = -Nc*c; z < (Nc+0.5)*c; z+=c)
            for(double x = -Na*a; x < (Na+0.5)*a; x+=a)
                for(double y = -Na*a; y < (Na+0.5)*a; y+=a){
                    add_point_charge(x,y,z,q,Vij);
                    add_point_charge(x+a/2.,y+a/2.,z,q,Vij);
                    if(gsl_rng_uniform(r) < 1-SeVacancies)
                        add_point_charge(x+a/2.,y,z+Se_z,-q,Vij);
                    if(gsl_rng_uniform(r) < 1-SeVacancies)
                        add_point_charge(x,y+a/2.,z-Se_z,-q,Vij);
                }
    }
}
double* CrystalFields::Vzz_FeSe_excessFe(double q, double alpha,
                                         double SeVacancies, int cycles,
                                         double FeExcess, double zFeExcess){
    double *V = new double[6];
    double Se_z = 0.5*a/tan(alpha/180.*M_PI);
    init_array(V,6,0);
    //add_point_charge(-a/2.,   0.,+Se_z,q,V);
    q /= cycles;
    double zexcess = c * zFeExcess;

    for(int i = 0; i < cycles; i++){
        for(double z = -Nc*c-zexcess; z < (Nc+0.5)*c - zexcess; z+=c)
            for(double x = -Na*a; x < (Na+0.5)*a; x+=a)
                for(double y = -Na*a-a/2; y < (Na+0.5)*a; y+=a){
                    add_point_charge(x,y,z,q,V);
                    add_point_charge(x+a/2.,y+a/2.,z,q,V);
                    if(gsl_rng_uniform(r) < 1-SeVacancies)
                        add_point_charge(x+a/2.,y,z+Se_z,-q,V);
                    if(gsl_rng_uniform(r) < 1-SeVacancies)
                        add_point_charge(x,y+a/2.,z-Se_z,-q,V);
                    if(gsl_rng_uniform(r) < FeExcess)
                        add_point_charge(x,y+a/2.,z+zexcess,q,V);
                    if(gsl_rng_uniform(r) < FeExcess)
                        add_point_charge(x+a/2.,y,z-zexcess,q,V);
                }
    }
    return V;
}

void CrystalFields::add_point_charge(double x, double y, double z, double q, double *Vij){
    double C = q*1.602176565e-19/(4*M_PI*8.85418781762e-12*1e-10);
    double r = sqrt(x*x+y*y+z*z);
    if(r > 0.05){
        ;//log.append(QString("%1\t%2\t%3\t").arg(x).arg(y).arg(z));
        Vij[0] += (3*x*x*C)/(r*r*r*r*r)-C/(r*r*r);
        Vij[1] += (3*y*y*C)/(r*r*r*r*r)-C/(r*r*r);
        Vij[2] += (3*z*z*C)/(r*r*r*r*r)-C/(r*r*r);
        Vij[3] += (3*x*y*C)/(r*r*r*r*r);
        Vij[4] += (3*x*z*C)/(r*r*r*r*r);
        Vij[5] += (3*y*z*C)/(r*r*r*r*r);
    }
    else
        ;//log.append(QString("\t\t\t"));

}
void CrystalFields::eigen_3x3(double* Vij,double *EW, double* EV){
    gsl_eigen_symmv_workspace * w1 = gsl_eigen_symmv_alloc (3);
    gsl_matrix* ssg = gsl_matrix_alloc(3,3);
    gsl_matrix* ssEV = gsl_matrix_alloc(3,3);
    gsl_vector* ssEW = gsl_vector_alloc(3);
    gsl_matrix_set(ssg,0,0,Vij[0]);
    gsl_matrix_set(ssg,0,1,Vij[3]);
    gsl_matrix_set(ssg,0,2,Vij[4]);
    gsl_matrix_set(ssg,1,0,Vij[3]);
    gsl_matrix_set(ssg,1,1,Vij[1]);
    gsl_matrix_set(ssg,1,2,Vij[5]);
    gsl_matrix_set(ssg,2,0,Vij[4]);
    gsl_matrix_set(ssg,2,1,Vij[5]);
    gsl_matrix_set(ssg,2,2,Vij[2]);
    gsl_eigen_symmv (ssg, ssEW, ssEV, w1);
    // now sort by EW
    int is[3] = {0,0,0};
    for(int i = 1; i < 3; i++){
        if(abs(gsl_vector_get(ssEW,i)) > abs(gsl_vector_get(ssEW,is[0])))
            is[0] = i;
        if(abs(gsl_vector_get(ssEW,i)) < abs(gsl_vector_get(ssEW,is[2])))
            is[2] = i;
    }
    is[1] = 3-is[0]-is[2];
    // return values
    for(int i = 0; i < 3; i++){
        EW[i] = gsl_vector_get(ssEW,is[i]);
        EV[i*3+0] = gsl_matrix_get(ssEV,0,is[i]);
        EV[i*3+1] = gsl_matrix_get(ssEV,1,is[i]);
        EV[i*3+2] = gsl_matrix_get(ssEV,2,is[i]);
    }
    gsl_eigen_symmv_free(w1);
    gsl_matrix_free(ssg);
    gsl_matrix_free(ssEV);
    gsl_vector_free(ssEW);
}
