#ifndef CRYSTALFIELDS_H
#define CRYSTALFIELDS_H
#define _USE_MATH_DEFINES
#define DegRad 0.01745329251994329576923690768489
#define RadDeg 57.295779513082320876798154814105

#include <cmath>
#include <QApplication>
#include <QtGlobal>
#include <QString>
#include "array.h"
#include <gsl/gsl_rng.h>
#include <gsl/gsl_eigen.h>


enum MagStructure{stripeAFM,bicolinearAFM,aFM,cFM,stripeAFM_c_axis,checkerboard};

class CrystalFields
{
public:
    CrystalFields(double a, double c, int na, int nc);
    double B_FeSeIntercalated_fm(double m, int cycles);
    double B_FeSeIntercalated_fm_Li(double m, int cycles);
    void Vzz_FeSe(double q, double alpha, double SeVacancies, int cycles,
                     double* Vij);
    double* Vzz_FeSe_excessFe(double q, double alpha,
                              double SeVacancies, int cycles,
                              double FeExcess, double zFeExcess);
    void add_dipol(double x, double y, double z, double m,
                   double mtheta, double mphi, double* B);
    void add_point_charge(double x, double y, double z, double q, double *Vzz);
    void eigen_3x3(double* Vij,double *EW, double* EV);
    //! calculates the dipol field for FeAs- or FeCh-compound, mtype order
    /*! Lattice parameters (a,b,c) are required for the orthorhombic lattice.
     *  At (0,0,0) position iron is to be found, thus a usual muon dipole field
     *  calculation may have the parameters (x,y,z)=(0.25*a,0.25*b,0.6). With
     *  i, j, and k and count of magnetic unit cells in x,y and z-direction has
     *  to be specified. Set AFM_interlayer_coupling=true for normal AFM
     *  coupling along the c-axis, else FM coupling is assumed. If the single
     *  vector components of the B-field are required you should suply Bx, By
     *  and Bz double-pointers, if not type NULL as argument
     */
    double B_dipol_pnictide(double a, double b, double c,
                              double x, double y, double z, double m,
                              int i, int j, int k,
                              double* Bx, double* By, double* Bz,
                              bool AFM_interlayer_coupling, MagStructure mtype);
    //! defines a canting of moments, applied in B_dipol_pnictides
    /*! set canting before dipole field calculation, check implementation
     *  at the moment canting is available only in stripeAFM
     */
    void set_canting(double theta_canting, double phi_canting);
    double B_dipol_stripe_AFM(double a, double b, double c,
                              double x, double y, double z, double m,
                              int i, int j, int k,
                              double* Bx, double* By, double* Bz,
                              bool AFM_interlayer_coupling);
    double B_dipol_bicolinear_AFM(double a, double b, double c,
                              double x, double y, double z, double m,
                              int i, int j, int k,
                              double* Bx, double* By, double* Bz,
                              bool AFM_interlayer_coupling);
    //! Fe3PO4O3 dipole field calculation
    double Fe3PO4O3_dipol(double a, double c,
                          double x, double y, double z, double m,
                          int i, int j, int k,
                          double* Bx, double* By, double* Bz,
                          double* P);
    //! Fe3PO4O3 field gradient calculation
    QString Fe3PO4O3_Vii(int EZ, double* EW, double *EV);
    //! Dipolcontribution to Bloc Mössbauer
    double TriangleDipoleFieldchange(double m, double Bhyp, double d,
                                     double theta, double phi);

private:
    double a,c;
    int Na,Nc;
    double mu_0;
    double d_theta, d_phi,maxd;
    const gsl_rng_type * T;
    gsl_rng * r;
    QString log;

};

#endif // CRYSTALFIELDS_H
