﻿#include "fitmodel.h"


/* Instruction to include a spectrum constructor
 * 1.) add to read_theory()
 * 2.) add MStyp at fitmodel.h
 * 3.) add to pc() function
 * 4.) add to build_spectrum() at "to now the spectrum has to be calculated"
 *     positions
 */
double operate_fitmodel(void* instance);
double operate_single_run(void* instance, int runi);
double operate_fitting(void* instance);
bool operate_correlation(void* instance, int par1, int par2);
fitmodel::fitmodel(QString filename){
    init();
    InputFile.append(filename);
    int FitparametersIn = 0,cparablock = 0,
        TheoryIn = 0,ctheoryblock = 0,
        FunctionsIn = 1, cfunblock = 0,
        RunIn = 0, crunblock = 0,
        PlotIn = 0, cplotblock = 0,
        Commands = 1, ccomblock = 0;
    FILE* f = fopen(filename.toLocal8Bit().data(),"r");
    if(f != NULL){
        char buffer[10000];
        while(!feof(f)){
            fgetss(buffer,10000,f);
            if(!strcmp(buffer,"COMMANDS\n"))
                Commands = (ccomblock++ < 1) * read_commands(f);
            if(!strcmp(buffer,"FITPARAMETER\n"))
                FitparametersIn = (cparablock++ < 1) * read_fitparameters(f);
            if(!strcmp(buffer,"THEORY\n"))
                TheoryIn = (ctheoryblock++ < 1) * read_theory(f);
            if(!strcmp(buffer,"FUNCTIONS\n"))
                FunctionsIn = (cfunblock++ < 1) * read_functions(f);
            if(!strcmp(buffer,"RUN\n"))
                RunIn = (crunblock++ < 1) * read_run(filename.toLocal8Bit().data(),f);
            if(!strcmp(buffer,"PLOT\n"))
                PlotIn = (cplotblock++ < 1) * read_plot(f);
            buffer[0]='\0';
        }
        fclose(f);
    }
    else{
        ErrorMessage.append("File couldn't be opened.\n");
    }
    if(FitparametersIn == 1 && TheoryIn == 1 && FunctionsIn == 1 && RunIn == 1 && Commands == 1)
        if(link()){
            if(process_theory()){
                if(!read_fitdata()|| !read_MEMdata())
                    QMessageBox(QMessageBox::NoIcon,"Parse-Errors",ErrorMessage,0,0).exec();
                //QTime myTimer;
                //myTimer.start();
                for(int i = 0; i < crun; i++)
                    //for(int j = 0; j < 500; j++)
                        calc_run(i);
                //int nMilliseconds = myTimer.elapsed();
                //QMessageBox(QMessageBox::NoIcon,QString("Time"),QString("%1").arg(nMilliseconds)).exec();
                isvalid = 1;
            }
            else{
                isvalid = 0;
                ErrorMessage.append("Processing of THEORY failed.\n");
            }
        }
        else{
            isvalid = 0;
            ErrorMessage.append("Failed to link all symbols\n");
        }
    else{
        isvalid = 0;
        if(!FitparametersIn){
            if(cparablock < 2)  ErrorMessage.append("FITPARAMETER block is not valid.\n");
            else                ErrorMessage.append("Too many FITPARAMETER blocks\n");}
        if(!TheoryIn){
            if(ctheoryblock < 2)  ErrorMessage.append("THEORY block is not valid.\n");
            else                ErrorMessage.append("Too many THEORY blocks\n");}
        if(!FunctionsIn){
            if(cfunblock < 2)  ErrorMessage.append("FUNCTIONS block is not valid.\n");
            else                ErrorMessage.append("Too many FUNCTIONS blocks\n");}
        if(!RunIn){
            if(crunblock < 2)  ErrorMessage.append("RUN block is not valid.\n");
            else                ErrorMessage.append("Too many RUN blocks\n");}
        if(!Commands){
            if(ccomblock < 2)  ErrorMessage.append("COMMANDS block is not valid.\n");
            else                ErrorMessage.append("Too many COMMANDS blocks\n");}
    }
    if(! isvalid)
        QMessageBox(QMessageBox::NoIcon,"Parse-Errors",
                    ErrorMessage,0,0).exec();
    else{
        /*play pannel anlegen*/
        double* T = new double[crun];
        QStringList sl;
        for(int i = 0; i < crun; i++)
            T[i] = run[i].T;
        for(int i = 0; i < cpara; i++)
            sl.append(fmp[i].name);
        fmpa = new fmPanel(rData,rError,cpara,crun,T,&sl);
        connect(fmpa,SIGNAL(recalc_baseline_area(int)),this,SLOT(calc_baseline_area(int)));
        connect(fmpa,SIGNAL(rewrite_FITPARAMETER()),this,SLOT(rewrite_FITPARAMETER()));
        connect(fmpa,SIGNAL(recalc(int,int)),this,SLOT(panel_input(int,int)));
        connect(fmpa,SIGNAL(view_run(int)),this,SLOT(view_run(int)));
        connect(fmpa,SIGNAL(view_run_only(int)),this,SLOT(view_run_only(int)));
        connect(fmpa,SIGNAL(hide_run(int)),this,SLOT(hide_run(int)));
        connect(fmpa,SIGNAL(reset_error(int,int,bool)),this,SLOT(reset_error(int,int,bool)));
        //fmpa->show();
        //fmpa->lower();
    }
    fitruns = 0;
}
fitmodel::fitmodel(){init(); fitruns = 0;}
fitmodel::~fitmodel(){
    //delete play pannel
    //if(fmpa != NULL)
    //    fmpa->close();

    if(fitruns)
        while(fitruns)
            QMessageBox(QMessageBox::NoIcon,"Error","Wait for the fitting procedure to finish").exec();

    //runs löschen
    if(run != NULL)
        delete[] run ;

    //Fitparameter loeschen
    if(fmp != NULL)
        delete[] fmp;

    //Funktionen loeschen
    if(fun != NULL)
        delete[] fun;

    //Theorie loeschen
    if(spectra != NULL)
        delete[] spectra ;

    //LINK loeschen
    if(pData != NULL) delete pData;
    if(pError != NULL) delete  pError;
    if(pIsChanged != NULL) delete pIsChanged;
    if(affect != NULL) delete affect;
    if(rData != NULL){
        for(int i = 0; i < (cpara); i++)
            delete rData[i];
        delete rData ;
    }
    if(rError != NULL){
        for(int i = 0; i < (cpara); i++)
            delete rError[i];
        delete rError ;
    }
    if(raffect != NULL){
        for(int i = 0; i < (cpara); i++)
            delete raffect[i];
        delete raffect ;
    }
}
short fitmodel::from_fld(char* filename,double** xdata, double** ydata,
                        int* count){
    return from_fld(filename,xdata,ydata,NULL,count);
}
short fitmodel::from_fld(char* filename,double** xdata, double** ydata,
                         double**yerror2rec, int* count){
    FILE *f = open_file(filename);
    if(f != NULL){
        reset(xdata,ydata,yerror2rec);
        // count real data lines
        *count = 0;
        char buffer[1000];
        bool doubleOK[3];
        while(!feof(f)){
            buffer[0] = '\0';
            fgetss(buffer,10000,f);
            QStringList dl = QString(buffer).split("\t");
            // check for valid data
            if(dl.length()>1){
                dl[0].toDouble(&(doubleOK[0]));     // valid velocity data
                dl[1].toDouble(&(doubleOK[1]));     // valid count data
                if(yerror2rec != NULL && dl.length() > 2)
                    dl[2].toDouble(&(doubleOK[2])); // valid error data
                else
                    doubleOK[2] = true;
                if(doubleOK[0] == true && doubleOK[1] == true && doubleOK[2] == true)
                    // valid velocity-count(-error) data
                    (*count)++;
            }
        }
        if(*count > 0){
            *xdata = new double[*count];
            *ydata = new double[*count];
            if(yerror2rec != NULL)
                *yerror2rec = new double[*count];

            fseek(f,0,SEEK_SET);
            int i = 0;
            // read valid data
            while(!feof(f)){
                buffer[0] = '\0';
                fgetss(buffer,10000,f);
                QStringList dl = QString(buffer).split("\t");
                if(dl.length()>1){
                    (*xdata)[i] = dl[0].toDouble(&(doubleOK[0]));
                    (*ydata)[i] = dl[1].toDouble(&(doubleOK[1]));
                    if(yerror2rec != NULL && dl.length() > 2){
                        (*yerror2rec)[i] = dl[2].toDouble(&(doubleOK[2]));
                        (*yerror2rec)[i] *= (*yerror2rec)[i];
                    }
                    else{
                        if(yerror2rec != NULL)
                            (*yerror2rec)[i] = (*ydata)[i];
                        doubleOK[2] = true;
                    }
                    if(doubleOK[0] == true && doubleOK[1] == true && doubleOK[2] == true)
                        i++;
                }
            }
            fclose(f);
            // prepare error data
            if(yerror2rec != NULL)
                for(int i = 0; i < *count; i++)
                    (*yerror2rec)[i] = 1./(qMax(1e-50,(*yerror2rec)[i]));
            return 1;
        }
        else
            return from_fld_unfolded(filename,xdata,ydata,yerror2rec,count);
    }
    else{
        ErrorMessage.append("Could not open ").append(filename).append(".\n");
        *count = 0;
        return 0;
    }
}
int fitmodel::from_fld_unfolded(char* filename,double** xdata, double** ydata,
                      double**yerror2rec, int* count){
    FILE *f = open_file(filename);
    if(f != NULL){
        reset(xdata,ydata,yerror2rec);
        // count real data lines
        *count = 0;
        char buffer[1000];
        bool OK;
        while(!feof(f)){
            buffer[0] = '\0';
            fgetss(buffer,10000,f);
            QString(buffer).toDouble(&OK);
            if(OK) (*count)++;
        }
        if(*count > 0){
            *xdata = new double[*count];
            *ydata = new double[*count];
            fseek(f,0,SEEK_SET);
            int i = 0;
            // read valid data
            while(!feof(f)){
                buffer[0] = '\0';
                fgetss(buffer,10000,f);
                double val = QString(buffer).toDouble(&OK);
                if(OK)
                    (*ydata)[i] = val;
                i++;
            }
            fclose(f);
            // generate error data
            if(yerror2rec != NULL){
                *yerror2rec = new double[*count];
                for(i = 0; i < *count; i++)
                    (*yerror2rec)[i] = 1./qMax(1e-50,qAbs((*ydata)[i]));
            }
            // generate temporary xdata
            for(i = 0; i < *count; i++)
                (*xdata)[i] = 5.*i/(*count);
            return 1;
        }
        else{
            ErrorMessage.append(filename).append(" contains no valid data.\n");
            return 0;
        }
    }
    else{
        ErrorMessage.append("Could not open ").append(filename).append(".\n");
        *count = 0;
        return 0;
    }
}
void fitmodel::reset(double** xdata, double** ydata, double**yerror2rec){
    if (*xdata  != NULL){delete *xdata; *xdata = NULL;}
    if (*ydata  != NULL){delete *ydata; *ydata = NULL;}
    if ( yerror2rec != NULL
      &&*yerror2rec != NULL){delete *yerror2rec; *yerror2rec = NULL;}
}
FILE* fitmodel::open_file(char *filename){
    FILE *f = fopen(filename,"r");
    if(f == NULL){
        QString test = QFileInfo(InputFile).dir().filePath(QString(filename));
        f = fopen(test.toLocal8Bit().data(),"r");
    }
    return f;
}
void fitmodel::save_linear_v_spectra(){
    for(int i = 0; i < crun; i++){
        double minv = min_array(run[i].v,run[i].cc);
        double maxv = max_array(run[i].v,run[i].cc);
        double dv = (maxv-minv)/265.;
        QString newfilename = QString("%1.fld").arg(run[i].T);
        FILE* fnew = fopen(QFileInfo(InputFile).dir().filePath(newfilename).toLocal8Bit().data(),"w");
        for(int k = 0; k < 256; k++){
            double vmin = minv + k/256.*(maxv-minv);
            double vmax = vmin + dv;
            double v0 = vmin + dv/2.;
            double vl,vr;
            double Itemp = 0;
            double w = 0;
            for(int j = 1; j < run[i].cc-1; j++){
                double v0min = (run[i].v[j]+run[i].v[j-1])/2;
                double v0max = (run[i].v[j]+run[i].v[j+1])/2;
                if(v0min>vmin) vl = v0min;
                else           vl = vmin;
                if(v0max<vmax) vr = v0max;
                else           vr = vmax;
                if(vr-vl > 0){
                    w += (vr-vl)/(vmax-vmin);
                    Itemp += run[i].I[j]*(vr-vl)/(vmax-vmin);
                }
            }
            fprintf(fnew,"%f\t%f\n",v0,Itemp/w);
        }
        fclose(fnew);
    }
}

short fitmodel::load_run(char* mbspath, char* T,int index){
    QDir folder = QFileInfo(QString(mbspath)).dir();
    if(!folder.cd(QString(T).append("K")))
        if(!folder.cd(QString(T))){
            ErrorMessage.append(QString("The following RUN does not exist: ").append(T).append("\n"));
            return 0;
        }
    folder.setNameFilters(QStringList("*.fld"));
    folder.setFilter(QDir::Files);
    folder.setSorting(QDir::Time);

    if(folder.entryList().count() == 0){
        ErrorMessage.append(folder.path())
                    .append(" doesn't contain a *.fld file.\n");
        return 0;
    }

    if(from_fld(folder.path().append("/").append(folder.entryList().at(0))
                .toLocal8Bit().data(),
                &(run[index].v),
                &(run[index].I),
                &(run[index].err2rec),
                &(run[index].cc))){
        run[index].T = QString(T).replace(",",".").toDouble();
        //MoessCalc().baseline(run[index].v,run[index].I,run[index].cc,
        //                     &(run[index].I0),&(run[index].A),1./0.13,0.13,
        //                     NULL,NULL,NULL,NULL);
        return 1;
    }
    else{
        ErrorMessage.append(folder.path())
                    .append("/")
                    .append(folder.entryList().at(0))
                    .append(" does not provide run data.\n");
        return 0;
    }
}
int fitmodel::link(){
    qSort(obj.begin(),obj.end());
    int linkOK = 1;
    cSpec = cGlobal = cConst = 0;
    for(int i = 0 ; i < cpara; i++)
            if(fmp[i].step == 0)        cConst ++;
            else
                if(fmp[i].runspecific)  cSpec++;
                else                    cGlobal++;
    pData = new double[cConst*crun+cSpec*crun+cGlobal];
    pError = new double[cConst*crun+cSpec*crun+cGlobal];
    pIsChanged = new int[cConst*crun+cSpec*crun+cGlobal];
    affect = new int[cConst*crun+cSpec*crun+cGlobal];
    /*Erstelle Referenzpool*/
    rData =  new double**[cpara];
    rError = new double**[cpara];
    raffect =   new int**[cpara];
    for(int i = 0; i < (cpara); i++){
        rData[i] = new double*[crun+1];         //+1 to keep the initial value
        rError[i] = new double*[crun+1];        //+1 to keep the initial value
        raffect[i] = new int*[crun];
    }
    /*initialisiere Werte in Variablenpool und referenziere zu Referenzpool*/
    for(int j = 0; j < cGlobal + cSpec * crun; j++) pIsChanged[j] = 0;
    int ciGlobal = 0,
        ciSpec  = cGlobal,
        ciConst = cGlobal + cSpec * crun;
    for(int i = 0 ; i < cpara; i++){
        rData[i][crun] = &(fmp[i].value);   //to access initial value
        rError[i][crun] = &(fmp[i].step);   //to access initial value
        if(fmp[i].step == 0){
            for(int j = 0; j < crun; j++){
                pData[ciConst+j*cConst] = fmp[i].value;
                pError[ciConst+j*cConst] = 0;
                affect[ciConst+j*cConst] = -2;
                rData[i][j] = &(pData[ciConst+j*cConst]);
                rError[i][j] = &(pError[ciConst+j*cConst]);
                raffect[i][j] = &(affect[ciConst+j*cConst]);
            }
            ciConst ++;
        }
        else
            if(fmp[i].runspecific){
                for(int j = 0; j < crun; j++){
                    pData[ciSpec+j*cSpec] = fmp[i].value;
                    pError[ciSpec+j*cSpec] = fmp[i].step;
                    affect[ciSpec+j*cSpec] = j;
                    rData[i][j] = &(pData[ciSpec+j*cSpec]);
                    rError[i][j] = &(pError[ciSpec+j*cSpec]);
                    raffect[i][j] = &(affect[ciSpec+j*cSpec]);
                }
                ciSpec++;
            }
            else{
                pData[ciGlobal] = fmp[i].value;
                pError[ciGlobal] = fmp[i].step;
                affect[ciGlobal] = -1;
                for(int j = 0; j < crun; j++){
                    rData[i][j] = &(pData[ciGlobal]);
                    rError[i][j] = &(pError[ciGlobal]);
                    raffect[i][j] = &(affect[ciGlobal]);
                }
                ciGlobal++;
            }
    }
    /*referenziere Map-Abschnitt*/
    for(int i = 0; i < crun; i++){
        run[i].map = new int[run[i].maplist.length()];
        for(int j = 0; j < run[i].maplist.length(); j++){
            run[i].map[j] = find_obj(run[i].maplist[j]);
            if(run[i].map[j] == -1){
                linkOK = 0;
                ErrorMessage.append(QString("run%1, map %2 couldn't be linked.\n")
                                    .arg(i).arg(run[i].maplist.at(j)));
            }
        }
    }
    /*Referenziere alle Symbole in Theory */
    for(int i = 0; i < cmbs; i++){
        if(spectra[i].typ == Dyn || spectra[i].typ == Dynbeta )
            spectra[i].p[0].arg[0] = find_obj(spectra[i].p[0].argv.at(0));
        for(int j = 0; j < pc(i); j++)
            for(int k = 0; k < ac(spectra[i].p[j].typ); k++){
                    spectra[i].p[j].arg[k] = find_obj(spectra[i].p[j].argv.at(k));
                    if(spectra[i].p[j].arg[k] == -1){
                        ErrorMessage.append(QString("Subspectrum%1 Parameter%2 argument %3 couldn't be linked.\n")
                                            .arg(i)
                                            .arg(j)
                                            .arg(spectra[i].p[j].argv.at(k)));
                        linkOK = 0;
                    }
                }
    }
    // parse and reference all function
    find_obj("rsp");                    // add run specific parameter to obj
    parse_functions(&linkOK);

    /*Referenziere alle Plots*/
    for(int i = 0; i < plotrunlist.length(); i++){
        int runi = 0;
        while(runi < crun && run[runi].T != plotrunlist.at(i))
            runi++;
        if(runi < crun)
            plotrunlist.replace(i,double(runi));
        else{
            ErrorMessage.append(QString("Plotting: %1-run not found.\n")
                                .arg(plotrunlist.at(i)));
            linkOK = 0;
        }
    }

    //link commands
    interprete_commands(&linkOK);

    /*check whether user wants to fit anything, than y-errors have to exist for every run*/
    for(int i = 0; i < cmbs; i++)
        if(spectra[i].typ == AnyFit)
            {AnyFit_in = 1; i = cmbs;}
   return linkOK;
}
int fitmodel::load_run(char* fldpath, double UseT, int index){
    run[index].v = NULL;
    run[index].I = NULL;
    if(from_fld(fldpath,
                &(run[index].v),
                &(run[index].I),
                &(run[index].err2rec),
                &(run[index].cc))){
        clone(&(run[index].Iunsorted),run[index].I,run[index].cc);
        mergesort(index);
        run[index].T = UseT;
        //MoessCalc().baseline(run[index].v,run[index].I,run[index].cc,
        //                     &(run[index].I0),&(run[index].A),1./0.13,0.13,
        //                     NULL,NULL,NULL,NULL);
        return 1;
    }
    else{
        ErrorMessage.append(fldpath)
                    .append(" does not provide run data.\n");
        return 0;
    }

}
int fitmodel::read_fitparameters(FILE*f){
    long int start = ftell(f);
    char buffer[10000]; buffer[0]='\0';
    /*Parameter zaehlen*/
    cpara = 0;
    while(!feof(f) && !blockend(buffer)){
        if(validline(buffer))
            cpara++;
        fgetss(buffer,10000,f);
    }
    if(cpara == 0)
        return 0;

    /*Parameter einlesen*/
    int paraOK = 1;
    fmp = new fmPara[cpara];
    cpara = 0;
    fseek(f,start,SEEK_SET);
    fgetss(buffer,10000,f);
    while(!feof(f) && !blockend(buffer)){
        if(validline(buffer)){
            QStringList l = QString(buffer).remove("\n")
                            .split(" ",QString::SkipEmptyParts);
            bool OK[6];
            if(l.length()>= 3){
                fmp[cpara].name.append(l.at(0));
                if(obj.indexOf(fmp[cpara].name)>=0){
                    ErrorMessage.append(QString("%1 was multiply defined.\n").arg(fmp[cpara].name));
                    paraOK = 0;
                }
                fmp[cpara].value    = l.at(1).toDouble(&OK[2]);
                fmp[cpara].step     = l.at(2).toDouble(&OK[3]);
                if(OK[2])
                    if(OK[3]){
                        fmp[cpara].runspecific = true;
                        if(!fmp[cpara].name.leftRef(7).compare("global_"))
                            fmp[cpara].runspecific = false;
                        // Anzahl Digits fuertausgabe bestimmen
                        int icomma3 =    std::max(l.at(1).indexOf("."),
                                                  l.at(1).indexOf(","));
                        int icomma2 =    std::max(l.at(2).indexOf("."),
                                                  l.at(2).indexOf(","));
                        if(icomma2 == -1) icomma2 = l.at(2).length();
                        if(icomma3 == -1) icomma3 = l.at(1).length();
                        icomma3 = std::max(l.at(1).length()-icomma3,
                                           l.at(2).length()-icomma2);
                        if(icomma3 >= 0)
                            fmp[cpara].printDigits = icomma3 + 2;
                        else
                            fmp[cpara].printDigits = 0;
                        obj.append(Obj(fmp[cpara].name,partyp,cpara));
                        cpara++;
                    }
                    else{
                        ErrorMessage.append(QString("Parameter %1 step is not a number.\n").arg(l.at(2)));
                        paraOK = 0;
                    }
                else{
                    ErrorMessage.append(QString("Parameter %1 value is not a number.\n").arg(l.at(1)));
                    paraOK = 0;
                }
            }
            else{
                ErrorMessage.append(
                                QString("Parameter %1 (\"%2\") is not complete.\n")
                                    .arg(cpara+1).arg(QString(buffer).remove("\n")));
                 paraOK = 0;
            }
        }
        fgetss(buffer,10000,f);
    }
    return paraOK;
}
int fitmodel::read_functions(FILE*f){
    long int start = ftell(f);
    char buffer[10000]; buffer[0]='\0';
    /*Funktionen zaehlen*/
    cfun = 0;

    fgetss(buffer,10000,f);
    while(!feof(f) && !blockend(buffer)){
        if(validline(buffer))
            cfun++;
        fgetss(buffer,10000,f);
    }
    if(cfun == 0)
        return 1;

    /*Funktionen einlesen*/
    int funsOK = 1;
    fun = new FunctionStruct[cfun];
    cfun = 0;
    fseek(f,start,SEEK_SET);
    fgetss(buffer,10000,f);
    while(!feof(f) && !blockend(buffer)){
        if(validline(buffer)){
            QString function(buffer);
            int equalsi = function.indexOf("=");
            if(equalsi >= 0){
                fun[cfun].name = function.left(equalsi);
                if(obj.indexOf(fun[cfun].name)>=0){
                    ErrorMessage.append(QString("%1 was multiply defined.\n").arg(fun[cfun].name));
                    funsOK = 0;
                }
                fun[cfun].name.remove(" ").remove("\n");
                fun[cfun].body = function.mid(equalsi+1);
                fun[cfun].body.remove(" ").remove("\n");
                obj.append(Obj(fun[cfun].name,funtyp,cfun));
                cfun++;
            }
            else{
                ErrorMessage.append(QString("There was no ""=""-sign in funtion %1.\n").arg(function));
                funsOK = 0;
            }
        }
        fgetss(buffer,10000,f);
    }
    return funsOK;

}
int fitmodel::read_theory(FILE*f){
    long int start = ftell(f);
    char buffer[10000]; buffer[0]='\0';
    /*Subspektren zaehlen*/
    cmbs = 0;
    fgetss(buffer,10000,f);
    while(!feof(f) && !blockend(buffer)){
        if(validline(buffer))
            cmbs++;
        fgetss(buffer,10000,f);
    }
    if(cmbs == 0){
        ErrorMessage.append("No Theory input.\n");
        return 0;
    }

    /*Subspektren einlesen*/
    int spectraOK = 1;
    spectra = new MoessbauerSpectrum[cmbs];
    cmbs = 0;
    buffer[0]='\0';
    QRegExp rMEM("(MEM\\[)(.+)(,)(.+)(,)(.+)(,)(.+)(\\])");
    QRegExp rMEMEXT("(MEM\\[)(.+)(,)(.+)(,)(.+)(,)(.+)(,)(.+)(\\])");
    QRegExp rHR("(HR\\[)(.+)(,)(.+)(,)(.+)(,)(.+)(\\])");
    QRegExp rHREXT("(HR\\[)(.+)(,)(.+)(,)(.+)(,)(.+)(\\])");
    QRegExp rDISTR("(DISTR\\[)(.+)(,)(.+)(,)(.+)(,)(.+)(,)(.+)(\\])");
    QRegExp rDISTREXT("(DISTR\\[)(.+)(,)(.+)(,)(.+)(,)(.+)(,)(.+)(,)(.+)(\\])");

    fseek(f,start,SEEK_SET);
    fgetss(buffer,10000,f);
    while(!feof(f) && !blockend(buffer)){
        if(validline(buffer)){
            QStringList l = QString(buffer).remove('\n')
                            .split(" ",QString::SkipEmptyParts);
            int vaildInput = 0;
            /*Art des Moessbauerspektrums bestimmen*/
            if(!l.at(0).compare("SkyrmionMBS")){
                if(l.length() ==  pc(SkyrmionMBS)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = SkyrmionMBS;
                }
                else
                    ErrorMessage.append(QString("SkyrmionMBS-site needs %1 parameters.").arg(pc(SkyrmionMBS)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("SHc")){
                if(l.length() ==  pc(SHcrystal)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = SHcrystal;
                }
                else
                    ErrorMessage.append(QString("SHc-site needs %1 parameters: B, Vzz, eta, theta, phi, CS, omega, A, beta, gamma.").arg(pc(SHcrystal)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("SHcG")){
                if(l.length() ==  pc(SHcrystalG)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = SHcrystalG;
                }
                else
                    ErrorMessage.append(QString("SHcG-site needs %1 parameters: B, Vzz, eta, theta, phi, CS, omega, A, beta, gamma.").arg(pc(SHcrystalG)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("SHcB")){
                if(l.length() ==  pc(SHcrystalB)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = SHcrystalB;
                }
                else
                    ErrorMessage.append(QString("SHcB-site needs %1 parameters: B, Vzz, eta, theta, phi, CS, omega, A, beta, gamma.").arg(pc(SHcrystalG)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }

            if(!l.at(0).compare("fmBex")){
                if(l.length() ==  pc(fmBexpowder)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = fmBexpowder;
                }
                else
                    ErrorMessage.append(QString("fmBex-site needs %1 parameters: Bafm, BA, Bex, Vzz, theta, CS, omega, A, N, Nvzz.").arg(pc(fmBexpowder)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("afmBex")){
                if(l.length() ==  pc(afmBexpowder)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = afmBexpowder;
                }
                else
                    ErrorMessage.append(QString("afmBex-site needs %1 parameters: Bafm, BA, BJ, Bex, Vzz, theta, CS, omega, A, N, Nvzz.").arg(pc(afmBexpowder)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("afmBexLong")){
                if(l.length() ==  pc(afmBexpowderLong)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = afmBexpowderLong;
                }
                else
                    ErrorMessage.append(QString("afmBexLong-site needs %1 parameters: Bafm, BA, BJ, Bex, Vzz, theta, CS, omega, A, N, Nvzz.").arg(pc(afmBexpowderLong)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("PseudoVoigt")){
                if(l.length() ==  pc(PseudoVoigt)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = PseudoVoigt;
                }
                else
                    ErrorMessage.append(QString("PseudoVoigt-site needs %1 parameters: CS, omega, eta, A.").arg(pc(PseudoVoigt)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("FeAs")){
                if(l.length() ==  pc(FeAspowder)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = FeAspowder;
                }
                else
                    ErrorMessage.append(QString("FeAs-site needs %1 parameters: T, CS0, A0, ThetaD, Tn.").arg(pc(FeAspowder)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("Fe2As")){
                if(l.length() ==  pc(Fe2Aspowder)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = Fe2Aspowder;
                }
                else
                    ErrorMessage.append(QString("Fe2As-site needs %1 parameters: T, CS0, A0, ThetaD, Tn, omega.").arg(pc(Fe2Aspowder)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("DynDiagDiag")){
                if(l.length() ==  pc(DynDiagDiag)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = DynDiagDiag;
                }
                else
                    ErrorMessage.append(QString("DynDiagDiag-site needs %1 parameters: B1, B2, Vzz1, Vzz2, CS1, CS2, omega, A, beta, W12, W21.").arg(pc(DynDiagDiag)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("DynDiagDiagDiag")){
                if(l.length() ==  pc(DynDiagDiagDiag)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = DynDiagDiagDiag;
                }
                else
                    ErrorMessage.append(QString("DynDiagDiagDiag-site needs %1 parameters: B1, B2, B3, Vzz1, Vzz2, Vzz3, CS1, CS2, CS3, omega, A, beta, W12, W13, W21, W23, W31. W32.").arg(pc(DynDiagDiagDiag)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("Dynbeta")){
                if(l.length() == pc(Dynbeta)+1){
                    bool JOK;
                    int J = l.at(1).toInt(&JOK);
                    if(JOK){
                        if(J > 1){
                            bool AllStatesOK = true;
                            for(int j = 0; j < J; j++){
                                fgetss(buffer,10000,f);
                                QStringList ll = QString(buffer).remove('\n')
                                                 .split(" ",QString::SkipEmptyParts);
                                if(ll.length() != 6 + J - 1){
                                    ErrorMessage.append(QString("Wrong state %1: Expect %2 parameters\n")
                                            .arg(j).arg(6+J-1));
                                    AllStatesOK = false;
                                }
                                else
                                    // append to l to read all parameters
                                    l.append(ll);
                            }
                            if(AllStatesOK)
                                vaildInput = 1;
                            else
                                ErrorMessage.append("There were errors in the declaration of the dynamic states.\n");
                        }
                        else
                            ErrorMessage.append("Expected at least two states.\n");
                    }
                    else
                        ErrorMessage.append("Expected integer value as state count.\n");
                    vaildInput = 1;
                    spectra[cmbs].typ = Dynbeta;
                }
                else
                    ErrorMessage.append("Dynbeta-site needs 4 parameters: J, omega, A and beta.\n");
            }
            if(!l.at(0).compare("Dyn")){
                if(l.length() == pc(Dyn)+1){
                    bool JOK;
                    int J = l.at(1).toInt(&JOK);
                    if(JOK){
                        if(J > 1){
                            bool AllStatesOK = true;
                            for(int j = 0; j < J; j++){
                                fgetss(buffer,10000,f);
                                QStringList ll = QString(buffer).remove('\n')
                                                 .split(" ",QString::SkipEmptyParts);
                                if(ll.length() != 6 + J - 1){
                                    ErrorMessage.append(QString("Wrong state %1: Expect %2 parameters\n")
                                            .arg(j).arg(6+J-1));
                                    AllStatesOK = false;
                                }
                                else
                                    // append to l to read all parameters
                                    l.append(ll);
                            }
                            if(AllStatesOK)
                                vaildInput = 1;
                            else
                                ErrorMessage.append("There were errors in the declaration of the dynamic states.\n");
                        }
                        else
                            ErrorMessage.append("Expected at least two states.\n");
                    }
                    else
                        ErrorMessage.append("Expected integer value as state count.\n");
                    vaildInput = 1;
                    spectra[cmbs].typ = Dyn;
                }
                else
                    ErrorMessage.append("Dyn-site needs 3 parameters: J, omega and A.\n");
            }
            if(!l.at(0).compare("MLR")){
                if(l.length() == pc(MLR)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = MLR;
                }
                else
                    ErrorMessage.append(QString("MLR-site needs %1 parameters: B, Vzz, CS, omega, A, beta, KV/kbT, D, S.").arg(pc(MLR)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }

            if(!l.at(0).compare("SHp")){
                if(l.length() == pc(SHpowder)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = SHpowder;
                }
                else
                    ErrorMessage.append(QString("SHp-site needs %1 parameters: B, Vzz, eta, theta, phi, CS, omega, A.").arg(pc(SHpowder)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("SHpLev")){
                if(l.length() == pc(SHpowderLev)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = SHpowderLev;
                }
                else
                    ErrorMessage.append(QString("SHpLev-site needs %1 parameters: B, Vzz, eta, theta, phi, CS, omega, A, ta.").arg(pc(SHpowder)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("mSSH")){
                if(l.length() ==  pc(mSSH)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = mSSH;
                }
                else
                    ErrorMessage.append(QString("mSSH-site needs %1 parameters: Bspl, Vzz, eta, theta, phi, CS, omega, A, beta, gamma, Bsrc, betasrc, alpha, epsilon.").arg(pc(mSSH)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("BL")){
                if(l.length() == pc(BL)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = BL;
                }
                else
                    ErrorMessage.append("BL needs 1 parameter.\n");
            }
            if(!l.at(0).compare("FeCal")){
                if(l.length() == pc(FeCal)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = FeCal;
                }
                else
                    ErrorMessage.append(QString("FeCal-site needs %1 parameters: omega1, omega2, omega3, A1, A2, A3, alpha, CS").arg(pc(FeCal)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("FeCaltriang")){
                if(l.length() == pc(FeCaltriang)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = FeCaltriang;
                }
                else
                    ErrorMessage.append(QString("FeCaltriang-site needs %1 parameters: omega1, omega2, omega3, A1, A2, A3, alpha, CS").arg(pc(FeCaltriang)))
                                .append(QString("Found only %1.\n").arg(l.length()-1));
            }
            if(!l.at(0).compare("fit")){
                if(l.length() == pc(AnyFit)+1){
                    vaildInput = 1;
                    spectra[cmbs].typ = AnyFit;
                }
                else
                    ErrorMessage.append("""fit"" needs 2 parameters: xbuffer and model\n");
            }

            // und dann ggf. auslesen
            if(vaildInput){
                for(int i = 1; i < l.length(); i++){
                    spectra[cmbs].p[i-1].typ = pObj;
                    /*normale Eingabe*/
                    /*MEM Verteilung aequidistant*/
                    if(rMEM.indexIn(l.at(i)) == 0 &&
                       rMEMEXT.indexIn(l.at(i)) != 0){
                        QStringList Meml = rMEM.capturedTexts();
                        spectra[cmbs].p[i-1].argv.append(Meml.at(2));
                        spectra[cmbs].p[i-1].argv.append(Meml.at(4));
                        spectra[cmbs].p[i-1].argv.append(Meml.at(6));
                        spectra[cmbs].p[i-1].argv.append(Meml.at(8));
                        spectra[cmbs].p[i-1].typ = pMEM;
                    }
                    /*MEM Verteilung paramterized*/
                    if(rMEMEXT.indexIn(l.at(i)) == 0){
                        QStringList Meml = rMEMEXT.capturedTexts();
                        spectra[cmbs].p[i-1].argv.append(Meml.at(2));
                        spectra[cmbs].p[i-1].argv.append(Meml.at(4));
                        spectra[cmbs].p[i-1].argv.append(Meml.at(6));
                        spectra[cmbs].p[i-1].argv.append(Meml.at(8));
                        spectra[cmbs].p[i-1].argv.append(Meml.at(10));
                        spectra[cmbs].p[i-1].typ = pMEMEXT;
                    }
                    // Hesse-Rübartsch
                    if(rHR.indexIn(l.at(i)) == 0&&
                       rHREXT.indexIn(l.at(i)) != 0){
                        QStringList HRl = rHR.capturedTexts();
                        spectra[cmbs].p[i-1].argv.append(HRl.at(2));
                        spectra[cmbs].p[i-1].argv.append(HRl.at(4));
                        spectra[cmbs].p[i-1].argv.append(HRl.at(6));
                        spectra[cmbs].p[i-1].argv.append(HRl.at(8));
                        spectra[cmbs].p[i-1].typ = pHR;
                    }
                    // Hesse-Rübartsch parameterized
                    if(rHREXT.indexIn(l.at(i)) == 0){
                        QStringList HRl = rHREXT.capturedTexts();
                        spectra[cmbs].p[i-1].argv.append(HRl.at(2));
                        spectra[cmbs].p[i-1].argv.append(HRl.at(4));
                        spectra[cmbs].p[i-1].argv.append(HRl.at(6));
                        spectra[cmbs].p[i-1].argv.append(HRl.at(8));
                        spectra[cmbs].p[i-1].argv.append(HRl.at(10));
                        spectra[cmbs].p[i-1].typ = pHREXT;
                    }
                    /*funktionale Verteilung*/
                    /*funktionale, parametrserte Verteilung*/
                    if(rDISTREXT.indexIn(l.at(i)) == 0){
                        QStringList Meml = rDISTREXT.capturedTexts();
                        spectra[cmbs].p[i-1].argv.append(Meml.at(2) );
                        spectra[cmbs].p[i-1].argv.append(Meml.at(4) );
                        spectra[cmbs].p[i-1].argv.append(Meml.at(6) );
                        spectra[cmbs].p[i-1].argv.append(Meml.at(8) );
                        spectra[cmbs].p[i-1].argv.append(Meml.at(10));
                        spectra[cmbs].p[i-1].argv.append(Meml.at(12));
                        spectra[cmbs].p[i-1].typ = pDISTREXT;
                    }
                    if(rDISTR.indexIn(l.at(i)) == 0 &&
                       rDISTREXT.indexIn(l.at(i)) != 0){
                        QStringList Meml = rDISTR.capturedTexts();
                        spectra[cmbs].p[i-1].argv.append(Meml.at(2) );
                        spectra[cmbs].p[i-1].argv.append(Meml.at(4) );
                        spectra[cmbs].p[i-1].argv.append(Meml.at(6) );
                        spectra[cmbs].p[i-1].argv.append(Meml.at(8) );
                        spectra[cmbs].p[i-1].argv.append(Meml.at(10));
                        spectra[cmbs].p[i-1].typ = pDISTR;

                    }
                    /*oder ist es doch eine normale Eingabe?*/
                    if(spectra[cmbs].p[i-1].typ == pObj)
                        spectra[cmbs].p[i-1].argv.append(l.at(i));
                }
                cmbs++;
            }
            else{
                ErrorMessage.append(QString("""%1"" is no valid input.\n").arg(buffer));
                spectraOK = 0;
            }
        }
        fgetss(buffer,10000,f);
    }
    return spectraOK;
}
int fitmodel::process_theory(){
    int processionsucces = 1;
    int IsHR = 0;
    int IsMEM = 0;
    // MEM-Spektren auf gleiche MEM-Parameter kontrollieren
    int isMEM[MAX_PC]; for(int i = 0; i < MAX_PC; i++) isMEM[i] = 0;
    int* containsMEM = new int[cmbs];

    for(int i = 0; i < cmbs; i++){
        containsMEM[i] = 0;
        for(int j = 0; j < pc(i); j++){
            if(is_arbitary_distribution(i,j))
                containsMEM[i] = 1;
            if(spectra[i].p[j].typ == pMEM ||
               spectra[i].p[j].typ == pMEMEXT)
                IsMEM = isMEM[j] = 1;
            if(spectra[i].p[j].typ == pHR ||
               spectra[i].p[j].typ == pHREXT)
                IsHR = 1;
        }
    }

    // check for exclusive distribution determination
    if(IsMEM){
        if(IsHR){
            processionsucces = 0;
            ErrorMessage.append(QString("The THEORY block cannot contain both MEM[] and HR[] parameters.\n"));
        }
        else
            Distrtyp = pMEM;
    }
    else
        if(IsHR)
            Distrtyp = pHR;

    // check for consistent subspectra
    for(int i = 0; i < cmbs; i++){
        for(int j = 0; j < pc(i); j++)
            if(containsMEM[i] && isMEM[j] &&
               !is_arbitary_distribution(i,j)){
                ErrorMessage.append(QString("Spectra %1, parameter %2: MEM-Calculations need to have consistent dimensions.\n").arg(i).arg(j));
                processionsucces = 0;
            }
    }
    // MEM-Parameter zaehlen
    cMEMpara = 0;
    for(int i = 0; i < MAX_PC; i++)
        if(isMEM[i]) cMEMpara++;

    // Gesamtzahl an MEM-Subspektren bestimmen
    cMEMsubspec = 1;
    for(int i = 0; i < cmbs; i++){
        if(containsMEM[i]){
            for(int j = 0; j < pc(i); j++)
                if(is_arbitary_distribution(i,j))
                    cMEMsubspec = cMEMsubspec * eval(spectra[i].p[j].arg[3],0);
            i = cmbs;
        }
    }

    // alloc RUN data
    for(int i = 0; i < crun; i++)
        run[i].init(cmbs,cMEMsubspec);

    // Spektren sortieren, sodass MEM-Spektre am Ende
    MoessbauerSpectrum mbsbuf;
    i1stMEMspec = cmbs;
    for(int i = 0; i < cmbs; i++){
        if(containsMEM[i]){
            i1stMEMspec--;
            mbsbuf = spectra[i];
            for(int j = i; j < cmbs -1; j++)
                spectra[j] = spectra[j+1];
            spectra[cmbs-1] = mbsbuf;
        }
    }
    delete containsMEM;   
    return processionsucces;
}
int fitmodel::is_arbitary_distribution(int i, int j){
    if(spectra[i].p[j].typ == pMEM ||
       spectra[i].p[j].typ == pMEMEXT ||
       spectra[i].p[j].typ == pHR ||
       spectra[i].p[j].typ == pHREXT)
        return true;
    else
        return false;

}
int fitmodel::read_run(char* mbspath, FILE*f){
    long int start = ftell(f);
    char buffer[10000]; buffer[0]='\0';
    /*runs zaehlen*/
    crun = 0;
    fgetss(buffer,10000,f);
    while(!feof(f) && !blockend(buffer)){
        if(validline(buffer))
            crun++;
        fgetss(buffer,10000,f);
    }
    if(crun == 0){
        ErrorMessage.append("No runs stated.\n");
        return 0;
    }
    /*runs einlese*/
    int runsOK = 1;
    run = new RUN[crun];
    crun = 0;
    QRegExp reex("(((\")(.*)(\"\\s))?)(-?\\d+\\.?,?\\d*)((\\s\\[-?\\d+\\.?\\d*,-?\\d+\\.?\\d*\\])*)((\\s.*)*)");               //subfolder-run

    fseek(f,start,SEEK_SET);
    fgetss(buffer,10000,f);
    while(!feof(f) && !blockend(buffer)){
        if(validline(buffer)){
            buffer[strlen(buffer)-1]='\0';  //Zeilenwechselzeichen
            int validrun = 0;
            QStringList maplist;
            QStringList rangelist;
            if(reex.indexIn(buffer)==0){
                QStringList l = reex.capturedTexts();
                maplist = l.at(9).split(" ",QString::SkipEmptyParts);
                rangelist = l.at(7).split(" ",QString::SkipEmptyParts);
                if(l.at(4).isEmpty())
                    validrun = load_run(mbspath,l.at(6).toLocal8Bit().data(),crun);
                else
                    validrun = load_run(l.at(4).toLocal8Bit().data(),
                                        l.at(6).toDouble(),crun);
            }
            if(validrun){
                // note maps
                run[crun].maplist = maplist;
                // handle fitting range
                run[crun].ranges = rangelist.length();
                run[crun].rangemin = new double[run[crun].ranges];
                run[crun].rangemax = new double[run[crun].ranges];
                for(int i = 0; i < run[crun].ranges; i++){
                    run[crun].rangemin[i] = QString(rangelist.at(i))
                                                    .remove("[").remove("]")
                                                    .split(",")
                                                    .at(0).toDouble();
                    run[crun].rangemax[i] = QString(rangelist.at(i))
                                                    .remove("[").remove("]")
                                                    .split(",")
                                                    .at(1).toDouble();
                }
                restrict_fitrange(crun);
                /*Initialisierung*/
                crun++;
            }
            else{
                runsOK = 0;
                ErrorMessage.append(buffer).append(" is no valid run.\n");
            }
        }
        fgetss(buffer,10000,f);
    }

    /*Kontrolliere, ob alle Map-Parameter gleich sind und vorhanden*/
    for(int i = 1; i < crun; i++){
        if(run[i].maplist.length() != run[i-1].maplist.length()){
            ErrorMessage.append(QString("Run %1 and run %2 have differenz map count.\n").arg(run[i-1].T).arg(run[i].T));
            runsOK = 0;
        }
    }
    /* create maps as normal objects */
    for(int i = 0; i < run[0].maplist.length();i++)
        obj.append(Obj(QString("map%1").arg(i),maptyp,i));
    return runsOK;
}
int fitmodel::read_plot(FILE*f){
    char buffer[10000]; buffer[0]='\0';
    int PlotOK = 1;
    fgetss(buffer,10000,f);
    while(!feof(f) && !blockend(buffer)){
        if(validline(buffer)){
            QStringList l = QString(buffer).remove('\n').split(" ",QString::SkipEmptyParts);
            if(!l.at(0).compare("runs")){
                /*Run-Nummern einlesen*/
                for(int i = 1; i < l.length();i++){
                    bool OK;
                    double runnr = l[i].replace(",",".").toDouble(&OK);
                    if(OK)
                        plotrunlist.append(runnr);
                    else{
                        ErrorMessage.append("Run list has to contain a run specific parameter (temperature etc.).\n");
                        PlotOK = 0;
                    }

                }
            }
            if(!l.at(0).compare("range"))
                if(l.length()>4){
                    /*Range einlesen*/
                    bool OK[4];
                    plotxmin = l.at(1).toDouble(&OK[0]);
                    plotxmax = l.at(2).toDouble(&OK[1]);
                    plotymin = l.at(3).toDouble(&OK[2]);
                    plotymax = l.at(4).toDouble(&OK[3]);
                    autorange = 0;
                    if(!(OK[0] && OK[1] && OK[2] && OK[3])){
                        ErrorMessage.append("Plot rang must be of type xmin xmax ymin xmax.\n");
                        autorange = 1;
                        plotxmin = -5;
                        plotxmax = 5;
                        plotymin = 0;
                        plotymax = 1.1;
                        PlotOK = 0;
                    }
                }
        }
        fgetss(buffer,10000,f);
    }
    return PlotOK;

}
int fitmodel::read_commands(FILE*f){
    char buffer[10000]; buffer[0]='\0';
    int ComandsOK = 1, foundcommand;
    fgetss(buffer,10000,f);
    while(!feof(f) && !blockend(buffer)){
        if(validline(buffer)){
            QStringList l = QString(buffer).remove('\n').split(" ",QString::SkipEmptyParts);
            foundcommand = 0;
            if(!l.at(0).compare("I0")){
                foundcommand = 1;
                if(l.length()>1)
                    sautoI0= l.at(1);
                else{
                    ErrorMessage.append("I0-command needs parameter to store automatically determined I0.\n");
                    ComandsOK = 0;
                }
            }
            if(!l.at(0).compare("A")){
                foundcommand = 1;
                if(l.length()>1)
                    sautoA= l.at(1);
                else{
                    ErrorMessage.append("A-command needs parameter to store automatically determined A.\n");
                    ComandsOK = 0;
                }
            }
            if(!l.at(0).compare("MaxIter")){
                foundcommand = 1;
                if(l.length()>1)
                    sMaxIter= l.at(1);
                else{
                    ErrorMessage.append("MaxIter-command needs number or reference to number.\n");
                    ComandsOK = 0;
                }
            }
            if(!l.at(0).compare("Binning")){
                foundcommand = 1;
                if(l.length()>1)
                    sBinning= l.at(1);
                else{
                    ErrorMessage.append("Binning-command needs number or reference to number.\n");
                    ComandsOK = 0;
                }
            }
            if(!l.at(0).compare("ConvergenceCriterium")){
                foundcommand = 1;
                if(l.length()>1)
                    sconv= l.at(1);
                else{
                    ErrorMessage.append("ConvergenceCriterium needs number or reference to number.\n");
                    ComandsOK = 0;
                }
            }
            if(!l.at(0).compare("MEM")){
                foundcommand = 1;
                if(l.length()>5){
                    sMEMsteps = l.at(1);
                    sMEMstepsize = l.at(2);
                    sMEMtollerance = l.at(3);
                    sMEMmaxlambda = l.at(4);
                    sMEMmaxlambdaI0 = l.at(5);
                }
                else{
                    ErrorMessage.append("MEM-command needs maxIteration, stepsize, tollerace, maxlambda and maxlambda refering I0.\n");
                    ComandsOK = 0;
                }
            }
            if(!l.at(0).compare("HR")){
                foundcommand = 1;
                if(l.length()>2){
                    sHR = l.at(1);
                    sHRc = l.at(2);
                }
                else{
                    ErrorMessage.append("HR-command needs smoothness parameter and constraint attribut.\n");
                    ComandsOK = 0;
                }
            }
            if(!l.at(0).compare("TransmissionIntegral")){
                foundcommand = 1;
                if(l.length()>4){
                    sta = l.at(1);
                    somegasrc = l.at(2);
                    somegaabs = l.at(3);
                    sfr = l.at(4);
                }
                else{
                    ErrorMessage.append("TransmissionIntegral needs effectiv thickness, source line, absorber line width and resonant fraction.\n");
                    ComandsOK = 0;
                }
            }
            if(!l.at(0).compare("CosineSmearing")){
                            foundcommand = 1;
                            if(l.length()>5){
                                CosineSmearing = true;
                                sCSr0 = l.at(1);
                                sCSr1 = l.at(2);
                                sCSr2 = l.at(3);
                                sCSz1 = l.at(4);
                                sCSz2 = l.at(5);
                            }
                            else{
                                ErrorMessage.append("CosineSmearing needs source-, sample- and detector diameter as well as source-sample- and source-detector-distance.\n");
                                ComandsOK = 0;
                            }
                        }
            if(!l.at(0).compare("UnfoldedData")){
                foundcommand = 1;
                if(l.length()>3){
                    useUnfoldedData = true;
                    sUDf = l.at(1);
                    sUDI = l.at(2);
                    sUDc = l.at(3);
                }
                else{
                    ErrorMessage.append("UnfoldedData command needs velocity, intensity correction and channel parameter.\n");
                    ComandsOK = 0;
                }
            }
            if(!l.at(0).compare("Errors")){
                if(l.length()>1){
                    if(l.at(1) == "Uncorrelated"){
                        errormode = uncorrelated;
                        foundcommand = 1;
                    }
                    if(l.at(1) == "Hesse"){
                        errormode = covariant;
                        foundcommand = 1;
                    }
                    if(l.at(1) == "Minos"){
                        errormode = minos;
                        foundcommand = 1;
                    }
                    if(l.at(1) == "MonteCarlo"){
                        errormode = montecarlo;
                        if(l.length()>2)
                            sMCi = l.at(2);
                        foundcommand = 1;
                    }
                }
                else{
                    ErrorMessage.append("Add either of the following error calculations to the Errors-command:\nUncorrelated, Hesse, Minos or MonteCarlo");
                    ComandsOK = 0;
                }
            }
            if(!l.at(0).compare("ForceSimplex")){
                forcesimplex = true;
                foundcommand = 1;
            }
            if(!l.at(0).compare("Nucleus")){
                foundcommand = 1;
                if(l.length()>5){
                    sNEg = l.at(1);
                    sNg12 = l.at(2);
                    sNg32 = l.at(3);
                    sNQ = l.at(4);
                    sNM = l.at(5);
                }
                else{
                    ErrorMessage.append("Nucleus needs transition Energy, gyromangetic ratio ground of ground and excited state, quadrupolar moment and mass.\n");
                    ComandsOK = 0;
                }
            }
            if(!foundcommand){
                ErrorMessage.append(QString("Command ""%1"" unknown.").arg(l.at(0)));
                ComandsOK = 0;
            }
        }
        fgetss(buffer,10000,f);
    }
    return ComandsOK;
}
void fitmodel::interprete_commands(int* linkOK){
    /*interprete Commands*/
    if(!sautoA.isEmpty()){
        autoA = find_obj(sautoA);
        if(autoA == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("A-command couldn't be linked.\n"));
        }
        if(autoA >= 0 && obj.at(autoA).typ != partyp){
            autoA = -1;
            *linkOK = 0;
            ErrorMessage.append(QString("A-command argument must be a simple parameter\n"));
        }
    }
    else
        autoA = -1;
    if(!sautoI0.isEmpty()){
        autoI0 = find_obj(sautoI0);
        if(autoI0 == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("I0-command couldn't be linked.\n"));
        }
        if(autoI0 >= 0 && obj.at(autoI0).typ != partyp){
            autoI0 = -1;
            *linkOK = 0;
            ErrorMessage.append(QString("I0-command argument must be a simple parameter\n"));
        }
    }
    else
        autoI0 = -1;
    if(!sUDf.isEmpty()){
        UDf = find_obj(sUDf);
        if(UDf == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("UnfoldedData-command, velocity couldn't be linked.\n"));
        }
    }
    else
        UDf = -1;
    if(!sUDI.isEmpty()){
        UDI = find_obj(sUDI);
        if(UDI == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("UnfoldedData-command, intensity correction couldn't be linked.\n"));
        }
    }
    else
        UDf = -1;
    if(!sUDc.isEmpty()){
        UDc = find_obj(sUDc);
        if(UDc == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("UnfoldedData-command, channel parameter couldn't be linked.\n"));
        }
        if(UDc >= 0 && obj.at(UDc).typ != partyp){
            UDc = -1;
            *linkOK = 0;
            ErrorMessage.append(QString("UnfoldedData-command, channel parameter must be a simple parameter\n"));
        }
    }
    else
        UDc = -1;
    if(!sCSr0.isEmpty()){
        CSr0 = find_obj(sCSr0);
        if(CSr0 == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("CosineSmearing-command, source radius couldn't be linked.\n"));
        }
    }
    else
        CSr0 = -1;
    if(!sCSr1.isEmpty()){
        CSr1 = find_obj(sCSr1);
        if(CSr1 == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("CosineSmearing-command, sample radius couldn't be linked.\n"));
        }
    }
    else
        CSr1 = -1;
    if(!sCSr2.isEmpty()){
        CSr2 = find_obj(sCSr2);
        if(CSr2 == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("CosineSmearing-command, detector radius couldn't be linked.\n"));
        }
    }
    else
        CSr2 = -1;
    if(!sCSz1.isEmpty()){
        CSz1 = find_obj(sCSz1);
        if(CSz1 == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("CosineSmearing-command, sample distance couldn't be linked.\n"));
        }
    }
    else
        CSz1 = -1;
    if(!sCSz2.isEmpty()){
        CSz2 = find_obj(sCSz2);
        if(CSz2 == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("CosineSmearing-command, detector distance couldn't be linked.\n"));
        }
    }
    else
        CSz2 = -1;
    if(CosineSmearing && CSr0 >-1 && CSr1 >-1 && CSr2 >-1 &&
       CSz1 >-1 && CSz2 >-1){
        CosSm = new CosSmearing(eval(CSr0,0),eval(CSr1,0),eval(CSr2,0),
                      eval(CSz1,0),eval(CSz2,0));
    }
    if(!sNg12.isEmpty()){
        Ng12 = find_obj(sNg12);
        if(Ng12 == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("Nucleus-command, ground state gyromagnetic ratio couldn't be linked.\n"));
        }
    }
    else
        Ng12 = -1;
    if(!sNg32.isEmpty()){
        Ng32 = find_obj(sNg32);
        if(Ng32 == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("Nucleus-command, excited state gyromagnetic ratio couldn't be linked.\n"));
        }
    }
    else
        Ng32 = -1;
    if(!sNQ.isEmpty()){
        NQ = find_obj(sNQ);
        if(NQ == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("Nucleus-command,quadrupole moment couldn't be linked.\n"));
        }
    }
    else
        NQ = -1;
    if(!sNM.isEmpty()){
        NM = find_obj(sNM);
        if(NM == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("Nucleus-command, nucleus mass couldn't be linked.\n"));
        }
    }
    else
        NM = -1;
    if(!sNEg.isEmpty()){
        NEg = find_obj(sNEg);
        if(NEg == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("Nucleus-command, transition energy couldn't be linked.\n"));
        }
    }
    else
        NEg = -1;
    if(!sMaxIter.isEmpty()){
        MaxIter = find_obj(sMaxIter);
        if(MaxIter == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("MaxIter-command couldn't be linked.\n"));
        }
    }
    if(!sMaxIter.isEmpty()){
        MaxIter = find_obj(sMaxIter);
        if(MaxIter == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("MaxIter-command couldn't be linked.\n"));
        }
    }
    if(!sconv.isEmpty()){
        conv = find_obj(sconv);
        if(conv == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("Convergence cirterium couldn't be linked.\n"));
        }
    }
    else
        conv = -1;
    if(!sMEMsteps.isEmpty()){
        MEMsteps = find_obj(sMEMsteps);
        if(MEMsteps == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("MEMsteps-command couldn't be linked.\n"));
        }
    }
    else
        MEMsteps = -1;
    if(!sMEMstepsize.isEmpty()){
        MEMstepsize = find_obj(sMEMstepsize);
        if(MEMstepsize == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("MEMstepsize-command couldn't be linked.\n"));
        }
    }
    else
        MEMstepsize = -1;
    if(!sMEMtollerance.isEmpty()){
        MEMtollerance = find_obj(sMEMtollerance);
        if(MEMtollerance == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("MEMtollerance-command couldn't be linked.\n"));
        }
    }
    else
        MEMtollerance = -1;
    if(!sMEMmaxlambda.isEmpty()){
        MEMmaxlambda = find_obj(sMEMmaxlambda);
        if(MEMmaxlambda == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("MEMmaxlambda-command couldn't be linked.\n"));
        }
    }
    else
        MEMmaxlambda = -1;
    if(!sMEMmaxlambdaI0.isEmpty()){
        MEMmaxlambdaI0 = find_obj(sMEMmaxlambdaI0);
        if(MEMmaxlambdaI0 == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("MEMmaxlambdaI0-command couldn't be linked.\n"));
        }
    }
    else
        MEMmaxlambdaI0 = -1;
    if(!sHR.isEmpty()){
        HR = find_obj(sHR);
        if(HR == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("HR-smooth parameter couldn't be linked.\n"));
        }
    }
    else
        HR = -1;
    if(!sHRc.isEmpty()){
        HRc = find_obj(sHRc);
        if(HRc == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("HR-constraint parameter couldn't be linked.\n"));
        }
    }
    else
        HRc = -1;


    if(!sta.isEmpty()){
        ta = find_obj(sta);
        if(ta == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("Effective thickness object couldn't be linked.\n"));
        }
    }
    else
        ta = -1;
    if(!somegasrc.isEmpty()){
        omegasrc = find_obj(somegasrc);
        if(ta == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("Source linewidth object couldn't be linked.\n"));
        }
    }
    else
        omegasrc = -1;
    if(!somegaabs.isEmpty()){
        omegaabs = find_obj(somegaabs);
        if(ta == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("Absorber linewidth object couldn't be linked.\n"));
        }
    }
    else
        omegaabs = -1;
    if(!sfr.isEmpty()){
        fr = find_obj(sfr);
        if(fr == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("Resonant fraction object couldn't be linked.\n"));
        }
    }
    else
        fr = -1;
    if(!sMCi.isEmpty()){
        MCi = find_obj(sMCi);
        if(MCi == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("MonteCarlo iterations object couldn't be linked.\n"));
        }
    }
    else
        MCi = -1;
    if(!sBinning.isEmpty()){
        Binning = find_obj(sBinning);
        if(Binning == -1){
            *linkOK = 0;
            ErrorMessage.append(QString("Binning argument couldn't be linked.\n"));
        }
        else
            process_Binning();
    }
    else
        Binning = -1;
    if(autoA >= 0 || autoI0 >= 0)
        calc_baseline_area(-1);
}
void fitmodel::parse_functions(int* linkOK){
    for(int i = 0; i < cfun; i++){
        QString pstring;
        /* first check for additional function*/
        AdditionalFunTypes aftypes[10] = {TAbsDeb,TQuadDop,SwaveSFD,Hc2WHH,GLBrandttriang,GLBrandtquad,Errorfunction,OrderParameterDistrTN,FerroMag,Bezier4P};
        int afac[10] = {3,3,4,5,3,3,1,5,6,9};
        char afn [10][100] = {"AbsDeb(","QuadDop(","SwaveSFD(","Hc2WHH(","NGLsigmatri(","NGLsigmaquad(","erf(","OrdParaDistr(","FerroMag(","Bezier4P("};
        int AddFunID = 0;
        for(int j = 0; j < 10; j++){
            int pos0 = 0;
            //QRegExp r(&(afn[j][0]));
            QString r(&(afn[j][0]));
            QString funiter(fun[i].body.mid(pos0));
            while ((pos0 = funiter.indexOf(r)) != -1){
                AdditionalFun af;
                af.typ = aftypes[j];
                af.cpara = afac[j];
                int bracketlevel = 1, ipara = 0;
                int pos = pos0 + strlen(afn[j]);
                funiter = QString(fun[i].body.mid(pos));
                int as = pos;
                while(bracketlevel > 0 &&
                      pos < fun[i].body.length() &&
                      ipara < af.cpara){
                    if(fun[i].body[pos] == ')') bracketlevel--;
                    if(fun[i].body[pos] == '(') bracketlevel++;
                    if(fun[i].body[pos] == ',' || bracketlevel == 0){
                        QString arg = fun[i].body.mid(as,pos-as);
                        af.arg[ipara] = find_obj(arg);
                        if(af.arg[ipara] == -1){
                            ErrorMessage.append(QString("function %1, subfunction %2, argument %3 couldn't be linked.\n")
                                                .arg(fun[i].name)
                                                .arg(afn[j])
                                                .arg(arg));
                            *linkOK = 0;
                        }
                        as = pos+1;
                        ipara++;
                    }
                    pos++;
                }
                if(bracketlevel != 0){
                    ErrorMessage.append(QString("function %1, subfunction %2: expected %3 arguments, found more\n")
                                        .arg(fun[i].name)
                                        .arg(afn[j])
                                        .arg(af.cpara));
                    *linkOK = 0;
                }
                if(ipara < af.cpara){
                    ErrorMessage.append(QString("function %1, subfunction %2: expected %3 arguments, found only %3\n")
                                        .arg(fun[i].name)
                                        .arg(afn[j])
                                        .arg(af.cpara));
                    *linkOK = 0;
                }
                // prepare parser string
                fun[i].body.replace(pos0,pos-pos0,QString("AddFun%1").arg(AddFunID));
                pstring.append(QString("AddFun%1,").arg(AddFunID));
                AddFunID++;
                fun[i].addfunlist.append(af);
            }
        }
        /* second check for parameters*/
        QString parsedbody = fun[i].body;
        for(int j2 = 0; j2 < obj.length();j2++){
            if(  obj[j2].typ != numtyp
               &&parsedbody.indexOf(obj[j2].name)!= -1){
                fun[i].arg.append(j2);
                parsedbody.remove(obj[j2].name);
                pstring.append(QString("%1,").arg(obj[j2].name));
            }
        }
        /* parse function*/
        if(-1 < fun[i].f.Parse(fun[i].body.toStdString(),pstring.toStdString())){
            ErrorMessage.append(QString("function %1 couldn't be parsed. The following arguments were found: ").arg(fun[i].name));
            for(int j = 0; j < fun[i].addfunlist.length();j++){
                ErrorMessage.append(QString("%1(").arg(afn[fun[i].addfunlist.at(j).typ]));
                    for(int k = 0; k < fun[i].addfunlist.at(j).cpara; k++)
                        ErrorMessage.append(QString("%1, ").arg(obj.at(fun[i].addfunlist.at(j).arg[k]).name));
                ErrorMessage.append("), ");
            }
            for(int j = 0; j < fun[i].arg.length();j++)
                ErrorMessage.append(QString("%1,").arg(obj.at(fun[i].arg.at(j)).name));
            ErrorMessage.append("\n");
            *linkOK = 0;
        }
    }
}
int fitmodel::is_valid(){
    return isvalid;
}
int fitmodel::set_obj(int i, int irun, double v){
    if(i >= 0 && i < obj.length()){
        switch(obj[i].typ){
            case numtyp:
                obj[i].dnumber = v;
                return 1;
            case partyp:
                *rData[obj[i].ref][irun] = v;
                return 1;
            case maptyp:
                return set_obj(run[irun].map[obj[i].ref],irun,v);
            default:
                return 0;
        }
    }
    else
        return 0;
}
int fitmodel::set_error(int i, int irun, double v){
    if(i >= 0 && i < obj.length()){
        switch(obj[i].typ){
            case partyp:
                *rError[obj[i].ref][irun] = v;
                return 1;
            case maptyp:
                return set_error(run[irun].map[obj[i].ref],irun,v);
            default:
                return 0;
        }
    }
    else
        return 0;
}
int fitmodel::find_obj(QString name){
    int index = obj.indexOf(name);
    if(index == -1){
        /* name might be "rsp"*/
        if(name == "rsp"){
            obj.append(Obj(name,runspecificparameter,0));
            return obj.length()-1;
        }
        /* name might be a number, try and add to obj*/
        bool OK;
        double buf = name.toDouble(&OK);
        if(OK){
            obj.append(Obj(name,buf));
            return obj.length()-1;
        }
        else
            return -1;
    }
    else
        return index;
}
int fitmodel::calc_run(int i){
    if( i < crun){
        // if necessary recalculate the velocities
        if(useUnfoldedData)
            calculate_velocity_data(i);
        // now begin to process theory
        double p[100], sumweight = 0;
        run[i].A = 0;
        MoessCalc mc;
        // if necessary, set the nucleus
        if(Ng12 > -1 && Ng32 > -1 && NQ > -1 && NM > -1 && NEg > -1)
            mc.set_nucleus(eval(Ng12,i),eval(Ng32,i),eval(NEg,i),eval(NM,i),
                           eval(NQ,i));
        // first calc all none-MEM spectra
        for(int j = 0; j < i1stMEMspec; j++){
            // reset spectra to zero
            init_array(run[i].Ith[j+1],run[i].ccr,0);
            // get spectra
            sumweight = 0;
            switch(spectra[j].typ){
                case BL:
                    run[i].I0 = eval(spectra[j].p[0].arg[0],i);
                    for(int k = 0; k < run[i].ccr; k++)
                        run[i].Ith[j+1][k] += run[i].I0;
                    break;
                case AnyFit:
                    anyfit(i,j);
                    break;
                default:
                    WeightSum(&(p[0]),0,j,i,j,1,&sumweight);
                    build_spectrum(&(p[0]),0,j,
                                   i,j,&mc,1./sumweight,&sumweight,NULL,0,0);
                    break;
            }

        }

        /*normale Spektren addieren*/
        for(int k = 0; k < run[i].ccr; k++){
            run[i].Ith[0][k] = 0;
            for(int j = 1; j < i1stMEMspec+1; j++)
                run[i].Ith[0][k] += run[i].Ith[j][k];
        }
        /*jetzt MEM-Spektren zusammenfassen und MEM durchfuehren, falls MEM
        vorhanden ist*/
        if(i1stMEMspec < cmbs){
            double** MEMbuf;
            alloc_array(&MEMbuf,run[i].ccr,cMEMsubspec);
            init_array(MEMbuf,run[i].ccr,cMEMsubspec,0);
            for(int j = i1stMEMspec; j < cmbs; j++){
                /*Spektrum Nullen*/
                init_array(run[i].Ith[j+1],run[i].ccr,0);
                /*Spektrum berechnen*/
                sumweight = 0;
                switch(spectra[j].typ){
                    case BL:
                    case AnyFit:
                        break;
                    default:
                        WeightSum(&(p[0]),0,j,i,j,1,&sumweight);
                        build_spectrum(&(p[0]),0,j,i,j,&mc,1.*cMEMsubspec/sumweight,
                                       &sumweight,MEMbuf,0,cMEMsubspec);
                        break;
                }
                /*Spektrum unsichtbar machen*/
                init_array(run[i].Ith[j+1],run[i].ccr,0.);
            }
            /*Restmessspektrum berechnen*/
            double* tofit = new double[run[i].ccr];
            for(int k = 0; k < run[i].ccr; k++){
                tofit[k] = run[i].I[k];
                for(int j = 0; j < i1stMEMspec; j++)
                    tofit[k] = tofit[k] - run[i].Ith[j+1][k];
            }
            /*Rechnung durchfuehren*/
            switch(Distrtyp){
                case pMEMEXT:
                case pMEM:{
                    double stepsize = eval(MEMstepsize,i);
                    int steps = int(eval(MEMsteps,i));
                    double l0 = eval(MEMmaxlambda,i);
                    double lI0 = eval(MEMmaxlambdaI0,i);
                    double lnom =  l0*sqrt(lI0/run[i].I0)
                                 /(1+(sqrt(lI0/run[i].I0)-1)*l0);
                    double tollerance = eval(MEMtollerance,i);
                    mc.MEM_Cambridge(MEMbuf,run[i].rho,run[i].rhoerror,tofit,
                                     run[i].I0,cMEMsubspec,run[i].ccr,stepsize,
                                     steps,lnom,tollerance,run[i].MEMstatus);
                    }break;
                case pHREXT:
                case pHR:{
                    double constrain = eval(HRc,i);
                    if(constrain>0){
                        double smooth = eval(HR,i)*cMEMsubspec*run[i].ccr;
                        mc.LeCear(MEMbuf,run[i].rho,tofit,cMEMsubspec,
                                  run[i].ccr,smooth);
                    }
                    else{
                        double smooth = eval(HR,i)*cMEMsubspec*cMEMsubspec*run[i].ccr
                                        *pow(qMax(1.,run[i].I0),1.5)/32.;
                        mc.HesseRuebartsch(MEMbuf,run[i].rho,tofit,cMEMsubspec,
                                           run[i].ccr,smooth);
                    }
                    //mc.HesseRuebartsch(MEMbuf,run[i].rho,tofit,cMEMsubspec,run[i].ccr,smooth);

                    }break;
                default:break;
            }


            /*MEM-Verteilung normieren*/
            double sumMEM = 0;
            for(int j = 0; j < cMEMsubspec; j++)
                sumMEM += run[i].rho[j];
            for(int j = 0; j < cMEMsubspec; j++)
                run[i].rho[j] = run[i].rho[j] / sumMEM;
            /*MEM-Subspektrum zusammensetzen*/
            init_array(run[i].Ith[i1stMEMspec+1],run[i].ccr,0);
            for(int j = 0; j < cMEMsubspec; j++)
                for(int k = 0; k < run[i].ccr; k++)
                    run[i].Ith[i1stMEMspec+1][k] = run[i].Ith[i1stMEMspec+1][k]
                                                 + run[i].rho[j] * MEMbuf[k][j];
            /*Sichtkontrolle
            alloc_array(&MEMbuf2,cMEMsubspec,run[i].ccr);
            for(int j = 0; j < cMEMsubspec; j++)
                for(int k = 0; k < run[i].ccr; k++)
                    MEMbuf2[j][k] = MEMbuf[k][j];// + run[i].I0;*/
            free_array(&MEMbuf,run[i].ccr);
            delete tofit;
        }

        /*MEMspektrum addieren*/
        for(int j = i1stMEMspec+1; j < cmbs+1; j++)
            for(int k = 0; k < run[i].ccr; k++)
                run[i].Ith[0][k] += run[i].Ith[j][k];
        // if necessary execute the cosine smearing
        if(CosineSmearing){
            CosSm->update(eval(CSr0,i),eval(CSr1,i),eval(CSr2,i),
                          eval(CSz1,i),eval(CSz2,i));
            CosSm->smear(run[i].Ith[0],run[i].v,run[i].ccr,run[i].I0);
        }
        /*Transmissionsintegral ggf. durchführen*/
        if(ta >= 0 && omegasrc >= 0 && omegaabs >= 0 && fr >= 0)
            mc.TransmissionIntegral(run[i].Ith[0],run[i].v,run[i].ccr,
                                    run[i].A,run[i].I0,eval(ta,i),eval(omegasrc,i),
                                    eval(omegaabs,i),eval(fr,i));
        /*aktuelles chi2 berechnen*/
        run[i].chi2 = 0;
        for(int kk = 0; kk < run[i].ccr; kk++)
            run[i].chi2 +=(run[i].Ith[0][kk]-run[i].I[kk])
                         *(run[i].Ith[0][kk]-run[i].I[kk])
                         * run[i].err2rec[kk];

        return 1;
    }
    else
        return 0;
}
double fitmodel::fv(int ir, int ifun){
    double *p = new double[ fun[ifun].addfunlist.length()
                           +fun[ifun].arg.length()];
    int ci = 0;
    for(int i = 0; i < fun[ifun].addfunlist.length();i++){
        double P[9];
        for(int j = 0; j < fun[ifun].addfunlist[i].cpara; j++)
            P[j] = eval(fun[ifun].addfunlist[i].arg[j],ir);
        switch(fun[ifun].addfunlist[i].typ){
            case TAbsDeb:{
                MoessCalc mc;
                // if necessary, set the nucleus
                if(Ng12 > -1 && Ng32 > -1 && NQ > -1 && NM > -1 && NEg > -1)
                    mc.set_nucleus(eval(Ng12,i),eval(Ng32,i),eval(NEg,i),eval(NM,i),
                                   eval(NQ,i));
                p[ci] = mc.Abs_Debye(P[0],P[1],P[2]);
                break;
            }
            case TQuadDop:
                p[ci] = MoessCalc().dv_Debye(P[0],P[1],P[2]);
                break;
            case SwaveSFD:
                p[ci] = MoessCalc().sWave_SuperfluidDensity(P[2],P[1],P[0],P[3]);
                break;
            case Hc2WHH:
                p[ci] = MoessCalc().Hc2_WHH(P[0],P[1],P[2],P[3],P[4]);
                break;
            case GLBrandttriang:
                p[ci] = MuSR().sigma_triangBLAS(P[0],P[1],P[2]*1e-9);
                break;
            case GLBrandtquad:
                p[ci] = MuSR().sigma_quadBLAS(P[0],P[1],P[2]*1e-9);
                break;
            case Errorfunction:
                p[ci] = MoessCalc().errorfunction(P[0]);
                break;
            case OrderParameterDistrTN:
                p[ci] = MoessCalc().OrderParameterDistr(P[0],P[1],P[2],P[3],P[4]);
                break;
            case FerroMag:
                p[ci] = MoessCalc().Mmf(P[0],P[1],P[2],P[3],P[4],P[5]);
                break;
            case Bezier4P:
                p[ci] = MoessCalc().Bezier_y(P[0],P[1],P[2],P[3],P[4],P[5],P[6],P[7],P[8]);
                break;
            default:
                QMessageBox().exec();
                break;
        }
        ci++;
    }
    for(int i = 0; i < fun[ifun].arg.length(); i++){
        p[ci] = eval(fun[ifun].arg[i],ir);
        ci++;
    }

    double result;
    /*calc but protect fparser against parallel usage*/
    fmutex.lock();
    result= fun[ifun].f.Eval(p);
    fmutex.unlock();

    delete p;
    return result;
}
void fitmodel::plot(plot2D *plotter, plot2D *MEM){
    if(isvalid){
        QStringList legendlabels;
        if(plotrunlist.length() == 0)
            plotter->set_pointdata(0,0,NULL,NULL);
        if(plotrunlist.length() == 1){
            int irun = int(plotrunlist.at(0));
            plotter->set_pointdata(run[irun].cc, 1, run[irun].v, &run[irun].I);
            double **ybuf = new double*[cmbs+1];
            ybuf[0] = run[irun].Ith[0];
            legendlabels.append("TotalSpectrum");
            int SpecIndex = 0;
            double I0 = run[irun].I0;;
            //if(autoI0>0)    I0 = eval(autoI0,irun);
            //else            I0 = run[irun].I0;
            for(int i = 0; i < cmbs; i++){
                if(spectra[i].typ != BL){
                    SpecIndex++;
                    ybuf[SpecIndex] = new double[run[irun].cc];
                    for(int k = 0; k < run[irun].ccr; k++)
                        ybuf[SpecIndex][k] = I0 + run[irun].Ith[i+1][k];
                    legendlabels.append(QString("Subspectrum%1").arg(i+1));
                }
            }
            plotter->set_linedata(run[irun].ccr, SpecIndex+1, run[irun].v, ybuf);
            plotter->set_legend(&legendlabels);
            for(int i = 1; i < SpecIndex + 1; i++)
                delete ybuf[i];
            delete ybuf;

        }
        if(plotrunlist.length() > 1){ 
            int *cc = new int[plotrunlist.length()];
            int *ccr = new int[plotrunlist.length()];
            double**ypdata = new double*[plotrunlist.length()];
            double**yldata = new double*[plotrunlist.length()];
            double**xdata = new double*[plotrunlist.length()];    
            for(int i = 0; i < plotrunlist.length(); i++){
                int irun = int(plotrunlist.at(i));
                double I0 = run[irun].I0;;
                //if(autoI0>0)    I0 = eval(autoI0,irun);
                //else            I0 = run[irun].I0;
                cc[i] = run[irun].cc;
                ccr[i] = run[irun].ccr;
                ypdata[i] = run[irun].nI;
                yldata[i] = run[irun].nIth;
                xdata[i] = run[irun].v;
                if(!AnyFit_in)
                    for(int k = 0; k < run[irun].cc; k++){
                        run[irun].nI[k] = run[irun].I[k] / I0;
                        run[irun].nIth[k] = run[irun].Ith[0][k] / I0;
                    }
                else
                    for(int k = 0; k < run[irun].cc; k++){
                        run[irun].nI[k] = run[irun].I[k];
                        run[irun].nIth[k] = run[irun].Ith[0][k];
                    }
                legendlabels.append(QString("%1").arg(run[irun].T));
            }
            plotter->set_pointdata(plotrunlist.length(),cc,xdata,ypdata);
            plotter->set_linedata(plotrunlist.length(),ccr,xdata,yldata);
            plotter->set_legend(&legendlabels);
            delete ypdata;
            delete yldata;
            delete xdata;
            delete cc;
            delete ccr;
        }

        if(autorange)
            plotter->autoset_axis();
        else
            plotter->set_range(plotxmin,plotxmax,plotymin,plotymax);

        // care for secondary plot
        if(cMEMpara > 0 && MEM != NULL && plotrunlist.length() > 0){
            double** yMEM = new double*[plotrunlist.length()];
            QStringList MEMlegend;
            for(int i = 0; i < plotrunlist.length();i++){
                int irun = int(plotrunlist.at(i));
                yMEM[i] = run[irun].rho;
                MEMlegend.append(QString("%1").arg(run[irun].T));
            }
            double* a = new double[cMEMsubspec];
            if(cMEMpara == 1)
                for(int i = 0; i < cMEMsubspec; i++)
                    a[i] = get_MEMcode(i,0);
            else
                for(int i = 0; i < cMEMsubspec; i++)
                    a[i] = i;

            MEM->set_linedata(cMEMsubspec,plotrunlist.length(),a,yMEM);
            MEM->set_legend(&MEMlegend);
            MEM->setWindowTitle("MEM Distributions");
            MEM->set_labels("MEM Distributions","sampling point","rho");
            MEM->autoset_axis();
            delete yMEM;
            delete a;
        }
        else{
            // generate and normalize fitdata
            double** data = alloc_array(cpara,crun);
            QStringList legend;
            for(int i = 0; i < cpara; i++){
                legend.append(fmp[i].name);
                for(int j = 0; j < crun; j++)
                    data[i][j] = *(rData[i][j]);
            }
            double* xdata = new double[crun];
            for(int j = 0; j < crun; j++)
                xdata[j] = run[j].T;

            // plot it
            MEM->set_linedata(crun,cpara,xdata,data);
            MEM->set_legend(&legend);
            MEM->setWindowTitle("Parameter Dependencies");
            MEM->set_labels("Parameter Dependencies","run specific parameter","parameter range");
            MEM->plot_current_set();

            // free helper data
            delete xdata;
            free_array(&data,cSpec);
        }
    }
}
void fitmodel::init(){
    AnyFit_in = 0;
    fitruns = 1;
    fun = NULL;
    spectra = NULL;
    fmp = NULL;
    CosSm = NULL;
    sautoI0 = "";
    sautoA= "";
    sMEMsteps= "100";
    sMaxIter= "1000";
    sconv= "0.01";
    sMEMmaxlambda= "0.9";
    sMEMmaxlambdaI0= "50000";
    sMEMtollerance= "0.1";
    sMEMstepsize= "0.012";
    sHR="1";
    sHRc="0";
    sta= "";
    somegasrc= "";
    somegaabs= "";
    sfr= "";
    sCSr0 = "";
    sCSr1 = "";
    sCSr2 = "";
    sCSz1 = "";
    sCSz2 = "";
    sUDf = "";
    sUDI = "";
    sUDc = "";
    sBinning = "";
    errormode = covariant;
    ProgressPrint = true;
    forcesimplex = false;
    useUnfoldedData = false;
    CosineSmearing = false;
    errors_up_to_data = false;
    ErrorMessage.clear();
    plotxmin = -5;
    plotxmax = 5;
    plotymin = 0;
    plotymax = 1.1;
    autorange = 1;
    crun = cfun = cpara = 0;
    sMCi = "10000";
    run = NULL;
    pData = NULL;
    pError = NULL;
    pIsChanged = NULL;
    affect = NULL;
    rData = NULL;
    rError = NULL;
    raffect = NULL;
    fmpa = NULL;
}
void fitmodel::build_iterator( double* p, int ip, int spectrumi,
                               int irun, int ims,
                               MoessCalc *mc, bool sumweightonly,
                               double weight0, double* integratedweight,
                               double **MEMIm, int MEMIndex, int maxMEMIndex){
    if(ip < pc(spectrumi)){
        /*falls noch nicht bei letztem Parameter angelangt*/
        switch(spectra[ims].p[ip].typ){
            case pObj:
                p[ip] = eval(spectra[ims].p[ip].arg[0],irun);
                build_iterator(p,ip+1,spectrumi,irun,ims,mc,sumweightonly,weight0,
                               integratedweight,MEMIm,MEMIndex,maxMEMIndex);
                break;
            case pDISTR:{
                double min = eval(spectra[ims].p[ip].arg[1],irun);
                double max = eval(spectra[ims].p[ip].arg[2],irun);
                int steps = int(eval(spectra[ims].p[ip].arg[3],irun));
                for(int i = 0; i < steps; i++){
                    /*Wert setzen*/
                    p[ip] = min + 1.*i/qMax(1,steps-1)*(max-min);
                    set_obj(spectra[ims].p[ip].arg[0],irun,p[ip]);
                    /*Gewicht berechnen*/
                    build_iterator(p,ip+1,spectrumi,irun,ims,mc,sumweightonly,weight0,
                                   integratedweight,MEMIm,MEMIndex,maxMEMIndex);
                }}
                break;
            case pDISTREXT:{
                double min = eval(spectra[ims].p[ip].arg[1],irun);
                double max = eval(spectra[ims].p[ip].arg[2],irun);
                int steps = int(eval(spectra[ims].p[ip].arg[3],irun));
                for(int i = 0; i < steps; i++){
                    /*Wert setzen*/
                    double buf = min + 1.*i/qMax(1,steps-1)*(max-min);
                    set_obj(spectra[ims].p[ip].arg[0],irun,buf);
                    build_iterator(p,ip+1,spectrumi,irun,ims,mc,sumweightonly,weight0,
                                   integratedweight,MEMIm,MEMIndex,maxMEMIndex);
                }}
                break;
            case pHR:
            case pMEM:{
                double min = eval(spectra[ims].p[ip].arg[1],irun);
                double max = eval(spectra[ims].p[ip].arg[2],irun);
                int steps = int(eval(spectra[ims].p[ip].arg[3],irun));
                for(int i = 0; i < steps; i++){
                    /*Wert setzen*/
                    p[ip] = min + 1.*i/qMax(1,steps-1)*(max-min);
                    set_obj(spectra[ims].p[ip].arg[0],irun,p[ip]);
                    /*Rekorsionsaufruf*/
                    build_iterator(p,ip+1,spectrumi,irun,ims,mc,sumweightonly,weight0,
                                   integratedweight,MEMIm,MEMIndex * steps + i,maxMEMIndex);
                }}
                break;
            case pHREXT:
            case pMEMEXT:{
                double min = eval(spectra[ims].p[ip].arg[1],irun);
                double max = eval(spectra[ims].p[ip].arg[2],irun);
                int steps = int(eval(spectra[ims].p[ip].arg[3],irun));
                for(int i = 0; i < steps; i++){
                    /*Wert setzen*/
                    double buf = min + 1.*i/qMax(1,steps-1)*(max-min);
                    set_obj(spectra[ims].p[ip].arg[0],irun,buf);
                    /*Rekorsionsaufruf*/
                    build_iterator(p,ip+1,spectrumi,irun,ims,mc,sumweightonly,weight0,
                                   integratedweight,MEMIm,MEMIndex * steps + i,maxMEMIndex);
                }}
                break;
            default:
                break;
        }
    }
    if(ip == pc(spectrumi)){
        // recalc all weights/values as arguments might be ready just now
        double weight = weight0, Atemp = 0;
        MSTyp m = spectra[spectrumi].typ;
        for(int i = 0; i < pc(spectrumi); i++)
            switch(spectra[ims].p[i].typ){
                case pDISTR:
                    weight = weight*eval(spectra[ims].p[i].arg[4],irun);
                    break;
                case pDISTREXT:
                    p[i] = eval(spectra[ims].p[i].arg[5],irun);
                    weight = weight*eval(spectra[ims].p[i].arg[4],irun);
                    break;
                case pMEMEXT:
                    p[i] = eval(spectra[ims].p[i].arg[4],irun);
                    break;
                case pObj:
                    if(obj.at(spectra[ims].p[i].arg[0]).typ == funtyp)
                        p[i] = eval(spectra[ims].p[i].arg[0],irun);
                    break;
                default:
                    break;
            }
        // now the spectrum has to be calculated
        if(!sumweightonly){
            double *Ibuf = new double[run[irun].cc];
            for(int k = 0; k < run[irun].cc; k++) Ibuf[k] = 0;
            if(m == SHcrystal){
                mc->get_SC_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[7];
            }
            if(m == SHcrystalG){
                mc->get_SC_spectrum_G(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[7];
            }
            if(m == SHcrystalB){
                mc->get_SC_spectrum_B(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[7];
            }
            if(m == Dyn){
                mc->get_Dyn_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[2];
            }
            if(m == Dynbeta){
                mc->get_Dynbeta_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[2];
            }
            if(m == MLR){
                mc->get_MLR_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[4];
            }
            if(m == mSSH){
                mc->get_SrcFld_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[7];
            }
            if(m == SHpowder){
                mc->get_Powder_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[7];
            }
            if(m == SHpowderLev){
                mc->get_Powder_spectrum_lev(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[7];
            }
            if(m == FeCal){
                mc->get_FeCal_spectrum(Ibuf, run[irun].cc,run[irun].v,p,
                                       run[irun].T);
                Atemp += p[3] + p[4] + p[5];
            }
            if(m == FeCaltriang){
                mc->get_FeCaltriang_spectrum(Ibuf, run[irun].cc,run[irun].v,p,
                                       run[irun].T);
                Atemp += p[3] + p[4] + p[5];
            }
            if(m == afmBexpowder){
                mc->get_afm_Powder_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[8];
            }
            if(m == afmBexpowderLong){
                mc->get_afm_Powder_long_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[8];
            }
            if(m == fmBexpowder){
                mc->get_fm_Powder_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[7];
            }
            if(m == PseudoVoigt){
                mc->get_PseudoVoigt_profile(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[3];
            }
            if(m == FeAspowder){
                mc->get_FeAs_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[7];
            }
            if(m == Fe2Aspowder){
                mc->get_Fe2As_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[7];
            }
            if(m == DynDiagDiag){
                mc->get_DynDiagDiag_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[7];
            }
            if(m == DynDiagDiagDiag){
                mc->get_DynDiagDiagDiag_spectrum(Ibuf, run[irun].cc, run[irun].v,p);
                Atemp += p[10];
            }
            if(m == SkyrmionMBS){
                //mc->get_skyrmion(Ibuf, run[irun].cc, run[irun].v,p);
                //Atemp += p[6];
                double theta[90];
                mc->correlation_theta(theta,p[0],p[1],p[2],p[3],p[4],8.006);
                //mc->skyrmion_theta(theta,p[0],p[1],p[2],p[3],p[4],8.006);
                for(int u = 0; u < 90; u++)
                    Ibuf[u] = theta[u];
            }
            /*Uebertragen in Hauptarray bzw. MEM-Speicher, falls vorhanden*/
            run[irun].A += Atemp * weight;
            for(int i = 0; i < run[irun].cc; i++){
                run[irun].Ith[ims+1][i] += Ibuf[i] * weight;
                if(MEMIm != NULL)
                    MEMIm[i][MEMIndex] = MEMIm[i][MEMIndex] + Ibuf[i] * weight;
            }
            delete Ibuf;
        }

        /*Gewicht integrieren*/
        *integratedweight = *integratedweight + weight;
    }
}
void fitmodel::build_spectrum( double* p, int ip, int spectrumi,
                               int irun, int ims,
                               MoessCalc *mc,
                               double weight0, double* integratedweight,
                               double **MEMIm, int MEMIndex, int maxMEMIndex){
    build_iterator(p,ip,spectrumi,irun,ims,mc,false,weight0,integratedweight,
                   MEMIm,MEMIndex,maxMEMIndex);
}
void fitmodel::WeightSum(double* p,int ip, int spectrumi, int irun, int ims,
                         double weight, double* integratedweight){
    build_iterator(p,ip,spectrumi,irun,ims,NULL,true,weight,integratedweight,
                   NULL,0,0);
}
void fitmodel::fit(){
    fitthread** ft = new fitthread*[crun];
    pgvs = new int[crun];
    pgchi2 = new double[crun];
    for(int i = 0; i < crun; i++){
         pgvs[i] = 0;
         pgchi2[i] = 0.0;
    }
    fitruns = 1;
    runiterations = eval(MaxIter,0);
    if(is_seriell_fit()){
        if(ProgressPrint){
            pbinstance.setMaximum(crun*eval(MaxIter,0));
            pbinstance.setWindowTitle("Seriell fitting");
            pbinstance.setWindowModality(Qt::WindowModal);
            pbinstance.setValue(0);
            pbinstance.show();
        }
        for(int i = 0; i < crun; i++){
            ft[i] = new fitthread(&(pData[cGlobal+i*cSpec]),
                                  i,
                                  &(pError[cGlobal+i*cSpec]),
                                  cSpec,
                                  &operate_single_run,
                                  this,
                                  eval(MaxIter,0),
                                  eval(conv,0));
            if(ProgressPrint)
                connect(ft[i],SIGNAL(refresh_progress(int,int,double)),
                        this,SLOT(update_fit_progressbar(int,int,double))
                        ,Qt::BlockingQueuedConnection);
            connect(ft[i],SIGNAL(finished(int)),
                    this,SLOT(finish_fitting(int))
                    ,Qt::BlockingQueuedConnection);
            connect(&pbinstance,SIGNAL(canceled()),
                    ft[i],SLOT(stop_calculations()));
        }
        for(int i = 0; i < crun; i++)
            ft[i]->start();
    }
    else{
        if(ProgressPrint){
            pbinstance.setMaximum(eval(MaxIter,0));
            pbinstance.setWindowTitle("Global fitting");
            pbinstance.setWindowModality(Qt::WindowModal);
            pbinstance.setValue(0);
            pbinstance.show();
        }
        ft[0] = new fitthread(pData,
                              pIsChanged,
                              pError,cGlobal + cSpec * crun,
                              &operate_fitmodel,
                              (void*)this, eval(MaxIter,0),eval(conv,0),
                              forcesimplex);
        if(ProgressPrint)
            connect(ft[0],SIGNAL(refresh_progress(int,int,double)),
                    this,SLOT(update_fit_progressbar(int,int,double))
                    ,Qt::BlockingQueuedConnection);
        connect(ft[0],SIGNAL(finished(int)),
                this,SLOT(finish_fitting(int)));
        connect(&pbinstance,SIGNAL(canceled()),ft[0],SLOT(stop_calculations()));
        ft[0]->start();
    }
}
void fitmodel::fit_hidden_wait(){
    // save the values that will be changed by fit()
    int* keep1 = pgvs;
    double* keep2 = pgchi2;
    int keep3 = runiterations;
    // avoid progressbar
    ProgressPrint = false;
    // execute the fit and wait with loop for finish
    QEventLoop loop;
    connect(this, SIGNAL(all_fitting_finished()), &loop, SLOT(quit()));
    fit();
    loop.exec();
    // restore the initial state
    ProgressPrint = true;
    pgvs = keep1;
    pgchi2 = keep2;
    runiterations = keep3;
}
void fitmodel::errors(){
    // prepare progress control
    pgvs = new int[crun];
    pgchi2 = new double[crun];
    for(int i = 0; i < crun; i++){
         pgvs[i] = 0;
         pgchi2[i] = 0.0;
    }
    switch(errormode){
        case covariant: runiterations = cSpec*(cSpec+1)/2;
                        if(is_seriell_fit())
                            pbinstance.setMaximum(runiterations*crun+1);
                        else
                            pbinstance.setMaximum((cGlobal + cSpec * crun)
                                                 *(cGlobal + cSpec * crun+1)
                                                   *0.5);
                        pbinstance.setWindowTitle("Error calculation: Hesse");
                        break;
        case montecarlo:runiterations = eval(MCi,0);
                        pbinstance.setMaximum(qMax(1,(is_seriell_fit()*crun))
                                              *runiterations);
                        pbinstance.setWindowTitle("Error calculation: MonteCarlo");
                        break;
        case uncorrelated:
                        runiterations = cSpec;
                        pbinstance.setMaximum(cGlobal + cSpec * crun);
                        pbinstance.setWindowTitle("Error calculation: Uncorrelated");
                        break;
        case minos:     runiterations = cGlobal + cSpec * crun;
                        pbinstance.setMaximum(runiterations);
                        pbinstance.setWindowTitle("Error calculation: Minos");
                        break;
    }
    pbinstance.setWindowModality(Qt::WindowModal);
    pbinstance.setValue(0);
    pbinstance.show();
    // setup calulation
    ft = new fitthread*[crun];
    if(is_seriell_fit() && (errormode != minos)){
        for(int i = 0; i < crun; i++){
            ft[i] = new fitthread(&(pData[cGlobal+i*cSpec]),i,
                                  &(pError[cGlobal+i*cSpec]),
                                  cSpec,
                                  &operate_single_run,
                                  (void*)this,
                                  errormode,int(eval(MCi,0)));
            connect(ft[i],SIGNAL(refresh_progress(int,int,double)),
                    this,SLOT(update_fit_progressbar(int,int,double))
                    ,Qt::BlockingQueuedConnection);
            connect(ft[i],SIGNAL(finished(int)),
                    this,SLOT(finish_fitting(int))
                    ,Qt::BlockingQueuedConnection);
            connect(ft[i],SIGNAL(errors_calculated()),
                    this,SLOT(errors_updated()),
                    Qt::BlockingQueuedConnection);
            connect(ft[i],SIGNAL(error_message(int,QString)),
                    this,SLOT(error_message(int,QString)),
                    Qt::BlockingQueuedConnection);
            connect(&pbinstance,SIGNAL(canceled()),
                    ft[i],SLOT(stop_calculations()));
        }
        for(int i = 0; i < crun; i++)
            ft[i]->start();
    }
    else{
        // for correct excecution of update_fit_progressbar() if model is
        // seriell but Minos errors are executed, than -1 indicates Minos
        for(int i = 1; i < crun; i++)
            pgvs[i] = -1;
        ft[0] = new fitthread(pData,  pIsChanged,
                                      pError,
                                      cGlobal + cSpec * crun,
                                      &operate_fitmodel,(void*)this,errormode,
                                      &operate_fitting,eval(MCi,0),crun,cGlobal,
                                      &operate_correlation);
        connect(ft[0],SIGNAL(refresh_progress(int,int,double)),
                this,SLOT(update_fit_progressbar(int,int,double))
                ,Qt::BlockingQueuedConnection);
        connect(ft[0],SIGNAL(finished(int)),
                this,SLOT(finish_fitting(int)));
        connect(ft[0],SIGNAL(errors_calculated()),
                this,SLOT(errors_updated()));
        connect(ft[0],SIGNAL(error_message(int,QString)),
                this,SLOT(error_message(int,QString)));
        connect(&pbinstance,SIGNAL(canceled()),
                ft[0],SLOT(stop_calculations()));
        ft[0]->start();
    }
}
void calc_run_threated(int i, fitmodel* fm){
    fm->calc_run(i);
}
double fitmodel::calc_changed_runs(){
    QFutureSynchronizer<void> sync;
    for(int i = 0; i < crun; i++){
        for(int j = 0; j < cGlobal + cSpec * crun; j++)
            if(pIsChanged[j] && (affect[j] == i || affect[j] == -1)){
                /*Der Run muss neu berechnet werden*/
                QFuture<void> future = QtConcurrent::run(calc_run_threated,i,this);
                sync.addFuture(future);
                //calc_run(i);
                j = cGlobal + cSpec * crun; //Schleife verlassen
            }
    }
    sync.waitForFinished();
    return 1;
}
double fitmodel::get_total_chi2(){
    double chi2 = 0;
    for(int i = 0; i < crun; i++)
        chi2 = chi2 + run[i].chi2;
    return chi2;
}
double fitmodel::get_run_chi2(int runi){
    return run[runi].chi2;
}
int fitmodel::get_affect(int i){
    return affect[i];
}
double operate_fitmodel(void* instance){
    fitmodel *fm = (fitmodel*) instance;
    fm->calc_changed_runs();
    return fm->get_total_chi2();
}
double operate_single_run(void* instance, int runi){
    fitmodel *fm = (fitmodel*) instance;
    fm->calc_run(runi);
    return fm->get_run_chi2(runi);
}
double operate_fitting(void* instance){
    fitmodel *fm = (fitmodel*) instance;
    fm->fit_hidden_wait();
    return fm->get_total_chi2();
}

int fitmodel::read_fitdata(){
    FILE* f =fopen(QString(InputFile).toLocal8Bit().data(),"r");
    char buffer[10000];
    int readon = 1;
    while(!feof(f) && readon){
        fgetss(buffer,10000,f);
        if(!strcmp(buffer,"FITDATA\n"))
            readon = 0;
    }
    if(readon){
        /*keine alten Fitdaten gefunden*/
        fclose(f);
        return 1;
    }
    else{
        /*alte Fitdaten existieren, zuerst schauen, ob sie valid sind*/
        QString ExpectedHeadLine = QString("%1\t").arg("T[K]",15);
        QString RegExp("( *-?\\d+\\.?\\d*\\t)");
        for(int i = 0; i < cpara; i++){
            ExpectedHeadLine.append(QString("%1\t%2\t").arg(fmp[i].name,15).arg("Error",15));
            RegExp.append("( *-?\\d+\\.?\\d*\\t)( *-?\\d+\\.?\\d*\\t|      press 'e'\\t)");
            //RegExp.append("( *-?\\d+\\.?\\d*\\t)( *-?\\d+\\.?\\d*\\t)");
        }
        ExpectedHeadLine.append("\n");
        RegExp.append("\n");
        fgetss(buffer,10000,f);
        if(ExpectedHeadLine.compare(buffer)){
            ErrorMessage.append("Expected the following FITDATA headline:\n");
            ErrorMessage.append(ExpectedHeadLine);
            fclose(f);
            return 0;
        }
        else{
            /*Kopfzeile stimmt ueberein, pruefe Format und Anzahl der Daten*/
            long int start = ftell(f);  //Position merken
            QRegExp red(RegExp);
            int cregexp = 0;
            readon = 1;
            while(!feof(f) && readon){
                fgetss(buffer,10000,f);
                if(red.indexIn(buffer)==0)
                    cregexp++;
                else
                    readon = 0;
                buffer[0] = '\0';
            }
            if(cregexp != crun){
                ErrorMessage.append(QString("FITDATA: %1 datasets expected, but %2 valid found.\n").arg(crun).arg(cregexp));
                fclose(f);
                return 0;
            }
            else{
                /*Daten können übernommen werden*/
                fseek(f,start,SEEK_SET);
                for(int i = 0; i < crun; i++){
                    fgetss(buffer,10000,f);
                    red.indexIn(buffer);
                    QStringList sl = red.capturedTexts();
                    for(int j = 0; j < cpara; j++){
                        *(rData [j][i])= QString(sl.at(2+j*2  )).remove("\t")
                                                                .toDouble();
                        if(QString(sl.at(2+j*2+1)).remove("\t").toDouble()<0)
                            *(rError [j][i])=-1;
                    }
                }
                fclose(f);
                return 1;
            }
        }
    }
}
int fitmodel::read_MEMdata(){
    FILE* f =fopen(QString(InputFile).toLocal8Bit().data(),"r");
    char buffer[10000];
    int readon = 1;
    while(!feof(f) && readon){
        fgetss(buffer,10000,f);
        if(!strcmp(buffer,"MEM OR HR DATA\n"))
            readon = 0;
    }
    if(readon){
        /*keine alten MEMdaten gefunden*/
        fclose(f);
        return 1;
    }
    else{
        /*alte MEMdaten existieren, zuerst schauen, ob sie valid sind*/
        QString ExpectedHeadLine = QString("%1\t").arg("T[K]",15);
        QString RegExp("( *\\d+\\.?\\d*\\t)");
        char MEMcode[1000];
        for(int i = 0; i < cMEMsubspec; i++){
            get_MEMcode(i,MEMcode);
            ExpectedHeadLine.append(QString("%1\t%2\t").arg(MEMcode,15).arg("Error",15));
            RegExp.append("( *-?\\d+\\.?\\d*\\t)( *-?\\d+\\.?\\d*\\t)");
        }
        ExpectedHeadLine.append("\n");
        RegExp.append("\n");
        fgetss(buffer,10000,f);
        if(ExpectedHeadLine.compare(buffer)){
            ErrorMessage.append("Expected the following MEMDATA headline:\n");
            ErrorMessage.append(ExpectedHeadLine);
            fclose(f);
            return 0;
        }
        else{
            /*Kopfzeile stimmt ueberein, pruefe Format und Anzahl der Daten*/
            long int start = ftell(f);  //Position merken
            QRegExp red(RegExp);
            int cregexp = 0;
            readon = 1;
            while(!feof(f) && readon){
                fgetss(buffer,10000,f);
                if(red.indexIn(buffer)==0)
                    cregexp++;
                else
                    readon = 0;
                buffer[0] = '\0';
            }
            if(cregexp != crun){
                ErrorMessage.append(QString("MEMDATA: %1 datasets expected, but %2 valid found.\n").arg(crun).arg(cregexp));
                fclose(f);
                return 0;
            }
            else{
                /*Daten können übernommen werden*/
                fseek(f,start,SEEK_SET);
                for(int i = 0; i < crun; i++){
                    fgetss(buffer,10000,f);
                    red.indexIn(buffer);
                    QStringList sl = red.capturedTexts();
                    for(int j = 0; j < cMEMsubspec; j++){
                        run[i].rho[j] = QString(sl.at(2+j*2  )).remove("\t").toDouble();
                        run[i].rhoerror[j]= QString(sl.at(2+j*2+1)).remove("\t").toDouble();
                    }
                }
                fclose(f);
                return 1;
            }
        }
    }
}
void fitmodel::print_fitdata(){
    //QLocale::setDefault(QLocale::English);
    /*zunaechst alte Datei umbennen*/
        remove(QString(InputFile).append("old").toLocal8Bit().data());
    rename(QString(InputFile).toLocal8Bit().data(),
           QString(InputFile).append("old").toLocal8Bit().data());
    FILE* fnew =fopen(QString(InputFile).toLocal8Bit().data(),"w");
    FILE* fold =fopen(QString(InputFile).append("old").toLocal8Bit().data(),"r");
    char buffer[10000];
    int readon = 1;
    while(!feof(fold) && readon){
        fgetss(buffer,10000,fold);
        if(!strcmp(buffer,"FITDATA\n"))
            readon = 0;
        else
            fprintf(fnew,"%s",buffer);
    }
    if(feof(fold))
        fprintf(fnew,"\n");
    fclose(fold);
    /*jetzt fitdata schreiben*/
    /*Tabellenkopf*/
    fprintf(fnew,"FITDATA\n");
    fprintf(fnew,"%15s\t","T[K]");
    for(int i = 0; i < cpara; i++)
        fprintf(fnew,"%15s\t%15s\t",fmp[i].name.toLocal8Bit().data(),"Error");
    fprintf(fnew,"\n");
    /*Fit Daten*/
    char format[100];
    for(int i = 0; i < crun; i++){
        fprintf(fnew,"%15.2f\t",run[i].T);
        for(int j = 0; j < cpara; j++){
            sprintf(format,"%%%i.%if\t",
                    qMax(15,fmp[j].name.length()),
                    fmp[j].printDigits);
            fprintf(fnew,format,*(rData[j][i]));
            if(errors_up_to_data){
                sprintf(format,"%%15.%if\t", fmp[j].printDigits);
                fprintf(fnew,format,*(rError[j][i]));
            }
            else{
                if(*(rError[j][i]) == -1)   fprintf(fnew,"             -1\t");
                else                        fprintf(fnew,"      press 'e'\t");
            }
        }
        fprintf(fnew,"\n");
    }
    /*MEM Daten ggf.*/
    if(cMEMsubspec > 1){
        fprintf(fnew,"###################\nMEMDATA\n");
        /*Kopfzeile*/
        fprintf(fnew,"%15s\t","T[K]");
        char MEMcode[1000];
        for(int j = 0; j < cMEMsubspec; j++){
            get_MEMcode(j,MEMcode);
            fprintf(fnew,"%15s\t%15s\t",MEMcode,"Error");
        }
        fprintf(fnew,"\n");
        /*Daten*/
        for(int i = 0; i < crun; i++){
            fprintf(fnew,"%15.1f\t",run[i].T);
            for(int j = 0; j < cMEMsubspec; j++)
                fprintf(fnew,"%15.6f\t%15.6f\t",run[i].rho[j],run[i].rhoerror[j]);
            fprintf(fnew,"\n");
        }
    }
    /*FITstatistik ausgeben*/
    fprintf(fnew,"###################\nFITSTATISTICS\nTotal chi2: %f\n",get_total_chi2());
    fprintf(fnew,"%15s\t%15s\t%15s\t%15s\t%15s\t%15s\n",
            "T[K]","chi2_red","K-S-confidence","MEM-diff","MEM-lambda","MEM-iterations");
    for(int i = 0; i < crun; i++){
        fprintf(fnew,"%15.2f\t%15.2f\t%15.3f\t%15.3f\t%15.3f\t%15.0f\n",
                run[i].T,
                run[i].chi2/qMax(1e-10,1.*(run[i].ccr-1-cSpec-cConst)),
                KSconfidence(i),
                run[i].MEMstatus[3],
                run[i].MEMstatus[4],
                run[i].MEMstatus[0]);
    }

    fclose(fnew);
}
void fitmodel::get_MEMcode(int istep, char* code){
    QString result;
    for(int j = 0; j < cMEMpara; j++)
        result.append(QString("%1-").arg(get_MEMcode(istep,j),4));
    strcpy(code,result.toLocal8Bit().data());
}
double fitmodel::get_MEMcode(int istep, int ipara){
    if(cmbs > i1stMEMspec && cMEMpara > 0){
        // a MEM calculation does exist
        int j=-1;
        int divisor =  cMEMsubspec;
        while(j < pc(i1stMEMspec) &&
              ipara >= 0 ){
            j++;
            if(is_arbitary_distribution(i1stMEMspec,j)){
                ipara--;
                istep = istep % divisor;
                divisor = divisor / eval(spectra[i1stMEMspec].p[j].arg[3],0);
            }
        }
        istep = istep / divisor;
        double min = eval(spectra[i1stMEMspec].p[j].arg[1],0);
        double max = eval(spectra[i1stMEMspec].p[j].arg[2],0);
        int steps = int(eval(spectra[i1stMEMspec].p[j].arg[3],0));
        return min +(max-min)*istep/(qMax(2,steps)-1);
    }
    else
        return 0;
}
int fitmodel::ac(ParameterTyp m){
    switch(m){
        case pDISTR:
            return 5;
        case pMEM:
            return 4;
        case pMEMEXT:
            return 5;
        case pHR:
            return 4;
        case pHREXT:
            return 5;
        case pDISTREXT:
            return 6;
        case pObj:
            return 1;
        default:
            return 0;
    }
}

int fitmodel::pc(MSTyp m){
    // note the expected argument count in this function
    if(m == SHcrystal) return 10;
    if(m == SHcrystalG) return 10;
    if(m == SHcrystalB) return 10;
    if(m == SHpowder) return 8;
    if(m == SHpowderLev) return 9;
    if(m == mSSH) return 14;
    if(m == BL) return 1;
    if(m == FeCal) return 8;
    if(m == FeCaltriang) return 8;
    if(m == AnyFit) return 2;
    if(m == afmBexpowder) return 11;
    if(m == afmBexpowderLong) return 11;
    if(m == fmBexpowder) return 10;
    if(m == PseudoVoigt) return 4;
    if(m == FeAspowder) return 5;
    if(m == Fe2Aspowder) return 6;
    if(m == Dyn) return 3;
    if(m == Dynbeta) return 4;
    if(m == MLR) return 9;
    if(m == DynDiagDiag) return 11;
    if(m == DynDiagDiagDiag) return 18;
    if(m == SkyrmionMBS) return 5;//11;
    return 0;
}
int fitmodel::pc(int spectrumi){
    if(spectra[spectrumi].typ == Dyn || spectra[spectrumi].typ == Dynbeta){
        int J = eval(spectra[spectrumi].p[0].arg[0],0);
        return pc(spectra[spectrumi].typ) + J*(6+J-1);
    }
    else
        return pc(spectra[spectrumi].typ);
}

void fitmodel::update_fit_progressbar(int ID, int citer,double chi2){
    pgvs[ID] = citer;
    pgchi2[ID] = chi2;
    int newprgrval = 0;
    double newchi2 = 0;
    for(int i = 0; i < crun; i++){
        newprgrval += qMax(0,pgvs[i]);  // pgvs might be -1 in Minos-calulation
        newchi2 += pgchi2[i];
    }
    pbinstance.setLabelText(QString("chi2 = %1\nchi2_red= %2")
                            .arg(newchi2).arg(newchi2/crun/(run[0].cc-1)));
    pbinstance.setValue(newprgrval);
    }
void fitmodel::errors_updated(){
    errors_up_to_data = true;
}
void fitmodel::finish_fitting(int ID){
    if(is_seriell_fit()){
        int allfinsished = 1;
        pgvs[ID] = runiterations;
        for(int i = 0; i < crun; i++)
            if(pgvs[i] != runiterations && pgvs[i] >= 0)
                allfinsished = 0;
        if(allfinsished || ID == -1){
            fitruns = 0;
            if(ProgressPrint){
                pbinstance.hide();
                emit ready_to_plot();
                print_fitdata();
                if(fmpa != NULL)
                    fmpa->refresh();
            }
            emit all_fitting_finished();

        }
    }
    else{
        fitruns = 0;
        if(ProgressPrint){
            pbinstance.hide();
            emit ready_to_plot();
            print_fitdata();
            if(fmpa != NULL)
                fmpa->refresh();
        }
        emit all_fitting_finished();

    }
}
int fitmodel::is_seriell_fit(){
    int fitseriell = 1;
    for(int i = 0; i < cpara; i++)
        if(!fmp[i].runspecific)
            fitseriell = 0;
    return fitseriell;
}
void fitmodel::error_message(int runi, QString message){
    if(runi < crun && runi >= 0)
        message.prepend(QString("run %1: ").arg(run[runi].T));
    QMessageBox(QMessageBox::NoIcon,"Error Calculation",
                message,0,0).exec();
}
void fitmodel::panel_input(int irun, int ipara){
    if(irun < 0 || !fmp[ipara].runspecific)
        // all runs were affected by changes and are now to be recalculated or
        // the changed parameter was a global parameter
        for(int k = 0; k < crun; k++)
            calc_run(k);
    else
        // recalculate the changed run only
        calc_run(irun);
    errors_up_to_data = false;
    print_fitdata();
    emit ready_to_plot();
}
void fitmodel::close_fmpa(){
    if(fmpa != NULL){
        fmpa->close();
        fmpa = NULL;
    }

}
void fitmodel::restrict_fitrange(int irun){
    int crange = run[irun].ranges;
    double* mins = run[irun].rangemin;
    double* maxs = run[irun].rangemax;
    if(crange > 0){
        double *newv = new double[run[irun].cc];
        double *newI = new double[run[irun].cc];
        double *newerr = new double[run[irun].cc];
        /*mark channels to hide*/
        int is = 0, ip = run[irun].cc-1;
        for(int i = 0; i < run[irun].cc; i++){
            int show = 0;
            for(int j = 0; j < crange; j++)
                if(run[irun].v[i] > mins[j] && run[irun].v[i] < maxs[j]){
                    j = crange;
                    show = 1;
                }
            if(show){
                newv[is] = run[irun].v[i];
                newI[is] = run[irun].I[i];
                newerr[is] = run[irun].err2rec[i];
                is++;
            }
            else{
                newv[ip] = run[irun].v[i];
                newI[ip] = run[irun].I[i];
                newerr[ip] = run[irun].err2rec[i];
                ip--;
            }
        }
        delete run[irun].v;
        delete run[irun].I;
        delete run[irun].err2rec;
        run[irun].v = newv;
        run[irun].I = newI;
        run[irun].err2rec = newerr;
        run[irun].ccr = is;
    }
    else
        run[irun].ccr = run[irun].cc;
}
void fitmodel::anyfit(int irun, int ims){
    for(int i = 0; i < run[irun].cc; i++){
        set_obj(spectra[ims].p[0].arg[0],irun,run[irun].v[i]);
        run[irun].Ith[ims+1][i] += eval(spectra[ims].p[1].arg[0],irun);
    }
}
void fitmodel::calculate_velocity_data(int runi){
    for(int i = 0; i < run[runi].cc; i++){
        set_obj(UDc,runi,i);
        run[runi].v[i] = eval(UDf,runi);
        double check = eval(UDI,runi);
        run[runi].I[i] = run[runi].Iunsorted[i] * eval(UDI,runi);
    }
    if(run[runi].ranges > 0 ||
       (ta >= 0 && omegasrc >= 0 && omegaabs >= 0 && fr >= 0) ||
       CosineSmearing){
        // if restricted fit range or transmission integral, the data must be
        // sorted
        for(int i = 0; i < run[runi].cc; i++)
            run[runi].err2rec[i] = 1./qMax(1e-50,qAbs(run[runi].Iunsorted[i]));
        mergesort(runi);
        restrict_fitrange(runi);
    }
}
void fitmodel::mergesort(int runi){
    SortContainer* sc1 = new SortContainer[run[runi].cc];
    SortContainer* sc2 = new SortContainer[run[runi].cc];
    for(int i = 0; i < run[runi].cc; i++){
        sc1[i].x = run[runi].v[i];
        sc1[i].y = run[runi].I[i];
        sc1[i].err2 = run[runi].err2rec[i];
    }
    // sort data
    mergesort(sc1,sc2,0,run[runi].cc-1);
    // get sorted data
    for(int i = 0; i < run[runi].cc; i++){
        run[runi].v[i] = sc1[i].x;
        run[runi].I[i] = sc1[i].y;
        run[runi].err2rec[i] = sc1[i].err2;
    }
    delete sc1;
    delete sc2;
}

void fitmodel::mergesort(SortContainer *a, SortContainer*b, int low, int high){
    if(low<high){
        int pivot=(low+high)/2;
        mergesort(a,b,low,pivot);
        mergesort(a,b,pivot+1,high);
        merge(a,b,low,pivot,high);
    }
}
void fitmodel::merge(SortContainer *a, SortContainer *b, int low, int pivot, int high){
    int h = low,i = low,j = pivot+1;
    while((h<=pivot)&&(j<=high)){
        if(a[h].x<=a[j].x){
            b[i]=a[h];
            h++;
        }
        else{
            b[i]=a[j];
            j++;
        }
        i++;
    }
    if(h>pivot){
        for(int k=j; k<=high; k++){
            b[i]=a[k];
            i++;
        }
    }
    else{
        for(int k=h; k<=pivot; k++){
            b[i]=a[k];
            i++;
        }
    }
    for(int k=low; k<=high; k++)
        a[k]=b[k];
}
Obj::Obj(QString name,ObjTyp typ, int i){
    this->name = name;
    this->typ = typ;
    this->ref = i;
}
Obj::Obj(QString name){
    this->name = name;
}

Obj::Obj(QString name,double i){
    this->name = name;
    typ = numtyp;
    dnumber = i;
}
Obj::~Obj(){}
RUN::RUN(){
    T = 0;
    cc = ccr = cmbs = cMEM = 0;
    v = I = Iunsorted = err2rec = nI = nIth = rangemin = rangemax = NULL;
    ranges = 0;
    Ith = NULL;
    map = NULL;
    I0 = A = 0;
    chi2 = 1.;
    rho = rhoerror = NULL;
    MEMstatus[0]= MEMstatus[1]= MEMstatus[2]= MEMstatus[3]= MEMstatus[4]= 0.0;
}
RUN::~RUN(){
    if(Ith != NULL)
        for(int i = 0; i < cmbs+1; i++)
            delete Ith[i];

    if(nIth != NULL)    delete nIth;
    if(nI != NULL)      delete nI;
    if(rho != NULL)     delete rho;
    if(rhoerror != NULL)delete rhoerror;
    if(rangemin != NULL)delete rangemin;
    if(rangemax != NULL)delete rangemax;
}
void RUN::init(int Speccount, int MEMsamplecount){
    cmbs = Speccount;
    cMEM = MEMsamplecount;
    Ith = new double*[cmbs+1];
    if(cMEM > 0){
        rho = new double[cMEM];
        rhoerror = new double[cMEM];
        init_array(rho,cMEM,1./cMEM);
        init_array(rhoerror,cMEM,0);
    }
    for(int j = 0; j < cmbs+1; j++)
        Ith[j] = new double[cc];
    nIth = new double[cc];
    nI = new double[cc];
}
double fitmodel::eval_error(int iobj, int irun){
    switch(obj.at(iobj).typ){
        case partyp:
            return *rError[obj[iobj].ref][irun];
        case maptyp:
            return eval_error(run[irun].map[obj[iobj].ref],irun);
        default:
            return 0.;
    }
    return 0.;
}
double fitmodel::eval(int iobj, int irun){
    switch(obj.at(iobj).typ){
        case partyp:
            return *rData[obj[iobj].ref][irun];
        case maptyp:
            return eval(run[irun].map[obj[iobj].ref],irun);
        case numtyp:
            return obj[iobj].dnumber;
        case funtyp:
            return fv(irun,obj[iobj].ref);
        case runspecificparameter:
            return run[irun].T;
    }
    return 0.;
}
bool Obj::operator==(Obj o){
    if(this->name == o.name)
        return true;
    else
        return false;
}
bool Obj::operator<(const Obj o2)const{
    if(name.length() > o2.name.length())
        return true;
    else
        if(name.length() < o2.name.length())
            return false;
        else
            return (name < o2.name);
}
double fitmodel::KSconfidence(int irun){
    double *r = new double[run[irun].ccr];
    for(int k = 0; k < run[irun].ccr; k++)
        r[k] =        (run[irun].Ith[0][k]-run[irun].I[k])
                     * sqrt(run[irun].err2rec[k]);
    // sort the residuums
    sort(r,0,run[irun].ccr-1);

    // determine Dn and Kalpha
    double Dn = 0;
    for(int k = 0; k < run[irun].ccr; k++)
        Dn = qMax(Dn,qAbs(1.*k/(run[irun].ccr-1)-0.5*(1+gsl_sf_erf(r[k]/sqrt(2)))));
    double Kalpha = Dn*sqrt(run[irun].ccr)  ;

    // calculate confidence alpha from Kalpha;
    double Pri, Pr = 0;
    int i = 1;
    do{
        Pri = sqrt(2*M_PI)/Kalpha*exp(-((2*i-1)*(2*i-1)*M_PI*M_PI/(8*Kalpha*Kalpha)));
        i++;
        Pr += Pri;
    }while(Pri > 1e-10);
    double alpha = 1 - Pr;

    // finsih
    delete r;
    return alpha;
}
void fitmodel::fgetss(char* buffer,int max,FILE* f){
    fgets(buffer,max,f);
    int len = strlen(buffer);
    if(len > 1)
        if(buffer[len-2]=='\r'){
            buffer[len-2]='\n';
            buffer[len-1]='\0';
        }
}
void fitmodel::view_run(int i){
    if(i < 0){
        plotrunlist.clear();
        for(int i = 0; i < crun; i++)
            plotrunlist.append(i);
    }
    else
        if(!plotrunlist.contains(i))
            plotrunlist.append(i);

    emit ready_to_plot();
}
void fitmodel::view_run_only(int i){
    plotrunlist.clear();
    plotrunlist.append(i);
    emit ready_to_plot();
}
void fitmodel::hide_run(int i){
    int j = plotrunlist.indexOf(i);
    if(j > -1){
        plotrunlist.removeAt(j);
        emit ready_to_plot();
    }
}
void fitmodel::reset_error(int runi, int parai, bool fit){
    if(fit)
        *(rError[parai][runi]) = fmp[parai].step;
    else
        *(rError[parai][runi]) = -1.;
    errors_up_to_data = false;
    print_fitdata();
}
void fitmodel::generate_default(QStringList flds){
    QString mbspath = QString(flds[0]).left(flds[0].length()-3).append("mbs");
    if(QFile(mbspath).exists())
        if(QMessageBox(QMessageBox::Question,
                       "mbs-file creation error, overwrite file?",
                       QString(mbspath).append(" does already exist. Overwrite file with default model?"),
                       QMessageBox::Yes|QMessageBox::No).exec() == QMessageBox::No)
            return;
    FILE* f = fopen(mbspath.toLocal8Bit().data(),"w");
    fprintf(f,"######################\n");
    fprintf(f,"FITPARAMETER\n");
    fprintf(f,"B 33 1\n");
    fprintf(f,"Vzz 50 2\n");
    fprintf(f,"eta 0 0.1\n");
    fprintf(f,"theta 0 5\n");
    fprintf(f,"phi 0 5\n");
    fprintf(f,"CS 0.4 0.02\n");
    fprintf(f,"omega 0.15 0.01\n");
    fprintf(f,"I0 1 1\n");
    fprintf(f,"A0 1 1\n");
    fprintf(f,"beta 54.7 3\n");
    fprintf(f,"gamma 0 5\n");
    fprintf(f,"###############################################################\n");
    fprintf(f,"THEORY\n");
    fprintf(f,"BL I0\n");
    fprintf(f,"SHc B Vzz eta theta phi CS omega A0 beta gamma\n");
    fprintf(f,"###############################################################\n");
    fprintf(f,"FUNCTIONS\n");
    fprintf(f,"#A0 = I0 * 0.05\n");
    fprintf(f,"###############################################################\n");
    fprintf(f,"COMMANDS\n");
    fprintf(f,"A A0\n");
    fprintf(f,"I0 I0\n");
    fprintf(f,"MaxIter 5000\n");
    fprintf(f,"Errors Hesse\n");
    fprintf(f,"#ForceSimplex\n");
    fprintf(f,"#ConvergenceCriterium 0.01\n");
    fprintf(f,"#MEM 500 0.013 0.1 lambda 70000\n");
    fprintf(f,"#HR 0.1 lamda\n");
    fprintf(f,"#TransmissionIntegral ta 0.0515 omega_abs fr\n");
    fprintf(f,"#UnfoldedData velocity intensity chn\n");
    fprintf(f,"#CosineSmearing CosineSmearing r0 r1 r2 z1 z2\n");
    fprintf(f,"#Nucleus Egamma g12 g32 Q mass\n");
    fprintf(f,"###############################################################\n");
    fprintf(f,"RUN\n");
    for(int i = 0; i < flds.length();i++)
        fprintf(f,"\"%s\" %i\n",flds[i].toLocal8Bit().data(),i+1);
    fprintf(f,"###############################################################\n");
    fprintf(f,"PLOT\n");
    fprintf(f,"runs");
    for(int i = 0; i < flds.length();i++)
        fprintf(f," %i",i+1);
    fprintf(f,"\n");
    fprintf(f,"###############################################################\n");
    fclose(f);
}
void fitmodel::rewrite_FITPARAMETER(){
    //rename old file
    remove(QString(InputFile).append("old").toLocal8Bit().data());
    rename(QString(InputFile).toLocal8Bit().data(),
           QString(InputFile).append("old").toLocal8Bit().data());
    FILE* fnew =fopen(QString(InputFile).toLocal8Bit().data(),"w");
    FILE* fold =fopen(QString(InputFile).append("old").toLocal8Bit().data(),"r");
    char buffer[10000];
    int readon = 1;
    // copy until fitparameter block
    while(!feof(fold) && readon){
        fgetss(buffer,10000,fold);
        if(!strcmp(buffer,"FITPARAMETER\n"))
            readon = 0;
        else
            fprintf(fnew,"%s",buffer);
    }
    // generate new FITPARAMETER block
    fprintf(fnew,"FITPARAMETER\n",buffer);
    int ipara = 0;
    while(!feof(fold) && !blockend(buffer)){
        fgetss(buffer,10000,fold);
        if(validline(buffer) && ipara < cpara){
            fprintf(fnew,"%s %.*f %.*f\n",fmp[ipara].name.toLocal8Bit().data(),
                    fmp[ipara].printDigits-2,fmp[ipara].value,
                    fmp[ipara].printDigits-2,fmp[ipara].step);
            ipara++;
        }
        else
            fprintf(fnew,"%s",buffer);
    }

    // copy further
    while(!feof(fold)){
        fgetss(buffer,10000,fold);
        fprintf(fnew,"%s",buffer);
    }
    // close files
    fclose(fold);
    fclose(fnew);
}

fmPanel* fitmodel::get_pannel(){
    return fmpa;
}
void fitmodel::add_noise(){
    for(int i = 0; i < crun; i++)
        MoessCalc().add_noise(run[i].Ith[0],run[i].cc);
    emit ready_to_plot();
}
bool operate_correlation(void* instance, int par1, int par2){
    fitmodel *fm = (fitmodel*) instance;
    return    fm->get_affect(par1) == fm->get_affect(par2)  // same run
           || fm->get_affect(par1) == -1                    // is global
           || fm->get_affect(par2) == -1;                   // is global
}
bool fitmodel::blockend(char* line){
    if(line[0] == '\n')
        return false;
    else{
        if(line[0] == '#' && line[1] == '#')
            return true;
        else
            return false;
    }
}
bool fitmodel::validline(char* line){
    if(line[0] == '\n' || line[0] == '#')
        return false;
    else
        return true;
}
void fitmodel::calc_baseline_area(int runi){
    int imin = 0; int imax = crun-1;    // by default, all runs are calc
    if(runi >= 0) imin = imax = runi;   // if only one run should be calc
    for(int i = 0; i < crun; i++)
        MoessCalc().baseline(run[i].v,run[i].I,run[i].cc,
                             &(run[i].I0),&(run[i].A),1./0.13,0.13,
                             NULL,NULL,NULL,NULL);

    if(autoA >= 0)
        for(int i = 0; i < crun; i++){
            set_obj(autoA,i,run[i].A);
            if(eval_error(autoA,i) > 0)
                set_error(autoA,i,0.01*(qAbs(run[i].A)));
        }
    if(autoI0 >= 0)
        for(int i = 0; i < crun; i++){
            set_obj(autoI0,i,run[i].I0);
            if(eval_error(autoI0,i) > 0)
                set_error(autoI0,i,0.01*(qAbs(run[i].I0)));
        }
}
void fitmodel::process_Binning(){
    for(int i = 0; i < crun;i++){
        int b = qMax(1,int(eval(Binning,i)));
        if(b > 1){
            // process binning only if necessary
            run[i].ccr = run[i].cc = run[i].ccr/b;
            for(int k = 0; k < run[i].ccr; k++){
                double newv = 0;
                int newI = 0;
                for(int kk = k * b; kk < (k+1) * b; kk++){
                    newv += run[i].v[kk];
                    newI += run[i].I[kk];
                }
                run[i].v[k] = newv / b;
                run[i].I[k] = newI;
            }
        }
    }
}
