﻿#ifndef FITMODEL_H
#define FITMODEL_H
#define MAX_PC 200

#include "musr.h"
#include "fitthread.h"
#include "plot2D.h"
#include "Moesscalc.h"
#include "array.h"
#include "fparser.hh"
#include <stdio.h>
#include <stdlib.h>
#include <QDir>
#include <QFileInfo>
#include <QMessageBox>
#include <QStringList>
#include <QString>
#include <QtConcurrentRun>
#include <QFutureSynchronizer>
#include <QFutureWatcher>
#include <QTimer>
#include <QMutex>
#include <QProgressBar>
#include <QLocale>
#include "fmpanel.h"

/*Fitmodel definieren*/
enum ObjTyp{funtyp,partyp,maptyp,numtyp,runspecificparameter};
enum ParameterTyp{pObj,pDISTR, pDISTREXT,pMEM,pMEMEXT,pHR,pHREXT};
enum MSTyp{SHcrystal,SHcrystalG,SHcrystalB,afmBexpowder,afmBexpowderLong,fmBexpowder,PseudoVoigt,MLR,SHpowder,SHpowderLev,mSSH,BL, FeCal, FeCaltriang,AnyFit,FeAspowder,Fe2Aspowder,DynDiagDiagDiag,DynDiagDiag,Dyn,Dynbeta,SkyrmionMBS};
enum AdditionalFunTypes{TAbsDeb,TQuadDop,SwaveSFD,Hc2WHH,GLBrandttriang,GLBrandtquad,Errorfunction,OrderParameterDistrTN,FerroMag,Bezier4P};
//! symbol class to organize complex us er imput
/*! user imput is done by symbols only. That means on every place where finally
 *  a number is expected the user states a symbol which might be resolved to a
 *  number, map, function or fit parameter. An Object knows its type due to the
 *  place of declaration (i.g. function objects are declaredin the FUNCTIONS
 *  block only). Using the fitmodel::eval() function the the corresponding
 *  number to an object can be returned.
 */
class Obj{
    public:
        //! create am onject of specific type, with reference i fitmodel::rData
        Obj(QString name,ObjTyp typ, int i);
        Obj(QString name,double i);     //!< creates constant number, value i
        Obj(QString name);              //!< creates without functionality
        ~Obj();
        bool operator==(QString name); //!< to compare Obj with each other
        bool operator==(Obj o);        //!< to compare Obj with each other
        bool operator<(const Obj o2)const;
        double dnumber;                 //!< float value if it is const. number
        QString name;                   //!< declaration name
        ObjTyp typ;                     //!< specifies how to evaluate this Obj
        int ref;                        //!< index to fitmodel::rData it belongs to
};
typedef struct Parameter{
    ParameterTyp typ;
    QStringList argv;
    int arg[6];
}Parameter;

//! contains all information related to the run
/*! all charateristics which was specified concerning the run will be saved in
 *  this instance. This is primarily the data to fit, but also data range
 *  restrictions and map parameters. Several functions of fitmodel will work on
 *  entities like RUN::I0 or RUN::A, in particular will the lasts calculated
 *  spectrum be save in Run::Ith
 */
class RUN{
    public:
        RUN();
        ~RUN();
        void init(int Speccount, int MEMsamplecount);
        double T;                       //!< run specif. param. (usual. temp.)
        int cc;                         //!< channel count (= data point count)
        int ccr;                        //!< channel count, restricted by user
        double *v;                      //!< x-data (i.e. velocity for Moessb.)
        double *I;                      //!< y-data (i.e. intensity for Moessb.)
        double *Iunsorted;              //!< y-data (i.e. intensity for Moessb.)
        double *err2rec;                //!< y-data error, squared, reciprocal
        //! simulated spectra including RUN::cmbs subspectra
        /*! holds the simulated data, the first idex specifies the subspectra
         *  with index 0 being the total spectrum. Second index is the data
         *  index. Thus Ith = double[1+RUN::cmbs][cc]
         */
        double **Ith;
        double *nI;                     //!< normalized data
        double *nIth;                   //!< normalized total fit
        //! map parameter names
        /*! the name strings will be resolved by fitmodel::link to an index
         *  in the array fitmodel::obj, which is stored in RUN::map
         */
        QStringList maplist;
        int* map;                       //!< map indeces in fitmodel::obj
        double I0;                      //!< preestimate data base line
        double A;                       //!< preestimated/updated spectral area
        double chi2;                    //!< current fit quality measure
        double *rho;                    //!< MEM-distribution
        double *rhoerror;               //!< MEM-dostribution errors
        double MEMstatus[5];            //!< latest statistics of MEM calcul.
        //! number of intervalls to be fitted
        /*! the user can specify xdata ranges, that should be used during. By
         *  that he can cut the data. The number of the valid data intervals is
         *  saved in ranges
         */
        int ranges;
        double* rangemin;               //!< intervall start
        double* rangemax;               //!< intervall start
    private:
        int cmbs;                       //!< subspectra count, size of RUN::Ith
        int cMEM;                       //!< MEM sample count, size of RUN::rho
};

//! information of the subspectra defined in the THEORY block of a mbs file
typedef struct MoessbauerSpectrum{
    MSTyp typ;                          //!< model to describe the spectrum
    Parameter p[MAX_PC];                //!< parameters of the specific model
}MoessbauerSpectrum;

//! fitmodel parameters, as defines in the FITPARAMETER block of a mbs file
typedef struct fmPara{
    double value;                       //!< initial value
    double step;                        //!< stepsize, 0 corresponds to "const"
    QString name;                       //!< user defined parameter name
    bool runspecific;                   //!< name does not start with "global_"
    int printDigits;                    //!< number of digits at FITDATA output
}fmPara;

//! specific functions which are available for FUNCTIONS defintions
typedef struct AdditionalFun{
    AdditionalFunTypes typ;
    int cpara;                          //!< argument count
    int arg[9];                         //!< argument refer. to fitmodel::obj
}AdditionalFun;

//! functions as defined on FUNCTIONS block of a mbs file
typedef struct FunctionStruct{
    QString name;
    QString body;                       //!< function body string
    FunctionParser f;                   //!< parser working upon body
    QList<int> arg;                     //!< argument refer. to fitmodel::obj
    QList<AdditionalFun> addfunlist;    //!< list of used preimplemented funct.
}FunctionStruct;

//! enable sorting of data in RUN
typedef struct SortContainer{
    double x,y,err2;
}SortContainer;

//! parser and handler of user input mbs-files
/*! a mbs file if a well structured text file to handle a fitting task. It
 *  necessarily consists of a FITPARAMETER, a THEORY and RUN block. If read in
 *  succesfully the data construct is used to perform a fitting.
 *  Base entitiy of a fitodel is the fitmodel::run, a array of RUN struct to
 *  hold datasets to be fitted. The fitting function fitmodel::mbs is specified
 *  first independently and connect via the fitting parameters, being declared
 *  at once for all runs and later resolved to runspecific, global and constant
 *  parameters. Thus there exists a reference array fitmodel::rData which holds
 *  the current value of a fitting parameter concerning a specific run. It
 *  points to the fitmodel::pData, where these values are stored. Note, that in
 *  fitmodel::pData the elements are grouped together to run-bloccks allowing
 *  multithreaded calculation on this array if runspecific fitting task is
 *  executed.
 *  The inititial parsing is done in the fitmodel() constructor. Fitting itself
 *  is performed with the help of fitthread class. It requires the
 *  fitmodel::calc_run method calc the new chi2 to eveluate cahnged fitting
 *  parameters.
 */
class fitmodel:public QWidget{
    Q_OBJECT
public:
    fitmodel(QString filename);
    fitmodel();
    ~fitmodel();
    void plot(plot2D *plotter, plot2D *MEM);
    //! creates fitthread instances to fit the whole data set
    /*! The fitting is outsourced to the fitthread class. This class interacts
     *  with fitmodel by modifying fitmodel::pData, disposing the recalculation
     *  of the concerning runs and interrogating the new chi2 values. This is
     *  the basic organisation of multithreading in Moessfit for seriell fits.
     *  Every single run then has an individual fitthread.
     *  In case of global fits, the multithreading is applied when calculating
     *  the indivdual runs.
     */
    void fit();
    void fit_hidden_wait();             //!< like fit, but without progressbar
    void errors();                      //!< like fit, but error calculation
    double get_total_chi2();            //!< sum of chi2 values of all runs
    double get_run_chi2(int runi);      //!< chi2 value of the runi-th run
    double calc_changed_runs();         //!< based on pIsChanged information
    //! returns, which run is affected by the i-th parameter
    /*! the fitparameters are stored in pdata, starting with global parameters
     *  if available and than run-wise the run specific parameter. affect and
     *  get_affect give the run-index, to which the i-th parameter corresponds.
     *  In case of a global parameter -1 will be returned
     */
    int get_affect(int i);
    int calc_run(int i);                //!< recalculates the i-th run
    int is_valid();                     //!< returns fitmodel::isvalid
    void close_fmpa();                  //!< closes fitmodel::fmpa
    //! generates default mbs-file working upon the flds files
    void generate_default(QStringList flds);
    short from_fld(char* filename,double** xdata, double** ydata,
                   int* count);         //!< load data from file, overloaded

    //! return pointer to fitmodel::fmpa, used by mainwindow to set up widget
    fmPanel* get_pannel();
    //! count of fit parameters of model, size of fitmodel::fmp array
    int cpara;
    //! fitparameters used in model, count saved in fitmodel::cpara
    fmPara* fmp;
    //! count of runs in the model, size of fitmodel::run array
    int crun;
    //! runs used in model, count saved in fitmodel::crun
    RUN *run;
    //! transforms sinusoidal data in linear data
    void save_linear_v_spectra();
private slots:
    //! triggered by fitthread
    void update_fit_progressbar(int ID, int citer, double chi2);
    void finish_fitting(int ID);        //!< triggered by fitthread
    void errors_updated();              //!< triggered by fitthread
    //! run specific error message triggered by fitthread
    void error_message(int runi, QString message);
public slots:
    //!< parameter ipara of run irun was modified, recalc model
    void panel_input(int irun, int ipara);
    void view_run(int i);               //!< add run i to the plot
    void view_run_only(int i);          //!< show only run i on the plot
    void hide_run(int i);               //!< hide run i from the plot
    void rewrite_FITPARAMETER();        //!< rewrite fitparameter block
    void calc_baseline_area(int runi);//!< calculate baseline and area
    //!< set initial value from parameter declaration if fit, else set -1
    void reset_error(int runi, int parai, bool fit);
    void add_noise();                   //!< statistical noise to current model
private:
    //! generate the data structure from parsed file, resolve Obj declarations
    /*! this function might be considered as the kernel of fitmodel, as it
     *  processes the read in information to data structure in the memory to
     *  work with. This functions sets up the arrays fitmodel::pData and
     *  related data as fitmodel::pError and resolves all symbolic arguments
     *  (functions, maps, numbers and parameters) using fitmodel::find_obj()
     *  function. The reference of the two dimensional fitmodel::rData and
     *  fitmodel::rError is generated.
     */
    int link();
    void interprete_commands(int* linkOK);//!< part of fitmodel::link()
    void parse_functions(int* linkOK);  //!< part of fitmodel::link()
    //!< returns index of name in fitmodell::obj, and adds number-strings
    int find_obj(QString name);
    int set_obj(int i, int irun, double v);//!< change Obj's value
    int set_error(int i, int irun, double v);//!< change Obj's related error
    fmPanel* fmpa;                      //!< front end GUI created from mbsfile
    QString InputFile;                  //!< path to input mbs-file
    QString ErrorMessage;               //!< eventually shown at end of reading
    QList<Obj> obj;                     //!< symbolic objects declared by user
    CosSmearing* CosSm;
    int read_fitparameters(FILE*f);     //!< read FITPARAMETER block in mbs-file
    int read_functions(FILE*f);         //!< read FUNCTIONS block in mbs-file
    int read_theory(FILE*f);            //!< read THEORY block in mbs-file
    int read_run(char* mbspath, FILE*f);//!< read RUN block in mbs-file
    int read_plot(FILE*f);              //!< read PLOT block in mbs-file
    int read_commands(FILE*f);          //!< read COMMANDS block in mbs-file
    int read_fitdata();                 //!< read FITDATA block in mbs-file
    int read_MEMdata();                 //!< read MEMDATA block in mbs-file
    //! checks for consistency, prepares parameter distribution calculation
    int process_theory();
    //! execute data binning
    void process_Binning();
    bool blockend(char* line);          //!< true if line starts with "##"
    bool validline(char* line);         //!< false at "\\n" or "#"
    void print_fitdata();               //!< output of fit parameter valuess
    //! reads data from file
    short from_fld(char* filename,double** xdata, double** ydata,
                   double**yerror, int* count);
    //! reads data from pure count file, no velocities
    int from_fld_unfolded(char* filename,double** xdata, double** ydata,
                   double**yerror2rec, int* count);
    //! recursive spectrum builder processing complex arguments as MEM or DISTR
    /*! the functions will be executed ip times minimum, which is the parameter
     *  count. If all arguments are processed and converted into numbers stored
     *  in p, then the spectrum will be calculated. A DISTR or MEM command
     *  serve as a branch, generating multiple numbers and thus multiple
     *  spectra, eventually stored in the MEMIm array containing all possible
     *  sample spectra. With the sumweightonly tag the recorsive construction
     *  can be tested with respect to integrated weight. Thus a normalization
     *  for a spectrum constructor can achieved and must need not be given by
     *  the user
     */
    void build_iterator(double* p, int ip, int spectrumi,
                        int irun, int ims,
                        MoessCalc *mc, bool sumweightonly,
                        double weight, double* integratedweight,
                        double **MEMIm, int MEMIndex, int maxMEMIndex);
    //! start the build iterator with sumweightonly = false
    void build_spectrum(double* p, int ip, int spectrumi,
                        int irun, int ims,
                        MoessCalc *mc,
                        double weight, double* integratedweight,
                        double **MEMIm, int MEMIndex, int maxMEMIndex);
    //! start the build iterator with sumweightonly = true
    void   WeightSum(double* p, int ip, int spectrumi, int irun, int ims,
                     double weight, double* integratedweight);
    void init();                        //!< initialize fitmodel's parameters
    //! search for fld file in the mbs-path directory and load_run()
    short load_run(char* mbspath, char* T,int index);
    //! loads a fld file with full path and UseT (as rsp) given to run[index]
    int load_run(char* fldpath, double UseT, int index);
    //! tries to open a give file using different paths
    FILE* open_file(char* filename);
    //! resets the data array
    void reset(double** xdata, double** ydata, double**yerror2rec);

    //! Kolmogorow-Smirnow-Test of residual distribution
    /*! returns confidence value of fitted data when claiming normal
     *  distribution of residuals. This distribution is copared to a gaussian
     *  distriution with sigma = 1 and x0 = 0. Kalpha is calculated from
     *  Kolmogorov–Smirnov statistic, and from Kalpha the confidence alpha.
     */
    double KSconfidence(int irun);
    //! spectrum constructor to calc user defined fitfunction upon x-values
    void anyfit(int irun, int ims);
    //! calculates velocity data according to Unfolded data command
    void calculate_velocity_data(int runi);
    //! easy interface to sort the x-, v- and error-data of a run
    void mergesort(int runi);
    //! part of fitmodel::mergesort
    void merge(SortContainer*,SortContainer*,int,int,int);
    //! mergesort algorithm is superior to quicksort!
    void mergesort(SortContainer *a, SortContainer*b, int low, int high);
    double eval_error(int iobj, int irun);//!< get error value from object
    double eval(int iobj, int irun);    //!< get actual value from object
    //! calculate function value with respect to run number
    double fv(int ir, int ifun);
    //! generate the sample value(s) related to the MEM/HR sample istep
    void get_MEMcode(int istep, char* code);
    //! generate the sample value of parameter ipara related to istep-th sample
    double get_MEMcode(int istep, int ipara);

    int pc(MSTyp m);                    //!< argument count of m
    int pc(int spectrumi);              //!< argument count of spectra[i]
    int ac(ParameterTyp m);             //!< argument count of m
    //! true if argument is MEM or Hesse-Ruebartsch
    int is_arbitary_distribution(int i, int j);
    //! rearranges data to put unused data to the end of the array
    void restrict_fitrange(int irun);


    int isvalid;                        //!< fitmodel was correctly parsed
    int cfun;                           //!< function count
    //! array of functions, as defined in FUNCTIONS block
    FunctionStruct* fun;
    //! moessbauer spectra count, as defined in THEORY block
    int cmbs;
    //! Maximum Entropy Method (MEM) total sample count, see process_theory()
    int cMEMsubspec;
    //! index of first MEM based subspectra in fitmodel::spectra array
    int i1stMEMspec;
    //! Maximum Entropy Method (MEM) parameter count, dimensionality of MEM
    int cMEMpara;
    //! spectra as defined in THEORY block
    /*! this array of Moessbauer spectra contains the information to construct
     *  all Moessbauer spectra. The number of these subspectra and length of
     *  the array is stored in fitmodel::cmbs. The order of subspectra might
     *  differ from the declaration in the THEORY block, as the spectra are
     *  sorted in fitmodel::process_theory() to push Maximum Entropy Method
     *  (MEM) based spectra to the end of the array.
     */
    MoessbauerSpectrum* spectra;
    int AnyFit_in;                      //!< none Moessbauer data as input data
    //! parameter distribution typ, wither Hesse Ruebartsch or Maximum Entropy
    /*! with the HR or MEM directive at argument position in the THEORY block
     *  the user declares a method to apply a free parameter distribution to
     *  the spectrum. Only one of the two can be applied. Internally both
     *  methods are treated the same up to the final application of the model.
     *  Distrtyp then specifies the choice of method.
     */
    ParameterTyp Distrtyp;

    //! array of all parameter values
    /*! the array contains all parameter values as defined in the mbs files,
     *  i.e. global parameters and for every run the run specific parameters
     *  and constant parameters (constant parameters are treated run specific
     *  due to multithreating issues). Thus the array length is
     *  cGlobal + crun*(cSpec+cConst). First global parameters are noted,
     *  followed by run-blocks containing the run specific parameters. In case
     *  of a seriell (i.e. non-global fit) the blocks can be transmitted inde-
     *  pendently. At the end of the array the global parameters are saved.
     *  To not confuse the order and indeces the is a helper array
     *  rData[ipara][irun] which points to pData at the correct position.
     */
    double *pData;
    //! array of all parameter initial step size or errors, organized as pData
    double *pError;                     //!< all parameter errors

    //! register to speed up global fitting routine
    /*! this boolean array corresponds to pData. during a global fit only the
     *  runs with changed parameters have to be recalculated. By means of
     *  pIsChanged the function fitmodel::calc_changed_runs() decides which
     *  runs to recalculate
     */
    int *pIsChanged;
    //! pointer to pData to avoid order confusion
    /*! rError[irun][jpara] points to the value of the j-th parameter of the
     * i-th run
     */
    double ***rData;
    //! pointer to pError to avoid order confusion
    /*! rError[irun][jpara] points to the error of the j-th parameter of the
     * i-th run
     */
    double ***rError;
    int cSpec;                          //!< run specific parameter count
    int cGlobal;                        //!< global parameter count
    int cConst;                         //!< constant parameter count
    //! tells which run is affected by the refering fit parameter
    /*! -2 for constants, -1 for global fitting, else run number. The array is
     *  organized the same way as pData and pError
     */
    int *affect;
    //! reference to affect, similiar to rData or rError
    int ***raffect;
    int fitruns;                        //!< fit procedure still running
    //! specification of the error estimation
    /*! fitthread can handle different methods of error estimation: Covariant
     *  errors based on Hesse matrix, MINOS errors or monte carlo errors. The
     *  mode is specified with the Error command in the COMMANDS block.
     */
    ErrMod errormode;


    //! plot auto range of no range was specified by the user
    int autorange;

    //! Command argument strings, later resolved to obj index
    ///@{
    QString sautoI0, sautoA, sMEMsteps, sMaxIter, sconv, sMEMmaxlambda, sMEMmaxlambdaI0,
            sMEMtollerance, sMEMstepsize,sta,somegasrc,somegaabs,sfr,sHR,sHRc,sMCi,
            sUDf, sUDc, sUDI, sCSr0, sCSr1, sCSr2, sCSz1, sCSz2,sNg12,sNg32,sNQ,sNM,sNEg,
            sBinning;
    ///@}
    //! Command argument obj indeces
    ///@{
    int autoI0, autoA, MEMsteps, MaxIter, conv, MEMmaxlambda, MEMmaxlambdaI0,
        MEMtollerance, MEMstepsize,ta,omegasrc,omegaabs,fr,HR,HRc,MCi,UDf,UDc,
        UDI,CSr0, CSr1, CSr2, CSz1, CSz2,Ng12,Ng32,NQ,NM,NEg,Binning;
    ///@}
    //! true of ForceSimplex command was stated
    bool forcesimplex,useUnfoldedData,CosineSmearing,errors_up_to_data;
    //! list of all runs to be plotted
    QList<double> plotrunlist;
    double plotxmin;
    double plotxmax;
    double plotymin;
    double plotymax;

    //! if true the Progressbar is update and fitdata written at end of fitting
    /*! fitthreads may be used for MINOS error calculation. In that case, the
     *  progressbar belongs to the error calculation and not to the individual
     *  fits for the parameters. The same accounts for printing the values. The
     *  ProgressPring-directive tells, whether the progressbar and FITDATA is
     *  to be updated.
     */
    bool ProgressPrint;
    //! mutex to prevent write/read conflicts during multithreading
    QMutex fmutex;
    //! (global) error bar to show the state of error calculation or fittting
    QProgressDialog pbinstance;
    //! progress values, container for progressbar
    /*! array of succesful iterations for every run. pgvs is resetted when
     *  fitthrear is going to be used and updated on fitthread signal.
     */
    int* pgvs;
    //! progress chi2, container for progressbar
    /*! current chi2 for every run. pgvs is resetted when
     *  fitthrear is going to be used and updated on fitthread signal.
     */
    double* pgchi2;
    //! maximum iterations per run, set by MaxIter command
    int runiterations;

    int is_seriell_fit();               //!< true if model is not global
    //! array of fitthreads, used during error calculation and fitting
    fitthread** ft;
    //! save fgets, i.e. fgets() +  overrite "Carriage Return" (CR) if present
    /*! simplyfies cross-compilation by modifying Windows line end format to
     *  Linux format. Thus in the source code, a consistent C-strings of
     *  equal lenth on every platform can be assumed. This is useful for every
     *  parsing task.
     */
    void fgetss(char* buffer,int max,FILE* f);


signals:
    //! indicates that data is processed and can be viewed
    /*! fitmodel does not hold its own plot2D instances, but it is capable to
     *  used a foreign plot2D to plot its own data. At the moment, mainwindow
     *  is master of the plot windows, and triggers fitmodel::plot() on
     *  ready_to_plot signal, using the main window plot2D childs.
     */
    void ready_to_plot();
    //! uses in fit_hidden_wait() to stop QEventLoop and proceed in code
    void all_fitting_finished();
};

#endif // FITMODEL_H
