﻿#include "fitthreat.h"
double helpf_global(const gsl_vector * x, void * params);
double helpf_seriell(const gsl_vector * x, void * params);
double helpf_gauss(const gsl_vector * x, void * params);
fitthreat::fitthreat(double* para, int* changed, double* errors, int cpara,
                     double (*f)(void*), void* instance, int MaxIter,
                     double conv, bool forceSimplex){
    // global fitting
    if(cpara > 0 && para != NULL && changed != NULL && errors != NULL
            && f != NULL && instance != NULL && MaxIter > 0){
        proceed_calc = 1;
        fitmodel = fitglobal;
        this->runi = 0;
        this->para = para;
        this->changed = changed;
        this->errors = errors;
        this->cpara = cpara;
        this->f_global = f;
        this->instance = instance;
        this->MaxIter  = MaxIter;
        this->conv = conv;
        usesimplex = forceSimplex;
    }
    else
        fitmodel = notvalid;
}
fitthreat::fitthreat(double* para, int runi, double* errors, int cpara,
                     double (*f)(void*,int), void* instance, int MaxIter,
                     double conv){
    // seriel fitting
    if(cpara > 0 && para != NULL && errors != NULL
       && f != NULL && instance != NULL && MaxIter > 0){
        proceed_calc = 1;
        fitmodel = fitseriell;
        this->para = para;
        this->runi = runi;
        this->errors = errors;
        this->cpara = cpara;
        this->f_seriell = f;
        this->instance = instance;
        this->MaxIter  = MaxIter;
        this->conv = conv;
    }
    else
        fitmodel = notvalid;
}
fitthreat::fitthreat(double* para, int* changed, double* errors, int cpara,
                     double (*chi2)(void*), void* instance, ErrMod mode,
                     double (*fitting)(void*),int MonteCarloIterations,
                     int crun, int cglobal, bool (*correlated)(void*,int,int)){
    // error calculation global
    if(cpara > 0 && para != NULL && changed != NULL && errors != NULL
       && chi2 != NULL && instance != NULL){
        proceed_calc = 1;
        fitmodel = errorglobal;
        this->runi = 0;
        this->para = para;
        this->changed = changed;
        this->errors = errors;
        this->cpara = cpara;
        this->f_global = chi2;
        this->fit = fitting;
        this->correlated = correlated;
        this->instance = instance;
        this->errormode = mode;
        this->crun = crun;
        this->cglobal = cglobal;
        montecarloiterations = MonteCarloIterations;
    }
    else
        fitmodel = notvalid;
}
fitthreat::fitthreat(double* para, int runi, double* errors, int cpara,
                     double (*chi2)(void*,int), void* instance, ErrMod mode,
                     int MonteCarloIterations){
    // error calculation seriell
    if(cpara > 0 && para != NULL && errors != NULL
       && chi2 != NULL && instance != NULL){
        proceed_calc = 1;
        fitmodel = errorseriell;
        this->runi = runi;
        this->para = para;
        this->changed = NULL;
        this->errors = errors;
        this->cpara = cpara;
        this->f_seriell = chi2;
        this->instance = instance;
        this->errormode = mode;
        this->crun = 1;
        this->cglobal = 0;
        montecarloiterations = MonteCarloIterations;
    }
    else
        fitmodel = notvalid;
}
fitthreat::fitthreat(){}
fitthreat::~fitthreat(){}
void fitthreat::run(){
    double MtCarloSigmac = 1e-3;
    switch(fitmodel){
        case fitseriell:        fit_seriell();
                                break;
        case fitglobal:         if(usesimplex)
                                    fit_global_simplex();
                                else
                                    fit_global();
                                break;
        case errorglobal:       MtCarloSigmac = pow(0,3);
        case errorseriell:      switch(errormode){
                                    case uncorrelated: errors_uncorrelated(true);
                                                       break;
                                    case covariant:    errors_covariant();
                                                       break;
                                    case minos:        errors_MINOS();
                                                       break;
                                    case montecarlo:   errors_MonteCarlo(0,3);
                                                       break;
                                    default:           errors_covariant();
                                                       break;
                                }break;
        case notvalid:          runi = 0;break;
    }
    emit finished(runi);
}
void fitthreat::exit(){};
void fitthreat::fit_global_simplex(){
    fitcontent fc;
    fc.chi2 = f_global;
    fc.instance = instance;
    fc.cpara = cpara;
    fc.IsChanged = changed;
    fc.param = para;
    /*Parametervektor und Stepsizevektor initialisieren und füllen*/
    gsl_vector* xini = gsl_vector_alloc(cpara);
    gsl_vector* ss = gsl_vector_alloc(cpara);
    for(int i = 0; i < cpara; i++){
        gsl_vector_set(xini ,i,para[i]);
        gsl_vector_set(ss   ,i,qMax(0.,errors[i]));
    }
    /*Solver initialisieren*/
    const gsl_multimin_fminimizer_type *T = gsl_multimin_fminimizer_nmsimplex2;
    gsl_multimin_fminimizer *s = NULL;
    gsl_multimin_function fn;
    fn.n = cpara;
    fn.f = helpf_global;
    fn.params = (void*)&fc;
    s = gsl_multimin_fminimizer_alloc (T, cpara);
    gsl_multimin_fminimizer_set(s, &fn, xini, ss);
    /*Iteration ausfuehren*/
    int maxiterations = MaxIter, iterations = 0; int state;
    double size;
    do{ gsl_multimin_fminimizer_iterate(s);
        size = gsl_multimin_fminimizer_size (s);
        state = gsl_multimin_test_size (size, conv);
        iterations++;
        emit refresh_progress(runi,iterations,s->fval);
    }while(state == GSL_CONTINUE
           && maxiterations > iterations
           && proceed_calc);

    /*Parameter zurückgeben*/
    for(int i = 0; i < cpara; i++)
        para[i] = gsl_vector_get(s->x,i);

    // refresh fitmodel
    helpf_global(s->x,(void*)&fc);
    gsl_vector_free(xini);
    gsl_vector_free(ss);
    gsl_multimin_fminimizer_free(s);

}
void fitthreat::fit_global(){
    int maxtry = 20, ct;
    double* dchi2 = new double[cpara];
    double* step = new double[cpara];
    for(int iI= 0; iI < cpara; iI++)
        step[iI] = qMax(0.,errors[iI]);
    init_array(changed,cpara,1);
    double cchi2 = f_global((void*)instance);
    init_array(changed,cpara,0);
    double pold, nchi2,chi2t;
    double convcheck[3];
    convcheck[1] = 1e100;
    convcheck[2] = 1e99;
    FILE* f=fopen("/home/musr/Sirko/MossbauerSirko/CuFeAs-FerrocenReference/p(t).txt","a");
    for(int i = 0; i < MaxIter; i++){
        for(int j = 0; j < cpara; j++){
            pold = para[j];
            changed[j] = true;
            ct = 0;
            do{ para[j] = pold + step[j];
                nchi2 = f_global((void*)instance);
                if(j>0) changed[j-1] = false;
                if(nchi2 > cchi2){
                    step[j] = -step[j];
                    para[j] = pold + step[j];
                    nchi2 = f_global((void*)instance);
                    if(nchi2 > cchi2)
                        step[j] = -0.7*step[j];
                }
                else
                    if(cchi2 - nchi2 < 1)
                        step[j] = 1.2*step[j];

                ct++;
            }while(nchi2 > cchi2 && ct < maxtry);
            if(ct == maxtry){
                step[j] = step[j]/(pow(0.7,1.*maxtry));
                dchi2[j] = 0;
            }
            else
                dchi2[j] = nchi2 - cchi2;
            para[j] = pold;
        }
        nchi2 = min_array(dchi2,cpara);
        int stepsdone;
        if(nchi2 != 0 && proceed_calc){
            stepsdone = 0;
            do{
                chi2t = cchi2;
                for(int j = 0; j < cpara; j++)
                    para[j] += step[j] * dchi2[j]/nchi2;
                init_array(changed,cpara,1);
                cchi2 = f_global((void*)instance);
                stepsdone++;
            }while(cchi2 < chi2t);
            if( stepsdone > 1){
                for(int j = 0; j < cpara; j++)
                    para[j] -= step[j] * dchi2[j]/nchi2;
                cchi2 = f_global((void*)instance);
            }
            init_array(changed,cpara,0);
        }
        else
            i = MaxIter;

        emit refresh_progress(runi,i,cchi2);

        // check for convergence
        convcheck[0] = convcheck[1];
        convcheck[1] = convcheck[2];
        convcheck[2] = cchi2;
        if(convcheck[0] - convcheck[2] < conv)
            i = MaxIter;
            for(int j = 0; j < cpara; j++)
                fprintf(f,"%f\t",step[j]);
            fprintf(f,"\n");

    }
    fclose(f);
    f_global((void*)instance);
    delete(dchi2);
}
void fitthreat::fit_seriell(){
    if(proceed_calc){
        fitcontent fc;
        fc.runchi2 = f_seriell;
        fc.instance = instance;
        fc.cpara = cpara;
        fc.IsChanged = NULL;
        fc.param = para;
        fc.runi = runi;
        /*Parametervektor und Stepsizevektor initialisieren und füllen*/
        gsl_vector* xini = gsl_vector_alloc(cpara);
        gsl_vector* ss = gsl_vector_alloc(cpara);
        for(int i = 0; i < cpara; i++){
            gsl_vector_set(xini ,i,para[i]);
            gsl_vector_set(ss   ,i,qMax(0.,errors[i]));
        }
        /*Solver initialisieren*/
        const gsl_multimin_fminimizer_type *T = gsl_multimin_fminimizer_nmsimplex2;
        gsl_multimin_fminimizer *s = NULL;
        gsl_multimin_function fn;
        fn.n = cpara;
        fn.f = helpf_seriell;
        fn.params = (void*)&fc;
        s = gsl_multimin_fminimizer_alloc (T, cpara);
        gsl_multimin_fminimizer_set(s, &fn, xini, ss);

        /*Iteration ausfuehren*/
        int maxiterations = MaxIter, iterations = 0; int state;
        double size;
        do{ gsl_multimin_fminimizer_iterate(s);
            size = gsl_multimin_fminimizer_size (s);
            state = gsl_multimin_test_size (size, conv);
            iterations++;
            emit refresh_progress(runi,iterations,s->fval);
        }while(state == GSL_CONTINUE
               && maxiterations > iterations
               && proceed_calc);

        /*Parameter zurückgeben*/
        for(int i = 0; i < cpara; i++)
            para[i] = gsl_vector_get(s->x,i);

        // refresh fitmodel
        helpf_seriell(s->x,(void*)&fc);
        gsl_vector_free(xini);
        gsl_vector_free(ss);
        gsl_multimin_fminimizer_free(s);
    }
}
void fitthreat::errors_covariant(){
    // helper data
    double  *k = new double[cpara];;

    for(int i = 0; i < cpara; i++)
        k[i] = para[i];
    errors_uncorrelated(false);

    int progress = 0;
    double chi0 = f(),fxybuf;
    gsl_matrix *H = gsl_matrix_alloc(cpara,cpara);
    gsl_matrix_set_all(H,0);
    // calc diagonal elements
    for(int xi = 0; xi < cpara; xi ++)
        if(errors[xi] > 0)
            gsl_matrix_set(H,xi,xi,1./(errors[xi]*errors[xi]));
        else
            gsl_matrix_set(H,xi,xi,0);

    /*Hessematrix bestuecken*/
    for(int xi = 0; xi < cpara; xi ++){
        for(int yi = xi+1; yi < cpara; yi++){
            if(correlation_possible(xi,yi)){
                if(changed != NULL) changed[xi] = changed[yi] = 1;
                fxybuf = fxy(&(para[xi]),&(para[yi]),k[xi],k[yi],
                             gsl_matrix_get(H,xi,xi),
                             gsl_matrix_get(H,yi,yi),
                             errors[xi],errors[yi],chi0);
                if(changed != NULL) changed[xi] = changed[yi] = 0;
                }
            else
                fxybuf = 0;

            gsl_matrix_set(H,xi,yi,fxybuf);
            gsl_matrix_set(H,yi,xi,fxybuf);
            progress++;
            if(progress % cpara == 0)
                emit refresh_progress(runi,progress,0.0);
            if(!proceed_calc) xi = yi = cpara;
        }
    }
    /*reset fitmodel to original values*/
    f();

    /*Hessematrix anzeige*/
    if(runi < 1){
        //FILE* fH = fopen("C:\\Users\\User\\Desktop\\Hesse2.txt","w");
        FILE* fH = fopen("C:\\Users\\test\\Desktop\\Hesse.txt","w");
        if(fH != NULL){
            gsl_matrix *Hh = gsl_matrix_alloc(cpara,cpara);
            gsl_matrix_memcpy(Hh,H);
            gsl_eigen_symm_workspace *ws1 = gsl_eigen_symm_alloc (cpara);
            gsl_vector* eval1 = gsl_vector_alloc(cpara);
            gsl_eigen_symm(Hh,eval1,ws1);
            for(int i = 0; i < cpara; i++){
               for(int j = 0; j < cpara; j++)
                   fprintf(fH,"%f\t",gsl_matrix_get(H,i,j));
               fprintf(fH,"\n");
            }
            fprintf(fH,"\n");
            for(int j = 0; j < cpara; j++)
                fprintf(fH,"%f\n",gsl_vector_get(eval1,j));
            fprintf(fH,"\n");
            for(int j = 0; j < cpara; j++)
                fprintf(fH,"%f\n",errors[j]);
            fclose(fH);
        }
    }
    /*Die Hessematrix invertieren um die Kovarianzmatrix zu erhalten*/
    Hesse_to_covariant_errors(H,errors,k,cpara);

    // Hilfsstrukturen loeschen
    gsl_matrix_free(H);
    delete k;
    emit refresh_progress(runi,cpara*(cpara+1)/2,0.0);
}
void fitthreat::errors_uncorrelated(bool EmitSignals){
    double  *k = new double[cpara];;
    for(int i = 0; i < cpara; i++)
        k[i] = para[i];

    double chi0 = f();
    // calc derivative which automatically adapts stepsize to dchi2 = 1
    for(int xi = 0; xi < cpara; xi ++){
        if(changed != NULL) changed[xi] = 1;
        fxx(&(para[xi]),k[xi],&(errors[xi]),chi0);
        if(EmitSignals) emit refresh_progress(runi,xi,0.0);
        if(changed != NULL) changed[xi] = 0;
        if(!proceed_calc) xi = cpara;
    }
    delete k;
}
void fitthreat::errors_MINOS(){
    // save initial values
    double* k, *errors0, chi0 = f(), chi[2];
    clone(&k,para,cpara);
    // calc none correlation error estimate
    errors_uncorrelated(false);
    clone(&errors0,errors,cpara);

    double ss[2];
    int maxtry = 3;
    // now change the Parameter and look for correlations
    for(int i = 0; i < cpara; i++){
        if(errors[i] > 0){
            ss[0] = -errors0[i];
            ss[1] =  errors0[i];
            for(int ii = 0; ii < 2; ii++){
                int cc = 0;
                do{
                    cc++;
                    ss[ii] *= 2;
                    para[i] = k[i] + ss[ii];
                    errors[i] = 0;
                    chi[ii] = fit(instance);
                }while(cc < maxtry && chi[ii] - chi0 < 1);
                copy(para,k,cpara);
            }
            errors[i] = errors0[i];
            if(chi[1] > chi0 && chi[0] > chi0)
                errors0[i] = (-ss[0] / sqrt(chi[0]-chi0)
                             +ss[1] / sqrt(chi[1]-chi0)) / 2.;
            else
                errors0[i] = -1;
        }
        emit refresh_progress(0,i,errors0[i]);
        if(!proceed_calc) i = cpara;
    }
    // reset
    f_global(instance);
    // transmit error calculation to datastrage
    //double buf1[9];
    //copy(buf1,errors0,cpara);
    //double buf2[9];
    //copy(buf2,errors,cpara);
    copy(errors,errors0,cpara);
    // free
    delete k;
    delete errors0;
}
void fitthreat::errors_MonteCarlo(double pdfmin, double sigmac){
    // helper data
    //int cspec = (cpara - cglobal)/crun;
    double Rmin = sqrt(gsl_cdf_chisq_Pinv(0.003,cpara));
    double Rmax = 1.2*sqrt(gsl_cdf_chisq_Pinv(0.997,cpara));
    sigmac = 5;
    double  *k = new double[cpara];;
    double *ss = new double[cpara];;

    for(int j = 0; j < cpara; j++)
        k[j] = para[j];
    bool full_pdminArea=true;
    // calc uncorrelatet sigma to ss
    errors_uncorrelated(false);
    //for(int j = 0; j < cglobal; j++)
    //    errors[j]*=sqrt(crun);
    copy(ss,errors,cpara);
    if(changed != NULL) init_array(changed,cpara,1);
    double chi0 = f(),pdf;
    // prepare the random number generation
    int m = montecarloiterations;
    double **x = alloc_array(cpara,80);
    init_array(x,cpara,80,0);
    double **sc = alloc_array(cpara,80);
    double ***xg = new double**[crun];
    for(int i = 0; i < crun; i++){
        xg[i] = alloc_array(cpara,80);
        init_array(xg[i],cpara,80,0);
    }
    init_array(sc,cpara,80,0);
    double *dx = new double[cpara];
    double *bins = new double[80];
    for(int j = 0; j < 80; j++)
        bins[j] = (1./80.*j-0.5)*2*sigmac;
    // setup random number generator
    const gsl_rng_type * T;
    gsl_rng * r;
    gsl_rng_env_setup();
    T = gsl_rng_default;
    r = gsl_rng_alloc (T);
    // generate parameter combinations
    //FILE* fH = fopen("C:\\Users\\test\\Desktop\\Hesse3.txt","w");
    FILE* fH = fopen("/home/musr/Sirko/Mossbauer/FeSe0.88S0.12-LiOH-intercalated/hesse.txt","w");
    //FILE* fH = fopen("C:\\Users\\User\\Desktop\\Hesse2.txt","w");
    int i = 0;
    while(i < m && proceed_calc){
        double R = Rmin + (Rmax-Rmin)*gsl_rng_uniform(r);
        double VolumeElement = pow(R/Rmax,cpara-1);
        // calc parameter vetor
        for(int j = 0; j < cpara; j++)
            dx[j] = gsl_cdf_gaussian_Pinv(gsl_rng_uniform(r),1);
        // normalize specific parameters
        for(int r = 0; r < crun; r++)
            scale(dx,cpara,R/norm(dx,cpara));
        // set parameters
        for(int j = 0; j < cpara; j++)
            para[j] = k[j] + ss[j] * dx[j];
        pdf = exp(-0.5*(f()-chi0));
        pdf *= VolumeElement;
        for(int j = 0; j < cpara; j++){
            int jsigma = (dx[j] /(2*sigmac)+0.5)*80;
            if(jsigma >= 0 && jsigma < 80){
                x[j][jsigma] += pdf;
                sc[j][jsigma] ++;
            }
        }
        // iterate and status
        i++;
        if(i % 100 == 0)
            emit refresh_progress(runi,i,0);
    }
    /*while(i < m && proceed_calc){
        double R = Rmin + (Rmax-Rmin)*gsl_rng_uniform(r);
        double VolumeElement = pow(R/Rmax,cglobal+cspec-1);
        // reset
        copy(para,k,cpara);
        if(changed != NULL) init_array(changed,cpara,1);
        chi0 = f();
        if(changed != NULL) init_array(changed,cpara,0);
        // calc parameter vetor
        for(int j = 0; j < cglobal; j++)
            dx[j] = (gsl_rng_uniform(r)-0.5)*2*Rmax;
        for(int j = cglobal; j < cpara; j++)
            dx[j] = gsl_cdf_gaussian_Pinv(gsl_rng_uniform(r),1);
        // normalize specific parameters
        for(int r = 0; r < crun; r++)
            scale(&(dx[cglobal+r*cspec]),cspec,
                  R/norm(&(dx[cglobal+r*cspec]),cspec));
        // set parameters
        for(int j = 0; j < cpara; j++)
            para[j] = k[j] + ss[j] * dx[j];
        // iterate over runs
        double chiold = chi0;
        for(int r = 0; r < crun; r++){
            if(changed != NULL) changed[cglobal+r*cspec] = 1;
            // calc propability density
            double chit = f() - chiold;
            chiold += chit;
            pdf = exp(-0.5*chit);
            pdf *= VolumeElement;
            // integrate the pdf in sigma-bins (jsigma) for run specifics
            for(int j = 0; j < cspec; j++){
                ti = cglobal + r*cspec + j;
                int jsigma = qMin(79,qMax(0,int((dx[ti] /(2*sigmac)+0.5)*80)));
                x[ti][jsigma] += pdf;
                sc[ti][jsigma] ++;
            }
            // intergrate pdf in the globals
            for(int j = 0; j < cglobal; j++){
                int jsigma = qMin(79,qMax(0,int((dx[j] /(2*sigmac)+0.5)*80)));
                xg[r][j][jsigma] += pdf;
            }
            if(changed != NULL) changed[cglobal+r*cspec] = 0;
        }
        // mark the global ounts
        for(int j = 0; j < cglobal; j++){
            int jsigma = qMin(79,qMax(0,int((dx[j] /(2*sigmac)+0.5)*80)));
            sc[j][jsigma] ++;
        }
        // iterate and status
        i++;
        if(i % 100 == 0)
            emit refresh_progress(runi,i,0);
        if(i > imin){
            imin = qMax(imin+1,imin*1.1);
            emit refresh_progress(runi,i,0);
            fprintf(fH,"\n%i\t",i);
            for(int j = 0; j < cpara; j++){
                init_array(sc,cpara,80,1);
                double sigma = 2;
                double x0 = 0;
                double y0 = x[j][80/2];
                double sctemp[80];
                for(int i = 0; i < 80; i++){
                    if(sc[j][i] < 10)
                        sctemp[i] = 1e50;
                    else
                        sctemp[i] = 1;
                }
                fit_gaussian(bins,x[j],sctemp,80,&sigma,&y0,&x0);
                errors[j] = qAbs(ss[j] * sqrt(sigma*sigma+x0*x0));
                fprintf(fH,"%f\t",errors[j]);
            }
        }
    }*/

    /* get total global pdfs
    for(int j = 0; j < 0*cglobal; j++)
        for(int i = 0; i < 80; i++){
            x[j][i] = 1;
            for(int r = 0; r < crun; r++)
                x[j][i] *= xg[r][j][i];
        }*/

    // normalize
    for(int i = 0; i < 80; i++){
        for(int j = 0; j < cpara; j++){
            if(sc[j][i] < 10)
                sc[j][i] = 1e50;
            else
                sc[j][i] = 1;
        }
    }

    double **xfit = x;
    double **scfit = sc;
    int steps = 80;
    if(!full_pdminArea){
        steps = 60;
        xfit = &(x[10]);
        scfit = &(sc[10]);
    }
    // plot statistics
    if(fH != NULL){
        for(int j = 0; j < cpara; j++)
            fprintf(fH,"%f\t",ss[j]);
        fprintf(fH,"\n");
        for(int j = 0; j < cpara; j++)
            fprintf(fH,"%f\t",errors[j]);
        fprintf(fH,"\n");
        for(int j = 0; j < cpara; j++)
            fprintf(fH,"%f\t",errors[j]/ss[j]);
        fprintf(fH,"\n");
        for(int i = 0; i < 80; i++){
            fprintf(fH,"%f\t",bins[i]);
            for(int j = 0; j < cpara; j++){
                fprintf(fH,"%e\t",x[j][i]);
                fprintf(fH,"%e\t",sc[j][i]);

            }
            fprintf(fH,"\n");
        }
        if(0 && cglobal>0){
            fprintf(fH,"Global1, runspecific histograms\n");
            for(int i = 0; i < 80; i++){
                fprintf(fH,"%f\t",bins[i]);
                for(int j = 0; j < crun; j++){
                    fprintf(fH,"%e\t",xg[j][0][i]);
                }
                fprintf(fH,"\n");
            }
        }
        fclose(fH);
    }

    // fit gaussian to density distributions
    for(int j = 0; j < cpara; j++){
        double sigma = 2;
        double x0 = 0;
        double y0 = x[j][steps/2];
        fit_gaussian(bins,xfit[j],scfit[j],steps,&sigma,&y0,&x0);
        errors[j] = qAbs(ss[j] * sqrt(sigma*sigma+x0*x0));
    }

    // reset model
    copy(para,k,cpara);
    f();
    // free
    gsl_rng_free (r);
    free_array(&x,cpara);
    free_array(&sc,cpara);
    for(int r = 0; r < crun; r++)
        free_array(&(xg[r]),cpara);
    delete bins;
    delete dx;

    if(!full_pdminArea)
        emit error_message(runi,QString("The errors ellipsoid seems to extend the sample space due to high correltations.\nUse 'Error Minos' or 'Error Hesse'-Commands instead.\nAn error estimation is tried."));
}
void fitthreat::fit_gaussian(double *x, double* y, double* sigma, int count,
                             double* gsigma, double* gy0, double* gx0){
    // init data
    gsl_vector* xini = gsl_vector_alloc(3);
    gsl_vector* ss = gsl_vector_alloc(3);
    gsl_vector_set(xini ,0,*gy0);
    gsl_vector_set(xini ,1,*gsigma);
    gsl_vector_set(xini ,2,*gx0);
    gsl_vector_set(ss ,0,*gy0);
    gsl_vector_set(ss ,1,*gsigma);
    gsl_vector_set(ss ,2,0.1);
    fitcontentGauss fc;
    fc.count = count;
    fc.sigma = sigma;
    fc.x = x;
    fc.y = y;

    /*Solver initialisieren*/
    const gsl_multimin_fminimizer_type *T = gsl_multimin_fminimizer_nmsimplex2;
    gsl_multimin_fminimizer *s = NULL;
    gsl_multimin_function fn;
    fn.n = 3;
    fn.f = helpf_gauss;
    fn.params = (void*)&fc;
    s = gsl_multimin_fminimizer_alloc (T, 3);
    gsl_multimin_fminimizer_set(s, &fn, xini, ss);

    /*Iteration ausfuehren*/
    int maxiterations = 1000, iterations = 0; int state;
    double size;
    do{ gsl_multimin_fminimizer_iterate(s);
        size = gsl_multimin_fminimizer_size (s);
        state = gsl_multimin_test_size (size, 1e-3);
        iterations++;
    }while(state == GSL_CONTINUE
           && maxiterations > iterations);

    /*Parameter zurückgeben*/
    *gsigma = gsl_vector_get(s->x,1);
    *gy0 = gsl_vector_get(s->x,0);
    *gx0 = gsl_vector_get(s->x,2);

    // refresh fitmodel
    gsl_vector_free(xini);
    gsl_vector_free(ss);
    gsl_multimin_fminimizer_free(s);
}


int fitthreat::correlation_possible(int xi, int yi){
    switch (fitmodel) {
        case errorseriell:
        case fitseriell:
            return (errors[xi] > 0 && errors[yi] > 0);
            break;
        case errorglobal:
            return correlated(instance,xi,yi);
            break;
        default:
            return 0;
            break;
    }
}
void fitthreat::get_Hesse(gsl_matrix *H, double *ss, double *k){
    int m = 10000;
    double **x = alloc_array(m,cpara);
    double *y = new double[m];
    copy(para,k,cpara);
    double chi0 = f();
    // setup random number generator
    const gsl_rng_type * T;
    gsl_rng * r;
    gsl_rng_env_setup();
    T = gsl_rng_default;
    r = gsl_rng_alloc (T);
    // generate parameter combinations
    int vs=0;
    for(int i = 0; i < m; i++){
        for(int j = 0; j < cpara; j++){
            x[vs][j] = (gsl_rng_uniform(r)-0.5)*2*ss[j];
            para[j] = k[j]+x[vs][j]*ss[j];
        }
        y[vs] = f()-chi0;
        if(y[vs] < 2)
            vs++;
    }
    double *fxy = new double[cpara*(cpara+1)/2];

    paraboloid(x,y,cpara,vs,fxy);
    int c = 0;
    for(int k = 0; k< cpara; k++)
        for(int l = k; l < cpara; l++){
            gsl_matrix_set(H,k,l,fxy[c]/(ss[k]*ss[l]));
            gsl_matrix_set(H,l,k,fxy[c]/(ss[k]*ss[l]));
            c++;
        }

    gsl_rng_free (r);
    delete y;
    delete fxy;
    free_array(&x,m);
}

void fitthreat::paraboloid(double **x, double *y, int N, int points,
                           double* fxy){
    int count = N*(N+1)/2;
    gsl_matrix* M = gsl_matrix_alloc(count,count);
    gsl_vector* Y = gsl_vector_alloc(count);
    // index helper
    int* i = new int [count];
    int* j = new int [count];
    int c = 0;
    for(int k = 0; k< N; k++)
        for(int l = k; l < N; l++){
            i[c] = k;
            j[c] = l;
            c++;
        }
    // fill Y
    for(int l = 0; l < count; l++){
        double Yi = 0;
        for(int k = 0; k < points; k++)
            Yi += x[k][i[l]]*x[k][j[l]]*y[k];
        gsl_vector_set(Y,l,Yi);
    }

    // fill M
    for(int r = 0; r < count; r++)
        for(int c = r; c < count; c++){
            double Mrc = 0;
            for(int k = 0; k < points; k++)
                Mrc += x[k][i[r]]*x[k][j[r]]*x[k][i[c]]*x[k][j[c]];
            if(i[r] == j[r]){
                gsl_matrix_set(M,r,c,0.5*Mrc);
                gsl_matrix_set(M,c,r,0.5*Mrc);
            }
            else{
                gsl_matrix_set(M,r,c,Mrc);
                gsl_matrix_set(M,c,r,Mrc);
            }
        }
    /**/FILE *f = fopen("C:\\Users\\User\\Desktop\\Hesse.txt","w");
    for(int i = 0; i < count; i++){
        for(int j = 0; j < count; j++)
            fprintf(f,"%f\t",gsl_matrix_get(M,i,j));
        fprintf(f,"\n");
    }

    // solve the linear equation
    gsl_vector* X = gsl_vector_alloc(count);
    gsl_linalg_HH_solve(M,Y,X);
    for(int k = 0; k < count; k++)
        fxy[k] = gsl_vector_get(X,k);

    /**/fprintf(f,"\n");
    for(int j = 0; j < count; j++)
        fprintf(f,"%f\n",gsl_vector_get(Y,j));
    fprintf(f,"\n");
    for(int j = 0; j < count; j++)
        fprintf(f,"%f\n",gsl_vector_get(X,j));
    fprintf(f,"Data\n");
    for(int i = 0; i < points; i++){
        for(int j = 0; j < N; j++)
            fprintf(f,"%f\t",x[i][j]);
        fprintf(f,"%f\n",y[i]);
    }
    fclose(f);

    gsl_matrix_free(M);
    gsl_vector_free(Y);
    gsl_vector_free(X);
    delete i;
    delete j;
}

void fitthreat::Hesse_to_covariant_errors(gsl_matrix *H0, double* errors,
                                          double* vals, int cpara){
    // look for complete-zero-rows-columns
    int* trans = new int[cpara];
    int cpara2 = 0;
    for(int i = 0; i < cpara; i++){
        int j = 0, iszero = 1;
        do{ if(gsl_matrix_get(H0,i,j) != 0){
                j = cpara;
                iszero = 0;
            }
            j++;
        }while(iszero && j < cpara);
        if(iszero)
            trans[i] = -1;
        else{
            trans[i] = cpara2;
            cpara2++;
        }
    }
    if(cpara2 == 0)
        return;
    // create new H-Matrix and covariance matrix without zero row-columns
    gsl_matrix* H = gsl_matrix_alloc(cpara2,cpara2);
    double* valt = new double[cpara2];
    for(int i = 0; i < cpara; i++){
       if(trans[i] != -1)
           valt[trans[i]] = vals[i];
       for(int j = 0; j < cpara; j++)
           if(trans[i] != -1 && trans[j] != -1)
               gsl_matrix_set(H,trans[i],trans[j],
                              gsl_matrix_get(H0,i,j));
    }
    gsl_matrix* covariance = gsl_matrix_alloc(cpara2,cpara2);

    // look for determinant
    gsl_matrix_memcpy(covariance,H);
    gsl_permutation*perm = gsl_permutation_alloc(cpara2);
    int s=0;
    gsl_linalg_LU_decomp(covariance, perm, &s);
    if(gsl_linalg_LU_det(covariance,s) == 0)
        emit error_message(runi,"Hessematrix has zero determinant. Uncorrelated errors are shown.");
    else{
        // try to get covariant matrix
        int tryagain = 15   ;
        double ml = 0., m = 1. ,mu = 2.;
        gsl_set_error_handler_off();
        do{
            // try as long as possible reducing offdiagonal terms to achieve
            // positive definite Hesse matrix
            m = (ml+mu)/2.;
            gsl_matrix_memcpy(covariance,H);
            gsl_matrix_scale(covariance,m);
            for(int i = 0; i < cpara2; i++)
                gsl_matrix_set(covariance,i,i,gsl_matrix_get(H,i,i));
            if(gsl_linalg_cholesky_decomp(covariance)== GSL_EDOM)
                mu = m;
            else
                ml = m;
            tryagain--;
            if(tryagain == -1){
                mu = m = 1;
                ml = -1;
            }
        }while(( tryagain > 0 ||
                (tryagain ==  0 && mu == m)
               ||tryagain == -1) && ml < 1.);
        // if positive definite matrixwas achieved
        /*Hessematrix anzeige
        FILE* fH = fopen("C:\\Users\\User\\Desktop\\Hesse.txt","w");
        if(fH != NULL){
            gsl_matrix *Hh = gsl_matrix_alloc(cpara2,cpara2);
            gsl_matrix_memcpy(Hh,H);
            gsl_matrix_scale(Hh,m);
            for(int i = 0; i < cpara2; i++)
                gsl_matrix_set(Hh,i,i,gsl_matrix_get(H,i,i));
            gsl_eigen_symm_workspace *ws1 = gsl_eigen_symm_alloc (cpara2);
            gsl_vector* eval1 = gsl_vector_alloc(cpara2);
            gsl_eigen_symm(Hh,eval1,ws1);
            for(int i = 0; i < cpara2; i++){
                for(int j = 0; j < cpara2; j++)
                    if(i==j)
                        fprintf(fH,"%f\t",gsl_matrix_get(H,i,j));
                    else
                        fprintf(fH,"%f\t",gsl_matrix_get(H,i,j)*m);
               fprintf(fH,"\n");
            }
            fprintf(fH,"Eigenvalues\n");
            for(int j = 0; j < cpara2; j++)
                fprintf(fH,"%f\n",gsl_vector_get(eval1,j));
            fclose(fH);
        }*/
        gsl_linalg_cholesky_invert(covariance);
        for(int i = 0; i < cpara; i++)
            if(trans[i] != -1)
                if(gsl_matrix_get(covariance,trans[i],trans[i]) > 0)
                    errors[i] = sqrt(gsl_matrix_get(covariance,
                                                    trans[i],trans[i]));

        if(m<1)
            emit error_message(runi,QString("Parameter correlations were to strong. Scaled offdiagonal terms of the Hesse matrix by an Factor of %1. The following high correlations are present:\n%2\nUse 'Error Minos' or 'Error MonteCarlo'-command instead.")
                               .arg(m)
                               .arg(correlations(H,cpara2,valt)));
    }
    gsl_matrix_free(covariance);
    gsl_matrix_free(H);
    gsl_permutation_free(perm);
    delete trans;
    delete valt;
}
QString fitthreat::correlations(gsl_matrix *H, int cpara, double* vals){
    QList<CorrelationEle> l;
    for(int i = 0; i < cpara; i++)
        for(int j = i+1; j < cpara; j++)
            l.append(CorrelationEle(i,j,
                                    gsl_matrix_get(H,i,i),
                                    gsl_matrix_get(H,j,j),
                                    gsl_matrix_get(H,i,j),
                                    vals[i],
                                    vals[j]));
    qSort(l.begin(),l.end());

    QString result;
    for(int i = 0; i < qMin(10,l.length());i++)
        result.append(QString("Parameter %1/%2 (%3/%4): %5\n")
                      .arg(l[i].a)
                      .arg(l[i].b)
                      .arg(l[i].va)
                      .arg(l[i].vb)
                      .arg(l[i].correlation));
    return result;
}
double fitthreat::f(){
    switch (fitmodel) {
    case errorseriell:
    case fitseriell:
        return f_seriell(instance,runi);
        break;
    case errorglobal:
    case fitglobal:
        return f_global(instance);
    default:
        return 0;
        break;
    }
}
double fitthreat::fx(double* v, double v0, double* ss, double chi0){
    int i=0;
    int maxi = 5;
    double dchi = 1;
    double chi = chi0 + dchi;

    do{
        if(chi-chi0 != 0)
            *ss *= dchi/qMax(0.5,chi-chi0);
        *v = v0 + *ss;
        chi = f();
        i++;
    }while(i < maxi && qAbs(chi-chi0)>dchi);
    // calculate the dervative
    *v = v0;

    if(*ss != 0 && (chi - chi0) != 0)
        return (chi - chi0)/ *ss;
    else
        return 0;
}
double fitthreat::fxx(double* v, double v0, double* ss, double chi0){
    int i=0;
    int maxi = 5;
    double chi = chi0 + 1,result[2], SS[2];
    // for both directions (forward, backward)
    for(int d = 0; d < 2; d++){
        *ss *= -1;
        // try t0o meet chi-increase of 0.5
        do{
            *ss /= sqrt(chi-chi0);
            *v = v0 + *ss;
            chi = f();
            if(chi > chi0)
                i++;
            else
                i = maxi;
        }while(i < maxi && qAbs(chi-chi0-1)>0.01);
        // calculate the dervative

        if(qAbs(*ss) > 0 && chi-chi0 > 0){
            result[d] = (chi-chi0)/(*ss * *ss);
            *ss /= sqrt(chi-chi0);
            SS[d] = qAbs(*ss);
        }
        else{
            // reset
            *v = v0;
            *ss = -1;
            return 0;
        }
    }
    // reset
    *v = v0;
    f();
    // mean of both directions
    *ss = (SS[0]+SS[1])/2.;
    return (result[0] + result[1])/2.;
}
double fitthreat::fxy(double* vx, double* vy,
                      double v0x, double v0y,
                      double fxx, double fyy,
                      double ssx, double ssy, double chi0){
    if(ssx == 0 || ssy == 0)
        return 0;

    chi0 += ssx*ssx*fxx + ssy*ssy*fyy;
    double chi[5];
    *vx = v0x+ssx; *vy = v0y+ssy; chi[0] =f()-chi0;
    *vx = v0x-ssx; *vy = v0y-ssy; chi[1] =f()-chi0;
    *vx = v0x-ssx; *vy = v0y+ssy; chi[2] =f()-chi0;
    *vx = v0x+ssx; *vy = v0y-ssy; chi[3] =f()-chi0;
    *vx = v0x;
    *vy = v0y;
    f();
    return (chi[0]+chi[1]-chi[2]-chi[3])/(8*ssx*ssy);
    double m[4][2] = {{1,1},{-1,1},{1,-1},{-1,-1}};
    // try all for quadrants an focus the one with least chi
    for(int i = 0; i < 4; i++){
        *vx = v0x + m[i][0] * ssx;
        *vy = v0y + m[i][1] * ssy;
        chi[i] = f();
    }
    int I = 0;
    for(int i = 1; i < 4; i++)
        if(chi[i] < chi[I])
            I = i;
    // reset
    *vx = v0x;
    *vy = v0y;
    f();
    return (chi[I] - chi0 - 0.5*fxx*m[I][0]*ssx*m[I][0]*ssx - 0.5*fyy*m[I][1]*ssy*m[I][1]*ssy)/(m[I][0]*ssx * m[I][1]*ssy);
    // short line scan, additional but unnecessary refinement
    double wx[5] = {0.0000000000000,0.3826834323651,0.7071067811865,0.9238795325113,1.0000000000000};
    double wy[5] = {1.0000000000000,0.9238795325113,0.7071067811865,0.3826834323651,0.0000000000000};
    for(int i = 0; i < 5; i++){
        *vx = v0x + m[I][0] * ssx * wx[i];
        *vy = v0y + m[I][1] * ssy * wy[i];
        chi[i] = f();
    }
    int J = 0;
    for(int i = 1; i < 5; i++)
        if(chi[i] < chi[J])
            J = i;

    double result;
    if(J == 0 || J == 5)
        // less correlated
        result = (2*(chi[2]-chi0)
                -fxx*ssx*wx[2]*ssx*wx[2]
                -fyy*ssy*wy[2]*ssy*wy[2])
               /(2 * ssx * wx[2] * ssy * wy[2]);
    else{
        // correlated, bracket algorithm to search minimum
        double J1 = M_PI*(J-1)/8, J2 = M_PI*J/8, J3 = M_PI*(J+1)/8;
        double c1 = chi[J-1], c2 = chi[J], c3 = chi[J+1];
        for(int i = 0; i < 5; i++){
            if(c1 < c3){
                c3 = c2;
                J3 = J2;
            }
            else{
                c1 = c2;
                J1 = J2;
            }
            J2 = (J1+J3)/2;
            *vx = v0x + m[I][0] * ssx * sin(J2);
            *vy = v0y + m[I][1] * ssy * cos(J2);
            c2 = f();
        }
        double x = ssx * sin(J2);
        double y = ssy * cos(J2);
        result = (2*qMax(0.,c2-chi0)-fxx*x*x-fyy*y*y)/(2 * x * y);
    }
    // reset
    *vx = v0x;
    *vy = v0y;
    f();
    return result;
}

double helpf_global(const gsl_vector * x, void * params){
    fitcontent* fc = (fitcontent*) params;
    for(int i = 0; i < fc->cpara; i++)
        if(x->data[i] != fc->param[i]){
            fc->IsChanged[i] = 1;
            fc->param[i] = x->data[i];
        }
        else
            fc->IsChanged[i] = 0;
    return fc->chi2(fc->instance);
}
double helpf_gauss(const gsl_vector * x, void * params){
    fitcontentGauss* fc = (fitcontentGauss*) params;
    double result = 0,ym;
    double sigmag = gsl_vector_get(x,1);
    double x0 = gsl_vector_get(x,2);
    double y02 = gsl_vector_get(x,0)*gsl_vector_get(x,0);
    for(int i = 0; i < fc->count; i++){
        ym =y02*exp(-0.5*(fc->x[i]-x0)*(fc->x[i]-x0)/(sigmag*sigmag));
        result += (ym-fc->y[i])*(ym-fc->y[i])/(qMax(1e-50,(fc->sigma[i]*fc->sigma[i])));
    }
    return result;
}

double helpf_seriell(const gsl_vector * x, void * params){
    fitcontent* fc = (fitcontent*) params;
    for(int i = 0; i < fc->cpara; i++)
        if(x->data[i] != fc->param[i])
            fc->param[i] = x->data[i];
    return fc->runchi2(fc->instance,fc->runi);
}

void fitthreat::stop_calculations(){
    proceed_calc = 0;
}

CorrelationEle::CorrelationEle(int ia, int ib, double vala, double valb, double valab, double xa, double xb){
    a = ia;
    b = ib;
    va = xa;
    vb = xb;
    correlation = valab*valab/(vala*valb);
}
CorrelationEle::~CorrelationEle(){};
bool CorrelationEle::operator<(const CorrelationEle o2)const{
    return qAbs(correlation) > qAbs(o2.correlation);
}
