﻿#ifndef FITTHREAT_H
#define FITTHREAT_H
#include <gsl/gsl_vector.h>
#include <gsl/gsl_complex.h>
#include <gsl/gsl_complex_math.h>
#include <gsl/gsl_eigen.h>
#include <gsl/gsl_sort_vector.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>
#include <gsl/gsl_multimin.h>
#include <gsl/gsl_multiroots.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_cdf.h>
#include <QProgressDialog>
#include <QMessageBox>
#include <QThread>
#include <QMutex>
#include <QTime>
#include "array.h"
#include "math.h"

enum ErrMod{uncorrelated,covariant,minos,montecarlo};
typedef struct fitcontent{
    double (*chi2)(void*);
    double (*runchi2)(void*,int);
    void* instance;
    int cpara;
    int* IsChanged;
    double* param;
    int runi;
}fitcontent;
typedef struct fitcontentGauss{
    double* x;
    double* y;
    double* sigma;
    int count;
}fitcontentGauss;
enum threattyp {fitseriell,fitglobal,errorseriell,errorglobal,notvalid};

class CorrelationEle{
    public:
        CorrelationEle(int ia, int ib, double vala,
                       double valb, double valab,
                       double xa, double xb);
        ~CorrelationEle();
        bool operator<(const CorrelationEle o2)const;
        int a,b;
        double va,vb;
        double correlation;
};


class fitthreat: public QThread{
Q_OBJECT
public:
    void run();
    void exit();
    fitthreat(double* para, int* changed, double* errors, int cpara,
              double (*f)(void*), void* instance, int MaxIter,
              double conv, bool forceSimplex);
    fitthreat(double* para, int runi, double* errors, int cpara,
              double (*f)(void*,int), void* instance, int MaxIter,
              double conv);
    fitthreat();
    ~fitthreat();
    fitthreat(double* para, int* changed, double *errors, int cpara,
              double (*chi2)(void*), void* instance, ErrMod mode,
              double (*fitting)(void*), int MonteCarloIterations,
              int crun, int cglobal, bool (*correlated)(void*,int,int));
    fitthreat(double* para, int runi, double *errors, int cpara,
              double (*chi2)(void*,int), void* instance, ErrMod mode,
              int MonteCarloIterations);
    void errors_covariant();
    void errors_MonteCarlo(double pdfmin, double sigmac);
    void errors_uncorrelated(bool EmitSignals);
    void errors_MINOS();
public slots:
    void stop_calculations();
    void paraboloid(double **x, double *y, int N, int points, double* fxy);
private:
    void fit_gaussian(double *x, double* y, double* sigma, int count,
                      double* gsigma, double* gy0, double* gx0);
    void fit_seriell();
    void fit_global();
    void fit_global_simplex();
    int correlation_possible(int xi, int yi);
    void get_Hesse(gsl_matrix *H, double* ss, double *k);
    //! fits a dim-dimensional paraboloid centered at zero at points x
    /*! x is points*dim array holding the arguments, y are the function values
     *  at x (a points-dimensional vetctor and fxy returns the elements of the
     *  Hesse Matrix H in the following order:
     *  fx = (f11, f12, f13 ..., f22, f23,...,fNN))
     *  The data can be used in a quadradic model: f(x) = 0.5*xHx
     */

    void Hesse_to_covariant_errors(gsl_matrix *H, double* errors,
                                   double* vals, int cpara);
    QString correlations(gsl_matrix *H, int cpara, double* vals);
    double fxx(double* v, double v0, double* ss, double chi0);
    double fx(double* v, double v0, double* ss, double chi0);
    double fxy(double* vx, double* vy,
               double v0x, double v0y,
               double fxx, double fyy,
               double ssx, double ssy, double chi0);
    double* para;
    int runi;
    double* errors;
    int* changed;
    int cpara, crun, cglobal;
    double (*f_global)(void*);
    double (*fit)(void*);
    double (*f_seriell)(void*,int);
    bool (*correlated)(void*,int, int);
    double f();
    double (*chi2)(void*);
    void* instance;
    int MaxIter;
    threattyp fitmodel;
    int proceed_calc;
    double conv;
    bool usesimplex;
    int montecarloiterations;
   ErrMod errormode;
signals:
    void refresh_progress(int ID, int citer,double chi2);
    void finished(int ID);
    void error_message(int runi, QString message);

};

#endif // FITTHREAT_H
