#include "fmpanel.h"

fmPanel::~fmPanel(){
    delete T;
    for(int i = 0; i < cpara; i++){
        delete mins[i];
        delete maxs[i];
    }
    delete mins;
    delete maxs;
    delete signalMapper;
    delete signalMapperSlider;
    delete SignalmapperCheck;

}
fmPanel::fmPanel(double ***rdata, double ***rerror,
                 int cpara, int crun,
                 double* T, QStringList* pnames){
    this->rdata = rdata;
    this->rerror = rerror;
    this->cpara = cpara;
    this->crun = crun;
    this->T = T;
    setWindowTitle("play panel");
    mins = new double*[cpara];
    maxs = new double*[cpara];
    QGridLayout* L = new QGridLayout;
    // prepare 1st row of the panel
    for(int i = 0; i < crun; i++)
        cb.insertItem(i,QString("%1").arg(T[i],3));
    connect(&cb,SIGNAL(currentIndexChanged(int)),this,SLOT(show_run(int)));
    QLabel *runlabel = new QLabel("select run:");
    L->addWidget(runlabel,0,0);
    L->addWidget(&cb,0,1);
    connect(&showrun,SIGNAL(clicked()),this,SLOT(view_run()));
    connect(&showrunonly,SIGNAL(clicked()),this,SLOT(view_run_only()));
    connect(&hiderun,SIGNAL(clicked()),this,SLOT(hide_run()));
    connect(&showall,SIGNAL(clicked()),this,SLOT(view_all()));
    showrun.setText("Show run");
    showrunonly.setText("Show only this run");
    showall.setText("Show all");
    hiderun.setText("Hide run");
    L->addWidget(&showrunonly,0,4);
    L->addWidget(&showrun,    0,3);
    L->addWidget(&hiderun,    1,3);
    L->addWidget(&showall,    0,2);

    // prepare 2nd row of the panel
    columnlabel     << new QLabel("name")
                    << new QLabel("slider min")
                    << new QLabel("slider max")
                    << new QLabel("value")
                    << new QLabel("fit");
    L->addWidget(columnlabel[0],1,0);
    L->addWidget(columnlabel[1],1,1);
    L->addWidget(columnlabel[2],1,2);
    L->addWidget(columnlabel[3],1,4);
    L->addWidget(columnlabel[4],1,5);

    // prepare other rows of the panel
    signalMapper = new QSignalMapper(this);
    signalMapperSlider = new QSignalMapper(this);
    SignalmapperCheck = new QSignalMapper(this);
    connect(signalMapper, SIGNAL(mapped(int)), this, SLOT(change_value(int)));
    connect(signalMapperSlider, SIGNAL(mapped(int)), this, SLOT(change_slider(int)));
    connect(SignalmapperCheck, SIGNAL(mapped(int)), this, SLOT(change_checkfit(int)));

    for(int i = 0; i < cpara; i++){
        mins[i] = new double[crun];
        maxs[i] = new double[crun];
        for(int j = 0; j < crun; j++)
            if(*rerror[i][j] > 0){
                mins[i][j] = *rdata[i][j] - 10*(*rerror[i][j]);
                maxs[i][j] = *rdata[i][j] + 10*(*rerror[i][j]);
            }
            else{
                mins[i][j] = 0;
                maxs[i][j] = 2 * (*rdata[i][j]);
            }
        name << new QLabel;
        minlabel << new QLineEdit;
        maxlabel << new QLineEdit;
        clabel << new QLineEdit;
        slider << new QSlider;
        checkfit << new QCheckBox;
        slider[i]->setMinimum(0);
        slider[i]->setMaximum(1000);
        slider[i]->setValue(500);
        slider[i]->setOrientation(Qt::Horizontal);
        minlabel[i]->setText(QString("%1").arg(mins[i][0]));
        maxlabel[i]->setText(QString("%1").arg(maxs[i][0]));
        clabel[i]->setText(QString("%1").arg(*rdata[i][0]));
        name[i]->setText(pnames->at(i));
        checkfit[i]->setChecked(*(rerror[i][0]) > 0);
        L->addWidget(name[i]    ,i+2,0);
        L->addWidget(minlabel[i],i+2,1);
        L->addWidget(maxlabel[i],i+2,2);
        L->addWidget(slider[i]  ,i+2,3);
        L->addWidget(clabel[i]  ,i+2,4);
        L->addWidget(checkfit[i],i+2,5);
        connect(minlabel[i], SIGNAL(editingFinished()), signalMapper, SLOT(map()));
        connect(maxlabel[i], SIGNAL(editingFinished()), signalMapper, SLOT(map()));
        connect(  clabel[i], SIGNAL(editingFinished()), signalMapper, SLOT(map()));
        connect(  slider[i], SIGNAL(sliderMoved(int)), signalMapperSlider, SLOT(map()));
        connect(checkfit[i], SIGNAL(clicked()), SignalmapperCheck, SLOT(map()));
        signalMapper->setMapping(minlabel[i], i);
        signalMapper->setMapping(maxlabel[i], i);
        signalMapper->setMapping(  clabel[i], i);
        signalMapperSlider->setMapping(  slider[i], i);
        SignalmapperCheck->setMapping( checkfit[i], i);
    }

    // prepare additional buttons
    connect(&resetall,SIGNAL(clicked()),this,SLOT(reset_all_runs()));
    connect(&resetrun,SIGNAL(clicked()),this,SLOT(reset_run()));
    connect(&takedefault,SIGNAL(clicked()),this,SLOT(take_run_values_as_default()));
    resetall.setText("reset all");
    resetrun.setText("reset run");
    takedefault.setText("take as default");
    L->addWidget(&takedefault,cpara+2,4);
    L->addWidget(&resetall,cpara+2,2);
    L->addWidget(&resetrun,cpara+2,3);



    L->setColumnStretch(0,1);
    L->setColumnStretch(1,1);
    L->setColumnStretch(2,1);
    L->setColumnStretch(3,2);
    L->setColumnStretch(4,1);
    setLayout(L);
}
void fmPanel::change_value(int i){
    int j = cb.currentIndex();
    mins[i][j] = minlabel[i]->text().toDouble();
    maxs[i][j] = maxlabel[i]->text().toDouble();
    if(*rdata[i][j] != clabel[i]->text().toDouble()){
        *rdata[i][j] = clabel[i]->text().toDouble();
         emit recalc(j,i);
    }

}
void fmPanel::change_slider(int i){
    int j = cb.currentIndex();
    *rdata[i][j] = slider[i]->value()/1000.*(maxs[i][j]-mins[i][j])+mins[i][j];
    clabel[i]->setText(QString("%1").arg(*rdata[i][j]));
    emit recalc(j,i);
}
void fmPanel::show_run(int i){
    for(int j = 0; j < cpara; j++){
        int slidervalue = (*rdata[j][i]-mins[j][i])/(maxs[j][i]-mins[j][i])*1000.;
        slider[j]->setValue(slidervalue);
        minlabel[j]->setText(QString("%1").arg(mins[j][i]));
        maxlabel[j]->setText(QString("%1").arg(maxs[j][i]));
        clabel[j]->setText(QString("%1").arg(*(rdata[j][i])));
        checkfit[j]->setChecked(*(rerror[j][i]) != -1);
   }
}
void fmPanel::refresh(){
    for(int i = 0; i < cpara; i++){
        for(int j = 0; j < crun; j++)
            if(*rerror[i][j] > 0){
                mins[i][j] = *rdata[i][j] - 10*(*rerror[i][j]);
                maxs[i][j] = *rdata[i][j] + 10*(*rerror[i][j]);
            }
            else{
                mins[i][j] = 0;
                maxs[i][j] = 2 * (*rdata[i][j]);
            }
        slider[i]->setMinimum(0);
        slider[i]->setMaximum(1000);
        slider[i]->setValue(500);
        slider[i]->setOrientation(Qt::Horizontal);
        minlabel[i]->setText(QString("%1").arg(mins[i][cb.currentIndex()]));
        maxlabel[i]->setText(QString("%1").arg(maxs[i][cb.currentIndex()]));
        clabel[i]->setText(QString("%1").arg(*rdata[i][cb.currentIndex()]));
    }
}
void fmPanel::view_run(){
    emit view_run(cb.currentIndex());
}
void fmPanel::view_all(){
    emit view_run(-1);
}


void fmPanel::view_run_only(){
    emit view_run_only(cb.currentIndex());
}

void fmPanel::hide_run(){
    emit hide_run(cb.currentIndex());
}
void fmPanel::reset_all_runs(){
    int j = cb.currentIndex();
    for(int j = 0; j < crun; j++)
        for(int i = 0; i < cpara; i++){
            *(rdata[i][j]) = *(rdata[i][crun]);
            *(rerror[i][j]) = *(rerror[i][crun]);
        }
    show_run(cb.currentIndex());
    emit recalc_baseline_area(-1);
    emit recalc(-1,0);
}

void fmPanel::reset_run(){
    int j = cb.currentIndex();
    for(int i = 0; i < cpara; i++)
        *(rdata[i][j]) = *(rdata[i][crun]);
    show_run(j);
    emit recalc_baseline_area(j);
    emit recalc(j,0);
}

void fmPanel::take_run_values_as_default(){
    int j = cb.currentIndex();
    for(int i = 0; i < cpara; i++)
        *(rdata[i][crun]) = *(rdata[i][j]);
    emit rewrite_FITPARAMETER();
}

void fmPanel::wheelEvent(QWheelEvent *event ){
    int newindex = cb.currentIndex()+event->delta()/120;
    newindex = qMin(cb.count()-1,qMax(0,newindex));
    cb.setCurrentIndex(newindex);
}
void fmPanel::mousePressEvent(QMouseEvent  *event){
    setFocus();
}

void fmPanel::change_checkfit(int i){
    if(checkfit[i]->isChecked())
        emit reset_error(cb.currentIndex(),i,true);
    else
        emit reset_error(cb.currentIndex(),i,false);
}
