#ifndef FMPANEL_H
#define FMPANEL_H
#include <QComboBox>
#include <QSlider>
#include <QLineEdit>
#include <QGridLayout>
#include <QLabel>
#include <QSignalMapper>
#include <QPushButton>
#include <QWheelEvent>
#include <QCheckBox>
#include <QMessageBox>

//! Graphical User Interface generated upon fitmodel
/*! this widget is intended for fast user input based on a specific fitmodel.
 *  The Widget provides line edits and sliders to modify the parameter values.
 *  It directly changes the values and emits signals to repaint plots or update
 *  mbs-files.
 */
class fmPanel : public QWidget{
public:
    /*!
     * \param rdata: pointer to data values
     * \param rerror: pointer to data errors
     * \param cpara: parameter count
     * \param crun: run count
     * \param T: run specfic parameters, noramlly temperature
     * \param pnames: parameter names
     */
    fmPanel(double ***rdata, double ***rerror,
                     int cpara, int crun,
                     double* T, QStringList* pnames);
    ~fmPanel();
    //! update line edits and slider wit regard on rdata
    void refresh();
private:
    Q_OBJECT
    double ***rdata;                    //!< parameter value array
    double ***rerror;                   //!< parameter error array
    double *T;                          //!< run specific parameters
    double ** mins;                     //!< slider minima
    double **maxs;                      //!< slider maxima
    int cpara;                          //!< parameter count
    int crun;                           //!< run count

    QList<QLineEdit*> minlabel;
    QList<QLineEdit*> maxlabel;
    QList<QLineEdit*> clabel;
    QList<QSlider*> slider;
    QList<QCheckBox*> checkfit;
    //! allows that multiple signals can be distinguished by the sender
    /*! the fmPanel contains multiple sliders/line edits/buttons, which might
     *  trigger the same slot, only with different index. To avoid the
     *  implementation of multiple slots to handle different signal senders,
     *  Qt class QSignalMapper is used to use the job. It will identify the
     *  sender and reemit the signal, but now with a previously set id.
     */
    QSignalMapper *signalMapper;
    QSignalMapper *signalMapperSlider;  //! see signalMapper
    QSignalMapper *SignalmapperCheck;   //! see signalMapper
    QList<QLabel*> name;                //!< parameter names
    QList<QLabel*> columnlabel;
    QComboBox cb;
    QPushButton showall;
    QPushButton showrun;
    QPushButton hiderun;
    QPushButton showrunonly;
    QPushButton resetall;
    QPushButton resetrun;
    QPushButton takedefault;
    void wheelEvent(QWheelEvent *event);
    void mousePressEvent(QMouseEvent  *event);

private slots:
    void view_run();
    void view_run_only();
    void hide_run();
    void view_all();
    void show_run(int i);
    void change_value(int i);
    void change_slider(int i);
    void change_checkfit(int i);
    void reset_all_runs();
    void reset_run();
    void take_run_values_as_default();
signals:
    void rewrite_FITPARAMETER();
    void recalc(int irun, int ipara);
    void view_run(int i);
    void view_run_only(int i);
    void hide_run(int i);
    void reset_error(int runi, int parai, bool fit);
    void recalc_baseline_area(int runi);
};

#endif // FMPANEL_H
