#include "fold.h"
#include "QString"

Fold::Fold(QString filename, QString inipath){
    setAttribute( Qt::WA_DeleteOnClose );
    setWindowTitle(QString("fold %1").arg(filename));
    this->inipath = inipath;
    this->filename = filename;
    init(inipath);
    load_data();
    autochannel();

    QGridLayout* L = new QGridLayout;
    editCS.setText(QString("%1").arg(CS));
    editalpha.setText(QString("%1").arg(alpha));
    editvoltage.setText(QString("%1").arg(voltage));
    editchnfold.setText(QString("%1").arg(chnfold));
    editautochn.setText(QString("%1").arg(autochn));

    lblalpha.setText("alpha [mm/s / mV]");
    lblCS.setText("zero - center shift [mm/s]");
    lblvoltage.setText("Umonitor [mV]");
    lblautochn.setText("propable fold channel");
    lblchnfold.setText("user fold channel");
    useautochn.setText("use propable fold channel");
    btnfold.setText("fold");
    btncancel.setText("cancel");
    connect(&btnfold,SIGNAL(clicked()),this,SLOT(execute_fold()));
    connect(&btncancel,SIGNAL(clicked()),this,SLOT(close()));

    L->addWidget(&lblalpha      ,0,0);
    L->addWidget(&editalpha     ,0,1);
    L->addWidget(&lblCS         ,1,0);
    L->addWidget(&editCS        ,1,1);
    L->addWidget(&lblvoltage    ,2,0);
    L->addWidget(&editvoltage   ,2,1);
    L->addWidget(&lblchnfold    ,3,0);
    L->addWidget(&editchnfold   ,3,1);
    L->addWidget(&lblautochn    ,4,0);
    L->addWidget(&editautochn   ,4,1);
    L->addWidget(&useautochn    ,5,0);
    L->addWidget(&btnfold       ,6,0);
    L->addWidget(&btncancel     ,6,1);

    setLayout(L);
}
Fold::~Fold(){
    if(data != NULL)
        delete data;
}
void Fold::init( QString inipath){
    data = NULL;
    if(QFile(inipath).exists()){
        FILE* f = fopen(inipath.toLocal8Bit().data(),"rb");
        fread(&alpha,sizeof(double),1,f);
        fread(&CS,sizeof(double),1,f);
        fread(&voltage,sizeof(double),1,f);
        fread(&chnfold,sizeof(double),1,f);
        bool buf;
        fread(&buf,sizeof(bool),1,f);
        useautochn.setChecked(buf);
        fclose(f);
    }
    else{
        CS = 0.1;
        alpha = 0.035;
        voltage = 200;
        chnfold = 512;
        useautochn.setChecked(true);
    }


}
void Fold::execute_fold(){
    // read parameters;
    QString Errormessage;
    bool OK;
    CS = -editCS.text().toDouble(&OK);
    if(!OK){
        Errormessage.append(QString("CS source: %1 is no valid number.\n")
                            .arg(editCS.text()));
    }
    alpha = editalpha.text().toDouble(&OK);
    if(!OK){
        Errormessage.append(QString("alpha: %1 is no valid number.\n")
                            .arg(editalpha.text()));
    }
    voltage = editvoltage.text().toDouble(&OK);
    if(!OK){
        Errormessage.append(QString("Umonitor: %1 is no valid number.\n")
                            .arg(editvoltage.text()));
    }
    chnfold = editchnfold.text().toDouble(&OK);
    if(!OK){
        Errormessage.append(QString("user fold channel: %1 is no valid number.\n")
                            .arg(editchnfold.text()));
    }

    if(Errormessage.isEmpty()){
        QString newfilename = QString(filename).left(filename.length()-3)
                              .append("fld");
        FILE* f = fopen(newfilename.toLocal8Bit(),"w");
        double v;
        int chn0,cc05 = cc/2;
        if(useautochn.isChecked())
            chn0 = autochn;
        else
            chn0 = chnfold;

        if(chnfold == floor(chnfold))
            // fold channel is integer, execute folding
            for(int i = 0; i < cc05; i++){
                v = -alpha*voltage*cos((i+0.5)/cc*2.*3.14159265358979)+CS;
                fprintf(f,"%f\t%i\n",v,data[chn0-cc05+i]+data[chn0+cc05-i-1]);
            }
        else
            // fold channel is float, no folding, just calculation
            for(int i = 0; i < cc; i++){
                v = alpha*voltage*cos((i+0.5+chnfold)/cc*2.*3.14159265358979)+CS;
                fprintf(f,"%f\t%i\n",v,data[i]);
            }

        fclose(f);

        // write ini file
        f = fopen(inipath.toLocal8Bit().data(),"wb");
        fwrite(&alpha,sizeof(double),1,f);
        fwrite(&CS,sizeof(double),1,f);
        fwrite(&voltage,sizeof(double),1,f);
        fwrite(&chnfold,sizeof(double),1,f);
        bool buf = useautochn.isChecked();
        fwrite(&buf,sizeof(bool),1,f);
        fclose(f);

        close();
    }
    else
        QMessageBox(QMessageBox::NoIcon,"Error",Errormessage).exec();
}
void Fold::autochannel(){
    int min = 0;
    int cc05 = cc/2;
    double chi2 = 1e300,cchi2;
    for(int chn0 = cc/2; chn0 < cc; chn0++){
        cchi2 = 0;
        for(int i = 0; i < cc/2; i++)
            cchi2 += (data[chn0-cc05+i]-data[chn0+cc05-i-1])
                    *(data[chn0-cc05+i]-data[chn0+cc05-i-1]);
        if(cchi2 < chi2){
            min = chn0;
            chi2 = cchi2;
        }
    }
    autochn = min;
}
void Fold::load_data(){
    FILE *f = fopen(filename.toLocal8Bit().data(),"r");
    cc = 0;
    QRegExp r("(\\d+)(\\n)");
    char buffer[10000];
    if(f != NULL){
        while(!feof(f)){
            buffer[0] = '\0';
            fgets(buffer,10000,f);
            if(r.indexIn(QString(buffer)) == 0)
                cc++;

        }
        if(cc > 0){
            data = new int[2*cc];
            int cct = 0;
            fseek(f,0,SEEK_SET);
            while(!feof(f)){
                buffer[0] = '\0';
                fgets(buffer,10000,f);
                if(r.indexIn(QString(buffer)) == 0){
                    data[cct+cc] = data[cct] = r.capturedTexts().at(1)
                                               .toDouble();
                    cct++;
                }
            }
            fclose(f);
        }
        else{
            QMessageBox(QMessageBox::NoIcon,"Error","Found no data").exec();
            fclose(f);
            close();
        }
    }
    else{
        QMessageBox(QMessageBox::NoIcon,"Error",
                    QString("%1: file not found.").arg(filename)).exec();
        close();
    }
}
