﻿/* LINUX issues: replace line end by using fgetss instead of fgets
 * Qt4/Qt5: - Qt5 needs "QT += widgets" and "QT += concurrent" in MoessFit.pro
 *
 */
/* Version History:
 *  015: - improved performance of SHc and SHp by factor three by
 *          a) introducing MoessCalc::f_lorenz_fast() (less divsion operation)
 *          b) moving "new"-commands from methods to constructor
 *          c) replacing the long Barb formula in MoessCalc::I(beta,gamma,f,i)
 *          d) removing redundant calc() command in MoessCalc::get_SC_spectrum()
 *       - introduced MoessCalc::get_afm_Powder_spectrum() for afm-powders in
 *         external field in transverse geometry
 *       - first LINUX compilation, substituted fgets() by fitmodel::fgetss()
 *         to get rid of line-end-charcter issues
 *  016: - corrected fitmodel::interprete_commands() concerning "A0" and "I0"
 *         not to crash if linking of symbols fails
 *       - simplyfied source code by removing AnyFit_in-exceptations, i.g.
 *         assigning errors to every kind of data (RUN.err2rec) and using it
 *       - simplified source code substitution RUN struct by class with
 *         appropriated constructor
 *       - improved performance of TransmissionIntegral by factor 3
 *       - corrected global error calculation progress bar
 *       - improved global fitting (maxtry=20 & introduced line search)
 *       - introduced convergence criterium for fitting algorithm, set COMMANDS
 *         "ConvergenceCriterium"
 *       - added "plot"-buttons to play panel
 *       - added scroll option to whole play panel
 *       - added fit-checkbox to play panel
 *       - added information label to progress bar
 *       - assigned play panel to main window,
 *       - rewrite mainwindow.cpp and mainwindow.h to get rid of old unattended
 *         error-prone moessfit user interface
 *       - check for block count in mbs-file
 *       - panel update at global fitting
 *       - obj resolving in FUNCTIONS-parser corrected by sorting obj in
 *         fitmodel::link()
 *       - introduced MoessCalc::get_fm_Powder_spectrum()
 *       - corrected multithreading-TransmissionIntegral but by using run.A
 *         instead of fitmodel.Asum (Asum does not exist anymore)
 *       - introduced Voigt-profil model
 *       - introduced FeAs-model
 *       - rebuild "fit"-handling on Mainpanel
 *       - panelclick removes focus from lineedit
 *  017: - fit errors now show 1-sigma-confidence (68%), and no more 95&
 *         confidence (28.11.2014)
 *       - shift FeAs-spectra to Moesscalc class
 *       - add noise
 *       - introduced Hesse-Ruebartsch constructor HR
 *       - improved covariant error calculation by line scan for diraction of
 *         highest correlation and downscaling of off diagonal matrix elements
 *         in case of ininvertability
 *       - ForceSimplex command
 *       - introduced maximum Error calcultion MINOS
 *       - introduced MonteCarlo Error calculation
 *       - show fit data in secondary plot if no MEM
 *       - enable mouse position in plot window
 *       - scrolling through data sets in plot2D
 *       - dataset links in plot 2D
 *       - new block definition: block ends at "##", single "#" or "\n" will
 *         continue the blok but disable the line
 *  018: - reorganized read_theory error message handling
 *       - introduced Dyn(DiagDiag(Diag)) models
 *       - introduced MLR models
 *       - introduced mainwindow::run_testsuite() to verify new code
 *       - changed number parsing in from_fld from RegExp to trial-error
 *       . Fe2As model
 *       - checking for multiple definition of parameters and functions
 *       - adding backup AutoA0 and AutoI0
 */


#ifndef MAINWINDOW_H
#define MAINWINDOW_H
#define SPECTRUMWIDTH 2048
#include <QMessageBox>
#include <QPainter>
#include <QColor>
#include <QBrush>
#include <QClipboard>
#include <QFileDialog>
#include "Moesscalc.h"
#include <QMainWindow>
#include "plot2D.h"
#include <QStringList>
#include <QProgressDialog>
#include "fitmodel.h"
#include "fold.h"
#include "DataTif.h"
#include "crystalfields.h"
#include "runlog_merge.h"
#include <QLayout>



namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0, int argc = 0, char *argv[] = NULL);
    ~MainWindow();

private:
    void keyPressEvent(QKeyEvent *event);
    Ui::MainWindow *ui;
    //! create startspectrum and start mbs file to show something
    void start_empty();
    //! generates a mbs file based on fld files and load_from_mbs
    bool load_from_fld(QStringList filelist);
    //! generates fitmodel from mbs file
    bool load_from_mbs(QString filename);
    void clear_panel();                 //!< deletes current GUI
    void run_test_suite();              //!< Developper testing purpose
    fitmodel* cfm;                      //!< current fitmodel
    QString cfmfile;                    //!< current fitmodel file name
    plot2D MoessSpectrum;               //!< window to plot spectra
    plot2D EnergySpectrum;              //!< window to plot seondary data

private slots:
    //! combines data sets with lowest line widths to new fld files
    void folder_refine();
    //! shows data chronologically with help of multilayer TIFF file
    void folder_to_tiff();
    //! creates data files with 256 linear velocity channels
    void linearize_data();
    //! use cfm to plot data in MoessSpectrum and EnergySpectrum
    void plot_current_fit_modell();
    void toggle_EnergyPlot();           //!< plot on/off
    void toggle_AbsorbtionPlot();       //!< plot on/off
    //! copy data of MoessSpectrum to clipboard
    void Absorbtion_to_clipboard();
    //! copy data of EnergySpectrum to clipboard
    void Energy_to_clipboard();
    //! forward key press events from child classes
    void child_keyevent(QKeyEvent *event);
    void open_dialog();
    //! add statistial noise to current fitmodel theory spectra
    void add_noise();
    //! merges and analyses runlog.txt
    void merge_runlog(QString path);

protected:
    void closeEvent(QCloseEvent *);
};

#endif // MAINWINDOW_H


