#define _USE_MATH_DEFINES
#include "musr.h"

MuSR::MuSR(){
    Phi0 = 2.067833758e-15;
}
double MuSR::sigma_triang(double Bex, double Bc, double kappa){
    b = Bex/Bc;
    this->Bex = Bex;
    N = 2000;
    Phi0 = 2*M_PI/kappa;
    this->kappa = kappa;
    S = Phi0 / Bex;
    x1 = sqrt(2/sqrt(3.)*S);
    x2 = x1/2.;
    y2 = sqrt(sqrt(3.)/2*S);
    init_kspace();
    init_realspace();


    for(int i = 0; i < 45; i++){
        // precalculate realspace values
        //Eq 1
        recalc_kspace_funtions();
        // iterate 1
        for(int ik = 0; ik < NK; ik++){
            ak[ik] = 0;
            for(int ix = 0; ix < Nx; ix++)
                for(int iy = 0; iy < Ny; iy++)
                    ak[ik] += (w[ix][iy]*(w[ix][iy]+Q2[ix][iy]-2)+g[ix][iy])
                            * cos(Kr(ik,ix,iy));
            ak[ik] = ak[ik]/ (Nx*Ny)
                    *4*kappa*kappa/(K[ik].sqr+2*kappa*kappa);
        }
        // precalculate realspace values
        recalc_kspace_funtions();
        //iterate 2
        for(int ik = 0; ik < NK; ik++){
            ak[ik] = ak[ik]*avgw_wQ2_g/avgw2;
        }
        // precalculate realspace values
        //Eq 3
        recalc_kspace_funtions();
        //iterate
        for(int ik = 0; ik < NK; ik++){
            bk[ik] = 0;
            for(int ix = 0; ix < Nx; ix++)
                for(int iy = 0; iy < Ny; iy++)
                    bk[ik] += ((w[ix][iy]-avgw)*B[ix][iy]+p[ix][iy])
                            * cos(Kr(ik,ix,iy));
            bk[ik] = bk[ik]/ (Nx*Ny)
                   * (-2/(K[ik].sqr+avgw));
        }
    }

    double result = 0;
    for(int ik = 0 ; ik < NK; ik++)
        result += bk[ik]*bk[ik];


    //Test
    FILE* f = fopen("C:\\Users\\User\\Desktop\\BDistr.txt","w");
    for(int ix = 0; ix < Nx; ix++){
        for(int iy = 0; iy < Ny; iy++)
            fprintf(f,"%f\t",B[ix][iy]);
        fprintf(f,"\n");
    }
    for(int ik = 0; ik < NK; ik++)
        fprintf(f,"%f\t%f\t%i\t%i\t%e\n",K[ik].x,K[ik].y,K[ik].m,K[ik].n,aAk[ik]);

    fclose(f);

    free_kspace();
    free_realspace();


    return result;
}
vector MuSR::get_K(double m, double n){
    vector result;
    result.x = 2*M_PI/S * m*y2;
    result.y = 2*M_PI/S * (n*x1+m*x2);
    result.m = m;
    result.n = n;
    result.sqr = result.x*result.x + result.y*result.y;
    result.abs = sqrt(result.sqr);
    return result;
}
void MuSR::init_kspace(){
    // count reciprocal vectors
    double K2limit = 20*N/S*1;
    NK = 0;
    bool OK;
    double nmax = (y2*y2+x2*x2)*sqrt(K2limit)/(x1*y2*sqrt(y2*y2+x2*x2))/(2*M_PI/S);
    double mmax = sqrt(K2limit)/y2/(2*M_PI/S);
    for(int n = -floor(nmax); n < nmax; n++)
        for(int m = -floor(mmax); m < mmax; m++)
            if(get_K(m,n).sqr < K2limit &&
               (n != 0 || m != 0))
                NK++;
    // create K container
    K = new vector[NK];
    int iK = 0;
    for(int n = -floor(nmax); n < nmax; n++)
        for(int m = -floor(mmax); m < mmax; m++)
            if(get_K(m,n).sqr < K2limit &&
               (n != 0 || m != 0)){
                K[iK] = get_K(m,n);
                iK++;
            }
    //create aAk
    aAk = new double[NK];
    for(int i =0 ; i < NK; i++){
        int v2 = K[i].m*K[i].m+K[i].m*K[i].n+K[i].n*K[i].n;
        if(v2%2 == 0)
            aAk[i] = - exp(-M_PI*v2/sqrt(3));
        else
            aAk[i] = + exp(-M_PI*v2/sqrt(3));

    }
    //create ak
    ak = new double[NK];
    for(int i =0 ; i < NK; i++)
        ak[i] = (1-b)*aAk[i];
    //create bk
    bk = new double[NK];
    for(int i =0 ; i < NK; i++)
        bk[i] = 0;

}
void MuSR::free_kspace(){
    delete K;
    delete aAk;
    delete ak;
    delete bk;
}
void MuSR::init_realspace(){
    Nx = sqrt(N*2/sqrt(3));
    Ny = sqrt(N*sqrt(3)/2);
    // alloc R
    x = new double[Nx];
    for(int i = 0; i < Nx; i++)
        x[i] = (0.5+i)/Nx*x1;
    y = new double[Ny];
    for(int i = 0; i < Ny; i++)
        y[i] = (0.5+i)/2/Ny*y2;
    // alloc real space data
    alloc_array(&B   ,Nx,Ny);
    alloc_array(&QAx ,Nx,Ny);
    alloc_array(&QAy ,Nx,Ny);
    alloc_array(&Q2  ,Nx,Ny);
    alloc_array(&Qx  ,Nx,Ny);
    alloc_array(&Qy  ,Nx,Ny);
    alloc_array(&p   ,Nx,Ny);
    alloc_array(&g   ,Nx,Ny);
    alloc_array(&w   ,Nx,Ny);
    alloc_array(&dwx ,Nx,Ny);
    alloc_array(&dwy ,Nx,Ny);
    alloc_array(&wA  ,Nx,Ny);
    alloc_array(&dwAx,Nx,Ny);
    alloc_array(&dwAy,Nx,Ny);
    //init start values
    for(int ix = 0; ix < Nx; ix++)
        for(int iy = 0; iy < Ny; iy++){
            wA[ix][iy] = 0;
            dwAx[ix][iy] = 0;
            dwAy[ix][iy] = 0;
            for(int ik = 0; ik < NK; ik++){
                wA[ix][iy]   += aAk[ik]*(1-cos(Kr(ik,ix,iy)));
                dwAx[ix][iy] += aAk[ik]*   sin(Kr(ik,ix,iy)) *K[ik].x;
                dwAy[ix][iy] += aAk[ik]*   sin(Kr(ik,ix,iy)) *K[ik].y;
            }
            QAx[ix][iy] =+dwAy[ix][iy]/2/kappa/wA[ix][iy];
            QAy[ix][iy] =-dwAx[ix][iy]/2/kappa/wA[ix][iy];
        }
}
void MuSR::free_realspace(){
    delete x;
    delete y;
    free_array(&B   ,Nx);
    free_array(&QAx ,Nx);
    free_array(&QAy ,Nx);
    free_array(&Q2  ,Nx);
    free_array(&Qx  ,Nx);
    free_array(&Qy  ,Nx);
    free_array(&p   ,Nx);
    free_array(&g   ,Nx);
    free_array(&w   ,Nx);
    free_array(&dwx ,Nx);
    free_array(&dwy ,Nx);
    free_array(&wA  ,Nx);
    free_array(&dwAx,Nx);
    free_array(&dwAy,Nx);
}
inline double MuSR::Kr(int ik, int ix, int iy){
    return K[ik].x * x[ix] + K[ik].y * y[iy];
}
void MuSR::recalc_kspace_funtions(){
    avgw = 0;
    avgw2 = 0;
    avgw_wQ2_g = 0;
    //Eq 2
    for(int ix = 0; ix < Nx; ix++)
        for(int iy = 0; iy < Ny; iy++){
            B[ix][iy] = Bex;
            w[ix][iy] = 0;
            dwx[ix][iy] = 0;
            dwy[ix][iy] = 0;
            Qx[ix][iy] = QAx[ix][iy];
            Qy[ix][iy] = QAy[ix][iy];

            for(int ik = 0; ik < NK; ik++){
                B[ix][iy]   += bk[ik]*cos(Kr(ik,ix,iy));
                w[ix][iy]   += ak[ik]*(1-cos(Kr(ik,ix,iy)));
                dwx[ix][iy] += ak[ik]*   sin(Kr(ik,ix,iy)) *K[ik].x;
                dwy[ix][iy] += ak[ik]*   sin(Kr(ik,ix,iy)) *K[ik].y;
                Qx[ix][iy]  +=-bk[ik]*K[ik].y/K[ik].sqr*sin(Kr(ik,ix,iy));
                Qy[ix][iy]  +=+bk[ik]*K[ik].x/K[ik].sqr*sin(Kr(ik,ix,iy));
            }
            Q2[ix][iy] = Qx[ix][iy]*Qx[ix][iy] +Qy[ix][iy]*Qy[ix][iy];
            g[ix][iy]  =(dwx[ix][iy]*dwx[ix][iy]+dwy[ix][iy]*dwy[ix][iy])
                        /(4*kappa*kappa*w[ix][iy]);
            p[ix][iy]  = -Qx[ix][iy]*dwy[ix][iy]+Qy[ix][iy]*dwx[ix][iy];
            avgw += w[ix][iy];
            avgw2 += w[ix][iy] * w[ix][iy];
            avgw_wQ2_g += w[ix][iy]*(1-Q2[ix][iy])-g[ix][iy];
        }
    avgw = avgw / (Nx*Ny);
    avgw2 = avgw2 / (Nx*Ny);
    avgw_wQ2_g = avgw_wQ2_g / (Nx*Ny);
}
