#define _USE_MATH_DEFINES
#include "musr.h"

MuSR::MuSR(){
    Phi0 = 2.067833758e-15;
    MaxIteration = 25;
    N = 500;
}
double MuSR::sigma_triangBLAS(double B, double Bc2, double lambda){
    init_BLAS(B,Bc2,lambda);
    for(int i = 0; i < MaxIteration; i++){
        // Eq 1
        gsl_blas_dgemv(CblasNoTrans,1,one_cosKR,aK,0,WR);
        gsl_vector_memcpy(QxR,QAxR);
        gsl_vector_memcpy(QyR,QAyR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRQx  ,bK,1,QxR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRQy  ,bK,1,QyR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRkx  ,aK,0,dWxR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRky  ,aK,0,dWyR);
        for(int iR = 0; iR < NR; iR++){
            gsl_vector_set(vt,iR,gsl_vector_get(WR,iR)*
                                (gsl_vector_get(WR,iR)+get_Q2(iR)-2)
                                +get_g(iR));
       }

        gsl_blas_dgemv(CblasTrans,1,cosKR,vt,0,aK);
        for(int iK = 0; iK < NK; iK++)
            gsl_vector_set(aK,iK,gsl_vector_get(aK,iK)*4*kappa*kappa/(K[iK].sqr+2*kappa*kappa)/NR);
        // Eq 2
        gsl_blas_dgemv(CblasNoTrans,1,one_cosKR,aK,0,WR);
        gsl_vector_memcpy(QxR,QAxR);
        gsl_vector_memcpy(QyR,QAyR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRQx,bK,1,QxR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRQy,bK,1,QyR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRkx,aK,0,dWxR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRky,aK,0,dWyR);
        avgw = avgw2 = avgw_wQ2_g = 0;
        for(int iR = 0; iR < NR; iR++){
            avgw       += gsl_vector_get(WR,iR);
            avgw2      += gsl_vector_get(WR,iR) * gsl_vector_get(WR,iR);
            avgw_wQ2_g += gsl_vector_get(WR,iR) * (1-get_Q2(iR))-get_g(iR);
        }
        avgw = avgw / NR;
        gsl_vector_scale(aK,avgw_wQ2_g/avgw2);
        // Eq 3
        gsl_blas_dgemv(CblasNoTrans,1,one_cosKR,aK,0,WR);
        gsl_vector_set_all(BR,Bex);
        gsl_blas_dgemv(CblasNoTrans,1,cosKR,bK,1,BR);
        gsl_vector_memcpy(QxR,QAxR);
        gsl_vector_memcpy(QyR,QAyR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRQx,bK,1,QxR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRQy,bK,1,QyR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRkx,aK,0,dWxR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRky,aK,0,dWyR);
        avgw = 0;
        for(int iR = 0; iR < NR; iR++)
            avgw       += gsl_vector_get(WR,iR);
        avgw = avgw / NR;
        for(int iR = 0; iR < NR; iR++){
            gsl_vector_set(vt,iR,(gsl_vector_get(WR,iR)-avgw)*gsl_vector_get(BR,iR)+get_p(iR));
        }
        gsl_blas_dgemv(CblasTrans,1,cosKR,vt,0,bK);
        for(int iK = 0; iK < NK; iK++)
            gsl_vector_set(bK,iK,-2.*gsl_vector_get(bK,iK)/(K[iK].sqr+avgw)/NR);
    }

    double result = 0;
    for(int ik = 0 ; ik < NK; ik++)
        result += gsl_vector_get(bK,ik)*gsl_vector_get(bK,ik);

    //FILE* f = fopen("C:\\Users\\User\\Desktop\\BDistr.txt","w");
    FILE* f = fopen("C:\\Users\\test\\Desktop\\Lattice2.txt","w");
    int i = 0;
    for(int ix = 0; ix < Nx; ix++){
        for(int iy = 0; iy < Ny; iy++){
            fprintf(f,"%f\t",BR->data[i]*Bctherm*sqrt(2));
            i++;
        }
        fprintf(f,"\n");
    }
    fprintf(f,"%f\n",sqrt(result)*Bctherm*sqrt(2));
    fclose(f);



    free_BLAS();
    return sqrt(result/2)*Bctherm*sqrt(2)*135.538817*2*M_PI;
}
double MuSR::sigma_quadBLAS(double B, double Bc2, double lambda){
    init_quadBLAS(B,Bc2,lambda);
    for(int i = 0; i < MaxIteration; i++){
        // Eq 1
        gsl_blas_dgemv(CblasNoTrans,1,one_cosKR,aK,0,WR);
        gsl_vector_memcpy(QxR,QAxR);
        gsl_vector_memcpy(QyR,QAyR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRQx  ,bK,1,QxR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRQy  ,bK,1,QyR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRkx  ,aK,0,dWxR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRky  ,aK,0,dWyR);
        for(int iR = 0; iR < NR; iR++){
            gsl_vector_set(vt,iR,gsl_vector_get(WR,iR)*
                                (gsl_vector_get(WR,iR)+get_Q2(iR)-2)
                                +get_g(iR));
       }

        gsl_blas_dgemv(CblasTrans,1,cosKR,vt,0,aK);
        for(int iK = 0; iK < NK; iK++)
            gsl_vector_set(aK,iK,gsl_vector_get(aK,iK)*4*kappa*kappa/(K[iK].sqr+2*kappa*kappa)/NR);
        // Eq 2
        gsl_blas_dgemv(CblasNoTrans,1,one_cosKR,aK,0,WR);
        gsl_vector_memcpy(QxR,QAxR);
        gsl_vector_memcpy(QyR,QAyR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRQx,bK,1,QxR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRQy,bK,1,QyR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRkx,aK,0,dWxR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRky,aK,0,dWyR);
        avgw = avgw2 = avgw_wQ2_g = 0;
        for(int iR = 0; iR < NR; iR++){
            avgw       += gsl_vector_get(WR,iR);
            avgw2      += gsl_vector_get(WR,iR) * gsl_vector_get(WR,iR);
            avgw_wQ2_g += gsl_vector_get(WR,iR) * (1-get_Q2(iR))-get_g(iR);
        }
        avgw = avgw / NR;
        gsl_vector_scale(aK,avgw_wQ2_g/avgw2);
        // Eq 3
        gsl_blas_dgemv(CblasNoTrans,1,one_cosKR,aK,0,WR);
        gsl_vector_set_all(BR,Bex);
        gsl_blas_dgemv(CblasNoTrans,1,cosKR,bK,1,BR);
        gsl_vector_memcpy(QxR,QAxR);
        gsl_vector_memcpy(QyR,QAyR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRQx,bK,1,QxR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRQy,bK,1,QyR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRkx,aK,0,dWxR);
        gsl_blas_dgemv(CblasNoTrans,1,sinKRky,aK,0,dWyR);
        avgw = 0;
        for(int iR = 0; iR < NR; iR++)
            avgw       += gsl_vector_get(WR,iR);
        avgw = avgw / NR;
        for(int iR = 0; iR < NR; iR++)
            gsl_vector_set(vt,iR,(gsl_vector_get(WR,iR)-avgw)*gsl_vector_get(BR,iR)+get_p(iR));
        gsl_blas_dgemv(CblasTrans,1,cosKR,vt,0,bK);
        for(int iK = 0; iK < NK; iK++)
            gsl_vector_set(bK,iK,-2.*gsl_vector_get(bK,iK)/(K[iK].sqr+avgw)/NR);
    }

    /*FILE* f = fopen("C:\\Users\\User\\Desktop\\BDistr.txt","w");
    //FILE* f = fopen("C:\\Users\\test\\Desktop\\Lattice2.txt","w");
    int i = 0;
    for(int ix = 0; ix < Nx; ix++){
        for(int iy = 0; iy < Ny; iy++){
            fprintf(f,"%f\t",BR->data[i]*Bctherm*sqrt(2));
            i++;
        }
        fprintf(f,"\n");
    }

    fclose(f);*/

    double result = 0;
    for(int ik = 0 ; ik < NK; ik++)
        result += gsl_vector_get(bK,ik)*gsl_vector_get(bK,ik);

    free_BLAS();
    return sqrt(result/2)*Bctherm*sqrt(2)*0.0135538817e4*2*M_PI;
}

double MuSR::get_Q2(int iR){
    return  gsl_vector_get(QxR,iR) * gsl_vector_get(QxR,iR) +
            gsl_vector_get(QyR,iR) * gsl_vector_get(QyR,iR);
}

double MuSR::get_g(int iR){
    return (gsl_vector_get(dWxR,iR) * gsl_vector_get(dWxR,iR) +
            gsl_vector_get(dWyR,iR) * gsl_vector_get(dWyR,iR))
            /4/kappa/kappa/gsl_vector_get(WR,iR);
}

double MuSR::get_p(int iR){
    return -gsl_vector_get(QxR,iR) * gsl_vector_get(dWyR,iR) +
            gsl_vector_get(QyR,iR) * gsl_vector_get(dWxR,iR);
}

void MuSR::init_BLAS(double B, double Bc2, double lambda){
    init_kspace(B,Bc2,lambda);
    init_realspace();
    cosKR   = gsl_matrix_alloc(NR,NK);
    one_cosKR   = gsl_matrix_alloc(NR,NK);
    sinKRkx = gsl_matrix_alloc(NR,NK);
    sinKRky = gsl_matrix_alloc(NR,NK);
    sinKRQx = gsl_matrix_alloc(NR,NK);
    sinKRQy = gsl_matrix_alloc(NR,NK);

    aK = gsl_vector_alloc(NK);
    aAK = gsl_vector_alloc(NK);
    bK = gsl_vector_alloc(NK);
    WR = gsl_vector_alloc(NR);
    dWxR = gsl_vector_alloc(NR);
    dWyR = gsl_vector_alloc(NR);
    Q2R = gsl_vector_alloc(NR);
    QxR = gsl_vector_alloc(NR);
    QyR = gsl_vector_alloc(NR);
    QAxR = gsl_vector_alloc(NR);
    QAyR = gsl_vector_alloc(NR);
    gR = gsl_vector_alloc(NR);
    pR = gsl_vector_alloc(NR);
    BR = gsl_vector_alloc(NR);
    vt = gsl_vector_alloc(NR);

    int iR = 0;
    for(int ix = 0; ix < Nx; ix++)
        for(int iy = 0; iy < Ny; iy++){
            for(int ik = 0; ik < NK; ik++){
                gsl_matrix_set(cosKR    ,iR,ik,  cos(Kr(ik,ix,iy)));
                gsl_matrix_set(one_cosKR,iR,ik,1-cos(Kr(ik,ix,iy)));
                gsl_matrix_set(sinKRkx  ,iR,ik,+K[ik].x*sin(Kr(ik,ix,iy)));
                gsl_matrix_set(sinKRky  ,iR,ik,+K[ik].y*sin(Kr(ik,ix,iy)));
                gsl_matrix_set(sinKRQx  ,iR,ik,-K[ik].y/K[ik].sqr*sin(Kr(ik,ix,iy)));
                gsl_matrix_set(sinKRQy  ,iR,ik,+K[ik].x/K[ik].sqr*sin(Kr(ik,ix,iy)));
            }
            gsl_vector_set(QAxR,iR,QAx[ix][iy]);
            gsl_vector_set(QAyR,iR,QAy[ix][iy]);
            iR++;
            }
    for(int ik = 0; ik < NK; ik++){
        gsl_vector_set(aAK,ik,aAk[ik]);
        gsl_vector_set(aK,ik,ak[ik]);
        gsl_vector_set(bK,ik,bk[ik]);
    }
}
void MuSR::init_quadBLAS(double B, double Bc2, double lambda){
    init_quadkspace(B,Bc2,lambda);
    init_quadrealspace();
    cosKR   = gsl_matrix_alloc(NR,NK);
    one_cosKR   = gsl_matrix_alloc(NR,NK);
    sinKRkx = gsl_matrix_alloc(NR,NK);
    sinKRky = gsl_matrix_alloc(NR,NK);
    sinKRQx = gsl_matrix_alloc(NR,NK);
    sinKRQy = gsl_matrix_alloc(NR,NK);

    aK = gsl_vector_alloc(NK);
    aAK = gsl_vector_alloc(NK);
    bK = gsl_vector_alloc(NK);
    WR = gsl_vector_alloc(NR);
    dWxR = gsl_vector_alloc(NR);
    dWyR = gsl_vector_alloc(NR);
    Q2R = gsl_vector_alloc(NR);
    QxR = gsl_vector_alloc(NR);
    QyR = gsl_vector_alloc(NR);
    QAxR = gsl_vector_alloc(NR);
    QAyR = gsl_vector_alloc(NR);
    gR = gsl_vector_alloc(NR);
    pR = gsl_vector_alloc(NR);
    BR = gsl_vector_alloc(NR);
    vt = gsl_vector_alloc(NR);

    int iR = 0;
    for(int ix = 0; ix < Nx; ix++)
        for(int iy = 0; iy < Ny; iy++){
            for(int ik = 0; ik < NK; ik++){
                gsl_matrix_set(cosKR    ,iR,ik,  cos(Kr(ik,ix,iy)));
                gsl_matrix_set(one_cosKR,iR,ik,1-cos(Kr(ik,ix,iy)));
                gsl_matrix_set(sinKRkx  ,iR,ik,+K[ik].x*sin(Kr(ik,ix,iy)));
                gsl_matrix_set(sinKRky  ,iR,ik,+K[ik].y*sin(Kr(ik,ix,iy)));
                gsl_matrix_set(sinKRQx  ,iR,ik,-K[ik].y/K[ik].sqr*sin(Kr(ik,ix,iy)));
                gsl_matrix_set(sinKRQy  ,iR,ik,+K[ik].x/K[ik].sqr*sin(Kr(ik,ix,iy)));
            }
            gsl_vector_set(QAxR,iR,QAx[ix][iy]);
            gsl_vector_set(QAyR,iR,QAy[ix][iy]);
            iR++;
            }
    for(int ik = 0; ik < NK; ik++){
        gsl_vector_set(aAK,ik,aAk[ik]);
        gsl_vector_set(aK,ik,ak[ik]);
        gsl_vector_set(bK,ik,bk[ik]);
    }
}

void MuSR::free_BLAS(){
    free_kspace();
    free_realspace();

    gsl_matrix_free(cosKR   );
    gsl_matrix_free(one_cosKR);
    gsl_matrix_free(sinKRkx );
    gsl_matrix_free(sinKRky );
    gsl_matrix_free(sinKRQx );
    gsl_matrix_free(sinKRQy );

    gsl_vector_free(aK );
    gsl_vector_free(aAK);
    gsl_vector_free(bK );
    gsl_vector_free(WR );
    gsl_vector_free(dWxR);
    gsl_vector_free(dWyR);
    gsl_vector_free(Q2R);
    gsl_vector_free(QxR);
    gsl_vector_free(QyR);
    gsl_vector_free(QAxR);
    gsl_vector_free(QAyR);
    gsl_vector_free(gR );
    gsl_vector_free(pR );
    gsl_vector_free(BR );
    gsl_vector_free(vt );
}

double MuSR::sigma_triang(double Bex, double Bc2, double lambda){
    init_kspace(Bex,Bc2,lambda);
    init_realspace();


    for(int i = 0; i < MaxIteration; i++){
        // precalculate realspace values
        //Eq 1
        recalc_kspace_funtions();
        double read;
        // iterate 1
        for(int ik = 0; ik < NK; ik++){
            ak[ik] = 0;
            for(int ix = 0; ix < Nx; ix++)
                for(int iy = 0; iy < Ny; iy++){
                    read = w[ix][iy]*(w[ix][iy]+Q2[ix][iy]-2)+g[ix][iy];
                    ak[ik] += (w[ix][iy]*(w[ix][iy]+Q2[ix][iy]-2)+g[ix][iy])
                            * cos(Kr(ik,ix,iy));
                }
            ak[ik] = ak[ik]/ (Nx*Ny)
                    *4*kappa*kappa/(K[ik].sqr+2*kappa*kappa);
        }
        // precalculate realspace values
        recalc_kspace_funtions();
        //iterate 2
        for(int ik = 0; ik < NK; ik++){
            ak[ik] = ak[ik]*avgw_wQ2_g/avgw2;
        }
        // precalculate realspace values
        //Eq 3
        recalc_kspace_funtions();
        //iterate
        for(int ik = 0; ik < NK; ik++){
            bk[ik] = 0;
            for(int ix = 0; ix < Nx; ix++)
                for(int iy = 0; iy < Ny; iy++){
                    //double read = ((w[ix][iy]-avgw)*B[ix][iy]+p[ix][iy]);
                    bk[ik] += ((w[ix][iy]-avgw)*B[ix][iy]+p[ix][iy])
                            * cos(Kr(ik,ix,iy));
                }
            bk[ik] = bk[ik]/ (Nx*Ny)
                   * (-2/(K[ik].sqr+avgw));
        }
    }

    double result = 0;
    for(int ik = 0 ; ik < NK; ik++)
        result += bk[ik]*bk[ik];


    //Test
    /*FILE* f = fopen("C:\\Users\\User\\Desktop\\BDistr.txt","w");
    //FILE* f = fopen("C:\\Users\\test\\Desktop\\Lattice2.txt","w");

    for(int ix = 0; ix < Nx; ix++){
        for(int iy = 0; iy < Ny; iy++)
            fprintf(f,"%f\t",B[ix][iy]*Bctherm*sqrt(2));
        fprintf(f,"\n");
    }
    fclose(f);*/

    free_kspace();
    free_realspace();


    return result;
}
vector MuSR::get_K(double m, double n){
    vector result;
    result.x = 2*M_PI/S * m*y2;
    result.y = 2*M_PI/S * (n*x1+m*x2);
    result.m = int(m);
    result.n = int(n);
    result.sqr = result.x*result.x + result.y*result.y;
    result.abs = sqrt(result.sqr);
    return result;
}
void MuSR::init_kspace(double Bex, double Bc2, double lambda){
    // initial reduced value measures
    b = Bex/Bc2;
    double xi= sqrt(2.067833758e-15/2/M_PI/Bc2); //GL thery
    Bctherm = 2.067833758e-15/sqrt(8.)/lambda/xi/M_PI;
    this->Bex = Bex/Bctherm/sqrt(2);
    kappa = lambda / xi;
    Phi0 = 2*M_PI/kappa;
    S = Phi0 / this->Bex;
    x1 = sqrt(2/sqrt(3.)*S);
    x2 = x1/2.;
    y2 = sqrt(sqrt(3.)/2*S);
    // count reciprocal vectors
    double K2limit = 20*N/S*1;
    NK = 0;
    double nmax = (y2*y2+x2*x2)*sqrt(K2limit)/(x1*y2*sqrt(y2*y2+x2*x2))/(2*M_PI/S);
    double mmax = sqrt(K2limit)/y2/(2*M_PI/S);
    for(int n = -floor(nmax); n < nmax; n++)
        for(int m = -floor(mmax); m < mmax; m++)
            if(get_K(m,n).sqr < K2limit &&
               (n != 0 || m != 0)&&
               ((m > 0 && n >= 0) || m<=0 && n > 0))
                NK++;
    // create K container
    K = new vector[NK];
    int iK = 0;
    for(int n = -floor(nmax); n < nmax; n++)
        for(int m = -floor(mmax); m < mmax; m++)
            if(get_K(m,n).sqr < K2limit &&
               (n != 0 || m != 0)&&
               ((m > 0 && n >= 0) || m<=0 && n > 0)){
                K[iK] = get_K(m,n);
                iK++;
            }
    //create aAk
    aAk = new double[NK];
    for(int i =0 ; i < NK; i++){
        int v2 = K[i].m*K[i].m+K[i].m*K[i].n+K[i].n*K[i].n;
        if(v2%2 == 0)
            aAk[i] = - exp(-M_PI*v2/sqrt(3));
        else
            aAk[i] = + exp(-M_PI*v2/sqrt(3));

    }
    //create ak
    ak = new double[NK];
    for(int i =0 ; i < NK; i++)
        ak[i] = (1-b)*aAk[i];
    //create bk
    bk = new double[NK];
    for(int i =0 ; i < NK; i++)
        bk[i] = 0;

}
void MuSR::init_quadkspace(double Bex, double Bc2, double lambda){
    // initial reduced value measures
    b = Bex/Bc2;
    double xi= sqrt(2.067833758e-15/2/M_PI/Bc2); //GL thery
    Bctherm = 2.067833758e-15/sqrt(8.)/lambda/xi/M_PI;
    this->Bex = Bex/Bctherm/sqrt(2);
    kappa = lambda / xi;
    Phi0 = 2*M_PI/kappa;
    S = Phi0 / this->Bex;
    x1 = sqrt(S);
    x2 = 0;
    y2 = sqrt(S);
    // count reciprocal vectors
    double K2limit = 20*N/S*1;
    NK = 0;
    bool OK;
    double nmax = (y2*y2+x2*x2)*sqrt(K2limit)/(x1*y2*sqrt(y2*y2+x2*x2))/(2*M_PI/S);
    double mmax = sqrt(K2limit)/y2/(2*M_PI/S);
    for(int n = -floor(nmax); n < nmax; n++)
        for(int m = -floor(mmax); m < mmax; m++)
            if(get_K(m,n).sqr < K2limit &&
               (n != 0 || m != 0)&&
               ((m > 0 && n >= 0) || m<=0 && n > 0))
                NK++;
    // create K container
    K = new vector[NK];
    int iK = 0;
    for(int n = -floor(nmax); n < nmax; n++)
        for(int m = -floor(mmax); m < mmax; m++)
            if(get_K(m,n).sqr < K2limit &&
               (n != 0 || m != 0)&&
               ((m > 0 && n >= 0) || m<=0 && n > 0)){
                K[iK] = get_K(m,n);
                iK++;
            }
    //create aAk
    aAk = new double[NK];
    for(int i =0 ; i < NK; i++){
        int v2 = K[i].m*K[i].m+K[i].m*K[i].n+K[i].n*K[i].n;
            if(v2%2 == 0)
                aAk[i] = - exp(-K[i].sqr*S/8./M_PI);
            else
                aAk[i] = + exp(-K[i].sqr*S/8./M_PI);

    }
    //create ak
    ak = new double[NK];
    for(int i =0 ; i < NK; i++)
        ak[i] = (1-b)*aAk[i];
    //create bk
    bk = new double[NK];
    for(int i =0 ; i < NK; i++)
        bk[i] = 0;

}

void MuSR::free_kspace(){
    delete K;
    delete aAk;
    delete ak;
    delete bk;
}
void MuSR:: init_realspace(){
    Nx = sqrt(N*4/sqrt(3));
    Ny = sqrt(N*sqrt(3)/4);
    NR = Nx*Ny;
    // alloc R
    x = new double[Nx];
    for(int i = 0; i < Nx; i++)
        x[i] = (0.5+i)/Nx*x1;
    y = new double[Ny];
    for(int i = 0; i < Ny; i++)
        y[i] = (0.5+i)/Ny*y2/2.;
    // alloc real space data
    alloc_array(&B   ,Nx,Ny);
    alloc_array(&QAx ,Nx,Ny);
    alloc_array(&QAy ,Nx,Ny);
    alloc_array(&Q2  ,Nx,Ny);
    alloc_array(&Qx  ,Nx,Ny);
    alloc_array(&Qy  ,Nx,Ny);
    alloc_array(&p   ,Nx,Ny);
    alloc_array(&g   ,Nx,Ny);
    alloc_array(&w   ,Nx,Ny);
    alloc_array(&dwx ,Nx,Ny);
    alloc_array(&dwy ,Nx,Ny);
    alloc_array(&wA  ,Nx,Ny);
    alloc_array(&dwAx,Nx,Ny);
    alloc_array(&dwAy,Nx,Ny);
    //init start values
    for(int ix = 0; ix < Nx; ix++)
        for(int iy = 0; iy < Ny; iy++){
            wA[ix][iy] = 0;
            dwAx[ix][iy] = 0;
            dwAy[ix][iy] = 0;
            for(int ik = 0; ik < NK; ik++){
                wA[ix][iy]   += aAk[ik]*(1-cos(Kr(ik,ix,iy)));
                dwAx[ix][iy] += aAk[ik]*   sin(Kr(ik,ix,iy)) *K[ik].x;
                dwAy[ix][iy] += aAk[ik]*   sin(Kr(ik,ix,iy)) *K[ik].y;
            }
            QAx[ix][iy] =+dwAy[ix][iy]/2/kappa/wA[ix][iy];
            QAy[ix][iy] =-dwAx[ix][iy]/2/kappa/wA[ix][iy];
        }
}
void MuSR:: init_quadrealspace(){
    Nx = sqrt(N);
    Ny = sqrt(N);
    NR = Nx*Ny;
    // alloc R
    x = new double[Nx];
    for(int i = 0; i < Nx; i++)
        x[i] = (0.5+i)/Nx*x1;
    y = new double[Ny];
    for(int i = 0; i < Ny; i++)
        y[i] = (0.5+i)/Ny*y2;
    // alloc real space data
    alloc_array(&B   ,Nx,Ny);
    alloc_array(&QAx ,Nx,Ny);
    alloc_array(&QAy ,Nx,Ny);
    alloc_array(&Q2  ,Nx,Ny);
    alloc_array(&Qx  ,Nx,Ny);
    alloc_array(&Qy  ,Nx,Ny);
    alloc_array(&p   ,Nx,Ny);
    alloc_array(&g   ,Nx,Ny);
    alloc_array(&w   ,Nx,Ny);
    alloc_array(&dwx ,Nx,Ny);
    alloc_array(&dwy ,Nx,Ny);
    alloc_array(&wA  ,Nx,Ny);
    alloc_array(&dwAx,Nx,Ny);
    alloc_array(&dwAy,Nx,Ny);
    //init start values
    for(int ix = 0; ix < Nx; ix++)
        for(int iy = 0; iy < Ny; iy++){
            wA[ix][iy] = 0;
            dwAx[ix][iy] = 0;
            dwAy[ix][iy] = 0;
            for(int ik = 0; ik < NK; ik++){
                wA[ix][iy]   += aAk[ik]*(1-cos(Kr(ik,ix,iy)));
                dwAx[ix][iy] += aAk[ik]*   sin(Kr(ik,ix,iy)) *K[ik].x;
                dwAy[ix][iy] += aAk[ik]*   sin(Kr(ik,ix,iy)) *K[ik].y;
            }
            QAx[ix][iy] =+dwAy[ix][iy]/2/kappa/wA[ix][iy];
            QAy[ix][iy] =-dwAx[ix][iy]/2/kappa/wA[ix][iy];
        }
}

void MuSR::free_realspace(){
    delete x;
    delete y;
    free_array(&B   ,Nx);
    free_array(&QAx ,Nx);
    free_array(&QAy ,Nx);
    free_array(&Q2  ,Nx);
    free_array(&Qx  ,Nx);
    free_array(&Qy  ,Nx);
    free_array(&p   ,Nx);
    free_array(&g   ,Nx);
    free_array(&w   ,Nx);
    free_array(&dwx ,Nx);
    free_array(&dwy ,Nx);
    free_array(&wA  ,Nx);
    free_array(&dwAx,Nx);
    free_array(&dwAy,Nx);
}
inline double MuSR::Kr(int ik, int ix, int iy){
    return K[ik].x * x[ix] + K[ik].y * y[iy];
}
void MuSR::recalc_kspace_funtions(){
    avgw = 0;
    avgw2 = 0;
    avgw_wQ2_g = 0;
    for(int ix = 0; ix < Nx; ix++)
        for(int iy = 0; iy < Ny; iy++){
            B[ix][iy] = Bex;
            w[ix][iy] = 0;
            dwx[ix][iy] = 0;
            dwy[ix][iy] = 0;
            Qx[ix][iy] = QAx[ix][iy];
            Qy[ix][iy] = QAy[ix][iy];

            for(int ik = 0; ik < NK; ik++){
                B[ix][iy]   += bk[ik]*cos(Kr(ik,ix,iy));
                w[ix][iy]   += ak[ik]*(1-cos(Kr(ik,ix,iy)));
                dwx[ix][iy] += ak[ik]*   sin(Kr(ik,ix,iy)) *K[ik].x;
                dwy[ix][iy] += ak[ik]*   sin(Kr(ik,ix,iy)) *K[ik].y;
                Qx[ix][iy]  +=-bk[ik]*K[ik].y/K[ik].sqr*sin(Kr(ik,ix,iy));
                Qy[ix][iy]  +=+bk[ik]*K[ik].x/K[ik].sqr*sin(Kr(ik,ix,iy));
            }
            Q2[ix][iy] = Qx[ix][iy]*Qx[ix][iy] +Qy[ix][iy]*Qy[ix][iy];
            g[ix][iy]  =(dwx[ix][iy]*dwx[ix][iy]+dwy[ix][iy]*dwy[ix][iy])
                        /(4*kappa*kappa*w[ix][iy]);
            p[ix][iy]  = -Qx[ix][iy]*dwy[ix][iy]+Qy[ix][iy]*dwx[ix][iy];
            avgw += w[ix][iy];
            avgw2 += w[ix][iy] * w[ix][iy];
            avgw_wQ2_g += w[ix][iy]*(1-Q2[ix][iy])-g[ix][iy];
        }
    avgw = avgw / (Nx*Ny);
    avgw2 = avgw2 / (Nx*Ny);
    avgw_wQ2_g = avgw_wQ2_g / (Nx*Ny);
}
double MuSR::tail(double theta, double phi, double alpha, double b){
    return (sin(theta)*sin(phi)*sin(alpha)+cos(theta)*cos(alpha))*cos(theta)
           /(1+b*b+b*sin(theta)*cos(phi));
}
double MuSR::tail(double BexOverBin, double alpha){
    double result = 0;
    double sintheta;
    double sumsin = 0;
    int phim = 1000;
    int thetam = 1000;
    for(int phii=0; phii < phim; phii++)
        for(int thetai=0; thetai < thetam; thetai++){
            sintheta = sin(M_PI*thetai/thetam);
            sumsin += sintheta;
            result += tail(M_PI*thetai/thetam,
                           2*M_PI*phii/phim,
                           alpha,BexOverBin)*sintheta;
        }
    return result/cos(alpha)/sumsin;
}
double MuSR::sigma_nuclear_tetragonal(int nuclidcount, nuclid *n, double a,
                                      double c, int nn, double mux, double muy,
                                      double muz ){
    double sigma2 = 0;
    double mua = mux*a;
    double mub = muy*a;
    double muc = muz*c;

    for(int ix = -nn; ix <= nn; ix++)
        for(int iy = -nn; iy <= nn; iy++)
            for(int iz = -nn; iz <= nn; iz++)
                for(int in = 0; in < nuclidcount; in++){
                    double rx = (n[in].x+ix)*a - mua;
                    double ry = (n[in].y+iy)*a - mub;
                    double rz = (n[in].z+iz)*c - muc;
                    double r = sqrt(rx*rx+ry*ry+rz*rz);
                    double r6 = r*r*r*r*r*r;
                    sigma2 += n[in].gamma*n[in].gamma*n[in].I*(1+n[in].I)/r6;
                }
    sigma2 *= 4./15*135.5*2*3.1415*135.5*2*3.1415*
            (1.2566e-6/(4*3.1415))*(1.2566e-6/(4*3.1415))*  //(mu_0/4pi)^2
            1.11212168135524E-008;                          //(1e-10)^6
    return sigma2;
}
void MuSR::add_dipol(double x, double y, double z, double m,
                     double mtheta, double mphi, double* B){
    double mx = m * sin(mtheta)*cos(mphi);
    double my = m * sin(mtheta)*sin(mphi);
    double mz = m * cos(mtheta);
    double r = sqrt(x*x+y*y+z*z)*1e-10;
    double ex = x*1e-10/r;
    double ey = y*1e-10/r;
    double ez = z*1e-10/r;

    B[0] += 9.27499994e-31/(r*r*r)*(3*(ex*mx+ex*my+ez*mz)*ex-mx);
    B[1] += 9.27499994e-31/(r*r*r)*(3*(ex*mx+ex*my+ez*mz)*ey-my);
    B[2] += 9.27499994e-31/(r*r*r)*(3*(ex*mx+ex*my+ez*mz)*ez-mz);
}
