#ifndef MUSR_H
#define MUSR_H

#include <cmath>
#include <stdio.h>
#include "array.h"
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_integration.h>

typedef struct vector{
    double x,y,sqr,abs;
    int m,n;
}vector;

typedef struct nuclid{
    double x,y,z;
    double I;
    double gamma;
}nuclid;

class MuSR{
public:
    MuSR();
    double sigma_triang(double Bex, double Bc2, double lambda);
    double sigma_triangBLAS(double Bex, double Bc2, double lambda);
    double sigma_quadBLAS(double Bex, double Bc2, double lambda);
    double tail(double BexOverBin, double alpha);
    double tail(double theta, double phi, double alpha, double b);
    double sigma_nuclear_tetragonal(int nuclidcount, nuclid *n, double a,
                                    double c, int nn, double mux, double muy,
                                    double muz);
    void add_dipol(double x, double y, double z, double m,
                   double mtheta, double mphi, double* B);
private:
    double Phi0;
    double kappa;
    double b;
    double x1;
    double x2;
    double y2;
    double S;
    double Bctherm;
    int N;
    int NR;
    int MaxIteration;
    //reciproal space
    int NK;
    vector* K;
    double* aAk;
    double* ak;
    double* bk;
    // real space
    int Nx;
    int Ny;
    double* x;
    double* y;
    double **B;
    double **QAx;
    double **QAy;
    double **Q2;
    double **Qx;
    double **Qy;
    double **p;
    double **g;
    double **w;
    double **dwx;
    double **dwy;
    double **wA;
    double **dwAx;
    double **dwAy;

    double avgw;
    double avgw2;
    double avgw_wQ2_g;
    double Bex;

    gsl_matrix* cosKR;
    gsl_matrix* one_cosKR;
    gsl_matrix* sinKRkx;
    gsl_matrix* sinKRky;
    gsl_matrix* sinKRQx;
    gsl_matrix* sinKRQy;

    gsl_vector* vt;
    gsl_vector* aK;
    gsl_vector* aAK;
    gsl_vector* bK;
    gsl_vector* WR;
    gsl_vector* dWxR;
    gsl_vector* dWyR;
    gsl_vector* Q2R;
    gsl_vector* QxR;
    gsl_vector* QyR;
    gsl_vector* QAxR;
    gsl_vector* QAyR;
    gsl_vector* gR;
    gsl_vector* pR;
    gsl_vector* BR;




    vector get_K(double m, double n);
    double get_Q2(int iR);
    double get_g(int iR);
    double get_p(int iR);
    void recalc_kspace_funtions();
    void init_kspace(double Bex, double Bc2, double lambda);
    void init_quadkspace(double Bex, double Bc2, double lambda);
    void free_kspace();
    void init_realspace();
    void init_quadrealspace();
    void free_realspace();
    void init_BLAS(double Bex, double Bc2, double lambda);
    void init_quadBLAS(double Bex, double Bc2, double lambda);
    void free_BLAS();
    double Kr(int ik, int ix, int iy);

};

#endif // MUSR_H
