﻿#ifndef PLOT2D_H
#define PLOT2D_H
#include <math.h>
#include "array.h"
#include <QWidget>
#include <QPixmap>
#include <QPainter>
#include <QRect>
#include <QBrush>
#include <QMouseEvent>
#include <QString>
#include <QFontMetricsF>
#include <QClipboard>
#include <QApplication>
#include <QMessageBox>


class plot2D : public QWidget{
    Q_OBJECT
public:
    plot2D(QWidget *parent = 0);
    void set_linedata(int cset, int* cc, double** xdata, double** ydata);
    void set_pointdata(int cset, int* cc, double** xdata, double** ydata);
    void set_linedata(int xcount, int ycount, double* xdata, double** ydata);
    void set_pointdata(int xcount, int ycount, double* xdata, double** ydata);
    void update_linedata(double** ydata, int ycount);
    void set_labels(QString title, QString xbarlabel, QString ybarlabel);
    void set_legend(QStringList* labels);
    QSize sizeHint() const;
    QSize minimumSizeHint() const;
    void autoset_axis();
    void set_linewidth(double linewidth);
    void set_range(double plotxmin, double plotxmax,
                   double plotymin, double plotymax);
    void data_to_clipboard();
    void show_residuum();
    void plot_current_set();
    unsigned char** get_bits();
public slots:
    void zoom_in();
    void zoom_out();
private:
    double **xpdata, **ypdata, **xldata, **yldata;
    int psets,lsets,*pcc,*lcc, freep, freel;
    int showResiduum,resheight;
    int showLegend;
    int wheelselection;
    QStringList leglabels;
    void refresh_pixmap();
    QPixmap pm;
    int margin, ticklength, xsteps, ysteps;
    double xmin,xmax,ymin,ymax;         //Diagrammbereich
    double xstep, ystep, xstep0, ystep0;//Achsskalen
    double xprecission, yprecission;    //Kommastellen der Beschriftung
    double textheight;
    double linewidth;
    QPoint mouseposition;
    QString title, xbarlabel, ybarlabel;
    QRect dataarea;
    QRect legendarea;
    QRect zoombox;
    int showzoombox;                    //Zoombox zeichnen oder nicht?
    double xtoxdata(int x);
    double ytoydata(int y);
    double xdatatox(double x);
    double ydatatoy(double y);
    double resy(int i);
    void add_axis_labels(QPainter* p);
    void add_grid(QPainter* p);
    void add_data(QPainter* p);
    void add_residuum(QPainter* p);
    void add_legend(QPainter* p);
    void plot_single_set(int i);
    QColor colorgradient(double x,double xmin,double xmax);
signals:
    void keypressed(QKeyEvent *event);
protected:
    void mouseMoveEvent(QMouseEvent *event);
    void paintEvent(QPaintEvent *);
    void resizeEvent(QResizeEvent *);
    void mousePressEvent(QMouseEvent *event);
    void wheelEvent(QWheelEvent * event );
    void mouseReleaseEvent(QMouseEvent *event);
    void keyPressEvent(QKeyEvent *event);

};

#endif // GRAPH_H
