﻿#define _USE_MATH_DEFINES
#include "Moesscalc.h"
#include "fitmodel.h"


MoessCalc::MoessCalc(){
    init();
    double* M[4];
    double M1[4] = {2,1,0,1}; M[0]=&M1[0];
    double M2[4] = {1,1,1,1}; M[1]=&M2[0];
    double M3[4] = {1,0,1,1}; M[2]=&M3[0];
    double M4[4] = {1,0,1,1}; M[3]=&M4[0];

    V0 = gsl_matrix_complex_alloc(2,2);
    C0 = gsl_matrix_complex_alloc(2,2);
    V1 = gsl_matrix_complex_alloc(4,4);
    C1 = gsl_matrix_complex_alloc(4,4);

    ws1 = gsl_eigen_hermv_alloc(4);
    eval1 = gsl_vector_alloc (4);
    ws0 = gsl_eigen_hermv_alloc(2);
    eval0 = gsl_vector_alloc (2);
}
MoessCalc::MoessCalc(MoessCalc *mc){
    MoessCalc();
    g12 = mc->g12;
    g32 = mc->g32;
    hbar_c_over_Egamma = mc->hbar_c_over_Egamma;
    Q = mc->Q;
    Erec = mc->Erec;

}
MoessCalc::~MoessCalc(){
    gsl_matrix_complex_free(V0);
    gsl_matrix_complex_free(C0);
    gsl_matrix_complex_free(V1);
    gsl_matrix_complex_free(C1);
    gsl_eigen_hermv_free(ws1);
    gsl_eigen_hermv_free(ws0);
    gsl_vector_free(eval1);
    gsl_vector_free(eval0);
}
void MoessCalc::set_nucleus(double g12, double g32, double Eg, double M,
                            double Q){
    this->g12 = g12/0.5;
    this->g32 = g32/1.5;
    hbar_c_over_Egamma = 4.135667662e-15/(2*M_PI)*c*1e-3/Eg;
    this->Q = Q*1e-28/(Eg*1000)*c*1e23;
    Erec = (1.60217656535e-19*Eg*1000/c)*
           (1.60217656535e-19*Eg*1000/c)/2./(M*1.66053892173e-27);
}

void MoessCalc::calc(){
    fill_V();
    /*I=3/2 berechnen*/
    gsl_eigen_hermv(V1,eval1,C1,ws1);
    /*I=1/2 berechnen*/
    gsl_eigen_hermv(V0,eval0,C0,ws0);

    E0[0]=eval0->data[0];
    E0[1]=eval0->data[1];
    E1[0]=eval1->data[0];
    E1[1]=eval1->data[1];
    E1[2]=eval1->data[2];
    E1[3]=eval1->data[3];

    //gsl_sort_vector(eval0);
    //gsl_sort_vector(eval1);

    firstcalc = 0;

    // precalculated coefficients for intensity calculation
    for(int i = 0; i < 2; i++){
        c12m12[i] = gsl_matrix_complex_get(C0,1,i);
        c12p12[i] = gsl_matrix_complex_get(C0,0,i);
    }
    for(int f = 0; f < 4; f++){
        c32m32[f] = gsl_matrix_complex_get(C1,3,f);
        c32m12[f] = gsl_matrix_complex_get(C1,2,f);
        c32p12[f] = gsl_matrix_complex_get(C1,1,f);
        c32p32[f] = gsl_matrix_complex_get(C1,0,f);
    }
    for(int i = 0; i < 2; i++)
        for(int f = 0; f < 4; f++){
            Km[i][f] = gsl_complex_add(
                        gsl_complex_mul_real(
                            gsl_complex_mul(
                                    gsl_complex_conjugate(c12m12[i]),
                                    c32m32[f]),
                            sqrt(3.)),
                        gsl_complex_mul(
                            gsl_complex_conjugate(c12p12[i]),
                            c32m12[f]));
            K0[i][f] = gsl_complex_mul_real(
                         gsl_complex_add(
                            gsl_complex_mul(
                                    gsl_complex_conjugate(c12m12[i]),
                                    c32m12[f]),
                            gsl_complex_mul(
                                    gsl_complex_conjugate(c12p12[i]),
                                    c32p12[f])),
                         sqrt(2.));
            Kp[i][f] = gsl_complex_add(
                        gsl_complex_mul(
                                gsl_complex_conjugate(c12m12[i]),
                                c32p12[f]),
                        gsl_complex_mul_real(
                            gsl_complex_mul(
                                    gsl_complex_conjugate(c12p12[i]),
                                    c32p32[f]),
                            sqrt(3.)));
        }
}
void MoessCalc::fill_V(){
    gsl_matrix_complex_set(V1,0,0,gsl_complex_polar(    3*a-1.5*b*cos(theta) ,   0));
    gsl_matrix_complex_set(V1,0,1,gsl_complex_polar(-sqrt(3.)/2.*b*sin(theta),-phi));
    gsl_matrix_complex_set(V1,0,2,gsl_complex_polar(           eta*a*sqrt(3.) ,   0));
    gsl_matrix_complex_set(V1,0,3,gsl_complex_polar(                       0 ,   0));

    gsl_matrix_complex_set(V1,1,0,gsl_complex_polar(-sqrt(3.)/2.*b*sin(theta),+phi));
    gsl_matrix_complex_set(V1,1,1,gsl_complex_polar(   -3*a-0.5*b*cos(theta) ,   0));
    gsl_matrix_complex_set(V1,1,2,gsl_complex_polar(           -b*sin(theta) ,-phi));
    gsl_matrix_complex_set(V1,1,3,gsl_complex_polar(           eta*a*sqrt(3.) ,   0));

    gsl_matrix_complex_set(V1,2,0,gsl_complex_polar(           eta*a*sqrt(3.),   0));
    gsl_matrix_complex_set(V1,2,1,gsl_complex_polar(           -b*sin(theta) ,+phi));
    gsl_matrix_complex_set(V1,2,2,gsl_complex_polar(   -3*a+0.5*b*cos(theta) ,   0));
    gsl_matrix_complex_set(V1,2,3,gsl_complex_polar(-sqrt(3.)/2.*b*sin(theta),-phi));

    gsl_matrix_complex_set(V1,3,0,gsl_complex_polar(                       0,   0));
    gsl_matrix_complex_set(V1,3,1,gsl_complex_polar(           eta*a*sqrt(3.),   0));
    gsl_matrix_complex_set(V1,3,2,gsl_complex_polar(-sqrt(3.)/2.*b*sin(theta),+phi));
    gsl_matrix_complex_set(V1,3,3,gsl_complex_polar(    3*a+1.5*b*cos(theta),   0));

    gsl_matrix_complex_set(V0,0,0,gsl_complex_polar(      -0.5*b0*cos(theta),   0));
    gsl_matrix_complex_set(V0,0,1,gsl_complex_polar(      -0.5*b0*sin(theta),-phi));
    gsl_matrix_complex_set(V0,1,0,gsl_complex_polar(      -0.5*b0*sin(theta),+phi));
    gsl_matrix_complex_set(V0,1,1,gsl_complex_polar(       0.5*b0*cos(theta),   0));
}
double MoessCalc::f_lorenz(double x, double dx,double x0,double I){
    double q = (x-x0)/dx;
    return I/((1+q*q)*M_PI*dx);
}
double MoessCalc::f_lorenz_fast(double x, double invdx,double x0,double I){
    double q = (x-x0)*invdx;
    return I*invdx/((1+q*q)*M_PI);
}
void MoessCalc::init(){
    g12 = 0.09044/0.5;//Nulear Data Sheets// 0.1806;Lang1970//0.09037/0.5;SchatzWeidinger// 0.09069/0.5;http://www.kayelaby.npl.co.uk/chemistry/3_8/3_8_4.html
    g32 = -0.1549/1.5;//Nulear Data Sheets//-0.1033;Lang1970//-0.1534/1.5;SchatzWeidinger//-0.1549/1.5;http://www.kayelaby.npl.co.uk/chemistry/3_8/3_8_4.html

    B = 0;
    Vzz = 0;
    phi = 0;
    theta = 0;
    eta = 0;
    hbar_c_over_Egamma = 1.36925078923599E-11;
    c = 299792458;
    mun =0.65571891279894;              //(mm/s)/T
    Q = 0.03328412576353570000;         //(mm/s)/(V/A^2), =Qec/Egamma, = 0.160 barn [Physical review letters 75 (1995), Nr. 19, S. 3545]

    Runi = 8.3144621;                   //universal gas constant
    a = 0;
    b = 0;
    b0= 0;
    Erec = 3.12740606464134E-22;        //J, =hbar^2k^2/2/M
    kbolz = 1.380648813E-23;            //J/K, Bolzmann constant

    firstcalc = 1;
}
void MoessCalc::set_parameters(double B, double Vzz, double eta, double theta,
                               double phi){
    this->B = B;
    this->Vzz = Vzz;
    this->theta = theta/180.*M_PI;
    this->phi = phi/180.*M_PI;
    this->eta = eta;
    a = 1./12.*Vzz*Q;                // (mm/s)
    b = g32 * mun*B;                 // (mm/s)
    b0= g12 * mun*B;                 // (mm/s)

}

double MoessCalc::get_dv_Douplett(){
    return a*6*sqrt(1+eta*eta/3.);
}
double MoessCalc::I(double beta, double gamma, int f, int i){
    return 0.5*(gsl_complex_abs2(Kp[i][f])+gsl_complex_abs2(Km[i][f]))*(1+cos(beta)*cos(beta))
         + sin(beta)*sin(beta) *  (gsl_complex_abs2(K0[i][f])+
                                   GSL_REAL(gsl_complex_mul(
                                                gsl_complex_conjugate(Km[i][f]),
                                                gsl_complex_mul(Kp[i][f],
                                                    gsl_complex_polar(1,2*gamma)))))
         - sqrt(0.5)*sin(2*beta)* (GSL_REAL(gsl_complex_mul(
                                                gsl_complex_conjugate(Km[i][f]),
                                                gsl_complex_mul(K0[i][f],
                                                    gsl_complex_polar(1,gamma))))
                                 - GSL_REAL(gsl_complex_mul(
                                                gsl_complex_conjugate(K0[i][f]),
                                                gsl_complex_mul(Kp[i][f],
                                                    gsl_complex_polar(1,gamma)))));
}
void MoessCalc::I(double beta, double gamma, double alpha, int f, int i,
                  double* Int, double* phi, double* xi){
    double qpp =   gsl_complex_abs2(Km[i][f])   *sin(beta/2.)*sin(beta/2.)*sin(beta/2.)*sin(beta/2.)
                  +gsl_complex_abs2(K0[i][f])/2.*sin(beta)*sin(beta)
                  +gsl_complex_abs2(Kp[i][f])   *cos(beta/2.)*cos(beta/2.)*cos(beta/2.)*cos(beta/2.)
                  +GSL_REAL(
                        gsl_complex_add(
                            gsl_complex_mul_real(
                                gsl_complex_mul(
                                    gsl_complex_conjugate(Km[i][f]),
                                    gsl_complex_mul(
                                       K0[i][f],
                                       gsl_complex_polar(1,gamma))),
                                sqrt(2.)*sin(beta/2.)*sin(beta/2.)),
                        gsl_complex_add(
                            gsl_complex_mul_real(
                                gsl_complex_mul(
                                    gsl_complex_conjugate(Km[i][f]),
                                    gsl_complex_mul(
                                       Kp[i][f],
                                       gsl_complex_polar(1,2*gamma))),
                                0.5*sin(beta)),
                            gsl_complex_mul_real(
                                gsl_complex_mul(
                                   gsl_complex_conjugate(K0[i][f]),
                                    gsl_complex_mul(
                                       Kp[i][f],
                                       gsl_complex_polar(1,gamma))),
                               sqrt(2.)*cos(beta/2.)*cos(beta/2.))))
                    )*sin(beta);
    double qmm =   gsl_complex_abs2(Km[i][f])   *cos(beta/2.)*cos(beta/2.)*cos(beta/2.)*cos(beta/2.)
                  +gsl_complex_abs2(K0[i][f])/2.*sin(beta)*sin(beta)
                  +gsl_complex_abs2(Kp[i][f])   *sin(beta/2.)*sin(beta/2.)*sin(beta/2.)*sin(beta/2.)
                  -GSL_REAL(
                        gsl_complex_add(
                            gsl_complex_mul_real(
                                gsl_complex_mul(
                                   gsl_complex_conjugate(Km[i][f]),
                                    gsl_complex_mul(
                                       K0[i][f],
                                       gsl_complex_polar(1,gamma))),
                                sqrt(2.)*cos(beta/2.)*cos(beta/2.)),
                        gsl_complex_add(
                            gsl_complex_mul_real(
                                gsl_complex_mul(
                                    gsl_complex_conjugate(Km[i][f]),
                                    gsl_complex_mul(
                                       Kp[i][f],
                                       gsl_complex_polar(1,2*gamma))),
                                -0.5*sin(beta)),
                            gsl_complex_mul_real(
                                gsl_complex_mul(
                                   gsl_complex_conjugate(K0[i][f]),
                                    gsl_complex_mul(
                                       Kp[i][f],
                                       gsl_complex_polar(1,gamma))),
                                sqrt(2.)*sin(beta/2.)*sin(beta/2.))))
                    )*sin(beta);
    gsl_complex qpm = gsl_complex_mul(
                        gsl_complex_polar(1,2*alpha+M_PI),
                        gsl_complex_add_real(
                            gsl_complex_add(
                                gsl_complex_mul_real(
                                    gsl_complex_mul(
                                        gsl_complex_sub(
                                            gsl_complex_mul(
                                                gsl_complex_conjugate(Km[i][f]),
                                                K0[i][f]),
                                            gsl_complex_mul(
                                                gsl_complex_conjugate(K0[i][f]),
                                                Kp[i][f])),
                                        gsl_complex_polar(1,gamma)),
                                    sin(beta)*cos(beta/2.)*cos(beta/2.)/sqrt(2.)),
                            gsl_complex_add(
                                gsl_complex_mul_real(
                                    gsl_complex_mul(
                                        gsl_complex_sub(
                                            gsl_complex_mul(
                                                gsl_complex_conjugate(Kp[i][f]),
                                                K0[i][f]),
                                            gsl_complex_mul(
                                                gsl_complex_conjugate(K0[i][f]),
                                                Km[i][f])),
                                        gsl_complex_polar(1,-gamma)),
                                    sin(beta)*sin(beta/2.)*sin(beta/2.)/sqrt(2.)),
                            gsl_complex_add(
                                gsl_complex_mul_real(
                                    gsl_complex_mul(
                                        gsl_complex_mul(
                                            gsl_complex_conjugate(Km[i][f]),
                                            Kp[i][f]),
                                        gsl_complex_polar(1,2.*gamma)),
                                    cos(beta/2.)*cos(beta/2.)*cos(beta/2.)*cos(beta/2.)),
                                gsl_complex_mul_real(
                                    gsl_complex_mul(
                                        gsl_complex_mul(
                                            gsl_complex_conjugate(Kp[i][f]),
                                            Km[i][f]),
                                        gsl_complex_polar(1,-2.*gamma)),
                                    sin(beta/2.)*sin(beta/2.)*sin(beta/2.)*sin(beta/2.))
                            ))),
                                (  gsl_complex_abs2(Km[i][f])
                                -2*gsl_complex_abs2(K0[i][f])
                                +  gsl_complex_abs2(Kp[i][f]))*sin(beta)*sin(beta)/4.)
                      );
    qpp = qMax(0.,qpp);
    qmm = qMax(0.,qmm);
    *Int = qpp + qmm;
    *phi = atan2(-GSL_IMAG(qpm),GSL_REAL(qpm))/2./M_PI*180;
    if(qpp != 0 || qmm != 0)
        *xi = atan((sqrt(qpp)-sqrt(qmm))/(sqrt(qpp)+sqrt(qmm)))/M_PI*180;
    else
        *xi = 0;
}
double MoessCalc::I(double ta){
    return ta*exp(-ta/2)*(gsl_sf_bessel_I0(ta/2.)+gsl_sf_bessel_I1(ta/2.));
}

double MoessCalc::get_Egamma(int f, int i){
    return E1[f]-E0[i];
}
double MoessCalc::I(int f, int i){
    return 2./3.*(gsl_complex_abs2(Km[i][f])+gsl_complex_abs2(K0[i][f])+gsl_complex_abs2(Kp[i][f]));
}
double MoessCalc::Idiag(double beta, int f, int i){
    switch(i*4+f){
        case 0: return 3*0.75*(1+cos(beta)*cos(beta));break;
        case 1: return 2*0.75*2*sin(beta)*sin(beta);break;
        case 2: return 1*0.75*(1+cos(beta)*cos(beta));break;
        case 3: return 0;break;
        case 4: return 0;break;
        case 5: return 1*0.75*(1+cos(beta)*cos(beta));break;
        case 6: return 2*0.75*2*sin(beta)*sin(beta);break;
        case 7: return 3*0.75*(1+cos(beta)*cos(beta));break;
        default: return 0;break;
    }
}

void MoessCalc::add_noise(double *I, int cc){
    const gsl_rng_type * T;
    gsl_rng * r;
    gsl_rng_env_setup();
    T = gsl_rng_default;
    r = gsl_rng_alloc (T);
    for (int i = 0; i < cc; i++){
        double err = gsl_cdf_gaussian_Pinv(gsl_rng_uniform(r),sqrt(I[i]));
        I[i] += err;
    }
    gsl_rng_free (r);
}

double LC_f(const gsl_vector * x, void * params ){
    //prepare
    LeCearContainer* p  = (LeCearContainer*)params;
    double result = 0;
    gsl_vector *x2 = gsl_vector_alloc (p->steps);
    for(int i = 0; i < p->steps;i++)
        gsl_vector_set(x2,i,gsl_vector_get(x,i)*gsl_vector_get(x,i));
    gsl_vector *tempc = gsl_vector_alloc (p->count);
    gsl_vector *temps = gsl_vector_alloc (p->steps);
    gsl_vector_memcpy(tempc,p->S);
    // calc
    gsl_blas_dgemv(CblasNoTrans,1,p->M,x2,-1,tempc);
    result += gsl_blas_dnrm2(tempc);
    gsl_blas_dgemv(CblasNoTrans,p->lambda,p->D0,x2,0,temps);
    result += gsl_blas_dnrm2(temps);
    // free
    gsl_vector_free(x2);
    gsl_vector_free(tempc);
    gsl_vector_free(temps);
    return result;
}
void LC_df(const gsl_vector * x, void * params, gsl_vector * g ){
    // prepare
    LeCearContainer* p  = (LeCearContainer*)params;
    gsl_vector *x2 = gsl_vector_alloc (p->steps);
    for(int i = 0; i < p->steps;i++)
        gsl_vector_set(x2,i,gsl_vector_get(x,i)*gsl_vector_get(x,i));
    // calc
    gsl_blas_dgemv(CblasNoTrans,p->lambda,p->D1,  x2,0,g);
    gsl_blas_dgemv(CblasNoTrans,        1,p->A ,  x2,1,g);
    gsl_blas_dgemv(CblasTrans,         -1,p->M ,p->S,1,g);
    // free
    gsl_vector_free(x2);
}
void LC_fdf(const gsl_vector * x, void * params,double * f, gsl_vector *g){
    *f = LC_f(x,params);
    LC_df(x,params,g);
}
void MoessCalc::LeCear(double**M0, double* rho, double *y,
                                int steps, int cc, double smooth){
    gsl_matrix *M = gsl_matrix_alloc(cc,steps);
    // GSL-Matrix M initialisieren mit den vorberechneten Subspektren
    for(int b = 0 ; b < steps; b++)
        for(int i = 0; i < cc; i++)
            gsl_matrix_set(M,i,b,M0[i][b]);
    // set A = M^T.M
    gsl_matrix *A = gsl_matrix_alloc(steps,steps);
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,M,M,0,A);

    // initialize D0
    double fac[9] = {-0.0833333333,1.3333333333,-2.5000000000,1.3333333333,-0.0833333333,0,0,0,0};
    gsl_matrix *D0 = gsl_matrix_alloc(steps,steps);
    gsl_matrix_set_zero(D0);
    for(int i = 0; i < steps; i++)
        for(int ii=i-2; ii < i+3; ii++)
            if(ii < 0)
                    gsl_matrix_set(D0,i,steps+ii,fac[ii-i+2]);
            else
                if(ii>=steps)
                    gsl_matrix_set(D0,i,ii-steps,fac[ii-i+2]);
                else
                    gsl_matrix_set(D0,i,      ii,fac[ii-i+2]);
    // initialize D1
    double fac2[9]= {0.16666667,-5.33333333,52.66666667,-165.33333333,
                  235.66666667,-165.33333333,52.66666667,-5.33333333,
                  0.16666667};
    gsl_matrix *D1 = gsl_matrix_alloc(steps,steps);
    gsl_matrix_set_zero(D1);
    for(int i = 0; i < steps; i++)
        for(int ii=i-4; ii < i+5; ii++)
            if(ii < 0)
                    gsl_matrix_set(D1,i,steps+ii,fac2[ii-i+4]);
            else
                if(ii>=steps)
                    gsl_matrix_set(D1,i,ii-steps,fac2[ii-i+4]);
                else
                    gsl_matrix_set(D1,i,      ii,fac2[ii-i+4]);

    // initialize spectrum
    gsl_vector * S = gsl_vector_alloc(cc);
    for(int i = 0; i < cc; i++)
        gsl_vector_set(S,i,y[i]);
    /*FILE* f = fopen("C:\\Users\\test\\Desktop\\MoessfitTestcase\\matrixcheck.txt","w");
    for(int j = 0; j < cc; j++){
        for(int i = 0; i < steps; i++)
            fprintf(f,"%f\t",gsl_matrix_get(M,j,i));
        fprintf(f,"\n");
    }
    fprintf(f,"\n");
    for(int j = 0; j < steps; j++){
        for(int i = 0; i < steps; i++)
            fprintf(f,"%f\t",gsl_matrix_get(A,j,i));
        fprintf(f,"\n");
    }
    fprintf(f,"\n");
    for(int j = 0; j < cc; j++){
        fprintf(f,"%f\t",gsl_vector_get(S,j));
        fprintf(f,"\n");
    }
    fprintf(f,"\n");
    fclose(f);*/
    // execute minimization
    LeCearContainer par;
    par.M = M;
    par.D0 = D0;
    par.D1 = D1;
    par.A = A;
    par.S = S;
    par.steps = steps;
    par.lambda = smooth;
    par.count = cc;
    gsl_multimin_function_fdf my_func;
    my_func.n = steps;
    my_func.f = LC_f;
    my_func.df = LC_df;
    my_func.fdf = LC_fdf;
    my_func.params = (void*)&par;
    //
    gsl_vector *x = gsl_vector_alloc(steps);
    gsl_vector_set_all(x,sqrt(1./steps));
    const gsl_multimin_fdfminimizer_type *T = gsl_multimin_fdfminimizer_conjugate_fr;
    gsl_multimin_fdfminimizer *s = gsl_multimin_fdfminimizer_alloc (T,steps);
    gsl_multimin_fdfminimizer_set (s, &my_func, x, 1, 0.1);
    int iter = 0, status;
    do{
        iter++;
        status = gsl_multimin_fdfminimizer_iterate (s);
        if (status)
        break;
        status = gsl_multimin_test_gradient (s->gradient, 1e-3);
    }while (status == GSL_CONTINUE && iter < 100);

    // read the result
    for(int k = 0; k < steps; k++)
        rho[k] = gsl_vector_get(s->x,k)*gsl_vector_get(s->x,k);

    // free
    gsl_multimin_fdfminimizer_free (s);
    gsl_vector_free (x);
    gsl_matrix_free(M);
    gsl_matrix_free(D0);
    gsl_matrix_free(D1);
    gsl_matrix_free(A);
    gsl_vector_free(S);
}
void MoessCalc::HesseRuebartsch(double**M0, double* rho, double *y,
                                int steps, int cc, double smooth){
    gsl_matrix *M = gsl_matrix_alloc(cc,steps);
    // GSL-Matrix M initialisieren mit den vorberechneten Subspektren
    for(int b = 0 ; b < steps; b++)
        for(int i = 0; i < cc; i++)
            gsl_matrix_set(M,i,b,M0[i][b]);

    gsl_matrix *Mtemp = gsl_matrix_alloc(steps,cc);
    gsl_matrix *D = gsl_matrix_alloc(steps,steps);
    int status;

    double* Ia = (double*)malloc(cc*sizeof(double));

    /*FILE* f = fopen("C:\\Users\\test\\Desktop\\MoessfitTestcase\\matrixcheck.txt","w");
    for(int j = 0; j < cc; j++){
        for(int i = 0; i < steps; i++)
            fprintf(f,"%f\t",gsl_matrix_get(M,j,i));
        fprintf(f,"\n");
    }
    fprintf(f,"\n");*/

    // initialize D with
    double fac[9] = {0.16666667,-5.33333333,52.66666667,-165.33333333,
                     235.66666667,-165.33333333,52.66666667,-5.33333333,
                     0.16666667};

    gsl_matrix_set_zero(D);
    for(int i = 0; i < steps; i++)
        for(int ii=i-4; ii < i+5; ii++)
            if(ii < 0)
                    gsl_matrix_set(D,i,steps+ii,fac[ii-i+4]);
            else
                if(ii>=steps)
                    gsl_matrix_set(D,i,ii-steps,fac[ii-i+4]);
                else
                    gsl_matrix_set(D,i,      ii,fac[ii-i+4]);
    /*for(int i = 2; i < steps - 2; i++){
        gsl_matrix_set(D,i,i-2,1);
        gsl_matrix_set(D,i,i-1,-4);
        gsl_matrix_set(D,i,i-0,6);
        gsl_matrix_set(D,i,i+1,-4);
        gsl_matrix_set(D,i,i+2,1);
    }
    gsl_matrix_set(D,0,0,1); gsl_matrix_set(D,0,1,-2);gsl_matrix_set(D,0,2,1);
    gsl_matrix_set(D,steps-1,steps-1,1); gsl_matrix_set(D,steps-1,steps-2,-2);gsl_matrix_set(D,steps-1,steps-3,1);
    gsl_matrix_set(D,1,0,-2);gsl_matrix_set(D,1,1,5); gsl_matrix_set(D,1,2,-4);gsl_matrix_set(D,1,3,1);
    gsl_matrix_set(D,steps-2,steps-1,-2);gsl_matrix_set(D,steps-2,steps-2,5); gsl_matrix_set(D,steps-2,steps-3,-4);gsl_matrix_set(D,steps-2,steps-4,1);
    */
    /*for(int j = 0; j < steps; j++){
        for(int i = 0; i < steps; i++)
            fprintf(f,"%f\t",gsl_matrix_get(D,j,i));
        fprintf(f,"\n");
    }
    fclose(f);*/
    // initialize spectrum
    gsl_vector * S = gsl_vector_alloc(cc);
    for(int i = 0; i < cc; i++)
        gsl_vector_set(S,i,y[i]);
    gsl_vector * result = gsl_vector_alloc(steps);

    // no calculate
    // calculate bracket to be inverted
    status = gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,M,M,smooth,D);
    // invert
    gsl_matrix *inverse = gsl_matrix_alloc(steps, steps);
    gsl_permutation*perm = gsl_permutation_alloc(steps);
    int s=0;
    status = gsl_linalg_LU_decomp(D, perm, &s);
    status = gsl_linalg_LU_invert(D, perm, inverse);

    // apply inverted matrix
    status = gsl_blas_dgemm(CblasNoTrans,CblasTrans,1,inverse,M,0,Mtemp);
    /*for(int j = 0; j < cc; j++){
        for(int i = 0; i < steps; i++)
            fprintf(f,"%f\t",gsl_matrix_get(Mtemp,i,j));
        fprintf(f,"\n");
    }
    fprintf(f,"\n");*/
    status = gsl_blas_dgemv(CblasNoTrans,1,Mtemp,S,0,result);

    // Ergebnis auslesen
    for(int i = 0; i < steps; i++)
        rho[i] = gsl_vector_get(result,i);

    gsl_matrix_free(M);
    gsl_matrix_free(Mtemp);
    gsl_matrix_free(D);
    gsl_matrix_free(inverse);
    gsl_permutation_free(perm);
    free(Ia);

}
double MoessCalc::baseline(double* x,double* y,int count,
                         double* Baseline,double* Face,
                         double bperx, double omega,
                         double** rho,double** rhox, int *rhosteps, double* INew){
    // ALTERNATIVER ANSATZ UEBER INVERTIERUNG
    if(count > 0 && x[count-1]-x[0] > 0){
        // search or suitable data intervals
        double* xi = (double*)malloc(int((max_array(x,count)-min_array(x,count))
                                         *2*bperx)*sizeof(double));
        double I0max = (mean(y,0,25)+mean(y,count-26,count-1))/2.;
        double dImax = 2*sqrt(I0max);
        int CSsteps = 0;
        int istart = -1, iend = -1;
        for(int i = 12; i < count-13; i++){
            if(abs(mean(y,i-12,i+12) - I0max) > dImax && i<count-12){
                // start a new data interval
                if(istart == -1)
                    istart = i;
                    iend = -1;          // intervall has started
            }
            else
                // finish data intervall
                if(iend == -1){
                    iend = i;
                    int newsteps = int((x[iend] - x[istart])*bperx)+1;
                    for(int j = 0; j < newsteps; j++){
                        xi[CSsteps] = x[istart] + (x[iend] - x[istart])*j/qMax(1,newsteps);
                        CSsteps++;
                    }
                    istart = -1;        // waiting for new intervall state
                    iend = 0;           // intervall has to started
                }
        }
        if(CSsteps > 0 && CSsteps < 200){
            double **M = (double**)malloc(count*sizeof(double*));

            steps = CSsteps + 1;
            double* rho2 = (double*)malloc(steps*sizeof(double));

            for(int i = 0; i < count; i++){
                M[i] = (double*) malloc(steps*sizeof(double));
            }
            for(int b = 0; b < CSsteps; b++){
                for(int i = 0; i < count; i++)
                    M[i][b] = -f_lorenz(x[i],omega,xi[b],M_PI*omega);
            }
            for(int i = 0; i < count; i++)
                M[i][CSsteps] = 1.;
            SimpleInvert(x,y,count,steps,M,rho2);

            *Baseline = rho2[CSsteps];
            *Face = 0;
            for(int b = 0; b < CSsteps; b++)
                *Face = *Face + rho2[b];
            *Face = *Face * M_PI * omega;

            double Ochi2 = 0;
            if(INew != NULL){
                for(int i = 0; i < count; i++){
                    INew[i] = rho2[CSsteps];
                    for(int b = 0; b < CSsteps; b++)
                       INew[i] = INew[i]+ M[i][b]*rho2[b];
                    Ochi2 = Ochi2 + (INew[i]-y[i])*(INew[i]-y[i])/rho2[CSsteps];
                }
                Ochi2 = Ochi2 / (count - 1);
            }
            if(rhosteps!= NULL)
                *rhosteps = CSsteps;
            if(rho != NULL){
                *rho = rho2;
                *rhox = xi;
            }
            else{
                free(rho2);
                free(xi);
            }

            for(int i = 0; i < count; i++)
                free(M[i]);
            free(M);
            // check if reasonable values were achieved and possibly executte
            // a more simple estimation
            if(*Baseline == 0 && *Face == 0)
                baseline(x,y,count,Baseline,Face);
            return Ochi2;
        }
        else{
            // naiv estimation has to be applied
            if(rho != NULL) *rho = NULL;
            if(rhox != NULL) *rhox = NULL;
            if(rhosteps!= NULL) *rhosteps = CSsteps;
            free(xi);
            baseline(x,y,count,Baseline,Face);
            return 0;
        }
    }
    else{
        // no valid data set present
        *Face = 0;
        *Baseline = 0;
        if(rho != NULL) *rho = NULL;
        if(rhox != NULL) *rhox = NULL;
        if(rhosteps != NULL) *rhosteps = 0;
        return 0;
    }
}
void MoessCalc::baseline(double* x,double* y,int count,
                         double* Baseline,double* Face){

    *Baseline = (mean(y,0,qMin(10,count-1))
                +mean(y,qMax(0,count-10),count-1))/2.;
    *Face = 0;
    // integrate phase under estimated baseline
    for(int i = 1; i < count-1; i++){
        *Face += (*Baseline-y[i])*(x[i+1]-x[i-1])*0.5;
    }
}

void MoessCalc::MEM_init(double**M0, double *y){
    P 	= gsl_vector_alloc(steps);
    vtemp 	= gsl_vector_alloc(steps);
    dchi2 	= gsl_vector_alloc(steps);
    dS 		= gsl_vector_alloc(steps);
    d2chi2 	= gsl_matrix_alloc(steps,steps);
    d2S 	= gsl_matrix_alloc(steps,steps);
    M   = gsl_matrix_alloc(count,steps);
    g 	= gsl_matrix_alloc(steps,steps);
    gsl_matrix_set_all(g,0);
    yS	= gsl_vector_alloc(count);
    chi	= gsl_vector_alloc(count);
    gsl_matrix_set_all(d2S,0);
    e[0] = gsl_vector_alloc(steps);
    e[1] = gsl_vector_alloc(steps);
    e[2] = gsl_vector_alloc(steps);
    e[3] = gsl_vector_alloc(steps);
    eON[0] = gsl_vector_alloc(steps);
    eON[1] = gsl_vector_alloc(steps);
    eON[2] = gsl_vector_alloc(steps);
    eON[3] = gsl_vector_alloc(steps);
    ssEW = gsl_vector_alloc(4);
    ssEV = gsl_matrix_alloc(4,4);

    /*GSL-Matrix M initialisieren mit den vorberechneten Subspektren*/
    for(int b = 0 ; b < steps; b++)
        for(int i = 0; i < count; i++)
            gsl_matrix_set(M,i,b,M0[i][b]);
    /*Messspektrum-Vector initialisieren*/
    for(int i = 0; i < count; i++)
        gsl_vector_set(yS,i,y[i]);
    /*Startverteilung initialisieren*/
    gsl_vector_set_all(P,1./steps);
    /*konstante chi2-Hessematrix berechnen*/
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,2./I0/count,M,M,0,d2chi2);
}
void MoessCalc::MEM_free(){
    gsl_vector_free(P      );
    gsl_vector_free(dchi2 	);
    gsl_vector_free(dS 	);
    gsl_matrix_free(d2chi2 );
    gsl_matrix_free(d2S 	);
    gsl_matrix_free(M      );
    gsl_matrix_free(g      );
    gsl_vector_free(yS    );
    gsl_vector_free(chi   );
    gsl_vector_free(e[0]  );
    gsl_vector_free(e[1]  );
    gsl_vector_free(e[2]  );
    gsl_vector_free(e[3]  );
    gsl_vector_free(eON[0]);
    gsl_vector_free(eON[1]);
    gsl_vector_free(eON[2]);
    gsl_vector_free(eON[3]);
    gsl_vector_free(ssEW  );
    gsl_matrix_free(ssEV  );
}
void MoessCalc::MEM_calc_derivatives(){
    /*Gradient der Entropie*/
    for(int b = 0; b < steps; b++)
        gsl_vector_set(dS,b,-log(gsl_vector_get(P,b)*steps));
    /*Hessematrix der Entropie*/
    for(int b = 0; b < steps; b++)
        gsl_matrix_set(d2S,b,b,-1./gsl_vector_get(P,b));
    /*aktuelle Abweichungen chi[i] in Kanaelen i des Fits vom Messspektrum*/
    gsl_vector_memcpy(chi,yS);
    gsl_blas_dgemv(CblasNoTrans,1,M,P,-1,chi);
    /*Gradient des chi2*/
    gsl_blas_dgemv(CblasTrans,1./I0/count,M,chi,0,dchi2);
    /*Hessematrix des Chi2 ist konstant*/
    /*ggf. den fixierten Parameter 0 setzen*/
}
void MoessCalc::MEM_calc_diff(){
    gsl_vector_set_all(vtemp,0);
    double norm1 = gsl_blas_dnrm2(dchi2);
    double norm2 = gsl_blas_dnrm2(dS);
    if(norm1 != 0 && norm2 != 0){
        gsl_blas_daxpy(-1./norm1,dchi2,vtemp);
        gsl_blas_daxpy(1./norm2,dS,vtemp);
        cdif = 0.5*gsl_blas_dnrm2(vtemp);
        gsl_blas_ddot(dS,dchi2,&cangle);//Skalarprodukt
        cangle = cangle / norm1/ norm2; //normieren
    }
    else{
        cdif = 0;
        if(norm1 == 0)
            cangle = -1;
        else
            cangle = 0;
    }
}

void MoessCalc::MEM_Cambridge(double**M0, double* rho, double* rhoerr, double *y,
                              double I0, int steps, int cc, double stepsize,
                              int MaxIterations, double MaxLambda,
                              double MaxDif, double status[5]){
    this->I0 = I0;
    this->steps = steps;
    this->count = cc;
    this->Pstepmax = stepsize;
    MEM_init(M0,y);
    double chi2min = 1e300, chi2;
    int iterations = 0;
    double lambda = 0;
    while(iterations  < MaxIterations   //Maximale Iterationsanzahl
          && lambda < MaxLambda){       //Minimale Glaettung
        iterations++;
        MEM_calc_derivatives();         //dS, d2S, chi, d2chi
        MEM_calc_diff();                //cangle, cdif
        lambda = MEM_Cambridge_lamda();
        chi2 = MEM_Cambridge_chi2();
        if(   chi2min > chi2            //neues Chi2-Minimum
           && cdif <= MaxDif            //mit Richtungstolleranz
           && lambda < MaxLambda){      //und ausreichend Glaettung
            /*das Zwischenergebnis wird als echtes MaxEnt-Ergebnis an-
            genommen*/
            chi2min = chi2;
            for(int b = 0; b < steps; b++)
                rho[b] = gsl_vector_get(this->P,b);
            status[0] = iterations + 1;
            status[1] = chi2;
            status[2] = cangle;
            status[3] = cdif;
            status[4] = lambda;
        }
        if(cangle >= 0){
            if(lambda < MaxLambda){
                MEM_Cambridge_create_subspace();
                MEM_Cambridge_calc_step();
                MEM_Cambridge_free_subspace();
            }
        }
        else
            iterations = MaxIterations;
    }

    if(iterations  < MaxIterations)
        MEM_Cambridge_tune_lambda(rho,MaxLambda,status);
    MEM_error_analysis(rho,rhoerr,status[4]);
    MEM_free();
}
double MoessCalc::MEM_Cambridge_lamda(){
    double dchi2nrm = gsl_blas_dnrm2(dchi2);
    double dSnrm = gsl_blas_dnrm2(dS);
    return dSnrm / (dSnrm+dchi2nrm);
}
double MoessCalc::MEM_Cambridge_chi2(){
    double chi2, nrm = gsl_blas_dasum(P);
    gsl_blas_ddot(chi,chi,&chi2);
    return chi2 / (count-1) / (I0*nrm);
}
void MoessCalc::MEM_Cambridge_tune_lambda(double* rho, double MaxLambda,
                                          double status[5]){
    /*try to match MaxLambda exactly after MEM_Cambridge has finished*/
    double lambda = MEM_Cambridge_lamda();
    double lambda0 = lambda;
    double* rho0 = new double[steps];
    for(int b = 0; b < steps; b++)
        rho0[b] = gsl_vector_get(P,b);

    for(int i = 0 ; i < 5; i++){
        /*estimate next step*/
        for(int b = 0; b < steps; b++)
             gsl_vector_set(P,b,(rho[b]*(lambda0-MaxLambda)
                                 +rho0[b]*(MaxLambda-status[4]))
                                /(lambda0-status[4]));
        MEM_calc_derivatives();
        lambda = MEM_Cambridge_lamda();
        /*evaluate new step*/
        if(lambda > MaxLambda){
            /*step still to big, try to reduce*/
            for(int b = 0; b < steps; b++)
                rho0[b] = gsl_vector_get(P,b);
            lambda0 = lambda;
        }
        else{
            /*succesfully tuned*/
            for(int b = 0; b < steps; b++)
                rho[b] = gsl_vector_get(P,b);
            MEM_calc_diff();
            status[1] = MEM_Cambridge_chi2();
            status[2] = cangle;
            status[3] = cdif;
            status[4] = lambda;
        }
    }
    /*check whether rho0 was the better solution*/
    if(status[4] != lambda && lambda0 - MaxLambda < MaxLambda - status[4]){
        for(int b = 0; b < steps; b++)
            rho[b] = rho0[b];
        MEM_calc_diff();
        status[1] = MEM_Cambridge_chi2();
        status[2] = cangle;
        status[3] = cdif;
        status[4] = lambda;
    }
}
void MoessCalc::MEM_Cambridge_create_subspace(){
    /*erzeuge den Unterraum aus 4 Richtungen, naemlich dS, dchi2,(d2chi2)dS
    und (d2chi2)dchi2*/
    /*Metrischen Tensor aktualisieren*/
    for(int b = 0; b < steps; b++)
        gsl_matrix_set(g,b,b,gsl_vector_get(P,b));

    /*Richtungen Berechnen*/
    gsl_blas_dgemv(CblasNoTrans,1,g,dchi2,0,e[0]);
    gsl_blas_dgemv(CblasNoTrans,1,g,dS,0,e[1]);
    gsl_blas_dgemv(CblasNoTrans,1,d2chi2,e[0],0,vtemp);
    gsl_blas_dgemv(CblasNoTrans,1,g,vtemp,0.,e[2]);
    gsl_blas_dgemv(CblasNoTrans,1,d2chi2,e[1],0,vtemp);
    gsl_blas_dgemv(CblasNoTrans,1,g,vtemp,0.,e[3]);

    /*Richtungen normalisieren*/
    for(int i = 0; i < 4; i++){
        double nrm = gsl_blas_dnrm2(e[i]);
        if(nrm > 0)
            gsl_blas_daxpy(1./nrm-1,e[i],e[i]);
    }

    /*Diagonalisieren des metrischen Tensor*/
    double vp;                          //Vektorprodukt
    ssg = gsl_matrix_alloc(4,4);
    for(int b1 = 0; b1 < 4; b1++)
        for(int b2 = 0; b2 < 4; b2++){
            gsl_blas_ddot(e[b1],e[b2],&vp);
            gsl_matrix_set(ssg,b1,b2,vp);
    }

    gsl_eigen_symmv_workspace * w1 = gsl_eigen_symmv_alloc (4);
    gsl_eigen_symmv (ssg, ssEW, ssEV, w1);
    gsl_eigen_symmv_free(w1);

    /*Neue Basisvektoren eON[] zusammensetzen*/
    for(int b = 0; b < 4; b++){
        gsl_vector_set_zero(eON[b]);
        for(int i = 0; i < 4; i++)
            gsl_blas_daxpy(gsl_matrix_get(ssEV,i,b),e[i],eON[b]);
    }

    /*M-Matrix initialisieren*/
    int t[4];                           //Zuordnungstabelle reduzierter Dimens.
    double EWlimit = 1e-5;
    ssEWc = 0;
    for(int i = 0; i < 4; i++)
        if(gsl_vector_get(ssEW,i)*gsl_vector_get(ssEW,i) > EWlimit*EWlimit){
            t[ssEWc] = i;
            ssEWc++;
        }
    ssM = gsl_matrix_alloc(ssEWc,ssEWc);
    for(int b1 = 0; b1 < ssEWc; b1++){
        gsl_blas_dgemv(CblasNoTrans,1,d2chi2,eON[t[b1]],0,vtemp);
        for(int b2 = 0; b2 < ssEWc; b2++){
            gsl_blas_ddot(vtemp,eON[t[b2]],&vp);
            gsl_matrix_set(ssM,b1,b2,vp);
        }
    }

    /*M diagonalisieren*/
    MEV = gsl_matrix_alloc(ssEWc,ssEWc);
    MEW = gsl_vector_alloc(ssEWc);
    gsl_eigen_symmv_workspace * w2 = gsl_eigen_symmv_alloc (ssEWc);
    gsl_eigen_symmv (ssM, MEW, MEV, w2);
    gsl_eigen_symmv_free(w2);


    /*und die finalen Basisvektoren e[] zusammensetzen*/
    for(int b = 0; b < ssEWc; b++){
        gsl_vector_set_zero(e[b]);
        for(int i = 0; i < ssEWc; i++)
            gsl_blas_daxpy(gsl_matrix_get(MEV,i,b),eON[i],e[b]);
    }
}
void MoessCalc::MEM_Cambridge_free_subspace(){
    gsl_matrix_free(MEV);
    gsl_matrix_free(ssM);
    gsl_matrix_free(ssg);
    gsl_vector_free(MEW);
}
void MoessCalc::MEM_Cambridge_calc_step(){
    /*im eigens erzeugten Unterraum wird die Optimierungsloesung des aktuellen
        Iterationsschritt mit Hilfe eines quadratischen Modells gesucht*/
        double vp;                          //Vektorprodukt
        /*chi2-Gradienten projezieren*/
        gsl_vector *dC = gsl_vector_alloc(ssEWc);
        for(int i = 0; i < ssEWc; i++){
            gsl_blas_ddot(dchi2,e[i],&vp);
            gsl_vector_set(dC,i,vp);
        }
        /*S-gradienten projezieren*/
        gsl_vector *dS = gsl_vector_alloc(ssEWc);
        for(int i = 0; i < ssEWc; i++){
            gsl_blas_ddot(this->dS,e[i],&vp);
            gsl_vector_set(dS,i,vp);
        }

        gsl_vector *Pstep = gsl_vector_alloc(ssEWc);
        double alpha, beta = 0, steplength, chi2, chi2beta,
               chi20, chi2aim;

        //Ziel-chi2aim ausrechnen
        gsl_blas_ddot(chi,chi,&chi20);
        chi20 = chi20 / count / I0;
        chi2aim = chi20;
        for(int i = 0; i < ssEWc; i++)
            chi2aim = chi2aim - 1./3.*gsl_vector_get(dC,i)
                                     *gsl_vector_get(dC,i)
                                     /gsl_vector_get(MEW,i);

        int alphafinished = 0, betafinished = 0;
        alphamode mode;
        do{ /*das kleinstmoegliche beta wird gesucht, sodass die Maximal-
            schrittweite eingehalten wird. Dazu muss der lagrangeparameter beta
            minimiert werden, ggf. ist der Startwert 0 ausreichend*/
            alpha = 1;
            alphafinished = 0;
            mode = initialize;
            do{ /*Entropie und chi2 werden so gewichtet, dass die Maximalschritt-
                weite eingehalten wird und chi2 minimal wird, d.h. alpha ist so
                klein wie möglich*/

                //Pstep berechnen
                gsl_vector_set_zero(Pstep);
                gsl_blas_daxpy(-1,dC,Pstep);
                gsl_blas_daxpy(alpha,dS,Pstep);
                for(int i = 0; i < ssEWc; i++)
                    gsl_vector_set(Pstep,i,gsl_vector_get(Pstep,i)
                                          /(beta+alpha+gsl_vector_get(MEW,i)));
                //Schrittweite Berechnen
                steplength = gsl_blas_dnrm2(Pstep);

                //chi2 berechnen
                gsl_blas_ddot(dC,Pstep,&chi2);
                chi2 = chi2 + chi20;
                for(int i = 0; i < ssEWc; i++)
                    chi2 = chi2 + 0.5*gsl_vector_get(Pstep,i)
                                     *gsl_vector_get(Pstep,i)
                                     *gsl_vector_get(MEW,i);
                //chi2beta berechnen
                gsl_blas_ddot(dC,Pstep,&chi2beta);
                chi2beta = chi2beta + chi20;
                for(int i = 0; i < ssEWc; i++)
                    chi2beta = chi2beta + 0.5*gsl_vector_get(Pstep,i)
                                             *gsl_vector_get(Pstep,i)
                                             *(beta+gsl_vector_get(MEW,i));

                //Erstschritt ggf. ausfuehren
                if(mode == initialize){
                    if(chi2 < chi2aim)                mode = exceed_aim;
                    else
                        if(chi2 <= chi20)
                            if(steplength > Pstepmax) mode = increase_out;
                            else                      mode = decrease_in;
                        else
                            if(steplength > Pstepmax) mode = decrease_out;
                            else                      mode = decrease_in;
                }

                /*je nach alpha-Modus abbrechen oder alpha veraendern*/
                 switch(mode){
                    case decrease_in:
                        if(chi2 < chi2aim ||//erfolgreicher Durchlauf
                           steplength > Pstepmax)
                            alphafinished = 1;
                        else
                            alpha = alpha * 0.95;
                        break;
                    case decrease_out:
                        if(steplength <= Pstepmax)
                            mode = decrease_in;
                        else
                            alpha = alpha * 0.9;
                        if(chi2 < chi2aim)  //vorbeigelaufen
                            alphafinished = 1;
                        break;
                    case increase_out:
                        if(steplength <= Pstepmax)
                            mode = decrease_in;
                        else
                            alpha = alpha*1.1;
                        if(chi2 >= chi20)   //vorbeigelaufen
                            alphafinished = 1;
                        break;
                    case exceed_aim:
                        if(chi2 >= chi2aim)
                            mode = increase_out;
                        else
                            alpha = alpha * 1.1;
                        break;
                    case increase_in:
                        break;
                    case initialize:
                        break;

                }
                    if(alpha < 1e-50){
                    alphafinished = 1;
                    betafinished = 1;
                }
            }while(!alphafinished);

            if(mode == decrease_in)
                betafinished = 1;
            else{
                if(beta == 0)
                    beta = 0.1;
                else
                    beta = beta*1.1;
            }
        }while(!betafinished);

        /*berechne nochmal mit akuellem alpha und beta P*/
        gsl_vector_set_zero(Pstep);
        gsl_blas_daxpy(-1,dC,Pstep);
        gsl_blas_daxpy(alpha,dS,Pstep);
        for(int i = 0; i < ssEWc; i++)
            gsl_vector_set(Pstep,i,gsl_vector_get(Pstep,i)
                                  /(beta+alpha+gsl_vector_get(MEW,i)));

        /*Rücktransformieren*/
        for(int i = 0; i < ssEWc; i++)
            gsl_blas_daxpy(gsl_vector_get(Pstep,i),e[i],this->P);

        /*Sicherung gegen negative Streuwerte*/
        for(int b = 0; b < steps; b++)
            if(gsl_vector_get(this->P,b)<= 0)
                gsl_vector_set(this->P,b,1e-20);

        gsl_vector_free(dC);
        gsl_vector_free(dS);
        gsl_vector_free(Pstep);
}
void MoessCalc::MEM_error_analysis(double* rho, double *rhoerror, double lambda){
    /*jetzt die Hessematrix des Lagranians aufstellen*/
    for(int b1 = 0; b1 < steps; b1++)
        for(int b2 = 0; b2 < steps; b2++)
            gsl_matrix_set(d2chi2,b1,b2,gsl_matrix_get(d2chi2,b1,b2)*count
                                         *lambda);
    for(int b = 0; b < steps; b++)
        gsl_matrix_set(d2chi2,b,b,(gsl_matrix_get(d2chi2,b,b)/count
                                  +(1-lambda)/rho[b])*count);
    /*Die Hessematrix invertieren um die Kovarianzmatrix zu erhalten*/
    gsl_matrix *covariance = gsl_matrix_alloc(steps, steps);
    gsl_permutation*perm = gsl_permutation_alloc(steps);
    int s=0;
    gsl_linalg_LU_decomp(d2chi2, perm, &s);
    gsl_linalg_LU_invert(d2chi2, perm, covariance);

    /*Diagonalelemente der Kovarianmatrix ablesen*/
    for(int b = 0; b < steps; b++)
        rhoerror[b] = 1.96*sqrt(gsl_matrix_get(covariance,b,b));

    /*Hilfsdaten löschen*/
    gsl_matrix_free(covariance);
    gsl_permutation_free(perm);
}

double MoessCalc::Vzz_mms(double Vzz){
    return 1./2.*Vzz*Q;
}
void MoessCalc::fitParaboloid(double** r, double* y, int N,int dim,
                              double* result, double* errors){
    /*r skalieren, sonst kann Householderalgorithmus zur Loesung linearer
    Gleichungssysteme nicht exakt arbeiten*/
    double* scalers = (double*)malloc(dim*sizeof(double));
    for(int j = 0; j < dim; j++){
        scalers[j] = 0;
        for(int i = 0; i < N; i++)
            if(r[j][i] < 0)
                scalers[j] = scalers[j] - r[j][i];
            else
                scalers[j] = scalers[j] + r[j][i];
        scalers[j] = scalers[j] / N;
        if(scalers[j] == 0)
            scalers[j] = 1;
        else
            scalers[j] = scalers[j];
        for(int i = 0; i < N; i++)
            r[j][i] = r[j][i]/ scalers[j];
    }

    /*legt Paraboloid in Punktwolke und bestimmt Extremum*/
    int d = 1 + dim + dim*(dim+1)/2.;
    gsl_matrix *M = gsl_matrix_alloc(d,d);
    gsl_vector *Y = gsl_vector_alloc(d);
    gsl_vector *X = gsl_vector_alloc(d);

    /*Y fuellen*/
    int indeces[4];
    double **r1 = (double**)malloc((dim+1)*sizeof(double*));
    for(int i = 0; i < dim; i++)
        r1[i] = r[i];
    r1[dim] = y;
    indeces[0] = dim;
    gsl_vector_set(Y,0,sum_up(r1,N,1,indeces));
    for(int i = 0; i < dim; i++){
        indeces[1] = i;
        gsl_vector_set(Y,i+1,sum_up(r1,N,2,indeces));
    }
    int index = dim;
    for(int i = 0; i < dim; i++){
        indeces[1] = i;
        for(int j = i; j < dim; j++){
            index++;
            indeces[2] = j;
            gsl_vector_set(Y,index,sum_up(r1,N,3,indeces));
        }
    }

    /*M befuellen, M besteht quasi aus 9 Bereichen:
      (1 2 3)
      (4 5 6)
      (7 8 9)
    */
    /*1:  */gsl_matrix_set(M,0,0,N);
    /*2,4:*/for(int i = 0; i < dim; i++){
                indeces[0] = i;
                gsl_matrix_set(M,0,i+1,sum_up(r1,N,1,indeces));
                gsl_matrix_set(M,i+1,0,gsl_matrix_get(M,0,i+1));
            }
    /*3,7:*/index = dim;
            for(int i = 0; i < dim; i++){
                indeces[0] = i;
                for(int j = i; j < dim; j++){
                    index++;
                    indeces[1] = j;
                    gsl_matrix_set(M,0,index,sum_up(r1,N,2,indeces));
                    gsl_matrix_set(M,index,0,gsl_matrix_get(M,0,index));
                }
            }
    /*5  :*/for(int i = 0; i < dim; i++){
                indeces[0] = i;
                for(int j = i; j < dim; j++){
                    index++;
                    indeces[1] = j;
                    gsl_matrix_set(M,1+i,1+j,sum_up(r1,N,2,indeces));
                    gsl_matrix_set(M,1+j,1+i,gsl_matrix_get(M,1+i,1+j));
                }
            }
    /*6,8*/ for(int h = 0; h <  dim; h++){
                indeces[2] = h;
                index = dim;
                for(int i = 0; i < dim; i++){
                    indeces[0] = i;
                    for(int j = i; j < dim; j++){
                        index++;
                        indeces[1] = j;
                        gsl_matrix_set(M,h+1,index,sum_up(r1,N,3,indeces));
                        gsl_matrix_set(M,index,1+h,
                                       gsl_matrix_get(M,h+1,index));

                    }
                }
            }
    /*9  :*/int index2;
            index = dim;
            for(int i1 = 0; i1 < dim; i1++){
                indeces[0] = i1;
                for(int j1 = i1; j1 < dim; j1++){
                    index++;
                    indeces[1] = j1;
                    index2 = dim;
                    for(int i2 = 0; i2 < dim; i2++){
                        indeces[2] = i2;
                        for(int j2 = i2; j2 < dim; j2++){
                            index2++;
                            indeces[3] = j2;
                            if(index2 >= index)
                                gsl_matrix_set(M,index,index2,
                                               sum_up(r1,N,4,indeces));
                            else
                                gsl_matrix_set(M,index,index2,
                                               gsl_matrix_get(M,index2,index));

                        }
                    }
                }
            }

    /*korrekturfaktor 0.5 fuer einige Spalten*/
    index = dim;
    for(int i = 0; i < dim; i++)
        for(int j = i; j < dim; j++){
            index++;
            if(i == j)
                for(int h = 0; h < d; h++)
                    gsl_matrix_set(M,h,index,
                                   gsl_matrix_get(M,h,index)/2.);
        }

    /*Paraboloiden berechnen*/
    gsl_linalg_HH_solve(M,Y,X);

    /*der Paraboloid ist berechnet, bestimme jetzt das Minimum, zuerst Gradient
    und Hessematrix auslesen*/
    gsl_vector *df = gsl_vector_alloc(dim);
    gsl_matrix *d2f = gsl_matrix_alloc(dim,dim);
    for(int i = 0; i < dim; i++)
        gsl_vector_set(df,i,gsl_vector_get(X,i+1));
    index = dim;
    for(int i = 0; i < dim; i++)
        for(int j = 0; j < dim; j++)
            if(j >= i){
                index++;
                gsl_matrix_set(d2f,i,j,gsl_vector_get(X,index));
                gsl_matrix_set(d2f,j,i,gsl_matrix_get(d2f,i,j));
            }
    gsl_matrix *d2f_copy = gsl_matrix_alloc(dim,dim);
    gsl_matrix_memcpy(d2f_copy,d2f);

    /*fals ein Resultatvektor existiert, das Paraboloidminimum finden:
    r_min = -d2f^-1 df*/
    gsl_vector *resultv = gsl_vector_alloc(dim);
    gsl_linalg_HH_solve(d2f,df,resultv);
    if(result != NULL)
        for(int i = 0; i < dim; i++)
            result[i] = - gsl_vector_get(resultv,i)*scalers[i];

    /*falls ein errorarray existiert Fehlerrechnung ausfuehren*/
    if(errors != NULL){
        gsl_matrix *covariance = gsl_matrix_alloc(dim, dim);
        gsl_permutation*perm = gsl_permutation_alloc(dim);
        int s=0;
        gsl_linalg_LU_decomp(d2f_copy, perm, &s);
        gsl_linalg_LU_invert(d2f_copy, perm, covariance);
        for(int i = 0; i < dim; i++)
            if(gsl_matrix_get(covariance,i,i) > 0)
                /*korrelierten Fehler auslesen*/
                errors[i] = 1.96 * sqrt(gsl_matrix_get(covariance,i,i))
                            *scalers[i];
            else
                if(gsl_matrix_get(d2f_copy,i,i) > 0)
                    /*unkorellierten Fehler auslesen*/
                    errors[i] = 2*1.96 / sqrt(gsl_matrix_get(d2f_copy,i,i))
                                *scalers[i];
                else
                    /*kein Fehler bestimmbar*/
                    errors[i] = -1;
        gsl_matrix_free(covariance);
    }
    /*Speicher freigeben*/
    gsl_matrix_free(d2f);
    gsl_vector_free(df);
    gsl_matrix_free(M);
    gsl_vector_free(Y);
    gsl_vector_free(X);
    gsl_vector_free(resultv);
    free(scalers);
}
double MoessCalc::sum_up(double **x, int N, int dim, int* indeces){
    double result = 0;
    for(int i = 0; i < N; i++){
        double s = 1;
        for(int j  = 0; j < dim; j++)
            s = s*x[indeces[j]][i];
        result = result + s;
    }
    return result;
}


void MoessCalc::SimpleInvert(double* x, double *y, int count, int steps,
                             double **Mspec, double* rho){
    gsl_matrix *M = gsl_matrix_alloc(count,steps);
    for(int b = 0 ; b < steps; b++)
        for(int i = 0; i < count; i++)
            gsl_matrix_set(M,i,b,Mspec[i][b]);

    gsl_matrix *Mtemp = gsl_matrix_alloc(steps,count);
    gsl_matrix *D = gsl_matrix_alloc(steps,steps);
    int status;

    /*Spektrumvektor initialisieren*/
    gsl_vector * S = gsl_vector_alloc(count);
    for(int i = 0; i < count; i++) gsl_vector_set(S,i,y[i]);
    gsl_vector * result = gsl_vector_alloc(steps);

    /*Rechnen */
    /*zu invertierende Klammer berechnen*/
    status = gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,M,M,0,D);
    /*Invertieren*/

    gsl_matrix *inverse = gsl_matrix_alloc(steps, steps);
        gsl_permutation*perm = gsl_permutation_alloc(steps);
        int s=0;
        status = gsl_linalg_LU_decomp(D, perm, &s);
    if(gsl_linalg_LU_det(D,s) != 0){
        status = gsl_linalg_LU_invert(D, perm, inverse);

        /*Inverse anwenden*/
        status = gsl_blas_dgemm(CblasNoTrans,CblasTrans,1,inverse,M,0,Mtemp);
        status = gsl_blas_dgemv(CblasNoTrans,1,Mtemp,S,0,result);

        /*Ergebnis auslesen*/
        for(int i = 0; i < steps; i++)
            rho[i] = gsl_vector_get(result,i);
    }
    else
        for(int i = 0; i < steps; i++)
            rho[i] = 0;

    /*Ergebnisspektrum ausgeben
    status = gsl_blas_dgemv(CblasNoTrans,1,M,result,0,S);
    for(int i = 0; i < count; i++)
        newfit[i] = gsl_vector_get(S,i);*/

    /*Ergebnis normieren
    double A = 0;
    for(int i = 0; i < steps; i++) A = A + B[i];
    for(int i = 0; i < steps; i++) B[i] = B[i]*steps/Bmax/A;*/


    gsl_matrix_free(M);
    gsl_matrix_free(Mtemp);
    gsl_matrix_free(D);
    gsl_matrix_free(inverse);
    gsl_permutation_free(perm);
}
void MoessCalc::get_SC_spectrum(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[7]==0) return;                 // no absorption at this site
    /*pruefen, ob Hamiltonia neu berechnet werden muss und dies ggf. tun*/
    if(p[0] != B || p[1] != Vzz || p[2] != eta || p[3]*DegRad != theta
       || p[4]*DegRad != phi || firstcalc){
        set_parameters(p[0],p[1],p[2],p[3],p[4]);
        calc();
    }
    /*Nicht-Hamiltonian-Elemente setzen*/
    double CS =         p[5];
    double omega =      p[6];
    double A =          p[7]/8.;
    double beta =       p[8]*DegRad;
    double gamma =      p[9]*DegRad;
    double omegarez = 1./omega;
    double pp[10]; copy(pp,p,10);
    /*Moessbauerspektrum generieren*/
    double Ic, vgamma;
    for(int i = 0; i < 2; i++)
        for(int f = 0; f < 4; f++){
            /*jeden Übergang berechnen*/
            Ic = this->I(beta,gamma,f,i)*A;
            vgamma = get_Egamma(f,i) + CS;
            for(int j = 0; j < cc; j++)
                /*Übergangswahrscheinlichkeit für jede Energie bei dem
                Uebergang*/
                I[j] -= f_lorenz_fast(v[j],omegarez,vgamma,Ic);
        }
}
void MoessCalc::get_SC_spectrum_G(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[7]==0) return;                 // no absorption at this site
    // these are the angles in the Gamma beam system
    double beta =       p[8]*DegRad;    // polar angle of Vzz to beam
    double gamma =      p[9]*DegRad;    // azimutal angle of Vxx to beam
    double Theta =      p[3]*DegRad;    // polar angle of B to beam
    double Phi =        p[4]*DegRad;    // azimutal angle of B to beam
    double Vz[3],Vx[3],Vy[3],Vx0[3],Vy0[3]={0,1,0},b[3];
    Vz[0] = cos(beta);
    Vz[1] = 0;
    Vz[2] = sin(beta);
    Vx0[0] = cos(beta-M_PI/2.);
    Vx0[1] = 0;
    Vx0[2] = sin(beta-M_PI/2.);
    Vx[0] =  cos(-gamma)*Vx0[0] + sin(-gamma)*Vy0[0];
    Vx[1] =  cos(-gamma)*Vx0[1] + sin(-gamma)*Vy0[1];
    Vx[2] =  cos(-gamma)*Vx0[2] + sin(-gamma)*Vy0[2];
    Vy[0] = -sin(-gamma)*Vx0[0] + cos(-gamma)*Vy0[0];
    Vy[1] = -sin(-gamma)*Vx0[1] + cos(-gamma)*Vy0[1];
    Vy[2] = -sin(-gamma)*Vx0[2] + cos(-gamma)*Vy0[2];
    b[0] = cos(Theta);
    b[1] = sin(Theta)*sin(Phi);
    b[2] = sin(Theta)*cos(Phi);
    double bx = Vx[0]*b[0]+Vx[1]*b[1]+Vx[2]*b[2];
    double by = Vy[0]*b[0]+Vy[1]*b[1]+Vy[2]*b[2];
    double bz = Vz[0]*b[0]+Vz[1]*b[1]+Vz[2]*b[2];

    double pnew[10]; copy(pnew,p,10);
    pnew[3] = acos(bz)*RadDeg;
    pnew[4] = atan2(by,bx)*RadDeg;

    return get_SC_spectrum(I,cc,v,pnew);
}
void MoessCalc::get_SC_spectrum_B(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[7]==0) return;                 // no absorption at this site
    // these are the angles in the Gamma beam system
    double beta =       p[8]*DegRad;    // polar angle of Vzz to beam
    double gamma =      p[9]*DegRad;    // azimutal angle of Vxx to beam
    double Theta =      p[3]*DegRad;    // polar angle of B to beam
    double Phi =        p[4]*DegRad;    // azimutal angle of B to beam
    double Vz[3],Vx[3],Vy[3],Vx0[3],Vy0[3]={0,1,0},g[3];
    Vz[0] = cos(Theta);
    Vz[1] = 0;
    Vz[2] = sin(Theta);
    Vx0[0] = cos(Theta-M_PI/2.);
    Vx0[1] = 0;
    Vx0[2] = sin(Theta-M_PI/2.);
    Vx[0] =  cos(-Phi)*Vx0[0] + sin(-Phi)*Vy0[0];
    Vx[1] =  cos(-Phi)*Vx0[1] + sin(-Phi)*Vy0[1];
    Vx[2] =  cos(-Phi)*Vx0[2] + sin(-Phi)*Vy0[2];
    Vy[0] = -sin(-Phi)*Vx0[0] + cos(-Phi)*Vy0[0];
    Vy[1] = -sin(-Phi)*Vx0[1] + cos(-Phi)*Vy0[1];
    Vy[2] = -sin(-Phi)*Vx0[2] + cos(-Phi)*Vy0[2];
    g[0] = cos(beta);
    g[1] = sin(beta)*sin(gamma);
    g[2] = sin(beta)*cos(gamma);
    double gx = Vx[0]*g[0]+Vx[1]*g[1]+Vx[2]*g[2];
    double gy = Vy[0]*g[0]+Vy[1]*g[1]+Vy[2]*g[2];
    double gz = Vz[0]*g[0]+Vz[1]*g[1]+Vz[2]*g[2];

    double pnew[10]; copy(pnew,p,10);
    pnew[8] = acos(gz)*RadDeg;
    pnew[9] = atan2(gy,gx)*RadDeg;

    return get_SC_spectrum(I,cc,v,pnew);
}
void MoessCalc::get_SrcFld_spectrum(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[7]==0) return;                 // no absorption at this site

    double xisrc[6],phisrc[6],vsrc[6],Isrc[6];
    double xispl[8],phispl[8],vspl[8],Ispl[8];
    double Ires[6][8];
    /*Nicht-Hamiltonian-Elemente setzen*/
    double CS =         p[5];
    double omega =      p[6];
    double A =          p[7];
    double beta =       p[8];
    double gamma =      p[9];
    double beta_src =   p[11];
    double alpha =      p[12];
    double pol =        p[13];
    /*Sample transition calculation*/
    if(p[0] != B || p[1] != Vzz || p[2] != eta || p[3]*DegRad != theta
            || p[4]*DegRad != phi || firstcalc){
        set_parameters(p[0],p[1],p[2],p[3],p[4]);
        calc();
    }
    for(int i = 0; i < 2; i++)
        for(int f = 0; f < 4; f++){
            this->I(beta/180.*M_PI,gamma/180.*M_PI,alpha,f,i,
                    &(Ispl[4*i+f]),&(phispl[4*i+f]),&(xispl[4*i+f]));
            vspl[4*i+f] = get_Egamma(f,i);
        }
    /*Source transition calculation*/
    if(p[10] != B || 0 != Vzz || 0 != eta || 0 != theta || 0 != phi
       || firstcalc){
        set_parameters(p[10],0,0,0,0);
        calc();
    }
    for(int i = 0; i < 2; i++)
        for(int f = 0; f < 3; f ++){
            this->I(beta_src/180.*M_PI,0.,0.,i+f,i,
                    &(Isrc[3*i+f]),&(phisrc[3*i+f]),&(xisrc[3*i+f]));
            vsrc[3*i+f] = get_Egamma(i+f,i);
        }
    /*calc total intensity*/
    double Itot = 0;
    for(int i = 0; i < 6; i++)
        for(int j = 0; j < 8; j++){
            Ires[i][j] = Isrc[i]*Ispl[j]/2.
                                        *(1+sin(2*xisrc[i]/180.*M_PI)
                                        *sin(2*xispl[j]/180.*M_PI)
                                        +cos(2*(phisrc[i]-phispl[j])/180.*M_PI)
                                        *cos(2*xispl[j]/180.*M_PI)
                                        *cos(2*xisrc[i]/180.*M_PI));
            Itot += Ires[i][j];
    }
    /*Moessbauerspektrum generieren*/
    for(int i = 0; i < 6; i++)
        for(int j = 0; j < 8; j++)
            if(Ires[i][j] > 0 || (pol < 1 && (Isrc[i] > 0 || Ispl[j] > 0)))
                for(int k = 0; k < cc; k++)
                    I[k] = I[k]
                        - f_lorenz(v[k],omega,vsrc[i]+vspl[j]+CS,Ires[i][j])
                          /Itot*A*pol
                        - f_lorenz(v[k],omega,vsrc[i]+vspl[j]+CS,Isrc[i]*Ispl[j])
                          /64.*A*(1-pol);
}
void MoessCalc::get_fm_Powder_spectrum(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[7]==0) return;                 // no absorption at this site
    /*pruefen, ob Hamiltonia neu berechnet werden muss und dies ggf. tun*/
    double Bafm =       p[0];
    double BA =         p[1];           // anisotropy field
    double Bex =        p[2];           // external field
    double Vzz =        p[3];
    double theta0 =     p[4];
    double CS =         p[5];
    double omega =      p[6];
    double A =          p[7];
    int N = qMax(2,(int) p[8]);
    int Nvzz = qMax(1,(int)p[9]);

    double thetai, zeta, phiprold, hk, varphi;
    double Btot, cOmega,thetapr,theta, gamma, beta;
    double phipr = 0;
    double SCpara[10]={0,0,0,0,0,0,0,0,0,0};
    SCpara[1] = Vzz;
    SCpara[5] = CS;
    SCpara[6] = omega;
    SCpara[7] = A;

    double* Itemp = new double[cc];

    //QTime myTimer;
    //myTimer.start();
    //    QString clipboard("Btot\tzeta\t\phipr\tthetai\tbeta\tgamma\ttheta\n");
    for(int i = 1; i <= N; i++){
        // generate N equally space directions on sphere by Saff & Kuijlaars
        hk = -1+2*(i-1)/(N-1.);
        zeta = acos(hk)*RadDeg;
        phiprold = phipr;
        if(abs(hk) != 1)
            phipr = fmod((phiprold + 3.6/sqrt(N)*1/sqrt(1-hk*hk)*RadDeg),360);
        else
            phipr = 0;
        if(i==1)
            thetai = zeta;
        fm_flip(Bex,BA,zeta, &(thetai));
        SCpara[0] = Btot = sqrt(Bex*Bex+Bafm*Bafm-2*Bex*Bafm*cos(thetai*DegRad));
        thetapr = atan2(Bafm*sin(thetai*DegRad),
                        Bafm*cos(thetai*DegRad)+Bex)*RadDeg;
        for(int ivzz = 0; ivzz < Nvzz; ivzz++){
            varphi = 360.*ivzz/Nvzz;//rand()%360;
            // simulate the spectrum for both spin directions
            double st0 = sin(theta0*DegRad);
            double ct0 = cos(theta0*DegRad);
            double sz = sin(zeta*DegRad);
            double cz = cos(zeta*DegRad);
            double svp = sin(varphi*DegRad);
            double cvp = cos(varphi*DegRad);
            double spp = sin(phipr*DegRad);
            double cpp = cos(phipr*DegRad);
            double stp = sin(thetapr*DegRad);
            double ctp = cos(thetapr*DegRad);
            SCpara[8] = beta  = acos(qMax(-1.,qMin(1.,cpp*(ct0*sz+st0*cvp*cz)+spp*st0*svp)))*RadDeg;
            SCpara[3] = theta = acos(qMax(-1.,qMin(1.,(ct0*sz+st0*cvp*cz)*stp+(ct0*cz-st0*cvp*sz)*ctp)))*RadDeg;
            cOmega = stp*cpp;
            SCpara[9] = gamma = acos(qMax(-1.,qMin(1.,(cOmega-cos(beta*DegRad)*cos(theta*DegRad))/(sin(theta*DegRad)*sin(beta*DegRad)))))*RadDeg;
            get_SC_spectrum(Itemp,cc,v,SCpara);
            for(int j = 0; j < cc; j++)
                I[j] += Itemp[j];
            //            clipboard.append(QString("%1\t%2\t%3\t%4\t%5\t%6\t%7\n").arg(SCpara[0]).arg(zeta).arg(phipr).arg(thetai).arg(beta).arg(gamma).arg(theta));
            }
        }
    //    QApplication::clipboard()->setText(clipboard);
    //int nMilliseconds = myTimer.elapsed();
    //QMessageBox(QMessageBox::NoIcon,QString("Time"),QString("%1").arg(nMilliseconds)).exec();
    // normalize
    for(int j = 0; j < cc; j++)
        I[j] = I[j]/(N*Nvzz);
    delete Itemp;
}
void MoessCalc::get_afm_Powder_long_spectrum(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[8]==0) return;                 // no absorption at this site
    /*pruefen, ob Hamiltonia neu berechnet werden muss und dies ggf. tun*/
    double Bafm =       p[0];
    double BA =         p[1];           // anisotropy field
    double BJ =         p[2];           // exchange field
    double Bex =        p[3];           // external field
    double Vzz =        p[4];
    double theta0 =     p[5];
    double CS =         p[6];
    double omega =      p[7];
    double A =          p[8];
    int N = qMax(2,(int) p[9]);
    int Nvzz = qMax(1,(int)p[10]);

    double thetai[2], zeta, varphi;
    double Btot, cOmega,thetapr,theta, gamma, beta;
    double phipr = 0;
    double SCpara[10]={0,0,0,0,0,0,0,0,0,0};
    SCpara[1] = Vzz;
    SCpara[5] = CS;
    SCpara[6] = omega;
    SCpara[7] = A;

    double* Itemp = new double[cc];
    //QTime myTimer;
    //myTimer.start();
    //    QString clipboard("Btot\tzeta\t\phipr\tthetai\tbeta\tgamma\ttheta\n");
    double sumsz = 0;
    for(int i = 0; i < N; i++){
        // consider all zeta values
        zeta = 180.*i/(N-1);
        double sz = sin(zeta*DegRad);
        sumsz += sz;
        // solve the Hamiltonian, i.g. get the spin directions
        if(i==0){
            thetai[0] = zeta;
            thetai[1] = zeta-180;
        }
        afm_flipflop(Bex,BJ,BA,zeta, &(thetai[0]), &(thetai[1]));
        for(int ivzz = 0; ivzz < Nvzz; ivzz++){
            varphi = 360.*ivzz/Nvzz;//rand()%360;
            // simulate the spectrum for both spin directions
            for(int j = 0; j < 2; j++){
                // for both moment directions
                SCpara[0] = Btot = sqrt(Bex*Bex+Bafm*Bafm-2*Bex*Bafm*cos(thetai[j]*DegRad));
                //thetapr = asin(Bafm/Btot*sin(thetai[j]*DegRad))*RadDeg;
                thetapr = atan2(Bafm*sin(thetai[j]*DegRad),
                               -Bafm*cos(thetai[j]*DegRad)+Bex)*RadDeg;
                double st0 = sin(theta0*DegRad);
                double ct0 = cos(theta0*DegRad);
                double cz = cos(zeta*DegRad);
                double cvp = cos(varphi*DegRad);
                double stp = sin(thetapr*DegRad);
                double ctp = cos(thetapr*DegRad);
                SCpara[8] = beta  = acos(qMax(-1.,qMin(1.,ct0*cz-st0*cvp*sz)))*RadDeg;
                SCpara[3] = theta = acos(qMax(-1.,qMin(1.,(ct0*sz+st0*cvp*cz)*stp+(ct0*cz-st0*cvp*sz)*ctp)))*RadDeg;
                cOmega = ctp;
                SCpara[9] = gamma = acos(qMax(-1.,qMin(1.,(cOmega-cos(beta*DegRad)*cos(theta*DegRad))/(sin(theta*DegRad)*sin(beta*DegRad)))))*RadDeg;
                get_SC_spectrum(Itemp,cc,v,SCpara);
                for(int j = 0; j < cc; j++)
                    I[j] += sz*Itemp[j];
                //                clipboard.append(QString("%1\t%2\t%3\t%4\t%5\t%6\t%7\n").arg(SCpara[0]).arg(zeta).arg(phipr).arg(thetai[j]).arg(beta).arg(gamma).arg(theta));
            }
        }
    }
    //    QApplication::clipboard()->setText(clipboard);
    //int nMilliseconds = myTimer.elapsed();
    //QMessageBox(QMessageBox::NoIcon,QString("Time"),QString("%1").arg(nMilliseconds)).exec();
    // normalize
    for(int j = 0; j < cc; j++)
        I[j] = I[j]/(2.*sumsz*Nvzz);
    delete Itemp;
}
void MoessCalc::get_afm_Powder_spectrum(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[8]==0) return;                 // no absorption at this site
    /*pruefen, ob Hamiltonia neu berechnet werden muss und dies ggf. tun*/
    double Bafm =       p[0];
    double BA =         p[1];           // anisotropy field
    double BJ =         p[2];           // exchange field
    double Bex =        p[3];           // external field
    double Vzz =        p[4];
    double theta0 =     p[5];
    double CS =         p[6];
    double omega =      p[7];
    double A =          p[8];
    int N = qMax(2,(int) p[9]);
    int Nvzz = qMax(1,(int)p[10]);

    double thetai[2], zeta, phiprold,hk, varphi;
    double Btot, cOmega,thetapr,theta, gamma, beta;
    double phipr = 0;
    double SCpara[10]={0,0,0,0,0,0,0,0,0,0};
    SCpara[1] = Vzz;
    SCpara[5] = CS;
    SCpara[6] = omega;
    SCpara[7] = A;

    double* Itemp = new double[cc];
    //QTime myTimer;
    //myTimer.start();
    //    QString clipboard("Btot\tzeta\t\phipr\tthetai\tbeta\tgamma\ttheta\n");
    for(int i = 1; i <= N; i++){
        // generate N equally space directions on sphere by Saff & Kuijlaars
        hk = -1+2*(i-1)/(N-1.);
        zeta = acos(hk)*RadDeg;
        phiprold = phipr;
        if(abs(hk) != 1)
            phipr = fmod((phiprold + 3.6/sqrt(N)*1/sqrt(1-hk*hk)*RadDeg),360);
        else
            phipr = 0;
        // solve the Hamiltonian, i.g. get the spin directions
        if(i==1){
            thetai[0] = zeta;
            thetai[1] = zeta-180;
        }
        afm_flipflop(Bex,BJ,BA,zeta, &(thetai[0]), &(thetai[1]));
        for(int ivzz = 0; ivzz < Nvzz; ivzz++){
            varphi = 360.*ivzz/Nvzz;//rand()%360;
            // simulate the spectrum for both spin directions
            for(int j = 0; j < 2; j++){
                // for both moment directions
                SCpara[0] = Btot = sqrt(Bex*Bex+Bafm*Bafm-2*Bex*Bafm*cos(thetai[j]*DegRad));
                //thetapr = asin(Bafm/Btot*sin(thetai[j]*DegRad))*RadDeg;
                thetapr = atan2(Bafm*sin(thetai[j]*DegRad),
                                Bafm*cos(thetai[j]*DegRad)+Bex)*RadDeg;
                double st0 = sin(theta0*DegRad);
                double ct0 = cos(theta0*DegRad);
                double sz = sin(zeta*DegRad);
                double cz = cos(zeta*DegRad);
                double svp = sin(varphi*DegRad);
                double cvp = cos(varphi*DegRad);
                double spp = sin(phipr*DegRad);
                double cpp = cos(phipr*DegRad);
                double stp = sin(thetapr*DegRad);
                double ctp = cos(thetapr*DegRad);
                SCpara[8] = beta  = acos(qMax(-1.,qMin(1.,cpp*(ct0*sz+st0*cvp*cz)+spp*st0*svp)))*RadDeg;
                SCpara[3] = theta = acos(qMax(-1.,qMin(1.,(ct0*sz+st0*cvp*cz)*stp+(ct0*cz-st0*cvp*sz)*ctp)))*RadDeg;
                cOmega = stp*cpp;
                SCpara[9] = gamma = acos(qMax(-1.,qMin(1.,(cOmega-cos(beta*DegRad)*cos(theta*DegRad))/(sin(theta*DegRad)*sin(beta*DegRad)))))*RadDeg;
                get_SC_spectrum(Itemp,cc,v,SCpara);
                for(int j = 0; j < cc; j++)
                    I[j] += Itemp[j];
                //                clipboard.append(QString("%1\t%2\t%3\t%4\t%5\t%6\t%7\n").arg(SCpara[0]).arg(zeta).arg(phipr).arg(thetai[j]).arg(beta).arg(gamma).arg(theta));
            }
        }
    }
    //    QApplication::clipboard()->setText(clipboard);
    //int nMilliseconds = myTimer.elapsed();
    //QMessageBox(QMessageBox::NoIcon,QString("Time"),QString("%1").arg(nMilliseconds)).exec();
    // normalize
    for(int j = 0; j < cc; j++)
        I[j] = I[j]/(2.*N*Nvzz);
    delete Itemp;
}
void MoessCalc::fm_flip(double Bex,double BA, double zeta, double* theta1){
    size_t iter = 0;
    int status;
    /* Position of the minimum (1,2), scale factors
    10,20, height 30. */
    double par[3];
    par[0] = Bex;
    par[1] = BA;
    par[2] = zeta;

    gsl_multimin_function_fdf my_func;
    my_func.n = 1;
    my_func.f = Hfm_f;
    my_func.df = Hfm_df;
    my_func.fdf = Hfm_fdf;
    my_func.params = par;
    /* Starting point, x = (5,7) */
    gsl_vector *x = gsl_vector_alloc (1);
    gsl_vector_set (x, 0, *theta1);
    const gsl_multimin_fdfminimizer_type *T = gsl_multimin_fdfminimizer_conjugate_fr;
    gsl_multimin_fdfminimizer *s = gsl_multimin_fdfminimizer_alloc (T, 1);
    gsl_multimin_fdfminimizer_set (s, &my_func, x, 1, 0.1);
    do{
        iter++;
        status = gsl_multimin_fdfminimizer_iterate (s);
        if (status)
        break;
        status = gsl_multimin_test_gradient (s->gradient, 1e-3);
    }while (status == GSL_CONTINUE && iter < 100);
    *theta1 = gsl_vector_get (s->x, 0);
    gsl_multimin_fdfminimizer_free (s);
    gsl_vector_free (x);
}
void MoessCalc::afm_flipflop(double Bex, double BJ, double BA, double zeta,
                  double* theta1, double*theta2){
    size_t iter = 0;
    int status;
    /* Position of the minimum (1,2), scale factors
    10,20, height 30. */
    double par[4];
    par[0] = Bex;
    par[1] = BJ;
    par[2] = BA;
    par[3] = zeta;

    gsl_multimin_function_fdf my_func;
    my_func.n = 2;
    my_func.f = Hafm_f;
    my_func.df = Hafm_df;
    my_func.fdf = Hafm_fdf;
    my_func.params = par;
    /* Starting point, x = (5,7) */
    gsl_vector *x = gsl_vector_alloc (2);
    gsl_vector_set (x, 0, *theta1);
    gsl_vector_set (x, 1, *theta2);
    const gsl_multimin_fdfminimizer_type *T = gsl_multimin_fdfminimizer_conjugate_fr;
    gsl_multimin_fdfminimizer *s = gsl_multimin_fdfminimizer_alloc (T, 2);
    gsl_multimin_fdfminimizer_set (s, &my_func, x, 1, 0.1);
    do{
        iter++;
        status = gsl_multimin_fdfminimizer_iterate (s);
        if (status)
        break;
        status = gsl_multimin_test_gradient (s->gradient, 1e-3);
    }while (status == GSL_CONTINUE && iter < 100);
    *theta1 = gsl_vector_get (s->x, 0);
    *theta2 = gsl_vector_get (s->x, 1);
    if(*theta1 > *theta2){
        *theta1 = gsl_vector_get (s->x, 1);
        *theta2 = gsl_vector_get (s->x, 0);
    }
    gsl_multimin_fdfminimizer_free (s);
    gsl_vector_free (x);
}
double Hafm_f(const gsl_vector * x, void * params ){
    double Bex  = ((double*)params)[0];
    double BJ   = ((double*)params)[1];
    double BA   = ((double*)params)[2];
    double zeta = ((double*)params)[3]/180.*M_PI;
    double theta1 = gsl_vector_get (x, 0)/180.*M_PI;
    double theta2 = gsl_vector_get (x, 1)/180.*M_PI;
    return     BJ* cos((theta1-theta2))
          -0.5*BA*(cos((theta1-zeta))*cos(theta1-zeta)+cos(theta2-zeta)*cos(theta2-zeta))
          -   Bex*(cos(theta1)+cos(theta2));
}
void Hafm_df(const gsl_vector * x, void * params, gsl_vector * g ){
    double Bex  = ((double*)params)[0];
    double BJ   = ((double*)params)[1];
    double BA   = ((double*)params)[2];
    double zeta = ((double*)params)[3]/180.*M_PI;
    double theta1 = gsl_vector_get (x, 0)/180.*M_PI;
    double theta2 = gsl_vector_get (x, 1)/180.*M_PI;
    gsl_vector_set(g,0,
              -BJ* sin(theta1-theta2)
          +    BA*(sin(theta1-zeta)*cos(theta1-zeta))
          +   Bex*(sin(theta1)));
    gsl_vector_set(g,1,
               BJ* sin(theta1-theta2)
          +    BA*(sin(theta2-zeta)*cos(theta2-zeta))
          +   Bex*(sin(theta2)));
}
void Hafm_fdf(const gsl_vector * x, void * params,double * f, gsl_vector *g){
    *f = Hafm_f(x,params);
    Hafm_df(x,params,g);
}
double Hfm_f(const gsl_vector * x, void * params ){
    double Bex  = ((double*)params)[0];
    double BA   = ((double*)params)[1];
    double zeta = ((double*)params)[2]/180.*M_PI;
    double theta1 = gsl_vector_get (x, 0)/180.*M_PI;
    return-0.5*BA*(cos((theta1-zeta))*cos(theta1-zeta))
          -   Bex*(cos(theta1));
}
void Hfm_df(const gsl_vector * x, void * params, gsl_vector * g ){
    double Bex  = ((double*)params)[0];
    double BA   = ((double*)params)[1];
    double zeta = ((double*)params)[2]/180.*M_PI;
    double theta1 = gsl_vector_get (x, 0)/180.*M_PI;
    gsl_vector_set(g,0,
               BA*(sin(theta1-zeta)*cos(theta1-zeta))
          +   Bex*(sin(theta1)));
}
void Hfm_fdf(const gsl_vector * x, void * params,double * f, gsl_vector *g){
    *f = Hfm_f(x,params);
    Hfm_df(x,params,g);
}
void MoessCalc::get_PseudoVoigt_profile(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[3]==0) return;                 // no absorption at this site
    double CS =         p[0];
    double omegarez = 1/p[1];
    double eta =qMin(1.,qMax(0.,p[2]));
    double Alorentz =   p[3]*eta;
    double Agauss =     p[3]*(1-eta)*sqrt(log(2)/M_PI)*omegarez;
    double log2oversqromega = log(2)*omegarez*omegarez;

    for(int j = 0; j < cc; j++)
        /*Übergangswahrscheinlichkeit für jede Energie bei dem
        Uebergang*/
        I[j] -=   f_lorenz_fast(v[j],omegarez,CS,Alorentz)
                + Agauss*exp(-log2oversqromega*(v[j]-CS)*(v[j]-CS));
}

void MoessCalc::get_Powder_spectrum(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[7]==0) return;                 // no absorption at this site
    /*pruefen, ob Hamiltonia neu berechnet werden muss und dies ggf. tun*/
    if(p[0] != B || p[1] != Vzz || p[2] != eta || p[3]*DegRad != theta
            || p[4]*DegRad != phi || firstcalc){
        set_parameters(p[0],p[1],p[2],p[3],p[4]);
        calc();
    }
    /*Nicht-Hamiltonian-Elemente setzen*/
    double CS =         p[5];
    double omega =      p[6];
    double A =          p[7]/8.;
    double omegarez = 1./omega;

    /*Moessbauerspektrum generieren*/
    double Ic, vgamma;
    for(int i = 0; i < 2; i++)
        for(int f = 0; f < 4; f++){
            /*jeden Übergang berechnen*/
            Ic = this->I(f,i)*A;
            vgamma = get_Egamma(f,i) + CS;
            for(int j = 0; j < cc; j++)
                /*Übergangswahrscheinlichkeit für jede Energie bei dem
                Uebergang*/
                I[j] -= f_lorenz_fast(v[j],omegarez,vgamma,Ic);
        }

}
void MoessCalc::get_Powder_spectrum_lev(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[7]==0) return;                 // no absorption at this site
    /*pruefen, ob Hamiltonia neu berechnet werden muss und dies ggf. tun*/
    if(p[0] != B || p[1] != Vzz || p[2] != eta || p[3]*DegRad != theta
            || p[4]*DegRad != phi || firstcalc){
        set_parameters(p[0],p[1],p[2],p[3],p[4]);
        calc();
    }
    /*Nicht-Hamiltonian-Elemente setzen*/
    double CS =         p[5];
    double omega =      p[6];
    double A =          p[7]/8.;
    double omegarez = 1./omega;

    // normalize according to saturation effect
    double taNorm = 0;
    for(int i = 0; i < 2; i++)
        for(int f = 0; f < 4; f++){
            double tai = p[8]*this->I(f,i)/8.;
            taNorm += this->I(tai)*0.125;
        }
    /*Moessbauerspektrum generieren*/
    double Ic, vgamma;
    for(int i = 0; i < 2; i++)
        for(int f = 0; f < 4; f++){
            /*jeden Übergang berechnen*/
            double tai = p[8]*this->I(f,i)/8.;
            Ic = this->I(tai)/taNorm*A;
            vgamma = get_Egamma(f,i) + CS;
            for(int j = 0; j < cc; j++)
                /*Übergangswahrscheinlichkeit für jede Energie bei dem
                Uebergang*/
                I[j] -= f_lorenz_fast(v[j],omegarez,vgamma,Ic);
        }

}
void MoessCalc::get_FeCal_spectrum(double *I, int cc , double* v, double* p,
                                   double U){
    init_array(I,cc,0);                 // reset spectrum
    if(p[3]==0 && p[4]==0 && p[5]== 0) return;    // no absorption at this site
    double omega[6];
    double A[6];
    double dv[6];
    omega[0]=omega[5]=p[0];
    omega[1]=omega[4]=p[1];
    omega[2]=omega[3]=p[2];
    A[0]=A[5]=p[3];
    A[1]=A[4]=p[4];
    A[2]=A[3]=p[5];
    double alpha = p[6];
    double CS = p[7];
    /*erzeuge neues Geschwindigkeitsspektrum*/
    for(int i = 0; i < cc; i++)
        v[i] = -cos(M_PI*(0.5+i)/cc)*U*alpha;
    /*Energieverschiebungen berechnen*/
    dv[0] = mun*33.3*(-1./2.*g12-(-3./2.*g32))+CS;
    dv[1] = mun*33.3*(-1./2.*g12-(-1./2.*g32))+CS;
    dv[2] = mun*33.3*(-1./2.*g12-(+1./2.*g32))+CS;
    dv[3] = mun*33.3*(+1./2.*g12-(-1./2.*g32))+CS;
    dv[4] = mun*33.3*(+1./2.*g12-(+1./2.*g32))+CS;
    dv[5] = mun*33.3*(+1./2.*g12-(+3./2.*g32))+CS;
    /*erzeuge Spektrum*/
    for(int i = 0; i < 6; i++){
        /*jeden Peak einzeichnen*/
        for(int j = 0; j < cc; j++)
            /*Übergangswahrscheinlichkeit für jede Energie bei dem
            Uebergang*/
            I[j] = I[j]
                    -f_lorenz(v[j],omega[i],dv[i],1)*A[i];
    }
}
void MoessCalc::get_FeCaltriang_spectrum(double *I, int cc , double* v, double* p,
                                   double U){
    init_array(I,cc,0);                 // reset spectrum
    if(p[3]==0 && p[4]==0 && p[5]== 0) return;    // no absorption at this site
    double omega[6];
    double A[6];
    double dv[6];
    omega[0]=omega[5]=p[0];
    omega[1]=omega[4]=p[1];
    omega[2]=omega[3]=p[2];
    A[0]=A[5]=p[3];
    A[1]=A[4]=p[4];
    A[2]=A[3]=p[5];
    double alpha = p[6];
    double CS = p[7];
    /*erzeuge neues Geschwindigkeitsspektrum*/
    for(int i = 0; i < cc; i++)
        v[i] = U*alpha*(0.5+i-cc/2)/cc;
    /*Energieverschiebungen berechnen*/
    dv[0] = mun*33.3*(-1./2.*g12-(-3./2.*g32))+CS;
    dv[1] = mun*33.3*(-1./2.*g12-(-1./2.*g32))+CS;
    dv[2] = mun*33.3*(-1./2.*g12-(+1./2.*g32))+CS;
    dv[3] = mun*33.3*(+1./2.*g12-(-1./2.*g32))+CS;
    dv[4] = mun*33.3*(+1./2.*g12-(+1./2.*g32))+CS;
    dv[5] = mun*33.3*(+1./2.*g12-(+3./2.*g32))+CS;
    /*erzeuge Spektrum*/
    for(int i = 0; i < 6; i++){
        /*jeden Peak einzeichnen*/
        for(int j = 0; j < cc; j++)
            /*Übergangswahrscheinlichkeit für jede Energie bei dem
            Uebergang*/
            I[j] = I[j]
                    -f_lorenz(v[j],omega[i],dv[i],1)*A[i];
    }
}
void MoessCalc::get_Dynbeta_spectrum(double *I, int cc , double* v, double* p){
    int J = int(p[0]);
    double *ptemp = new double[4+J*(6+J-1)];
    copy(ptemp,p,3+J*(6+J-1));
    for(int i = 3+J*(6+J-1); i>3; i--)
        ptemp[i] = ptemp[i-1];
    ptemp[3] = acos(sqrt(1./3.))*RadDeg;
    get_Dynbeta_spectrum(I,cc,v,ptemp);
    delete ptemp;
}

void MoessCalc::get_Dyn_spectrum(double *I, int cc , double* v, double* p){
    int N0 = 3;                             // fundamanetal model parameters
    init_array(I,cc,0);                 // reset spectrum
    if(p[2] == 0) return;               // no absorption at this site
    double omega =      p[1];
    double A     =      p[2]/(M_PI*12);
    int J        =  int(p[0]);
    //double beta  =      p[3]*DegRad;
    double pcheck[17];
    for(int i = 0; i < 17; i++)
        pcheck[i] = p[i];
    gsl_matrix_complex* M0 = gsl_matrix_complex_alloc(8*J,8*J);
    gsl_matrix_complex_set_all(M0,gsl_complex_rect(0,0));
    gsl_matrix_complex* Mt = gsl_matrix_complex_alloc(8*J,8*J);
    gsl_matrix_complex* M = gsl_matrix_complex_alloc(8*J,8*J);
    gsl_matrix* W = gsl_matrix_alloc(J,J);
    gsl_vector* pa = gsl_vector_alloc(J);
    for(int j = 0; j < J; j++){
        double Wsum = 0;
        for(int jj = 0; jj < J; jj++){
            double val = 0;
            if(jj < j)
                val = p[N0+j*(6+J-1)+6+jj  ];
            if(jj > j)
                val = p[N0+j*(6+J-1)+6+jj-1];
            val *=  1e6 * hbar_c_over_Egamma * 1e3;
            gsl_matrix_set(W,j,jj,val);
            Wsum -= val;
        }
        // set diagonal element
        gsl_matrix_set(W,j,j,Wsum);
    }


    // calulate equilibrium occupation pa
    gsl_vector* b = gsl_vector_alloc(J);
    gsl_vector_set_all(b,0);
    gsl_vector_set(b,0,1);
    gsl_matrix* Wt = gsl_matrix_alloc(J,J);
    gsl_matrix_memcpy(Wt,W);
    gsl_matrix_transpose(Wt);
    for(int j = 0; j < J; j++) gsl_matrix_set(Wt,0,j,1.);
    gsl_permutation*perm = gsl_permutation_alloc(J);
    int s=0;
    gsl_linalg_LU_decomp(Wt, perm, &s);
    if(gsl_linalg_LU_det(Wt,s) != 0)
        gsl_linalg_LU_solve(Wt,perm,b,pa);
    else
        gsl_vector_set_all(pa,1./J);

    double ClebschGordan[2][4] = {{1.7320508076,1.4142135624,1,0},{0,1,1.4142135624,1.7320508076}};
    //double Icl[2][4];
    //double cos2beta = cos(beta)*cos(beta);
    //Icl[0][0] = sqrt(0.75*  (1+cos2beta));
    //Icl[0][1] = sqrt(0.75*2*(1-cos2beta));
    //Icl[0][2] = sqrt(0.75*  (1+cos2beta));
    //Icl[0][3] = 0;
    //Icl[1][0] = 0;
    //Icl[1][1] = sqrt(0.75*2*(1+cos2beta));
    //Icl[1][2] = sqrt(0.75*  (1-cos2beta));
    //Icl[1][3] = sqrt(0.75*2*(1+cos2beta));



    // fill basic matrix fluctuations
    for(int i = 0; i < 2; i++)
        for(int f = 0; f < 4; f++)
            for(int a = 0; a < J; a++){
                //col finished
                int c = i*(4*J) + f*J + a;
                for(int ip = 0; ip < 2; ip++)
                    for(int fp = 0; fp < 4; fp++)
                        for(int ap = 0; ap < J; ap++){
                            //row finished
                            int r = ip*(4*J) + fp*J + ap;
                            //now fill
                            if(i == ip && f == fp)
                                gsl_matrix_complex_set(M0,r,c,
                                                       gsl_complex_rect(
                                                           gsl_matrix_get(W,a,ap),
                                                           0));
                        }
            }
    // fill basic matrix V
    for(int j = 0; j < J; j++){
        set_parameters(p[N0+j*(6+J-1)+0],
                       p[N0+j*(6+J-1)+1],
                       p[N0+j*(6+J-1)+2],
                       p[N0+j*(6+J-1)+3],
                       p[N0+j*(6+J-1)+4]);
        fill_V();
        for(int i = 0; i < 2; i++)
            for(int f = 0; f < 4; f++){
                int c = i*(4*J) + f*J + j;
                for(int ip = 0; ip < 2; ip++)
                    for(int fp = 0; fp < 4; fp++){
                            //row finished
                            int r = ip*(4*J) + fp*J + j;
                            // fill hamiltonian elements
                            gsl_complex C = gsl_matrix_complex_get(M0,r,c);
                            gsl_complex Ct = gsl_complex_rect(0,0);
                            if(f == fp)
                                Ct = gsl_complex_add(Ct,gsl_matrix_complex_get(V0,i,ip));
                            if(i == ip)
                                Ct = gsl_complex_sub(Ct,gsl_matrix_complex_get(V1,f,fp));
                            if(f == fp && i == ip)
                                Ct = gsl_complex_add_real(Ct,-p[N0+j*(6+J-1)+5]);
                            // add center shift
                            C = gsl_complex_add(C,
                                                gsl_complex_mul_imag(Ct,1));
                            gsl_matrix_complex_set(M0,r,c,C);
                        }
            }
    }

    // calculate the spectrum
    //FILE* file = fopen("C:\\Users\\test\\Desktop\\MoessfitTestcase\\matrixcheck.txt","w");
    init_array(I,cc,0);
    for(int k = 0; k < cc; k++){
        gsl_matrix_complex_memcpy(Mt,M0);
        gsl_matrix_complex_add_diagonal(Mt,gsl_complex_rect(-omega,v[k]));
        gsl_permutation*perm = gsl_permutation_alloc(8*J);
        int s=0;
        gsl_linalg_complex_LU_decomp(Mt, perm, &s);
        gsl_linalg_complex_LU_invert(Mt, perm, M);
        //gsl_matrix_complex_memcpy(M,Mt);
        gsl_complex C = gsl_complex_rect(0,0);
        //fprintf(file,"%i\t%f",k,v[k]);
        for(int i = 0; i < 2; i++)
            for(int f = 0; f < 4; f++)
                for(int a = 0; a < J; a++){
                    int c = i*(4*J) + f*J + a;
                    //fprintf(file,"\n");
                    for(int ip = 0; ip < 2; ip++)
                        for(int fp = 0; fp < 4; fp++)
                            for(int ap = 0; ap < J; ap++){
                                int r = ip*(4*J) + fp*J + ap;
                                //fprintf(file,"%f\t",GSL_REAL(gsl_matrix_complex_get(M,r,c)));
                                C = gsl_complex_add(
                                        C,
                                        gsl_complex_mul_real(
                                            gsl_matrix_complex_get(M,r,c),
                                            ClebschGordan[i ][f ] * //Icl[i ][f ] *
                                            ClebschGordan[ip][fp] * //Icl[ip][fp] *
                                            gsl_vector_get(pa,a)));
                            }
                }
        I[k] = A * GSL_REAL(C);
        gsl_permutation_free(perm);
         //fprintf(file,"\n%f\n\n",GSL_REAL(C));
    }
    //fclose(file);
    gsl_matrix_complex_free(M0);
    gsl_matrix_complex_free(Mt);
    gsl_matrix_complex_free(M);
    gsl_matrix_free(W);
    gsl_vector_free(pa);
    gsl_vector_free(b);
    gsl_matrix_free(Wt);
}
void MoessCalc::get_DynDiagDiag_spectrum(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[7] == 0) return;               // no absorption at this site
    double B[2];     B[0] = p[0];   B[1] = p[1];
    double Vzz[2]; Vzz[0] = p[2]; Vzz[1] = p[3];
    double CS[2];   CS[0] = p[4];  CS[1] = p[5];
    double w = p[6];
    double A     =      p[7]/(M_PI*12);
    double beta = p[8]/180.*M_PI;
    double W12 = p[9]*1e6 * hbar_c_over_Egamma * 1e3;;
    double W21 = p[10]*1e6 * hbar_c_over_Egamma * 1e3;;

    // precalculate coefficients
    double Cd[5][2][4];
    double Cn[3][2][4];
    double V[2][2][4];
    for(int j = 0; j < 2; j++){
        set_parameters(B[j],Vzz[j],0,0,0);
        fill_V();
        for(int i = 0; i < 2; i++)
            for(int f = 0; f < 4; f++)
                V[j][i][f] = + GSL_REAL(gsl_matrix_complex_get(V0,i,i))
                             - GSL_REAL(gsl_matrix_complex_get(V1,f,f))
                             - CS[j];
    }
    double p1,p2;
    if(W12+W21 != 0){
        p1 = W21/(W12+W21);
        p2 = W12/(W12+W21);
    }
    else
        p1 = p2 = 0.5;
    for(int i = 0; i < 2; i++)
        for(int f = 0; f < 4; f++)
            if(Idiag(beta,f,i)> 0){
                double V1 = V[0][i][f];
                double V2 = V[1][i][f];
                Cd[4][i][f] = 1;
                Cd[3][i][f] = 2*(V2+V1);
                Cd[2][i][f] = (W21+W12)*(W21+W12)+2*w*(W21+W12+w)+V2*V2+4*V1*V2+V1*V1;
                Cd[1][i][f] = V1*W21*(2*W21+4*w)+V2*W12*(2*W12+4*w)+2*(V1*V2+w*w+W12*W21)*(V1+V2);
                Cd[0][i][f] = (V1*V1+w*w)*W21*W21+(V2*V2+w*w)*W12*W12+V1*V1*V2*V2+w*w*(V1*V1+V2*V2)+2*(V1*V2+w*w)*W12*W21+2*(w*V1*V1+w*w*w)*W21+2*(w*V2*V2+w*w*w)*W12+w*w*w*w;
                Cn[2][i][f] = -w;
                Cn[1][i][f] = p2*(V2-V1)*W21+p1*(V1-V2)*W12-2*w*(p1*V2+p2*V1);
                Cn[0][i][f] = -w*(W21*W21+W12*W12)-2*w*W12*W21+(p2*V1*(V2-V1)-2*w*w)*W21+(p1*V2*(V1-V2)-2*w*w)*W12-p1*w*V2*V2-p2*w*V1*V1-w*w*w;
            }

    // calculate spectrum
    for(int k = 0; k < cc; k++){
        for(int i = 0; i < 2; i++)
            for(int f = 0; f < 4; f++)
                if(Idiag(beta,f,i) > 0){
                    double V1 = V[0][i][f];
                    double V2 = V[1][i][f];
                    double vk = v[k];
                    /*I[k] += Int[i][f]*
                            GSL_REAL(
                                gsl_complex_div(
                                    gsl_complex_rect(
                                        -W21-W12-w,
                                        p1*V2+p2*V1+vk),
                                    gsl_complex_rect(
                                        -W12*W21+(W12+w)*(W21+w)-(V1+vk)*(V2+vk),
                                        ((V1+vk)*(-W21-w)+(V2+vk)*(-W12-w)))
                                )
                            );*/
                    I[k] += Idiag(beta,f,i)
                                     *( Cn[2][i][f]*vk*vk+
                                        Cn[1][i][f]*vk+
                                        Cn[0][i][f])/
                                      ( Cd[4][i][f]*vk*vk*vk*vk+
                                        Cd[3][i][f]*vk*vk*vk+
                                        Cd[2][i][f]*vk*vk+
                                        Cd[1][i][f]*vk+
                                        Cd[0][i][f]);
                }
    }
    scale(I,cc,A);
}
void MoessCalc::get_MLR_spectrum(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[4] == 0) return;               // no absorption at this site
    double B = p[0];
    double Vzz = p[1];
    double CS = p[2];
    double w = p[3];
    double A     =      p[4]/(M_PI*12);
    double beta = p[5]/180.*M_PI;
    double KV_kbT = p[6];
    double D = p[7] *   1e6 * hbar_c_over_Egamma * 1e3;
    int S = int(p[8]);
    int J = 2*S+1;
    double pp,pm,p0,Ei,Ej;

    gsl_matrix_complex *A0 = gsl_matrix_complex_alloc(J,J);
    gsl_matrix_complex *At = gsl_matrix_complex_alloc(J,J);
    gsl_matrix_complex *Ai = gsl_matrix_complex_alloc(J,J);
    gsl_vector_complex *one = gsl_vector_complex_alloc(J);
    gsl_vector_complex *X = gsl_vector_complex_alloc(J);
    gsl_vector_complex *pav = gsl_vector_complex_alloc(J);
    // calclulate the level occupation
    double *pa = new double[J], pasum=0;
    for(int m = -S; m <= S; m++){
        pa[m+S] = exp(KV_kbT*m*m/(S*S));
        pasum += pa[m+S];
    }
    scale(pa,J,1./pasum);
    for(int m = -S; m <= S; m++)
        gsl_vector_complex_set(pav,m+S,gsl_complex_rect(pa[m+S],0));



    // calculate spectrum
    gsl_complex C;
    for(int i = 0; i < 2; i++)
        for(int f = 0; f < 4; f++)
            if(Idiag(beta,f,i) > 0){
                gsl_matrix_complex_set_all(A0,gsl_complex_rect(0,0));
                // fill Matrix
                for(int m = -S; m <= S; m++){
                    pp = pm = 0;
                    if(m > -S){
                        Ei = -m*m/(S*S);
                        Ej = -(m-1)*(m-1)/(S*S);
                        pm = D*(S*(S+1)-m*(m-1)) * qMin(1.,exp(-(Ej-Ei)*KV_kbT));
                        gsl_matrix_complex_set(A0,m+S,m+S-1,gsl_complex_rect(-pm,0));
                    }
                    if(m < S){
                        Ei = -m*m/(S*S);
                        Ej = -(m+1)*(m+1)/(S*S);
                        pp = D*(S*(S+1)-m*(m+1)) * qMin(1.,exp(-(Ej-Ei)*KV_kbT));
                        gsl_matrix_complex_set(A0,m+S,m+S+1,gsl_complex_rect(-pp,0));
                    }
                    p0 = pm + pp + w;
                    set_parameters(B*m/S,Vzz,0,0,0);
                    fill_V();
                    gsl_complex V  = gsl_complex_sub(
                                        gsl_matrix_complex_get(V1,f,f),
                                        gsl_matrix_complex_get(V0,i,i));
                    V = gsl_complex_add(V,gsl_complex_rect(-CS,0));
                    V = gsl_complex_mul_imag(V,1);
                    gsl_matrix_complex_set(A0,m+S,m+S,gsl_complex_add(
                                                    V,
                                                    gsl_complex_rect(p0,0)));
                }
                for(int k = 0; k < cc; k++){
                    gsl_matrix_complex_memcpy(At,A0);
                    gsl_matrix_complex_add_diagonal(At,gsl_complex_rect(0,v[k]));
                    gsl_vector_complex_set_all(one,gsl_complex_rect(1,0));
                    /*gsl_permutation*perm = gsl_permutation_alloc(J);
                    int s=0;
                    gsl_linalg_complex_LU_decomp(At, perm, &s);
                    gsl_vector_complex_set_all(one,gsl_complex_rect(1,0));
                    gsl_linalg_complex_LU_solve(At,perm,one,X);*/
                    SolveTridiagSys(At,X,one);
                    gsl_blas_zdotu(pav,X,&C);
                    /*gsl_linalg_complex_LU_invert(At, perm, Ai);
                    C = gsl_complex_rect(0,0);
                    for(int m1 = -S; m1 <= S; m1++)
                        for(int m2 = -S; m2 <= S; m2++)
                        C = gsl_complex_add(
                                C,
                                gsl_complex_mul_real(
                                    gsl_matrix_complex_get(Ai,m1+S,m2+S),
                                    Idiag(beta,f,i)*pa[m1+S]));*/
                    I[k] -= GSL_REAL(C)*Idiag(beta,f,i);
                }
    }
    scale(I,cc,A);
}
void MoessCalc::SolveTridiagSys(gsl_matrix_complex* M, gsl_vector_complex* X,
                                gsl_vector_complex* b){
    // forward1
    gsl_matrix_complex_set(M,0,1,
                           gsl_complex_div(gsl_matrix_complex_get(M,0,1),
                                           gsl_matrix_complex_get(M,0,0)));
    for(int i = 1; i < int(M->size1-1); i++)
        gsl_matrix_complex_set(M,i,i+1,
                               gsl_complex_div(gsl_matrix_complex_get(M,i,i+1),
                                               gsl_complex_sub(
                                                  gsl_matrix_complex_get(M,i,i),
                                                  gsl_complex_mul(
                                                       gsl_matrix_complex_get(M,i-1,i),
                                                       gsl_matrix_complex_get(M,i,i-1))
                                               )));
    // forward2
    gsl_vector_complex_set(b,0,
                           gsl_complex_div(gsl_vector_complex_get(b,0),
                                           gsl_matrix_complex_get(M,0,0)));
    for(int i = 1; i < int(b->size); i++)
        gsl_vector_complex_set(b,i,
                               gsl_complex_div(gsl_complex_sub(
                                                   gsl_vector_complex_get(b,i),
                                                   gsl_complex_mul(
                                                        gsl_vector_complex_get(b,i-1),
                                                        gsl_matrix_complex_get(M,i,i-1))),
                                               gsl_complex_sub(
                                                  gsl_matrix_complex_get(M,i,i),
                                                  gsl_complex_mul(
                                                       gsl_matrix_complex_get(M,i-1,i),
                                                       gsl_matrix_complex_get(M,i,i-1)))));
    // backsubstitution
    gsl_vector_complex_set(X,X->size-1, gsl_vector_complex_get(b,b->size-1));
    for(int i = X->size-2; i >= 0; i--)
        gsl_vector_complex_set(X,i,gsl_complex_sub(
                                       gsl_vector_complex_get(b,i),
                                       gsl_complex_mul(
                                            gsl_matrix_complex_get(M,i,i+1),
                                            gsl_vector_complex_get(X,i+1))));
}
void MoessCalc::get_DynDiagDiagDiag_spectrum(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[10] == 0) return;              // no absorption at this site
    double B[3];     B[0] = p[0];   B[1] = p[1];   B[2] = p[2];
    double Vzz[3]; Vzz[0] = p[3]; Vzz[1] = p[4]; Vzz[2] = p[5];
    double CS[3];   CS[0] = p[6];  CS[1] = p[7];  CS[2] = p[8];
    double w = p[9];
    double A     =      p[10]/(M_PI*12);
    double beta = p[11]/180.*M_PI;;
    double W12 = p[12]*1e6 * hbar_c_over_Egamma * 1e3;
    double W13 = p[13]*1e6 * hbar_c_over_Egamma * 1e3;
    double W21 = p[14]*1e6 * hbar_c_over_Egamma * 1e3;
    double W23 = p[15]*1e6 * hbar_c_over_Egamma * 1e3;
    double W31 = p[16]*1e6 * hbar_c_over_Egamma * 1e3;
    double W32 = p[17]*1e6 * hbar_c_over_Egamma * 1e3;

    // precalculate coefficients
    double V[3][2][4];
    for(int j = 0; j < 3; j++){
        set_parameters(B[j],Vzz[j],0,0,0);
        fill_V();
        for(int i = 0; i < 2; i++)
            for(int f = 0; f < 4; f++)
                V[j][i][f] = + GSL_REAL(gsl_matrix_complex_get(V0,i,i))
                             - GSL_REAL(gsl_matrix_complex_get(V1,f,f))
                             - CS[j];
    }
    double p1,p2,p3;
    double denominator = -W23*W32+W13*W32+(-W23-W21)*(-W32-W31)-W12*(-W32-W31)+W12*W23-W13*(-W23-W21);
    if(denominator != 0){
        p1 = (W21*W32+W23*W31+W21*W31)/denominator;
        p2 = (W13*W32+W12*W32+W12*W31)/denominator;
        p3 = (W13*W23+W12*W23+W13*W21)/denominator;
    }
    else
        p1 = p2 = p3 = 1./3.;
    /*for(int i = 0; i < -1; i++)
        for(int f = 0; f < 4; f++)
            if(Int[i][f] > 0){
                double V1 = V[0][i][f];
                double V2 = V[1][i][f];
                double V3 = V[2][i][f];
                Cn[0][i][f] = ((-1)*w*W21*W21+((-2)*w*W13+(-2)*w*W12+(p3+p2)*V1*V2+(-p3-p2)*V1*V1+(-2)*w*w)*W21+(-1)*w*W13*W13+((-2)*w*W12-p1*V2*V2+p1*V1*V2+(-2)*w*w)*W13+(-1)*w*W12*W12+(-p1*V2*V2+p1*V1*V2+(-2)*w*w)*W12-p1*w*V2*V2+(-p3-p2)*w*V1*V1+(-1)*w*w*w)*W32*W32+((((-2)*w*W21+(-2)*w*W13+(-2)*w*W12+(p3+p2)*V1*V2+(-p3-p2)*V1*V1+(-2)*w*w)*W23+(-2)*w*W21*W21+((-2)*w*W13+(-4*p3-4*p2-4*p1)*w*W12+(p3+2*p2)*V1*V2+(-p3-2*p2)*V1*V1+(-4*p3-4*p2-4*p1)*w*w)*W21+((-2)*w*W12+(-p3-p1)*V2*V2+(p3+p1)*V1*V2+(-2)*w*w)*W13+(-2)*w*W12*W12+((-p3-2*p1)*V2*V2+(p3+2*p1)*V1*V2+(-4*p3-4*p2-4*p1)*w*w)*W12+(-p3-2*p1)*w*V2*V2+(-p3-2*p2)*w*V1*V1+(-2)*w*w*w)*W31+(((-2)*w*W13+(-2)*w*W12+(p3+p2)*V1*V3+(-p3-p2)*V1*V1+(-2)*w*w)*W21+(-2)*w*W13*W13+((-4*p3-4*p2-4*p1)*w*W12+(p1*V1-2*p1*V2)*V3+p1*V1*V2+(-4*p3-4*p2-4*p1)*w*w)*W13+(-2)*w*W12*W12+((p1*V1-2*p1*V2)*V3+p1*V1*V2+(-4*p3-4*p2-4*p1)*w*w)*W12-2*p1*w*V2*V3+(-2*p3-2*p2)*w*V1*V1+(-2)*w*w*w)*W23+((-2)*w*W13+p3*V1*V3-p3*V1*V1+(-2)*w*w)*W21*W21+((-2)*w*W13*W13+((-2)*w*W12+((p3-p1)*V2+(p1-p2)*V1)*V3+(p2-p3)*V1*V2+(-6*p3-6*p2-6*p1)*w*w)*W13+((p3*V2+p3*V1)*V3-2*p3*V1*V2+(-4*p3-4*p2-4*p1)*w*w)*W12+(p3*w*V2+p3*w*V1)*V3+(p3+2*p2)*w*V1*V2+(-3*p3-2*p2)*w*V1*V1+(-4*p3-4*p2-4*p1)*w*w*w)*W21+((p3+p1)*V2*V3+(-p3-p1)*V2*V2+(-2)*w*w)*W13*W13+(((2*p3+p1)*V2*V3+(-2*p3-p1)*V2*V2+(-4*p3-4*p2-4*p1)*w*w)*W12+((2*p3+p1)*w*V2+p1*w*V1)*V3+(-2*p3-3*p1)*w*V2*V2+p1*w*V1*V2+(-4*p3-4*p2-4*p1)*w*w*w)*W13+(p3*V2*V3-p3*V2*V2+(-2)*w*w)*W12*W12+(2*p3*w*V2*V3+(-2*p3-2*p1)*w*V2*V2+2*p1*w*V1*V2+(-4*p3-4*p2-4*p1)*w*w*w)*W12+(p3*V1*V1+p3*w*w)*V2*V3+((-p3-2*p1)*w*w-p3*V1*V1)*V2*V2+(-2*p3-2*p2)*w*w*V1*V1+(-2)*w*w*w*w)*W32+((-1)*w*W23*W23+((-2)*w*W21+(-2)*w*W12+p2*V1*V2-p2*V1*V1+(-2)*w*w)*W23+(-1)*w*W21*W21+((-2)*w*W12+p2*V1*V2-p2*V1*V1+(-2)*w*w)*W21+(-1)*w*W12*W12+((-p3-p1)*V2*V2+(p3+p1)*V1*V2+(-2)*w*w)*W12+(-p3-p1)*w*V2*V2-p2*w*V1*V1+(-1)*w*w*w)*W31*W31+(((-2)*w*W13+(-2)*w*W12+(p3+p2)*V1*V3+(-p3-p2)*V1*V1+(-2)*w*w)*W23*W23+(((-4*p3-4*p2-4*p1)*w*W13+(-2)*w*W12+(2*p3+p2)*V1*V3+(-2*p3-p2)*V1*V1+(-4*p3-4*p2-4*p1)*w*w)*W21+((-2)*w*W12+(p2*V2-2*p2*V1)*V3+p2*V1*V2+(-4*p3-4*p2-4*p1)*w*w)*W13+(-2)*w*W12*W12+(((p2-p1)*V2+(p3-p2)*V1)*V3+(p1-p3)*V1*V2+(-6*p3-6*p2-6*p1)*w*w)*W12+(p2*w*V2+(2*p3+p2)*w*V1)*V3+p2*w*V1*V2+(-2*p3-3*p2)*w*V1*V1+(-4*p3-4*p2-4*p1)*w*w*w)*W23+((-2)*w*W13+p3*V1*V3-p3*V1*V1+(-2)*w*w)*W21*W21+(((-2)*w*W12+(p2*V2-2*p2*V1)*V3+p2*V1*V2+(-4*p3-4*p2-4*p1)*w*w)*W13+((p3*V2+p3*V1)*V3-2*p3*V1*V2+(-4*p3-4*p2-4*p1)*w*w)*W12+2*p3*w*V1*V3+2*p2*w*V1*V2+(-2*p3-2*p2)*w*V1*V1+(-4*p3-4*p2-4*p1)*w*w*w)*W21+(((p3+p1)*V2*V3+(-p3-p1)*V2*V2+(-2)*w*w)*W12-2*p2*w*V1*V3+(-2*p3-2*p1)*w*V2*V2+(-2)*w*w*w)*W13+(p3*V2*V3-p3*V2*V2+(-2)*w*w)*W12*W12+((p3*w*V2+p3*w*V1)*V3+(-3*p3-2*p1)*w*V2*V2+(p3+2*p1)*w*V1*V2+(-4*p3-4*p2-4*p1)*w*w*w)*W12+(p3*V1*V2*V2+p3*w*w*V1)*V3+((-2*p3-2*p1)*w*w-p3*V1*V1)*V2*V2+(-p3-2*p2)*w*w*V1*V1+(-2)*w*w*w*w)*W31+((-1)*w*W13*W13+((-2)*w*W12-p1*V3*V3+p1*V1*V3+(-2)*w*w)*W13+(-1)*w*W12*W12+(-p1*V3*V3+p1*V1*V3+(-2)*w*w)*W12-p1*w*V3*V3+(-p3-p2)*w*V1*V1+(-1)*w*w*w)*W23*W23+(((-2)*w*W13*W13+((-2)*w*W12+(-p2-2*p1)*V3*V3+(p2+2*p1)*V1*V3+(-4*p3-4*p2-4*p1)*w*w)*W13+((-p2-p1)*V3*V3+(p2+p1)*V1*V3+(-2)*w*w)*W12+(-p2-2*p1)*w*V3*V3+(-2*p3-p2)*w*V1*V1+(-2)*w*w*w)*W21+(-p2*V3*V3+p2*V2*V3+(-2)*w*w)*W13*W13+(((-2*p2-p1)*V3*V3+(2*p2+p1)*V2*V3+(-4*p3-4*p2-4*p1)*w*w)*W12+(-2*p2-2*p1)*w*V3*V3+(2*p2*w*V2+2*p1*w*V1)*V3+(-4*p3-4*p2-4*p1)*w*w*w)*W13+((-p2-p1)*V3*V3+(p2+p1)*V2*V3+(-2)*w*w)*W12*W12+((-2*p2-3*p1)*w*V3*V3+((2*p2+p1)*w*V2+p1*w*V1)*V3+p1*w*V1*V2+(-4*p3-4*p2-4*p1)*w*w*w)*W12+((-p2-2*p1)*w*w-p2*V1*V1)*V3*V3+(p2*V1*V1+p2*w*w)*V2*V3+(-2*p3-2*p2)*w*w*V1*V1+(-2)*w*w*w*w)*W23+((-1)*w*W13*W13+((-p2-p1)*V3*V3+(p2+p1)*V1*V3+(-2)*w*w)*W13+(-p2-p1)*w*V3*V3-p3*w*V1*V1+(-1)*w*w*w)*W21*W21+((-p2*V3*V3+p2*V2*V3+(-2)*w*w)*W13*W13+(((-p2-p1)*V3*V3+(p2+p1)*V2*V3+(-2)*w*w)*W12+(-3*p2-2*p1)*w*V3*V3+(p2*w*V2+(p2+2*p1)*w*V1)*V3+p2*w*V1*V2+(-4*p3-4*p2-4*p1)*w*w*w)*W13+((-2*p2-2*p1)*w*V3*V3-2*p3*w*V1*V2+(-2)*w*w*w)*W12+(p2*V1*V2-p2*V1*V1+(-2*p2-2*p1)*w*w)*V3*V3+p2*w*w*V1*V2+(-2*p3-p2)*w*w*V1*V1+(-2)*w*w*w*w)*W21+(-p2*w*V3*V3+(-p3-p1)*w*V2*V2+(-1)*w*w*w)*W13*W13+(((-2*p2-p1)*w*V3*V3+(-2*p3-p1)*w*V2*V2+(-2)*w*w*w)*W12+((-2*p2-p1)*w*w-p1*V2*V2)*V3*V3+(p1*V1*V2*V2+p1*w*w*V1)*V3+(-2*p3-2*p1)*w*w*V2*V2+(-2)*w*w*w*w)*W13+((-p2-p1)*w*V3*V3-p3*w*V2*V2+(-1)*w*w*w)*W12*W12+((-p1*V2*V2+p1*V1*V2+(-2*p2-2*p1)*w*w)*V3*V3+(-2*p3-p1)*w*w*V2*V2+p1*w*w*V1*V2+(-2)*w*w*w*w)*W12+(-p1*w*V2*V2-p2*w*V1*V1+(-p2-p1)*w*w*w)*V3*V3+((-p3-p1)*w*w*w-p3*w*V1*V1)*V2*V2+(-p3-p2)*w*w*w*V1*V1+(-1)*w*w*w*w*w;
                Cn[1][i][f] = (((p3+p2)*V2+(-p3-p2)*V1)*W21+(p1*V1-p1*V2)*W13+(p1*V1-p1*V2)*W12-2*p1*w*V2+(-2*p3-2*p2)*w*V1)*W32*W32+((((p3+p2)*V2+(-p3-p2)*V1)*W23+((p3+2*p2)*V2+(-p3-2*p2)*V1)*W21+((-p3-p1)*V2+(p3+p1)*V1)*W13+((-p3-2*p1)*V2+(p3+2*p1)*V1)*W12+(-2*p3-4*p1)*w*V2+(-2*p3-4*p2)*w*V1)*W31+(((p3+p2)*V3+(-p3-p2)*V1)*W21+(-p1*V3-p1*V2+2*p1*V1)*W13+(-p1*V3-p1*V2+2*p1*V1)*W12-2*p1*w*V3-2*p1*w*V2+(-4*p3-4*p2)*w*V1)*W23+(p3*V3-p3*V1)*W21*W21+(((p3-p2)*V3+(p2-p1)*V2+(p1-p3)*V1)*W13+(2*p3*V3-p3*V2-p3*V1)*W12+2*p3*w*V3+(2*p3+2*p2)*w*V2+(-4*p3-2*p2)*w*V1)*W21+((p3+p1)*V3+(-p3-p1)*V2)*W13*W13+(((2*p3+p1)*V3+(-2*p3-p1)*V2)*W12+(2*p3+2*p1)*w*V3+(-2*p3-4*p1)*w*V2+2*p1*w*V1)*W13+(p3*V3-p3*V2)*W12*W12+(2*p3*w*V3+(-2*p3-2*p1)*w*V2+2*p1*w*V1)*W12+(2*p3*V1*V2+p3*V1*V1+p3*w*w)*V3-2*p3*V1*V2*V2+((-p3-4*p1)*w*w-p3*V1*V1)*V2+(-4*p3-4*p2)*w*w*V1)*W32+((p2*V2-p2*V1)*W23+(p2*V2-p2*V1)*W21+((-p3-p1)*V2+(p3+p1)*V1)*W12+(-2*p3-2*p1)*w*V2-2*p2*w*V1)*W31*W31+(((p3+p2)*V3+(-p3-p2)*V1)*W23*W23+(((2*p3+p2)*V3+(-2*p3-p2)*V1)*W21+(-p2*V3+2*p2*V2-p2*V1)*W13+((p3-p1)*V3+(p2-p3)*V2+(p1-p2)*V1)*W12+(2*p3+2*p2)*w*V3+2*p2*w*V2+(-2*p3-4*p2)*w*V1)*W23+(p3*V3-p3*V1)*W21*W21+((-p2*V3+2*p2*V2-p2*V1)*W13+(2*p3*V3-p3*V2-p3*V1)*W12+2*p3*w*V3+2*p2*w*V2+(-2*p3-2*p2)*w*V1)*W21+(((p3+p1)*V3+(-p3-p1)*V2)*W12-2*p2*w*V3+(-4*p3-4*p1)*w*V2-2*p2*w*V1)*W13+(p3*V3-p3*V2)*W12*W12+(2*p3*w*V3+(-4*p3-2*p1)*w*V2+(2*p3+2*p1)*w*V1)*W12+(p3*V2*V2+2*p3*V1*V2+p3*w*w)*V3-p3*V1*V2*V2+((-4*p3-4*p1)*w*w-2*p3*V1*V1)*V2+(-p3-4*p2)*w*w*V1)*W31+((p1*V1-p1*V3)*W13+(p1*V1-p1*V3)*W12-2*p1*w*V3+(-2*p3-2*p2)*w*V1)*W23*W23+((((-p2-2*p1)*V3+(p2+2*p1)*V1)*W13+((-p2-p1)*V3+(p2+p1)*V1)*W12+(-2*p2-4*p1)*w*V3+(-4*p3-2*p2)*w*V1)*W21+(p2*V2-p2*V3)*W13*W13+(((-2*p2-p1)*V3+(2*p2+p1)*V2)*W12+(-2*p2-2*p1)*w*V3+2*p2*w*V2+2*p1*w*V1)*W13+((-p2-p1)*V3+(p2+p1)*V2)*W12*W12+((-2*p2-4*p1)*w*V3+(2*p2+2*p1)*w*V2+2*p1*w*V1)*W12-2*p2*V1*V3*V3+(2*p2*V1*V2-p2*V1*V1+(-p2-4*p1)*w*w)*V3+(p2*V1*V1+p2*w*w)*V2+(-4*p3-4*p2)*w*w*V1)*W23+(((-p2-p1)*V3+(p2+p1)*V1)*W13+(-2*p2-2*p1)*w*V3-2*p3*w*V1)*W21*W21+((p2*V2-p2*V3)*W13*W13+(((-p2-p1)*V3+(p2+p1)*V2)*W12+(-4*p2-2*p1)*w*V3+2*p2*w*V2+(2*p2+2*p1)*w*V1)*W13+((-4*p2-4*p1)*w*V3-2*p3*w*V2-2*p3*w*V1)*W12+(p2*V2-p2*V1)*V3*V3+(2*p2*V1*V2-2*p2*V1*V1+(-4*p2-4*p1)*w*w)*V3+p2*w*w*V2+(-4*p3-p2)*w*w*V1)*W21+((-2*p3-2*p1)*w*V2-2*p2*w*V3)*W13*W13+(((-4*p2-2*p1)*w*V3+(-4*p3-2*p1)*w*V2)*W12-2*p1*V2*V3*V3+(-p1*V2*V2+2*p1*V1*V2+(-4*p2-p1)*w*w)*V3+p1*V1*V2*V2+(-4*p3-4*p1)*w*w*V2+p1*w*w*V1)*W13+((-2*p2-2*p1)*w*V3-2*p3*w*V2)*W12*W12+((p1*V1-p1*V2)*V3*V3+(-2*p1*V2*V2+2*p1*V1*V2+(-4*p2-4*p1)*w*w)*V3+(-4*p3-p1)*w*w*V2+p1*w*w*V1)*W12+(-2*p1*w*V2-2*p2*w*V1)*V3*V3+(-2*p1*w*V2*V2-2*p2*w*V1*V1+(-2*p2-2*p1)*w*w*w)*V3-2*p3*w*V1*V2*V2+((-2*p3-2*p1)*w*w*w-2*p3*w*V1*V1)*V2+(-2*p3-2*p2)*w*w*w*V1;
                Cn[2][i][f] = (-1)*w*W32*W32+((-2)*w*W31+(-2)*w*W23+(p3*V2+2*p3*V1)*V3-p3*V2*V2-2*p3*V1*V2+(-2)*w*w)*W32+(-1)*w*W31*W31+((-2)*w*W13+(2*p3*V2+p3*V1)*V3-2*p3*V1*V2-p3*V1*V1+(-2)*w*w)*W31+(-1)*w*W23*W23+((-2)*w*W21-p2*V3*V3+(p2*V2-2*p2*V1)*V3+2*p2*V1*V2+(-2)*w*w)*W23+(-1)*w*W21*W21+((-2)*w*W12+(2*p2*V2-2*p2*V1)*V3+p2*V1*V2-p2*V1*V1+(-2)*w*w)*W21+(-1)*w*W13*W13+((-2)*w*W12-p1*V3*V3+(p1*V1-2*p1*V2)*V3+2*p1*V1*V2+(-2)*w*w)*W13+(-1)*w*W12*W12+((2*p1*V1-2*p1*V2)*V3-p1*V2*V2+p1*V1*V2+(-2)*w*w)*W12+(-p2-p1)*w*V3*V3+(-4*p1*w*V2-4*p2*w*V1)*V3+(-p3-p1)*w*V2*V2-4*p3*w*V1*V2+(-p3-p2)*w*V1*V1+(-2)*w*w*w;
                Cn[3][i][f] = (p3*V3-p3*V2)*W32+(p3*V3-p3*V1)*W31+(p2*V2-p2*V3)*W23+(p2*V2-p2*V1)*W21+(p1*V1-p1*V3)*W13+(p1*V1-p1*V2)*W12+(-2*p2-2*p1)*w*V3+(-2*p3-2*p1)*w*V2+(-2*p3-2*p2)*w*V1;
                Cn[4][i][f] = (-1)*w;

                Cd[0][i][f] = ((V1*V1+w*w)*W21*W21+((2*V1*V2+2*w*w)*W13+(2*V1*V2+2*w*w)*W12+2*w*V1*V1+2*w*w*w)*W21+(V2*V2+w*w)*W13*W13+((2*V2*V2+2*w*w)*W12+2*w*V2*V2+2*w*w*w)*W13+(V2*V2+w*w)*W12*W12+(2*w*V2*V2+2*w*w*w)*W12+(V1*V1+w*w)*V2*V2+w*w*V1*V1+w*w*w*w)*W32*W32+((((2*V1*V1+2*w*w)*W21+(2*V1*V2+2*w*w)*W13+(2*V1*V2+2*w*w)*W12+2*w*V1*V1+2*w*w*w)*W23+(2*V1*V1+2*w*w)*W21*W21+((2*V1*V2+2*w*w)*W13+(4*V1*V2+4*w*w)*W12+4*w*V1*V1+4*w*w*w)*W21+((2*V2*V2+2*w*w)*W12+2*w*V2*V2+2*w*w*w)*W13+(2*V2*V2+2*w*w)*W12*W12+(4*w*V2*V2+4*w*w*w)*W12+(2*V1*V1+2*w*w)*V2*V2+2*w*w*V1*V1+2*w*w*w*w)*W31+(((2*V1*V3+2*w*w)*W13+(2*V1*V3+2*w*w)*W12+2*w*V1*V1+2*w*w*w)*W21+(2*V2*V3+2*w*w)*W13*W13+((4*V2*V3+4*w*w)*W12+4*w*V2*V3+4*w*w*w)*W13+(2*V2*V3+2*w*w)*W12*W12+(4*w*V2*V3+4*w*w*w)*W12+(2*V1*V1+2*w*w)*V2*V3+2*w*w*V1*V1+2*w*w*w*w)*W23+((2*V1*V3+2*w*w)*W13+2*w*V1*V1+2*w*w*w)*W21*W21+((2*V2*V3+2*w*w)*W13*W13+((2*V2*V3+2*w*w)*W12+(2*w*V2+2*w*V1)*V3+2*w*V1*V2+6*w*w*w)*W13+(4*w*V1*V2+4*w*w*w)*W12+4*w*w*V1*V1+4*w*w*w*w)*W21+(2*w*V2*V2+2*w*w*w)*W13*W13+((4*w*V2*V2+4*w*w*w)*W12+4*w*w*V2*V2+4*w*w*w*w)*W13+(2*w*V2*V2+2*w*w*w)*W12*W12+(4*w*w*V2*V2+4*w*w*w*w)*W12+(2*w*V1*V1+2*w*w*w)*V2*V2+2*w*w*w*V1*V1+2*w*w*w*w*w)*W32+((V1*V1+w*w)*W23*W23+((2*V1*V1+2*w*w)*W21+(2*V1*V2+2*w*w)*W12+2*w*V1*V1+2*w*w*w)*W23+(V1*V1+w*w)*W21*W21+((2*V1*V2+2*w*w)*W12+2*w*V1*V1+2*w*w*w)*W21+(V2*V2+w*w)*W12*W12+(2*w*V2*V2+2*w*w*w)*W12+(V1*V1+w*w)*V2*V2+w*w*V1*V1+w*w*w*w)*W31*W31+(((2*V1*V3+2*w*w)*W13+(2*V1*V3+2*w*w)*W12+2*w*V1*V1+2*w*w*w)*W23*W23+(((4*V1*V3+4*w*w)*W13+(2*V1*V3+2*w*w)*W12+4*w*V1*V1+4*w*w*w)*W21+((2*V2*V3+2*w*w)*W12+4*w*V1*V3+4*w*w*w)*W13+(2*V2*V3+2*w*w)*W12*W12+((2*w*V2+2*w*V1)*V3+2*w*V1*V2+6*w*w*w)*W12+4*w*w*V1*V1+4*w*w*w*w)*W23+((2*V1*V3+2*w*w)*W13+2*w*V1*V1+2*w*w*w)*W21*W21+(((2*V2*V3+2*w*w)*W12+4*w*V1*V3+4*w*w*w)*W13+(4*w*V1*V2+4*w*w*w)*W12+4*w*w*V1*V1+4*w*w*w*w)*W21+((2*w*V2*V2+2*w*w*w)*W12+(2*V1*V2*V2+2*w*w*V1)*V3+2*w*w*V2*V2+2*w*w*w*w)*W13+(2*w*V2*V2+2*w*w*w)*W12*W12+(4*w*w*V2*V2+4*w*w*w*w)*W12+(2*w*V1*V1+2*w*w*w)*V2*V2+2*w*w*w*V1*V1+2*w*w*w*w*w)*W31+((V3*V3+w*w)*W13*W13+((2*V3*V3+2*w*w)*W12+2*w*V3*V3+2*w*w*w)*W13+(V3*V3+w*w)*W12*W12+(2*w*V3*V3+2*w*w*w)*W12+(V1*V1+w*w)*V3*V3+w*w*V1*V1+w*w*w*w)*W23*W23+(((2*V3*V3+2*w*w)*W13*W13+((2*V3*V3+2*w*w)*W12+4*w*V3*V3+4*w*w*w)*W13+(2*w*V3*V3+2*w*w*w)*W12+(2*V1*V1+2*w*w)*V3*V3+2*w*w*V1*V1+2*w*w*w*w)*W21+(2*w*V3*V3+2*w*w*w)*W13*W13+((4*w*V3*V3+4*w*w*w)*W12+4*w*w*V3*V3+4*w*w*w*w)*W13+(2*w*V3*V3+2*w*w*w)*W12*W12+(4*w*w*V3*V3+4*w*w*w*w)*W12+(2*w*V1*V1+2*w*w*w)*V3*V3+2*w*w*w*V1*V1+2*w*w*w*w*w)*W23+((V3*V3+w*w)*W13*W13+(2*w*V3*V3+2*w*w*w)*W13+(V1*V1+w*w)*V3*V3+w*w*V1*V1+w*w*w*w)*W21*W21+((2*w*V3*V3+2*w*w*w)*W13*W13+((2*w*V3*V3+2*w*w*w)*W12+4*w*w*V3*V3+4*w*w*w*w)*W13+((2*V1*V2+2*w*w)*V3*V3+2*w*w*V1*V2+2*w*w*w*w)*W12+(2*w*V1*V1+2*w*w*w)*V3*V3+2*w*w*w*V1*V1+2*w*w*w*w*w)*W21+((V2*V2+w*w)*V3*V3+w*w*V2*V2+w*w*w*w)*W13*W13+(((2*V2*V2+2*w*w)*V3*V3+2*w*w*V2*V2+2*w*w*w*w)*W12+(2*w*V2*V2+2*w*w*w)*V3*V3+2*w*w*w*V2*V2+2*w*w*w*w*w)*W13+((V2*V2+w*w)*V3*V3+w*w*V2*V2+w*w*w*w)*W12*W12+((2*w*V2*V2+2*w*w*w)*V3*V3+2*w*w*w*V2*V2+2*w*w*w*w*w)*W12+((V1*V1+w*w)*V2*V2+w*w*V1*V1+w*w*w*w)*V3*V3+(w*w*V1*V1+w*w*w*w)*V2*V2+w*w*w*w*V1*V1+w*w*w*w*w*w;
                Cd[1][i][f] = (2*V1*W21*W21+((2*V2+2*V1)*W13+(2*V2+2*V1)*W12+4*w*V1)*W21+2*V2*W13*W13+(4*V2*W12+4*w*V2)*W13+2*V2*W12*W12+4*w*V2*W12+2*V1*V2*V2+(2*V1*V1+2*w*w)*V2+2*w*w*V1)*W32*W32+(((4*V1*W21+(2*V2+2*V1)*W13+(2*V2+2*V1)*W12+4*w*V1)*W23+4*V1*W21*W21+((2*V2+2*V1)*W13+(4*V2+4*V1)*W12+8*w*V1)*W21+(4*V2*W12+4*w*V2)*W13+4*V2*W12*W12+8*w*V2*W12+4*V1*V2*V2+(4*V1*V1+4*w*w)*V2+4*w*w*V1)*W31+(((2*V3+2*V1)*W13+(2*V3+2*V1)*W12+4*w*V1)*W21+(2*V3+2*V2)*W13*W13+((4*V3+4*V2)*W12+4*w*V3+4*w*V2)*W13+(2*V3+2*V2)*W12*W12+(4*w*V3+4*w*V2)*W12+(4*V1*V2+2*V1*V1+2*w*w)*V3+(2*V1*V1+2*w*w)*V2+4*w*w*V1)*W23+((2*V3+2*V1)*W13+4*w*V1)*W21*W21+((2*V3+2*V2)*W13*W13+((2*V3+2*V2)*W12+4*w*V3+4*w*V2+4*w*V1)*W13+(4*w*V2+4*w*V1)*W12+8*w*w*V1)*W21+4*w*V2*W13*W13+(8*w*V2*W12+8*w*w*V2)*W13+4*w*V2*W12*W12+8*w*w*V2*W12+4*w*V1*V2*V2+(4*w*V1*V1+4*w*w*w)*V2+4*w*w*w*V1)*W32+(2*V1*W23*W23+(4*V1*W21+(2*V2+2*V1)*W12+4*w*V1)*W23+2*V1*W21*W21+((2*V2+2*V1)*W12+4*w*V1)*W21+2*V2*W12*W12+4*w*V2*W12+2*V1*V2*V2+(2*V1*V1+2*w*w)*V2+2*w*w*V1)*W31*W31+(((2*V3+2*V1)*W13+(2*V3+2*V1)*W12+4*w*V1)*W23*W23+(((4*V3+4*V1)*W13+(2*V3+2*V1)*W12+8*w*V1)*W21+((2*V3+2*V2)*W12+4*w*V3+4*w*V1)*W13+(2*V3+2*V2)*W12*W12+(4*w*V3+4*w*V2+4*w*V1)*W12+8*w*w*V1)*W23+((2*V3+2*V1)*W13+4*w*V1)*W21*W21+(((2*V3+2*V2)*W12+4*w*V3+4*w*V1)*W13+(4*w*V2+4*w*V1)*W12+8*w*w*V1)*W21+(4*w*V2*W12+(2*V2*V2+4*V1*V2+2*w*w)*V3+2*V1*V2*V2+4*w*w*V2+2*w*w*V1)*W13+4*w*V2*W12*W12+8*w*w*V2*W12+4*w*V1*V2*V2+(4*w*V1*V1+4*w*w*w)*V2+4*w*w*w*V1)*W31+(2*V3*W13*W13+(4*V3*W12+4*w*V3)*W13+2*V3*W12*W12+4*w*V3*W12+2*V1*V3*V3+(2*V1*V1+2*w*w)*V3+2*w*w*V1)*W23*W23+((4*V3*W13*W13+(4*V3*W12+8*w*V3)*W13+4*w*V3*W12+4*V1*V3*V3+(4*V1*V1+4*w*w)*V3+4*w*w*V1)*W21+4*w*V3*W13*W13+(8*w*V3*W12+8*w*w*V3)*W13+4*w*V3*W12*W12+8*w*w*V3*W12+4*w*V1*V3*V3+(4*w*V1*V1+4*w*w*w)*V3+4*w*w*w*V1)*W23+(2*V3*W13*W13+4*w*V3*W13+2*V1*V3*V3+(2*V1*V1+2*w*w)*V3+2*w*w*V1)*W21*W21+(4*w*V3*W13*W13+(4*w*V3*W12+8*w*w*V3)*W13+((2*V2+2*V1)*V3*V3+(4*V1*V2+4*w*w)*V3+2*w*w*V2+2*w*w*V1)*W12+4*w*V1*V3*V3+(4*w*V1*V1+4*w*w*w)*V3+4*w*w*w*V1)*W21+(2*V2*V3*V3+(2*V2*V2+2*w*w)*V3+2*w*w*V2)*W13*W13+((4*V2*V3*V3+(4*V2*V2+4*w*w)*V3+4*w*w*V2)*W12+4*w*V2*V3*V3+(4*w*V2*V2+4*w*w*w)*V3+4*w*w*w*V2)*W13+(2*V2*V3*V3+(2*V2*V2+2*w*w)*V3+2*w*w*V2)*W12*W12+(4*w*V2*V3*V3+(4*w*V2*V2+4*w*w*w)*V3+4*w*w*w*V2)*W12+(2*V1*V2*V2+(2*V1*V1+2*w*w)*V2+2*w*w*V1)*V3*V3+((2*V1*V1+2*w*w)*V2*V2+2*w*w*V1*V1+2*w*w*w*w)*V3+2*w*w*V1*V2*V2+(2*w*w*V1*V1+2*w*w*w*w)*V2+2*w*w*w*w*V1;
                Cd[2][i][f] = (W21*W21+(2*W13+2*W12+2*w)*W21+W13*W13+(2*W12+2*w)*W13+W12*W12+2*w*W12+V2*V2+4*V1*V2+V1*V1+2*w*w)*W32*W32+(((2*W21+2*W13+2*W12+2*w)*W23+2*W21*W21+(2*W13+4*W12+4*w)*W21+(2*W12+2*w)*W13+2*W12*W12+4*w*W12+2*V2*V2+8*V1*V2+2*V1*V1+4*w*w)*W31+((2*W13+2*W12+2*w)*W21+2*W13*W13+(4*W12+4*w)*W13+2*W12*W12+4*w*W12+(2*V2+4*V1)*V3+4*V1*V2+2*V1*V1+4*w*w)*W23+(2*W13+2*w)*W21*W21+(2*W13*W13+(2*W12+6*w)*W13+4*w*W12+4*w*w)*W21+2*w*W13*W13+(4*w*W12+4*w*w)*W13+2*w*W12*W12+4*w*w*W12+2*w*V2*V2+8*w*V1*V2+2*w*V1*V1+4*w*w*w)*W32+(W23*W23+(2*W21+2*W12+2*w)*W23+W21*W21+(2*W12+2*w)*W21+W12*W12+2*w*W12+V2*V2+4*V1*V2+V1*V1+2*w*w)*W31*W31+((2*W13+2*W12+2*w)*W23*W23+((4*W13+2*W12+4*w)*W21+(2*W12+4*w)*W13+2*W12*W12+6*w*W12+4*w*w)*W23+(2*W13+2*w)*W21*W21+((2*W12+4*w)*W13+4*w*W12+4*w*w)*W21+(2*w*W12+(4*V2+2*V1)*V3+2*V2*V2+4*V1*V2+4*w*w)*W13+2*w*W12*W12+4*w*w*W12+2*w*V2*V2+8*w*V1*V2+2*w*V1*V1+4*w*w*w)*W31+(W13*W13+(2*W12+2*w)*W13+W12*W12+2*w*W12+V3*V3+4*V1*V3+V1*V1+2*w*w)*W23*W23+((2*W13*W13+(2*W12+4*w)*W13+2*w*W12+2*V3*V3+8*V1*V3+2*V1*V1+4*w*w)*W21+2*w*W13*W13+(4*w*W12+4*w*w)*W13+2*w*W12*W12+4*w*w*W12+2*w*V3*V3+8*w*V1*V3+2*w*V1*V1+4*w*w*w)*W23+(W13*W13+2*w*W13+V3*V3+4*V1*V3+V1*V1+2*w*w)*W21*W21+(2*w*W13*W13+(2*w*W12+4*w*w)*W13+(2*V3*V3+(4*V2+4*V1)*V3+2*V1*V2+4*w*w)*W12+2*w*V3*V3+8*w*V1*V3+2*w*V1*V1+4*w*w*w)*W21+(V3*V3+4*V2*V3+V2*V2+2*w*w)*W13*W13+((2*V3*V3+8*V2*V3+2*V2*V2+4*w*w)*W12+2*w*V3*V3+8*w*V2*V3+2*w*V2*V2+4*w*w*w)*W13+(V3*V3+4*V2*V3+V2*V2+2*w*w)*W12*W12+(2*w*V3*V3+8*w*V2*V3+2*w*V2*V2+4*w*w*w)*W12+(V2*V2+4*V1*V2+V1*V1+2*w*w)*V3*V3+(4*V1*V2*V2+(4*V1*V1+4*w*w)*V2+4*w*w*V1)*V3+(V1*V1+2*w*w)*V2*V2+4*w*w*V1*V2+2*w*w*V1*V1+3*w*w*w*w;
                Cd[3][i][f] = (2*V2+2*V1)*W32*W32+((4*V2+4*V1)*W31+(2*V3+2*V2+4*V1)*W23+4*w*V2+4*w*V1)*W32+(2*V2+2*V1)*W31*W31+((2*V3+4*V2+2*V1)*W13+4*w*V2+4*w*V1)*W31+(2*V3+2*V1)*W23*W23+((4*V3+4*V1)*W21+4*w*V3+4*w*V1)*W23+(2*V3+2*V1)*W21*W21+((4*V3+2*V2+2*V1)*W12+4*w*V3+4*w*V1)*W21+(2*V3+2*V2)*W13*W13+((4*V3+4*V2)*W12+4*w*V3+4*w*V2)*W13+(2*V3+2*V2)*W12*W12+(4*w*V3+4*w*V2)*W12+(2*V2+2*V1)*V3*V3+(2*V2*V2+8*V1*V2+2*V1*V1+4*w*w)*V3+2*V1*V2*V2+(2*V1*V1+4*w*w)*V2+4*w*w*V1;
                Cd[4][i][f] = W32*W32+(2*W31+2*W23+2*w)*W32+W31*W31+(2*W13+2*w)*W31+W23*W23+(2*W21+2*w)*W23+W21*W21+(2*W12+2*w)*W21+W13*W13+(2*W12+2*w)*W13+W12*W12+2*w*W12+V3*V3+(4*V2+4*V1)*V3+V2*V2+4*V1*V2+V1*V1+3*w*w;
                Cd[5][i][f] = 2*V3+2*V2+2*V1;
                Cd[6][i][f] = 1;
            }*/

    // calculate spectrum
    for(int k = 0; k < cc; k++){
        for(int i = 0; i < 2; i++)
            for(int f = 0; f < 4; f++)
                if(Idiag(beta,f,i) > 0){
                    double V1 = V[0][i][f];
                    double V2 = V[1][i][f];
                    double V3 = V[2][i][f];
                    double vk = v[k];
                    I[k] += Idiag(beta,f,i)*
                            GSL_REAL(
                                gsl_complex_div(
                                    gsl_complex_rect(
                                        +p1*(-W23*W32+W13*W32+(-W23-W21-w)*(-W32-W31-w)-W12*(-W32-W31-w)+W12*W23-W13*(-W23-W21-w)-(V2+vk)*(V3+vk))+p3*(W21*W32-(-W13-W12-w)*W32-(-W23-W21-w)*W31+W12*W31+(-W13-W12-w)*(-W23-W21-w)-W12*W21-(V1+vk)*(V2+vk))+p2*(-W21*(-W32-W31-w)+(-W13-W12-w)*(-W32-W31-w)+W23*W31-W13*W31-(-W13-W12-w)*W23+W13*W21-(V1+vk)*(V3+vk)),
                                        (p3*(-(V1+vk)*W32-(V2+vk)*W31+(V1+vk)*(-W23-W21-w)+(V2+vk)*(-W13-W12-w))+p1*((V2+vk)*(-W32-W31-w)+(V3+vk)*(-W23-W21-w)-(V2+vk)*W13-(V3+vk)*W12)+p2*((V1+vk)*(-W32-W31-w)-(V1+vk)*W23-(V3+vk)*W21+(V3+vk)*(-W13-W12-w)))),
                                    gsl_complex_rect(
                                        +(-W13-W12-w)*(-W23*W32+(-W23-W21-w)*(-W32-W31-w)-(V2+vk)*(V3+vk))+W13*(W21*W32-(-W23-W21-w)*W31)+W12*(W23*W31-W21*(-W32-W31-w))-(V1+vk)*((V2+vk)*(-W32-W31-w)+(V3+vk)*(-W23-W21-w)),
                                        ((V1+vk)*(-W23*W32+(-W23-W21-w)*(-W32-W31-w)-(V2+vk)*(V3+vk))+(-W13-W12-w)*((V2+vk)*(-W32-W31-w)+(V3+vk)*(-W23-W21-w))-(V2+vk)*W13*W31-(V3+vk)*W12*W21))
                                )
                            );
                    /*I[k] += Idiag(beta,f,i)*(
                                        Cn[4][i][f]*vk*vk*vk*vk+
                                        Cn[3][i][f]*vk*vk*vk+
                                        Cn[2][i][f]*vk*vk+
                                        Cn[1][i][f]*vk+
                                        Cn[0][i][f])/
                                      ( Cd[6][i][f]*vk*vk*vk*vk*vk*vk+
                                        Cd[5][i][f]*vk*vk*vk*vk*vk+
                                        Cd[4][i][f]*vk*vk*vk*vk+
                                        Cd[3][i][f]*vk*vk*vk+
                                        Cd[2][i][f]*vk*vk+
                                        Cd[1][i][f]*vk+
                                        Cd[0][i][f]);*/
                }
    }
    scale(I,cc,A);
}

double MoessCalc::dv_Debye(double T, double TD, double Meff){
    if(TD < 0 || T <= 0)
        return 0;
    else
        return -9./2./Meff*1000.*1000/c*Runi*T/3.*gsl_sf_debye_3(TD/T);
}
double MoessCalc::Abs_Debye(double T, double TD, double Meff){
    if(TD <= 0 || T <= 0)
        return 0;
    else{
        // ATTENTION: gsl_sf_debye_1(x) = 1/x*int_0^x(x dx/(e^x-1))
        return exp(-3./2.*Erec*56.93539/Meff/(kbolz*TD)*(1.+4.*T/TD*gsl_sf_debye_1(TD/T)));
    }
}
double sWaveIntegrand(double E, void *params){
    double delta = ((double*)params)[1];
    double T = ((double*)params)[2];
    double kbT = 8.6173e-5*T;
    double expf;
    gsl_sf_result_e10 expr;
    if(gsl_sf_exp_e10_e(E/kbT,&expr) == GSL_SUCCESS
       && E > delta
       && expr.val < 1e150
       && expr.e10 < 100){
        expf = expr.val*pow(10,expr.e10);
        return -expf/(kbT*(expf+1)*(expf+1))*E/sqrt(E*E-delta*delta);
    }
    else
        return 0;
}
double MoessCalc::sWave_SuperfluidDensity(double SF0, double Tc, double T,
                                          double delta0){
    if( T < Tc && delta0 > 0){
        gsl_integration_workspace * w = gsl_integration_workspace_alloc(1000);
        double result, error;
        double delta = delta0*tanh(1.82*(1.018*pow(Tc/T-1,0.51)));
        double alpha[3];
        alpha[0] = Tc;
        alpha[1] = delta;
        alpha[2] = T;
        gsl_function F;
        F.function = &sWaveIntegrand;
        F.params = &alpha;
        double pts[3];
        pts[0] = delta;
        pts[1] = delta;
        pts[2] = std::max(100*8.6173e-5*T,2*delta);
        gsl_integration_qagp(&F,pts,3,0, 1e-7, 1000,  w, &result, &error);
        gsl_integration_workspace_free (w);
        return SF0*(1+2*result);
    }
    else
        return 0;
}
void MoessCalc::TransmissionIntegral(double *SHI,double *v, int count,
                                     double A, double I0, double ta,
                                     double omegasrc, double omegaabs,
                                     double fr){
    double* OneMinusT = new double[count];
    T_sigma(SHI,v,count,A,I0,OneMinusT,ta,omegasrc,omegaabs);
    for(int k = 0; k < count; k++)
        SHI[k] = I0*(1-fr*OneMinusT[k]);
    delete OneMinusT;
}
void MoessCalc::T_sigma(double *SHI,double *v, int count,
                        double A, double I0, double* OneMinusT,
                        double ta, double omegasrc, double omegaabs){
    double* expsigma05 = new double[count];
    double oataArec = -omegaabs*ta*M_PI/A;
    //oataArec = -5.7457983771948e-06;
    for(int k = 0; k < count; k++){
        expsigma05[k] = (1-exp((I0-SHI[k])*oataArec))*0.5;
        OneMinusT[k] = 0;
    }
    double omegasrcrec = 1./omegasrc;
    double florenz0 = f_lorenz_fast(0,omegasrcrec,0,1);
    double filtertemp;
    int kp,k2p;
    if(abs((v[3]-v[0])-(v[count-1]-v[count-4]))<0.000001){
        // optimized convolution with special regard on f_lorenz-reduction
        // needs exact symmetric velocity spacing!
        for(int k = 1; k < count/2; k++){
            kp = count-1-k;
            filtertemp = f_lorenz_fast(v[k],omegasrcrec,v[kp],1);
            OneMinusT[k] += florenz0  *expsigma05[k ]*(v[k +1]-v[k -1]);
            OneMinusT[k] += filtertemp*expsigma05[kp]*(v[kp+1]-v[kp-1]);
            OneMinusT[kp]+= florenz0  *expsigma05[kp]*(v[kp+1]-v[kp-1]);
            OneMinusT[kp]+= filtertemp*expsigma05[k ]*(v[k +1]-v[k -1]);
            for(int k2 = k+1; k2 < kp; k2++){
                k2p = count - 1 - k2;
                filtertemp = f_lorenz_fast(v[k],omegasrcrec,v[k2],1);
                OneMinusT[k  ] += filtertemp*expsigma05[k2 ]*(v[k2 +1]-v[k2 -1]);
                OneMinusT[k2 ] += filtertemp*expsigma05[k  ]*(v[k  +1]-v[k  -1]);
                OneMinusT[kp ] += filtertemp*expsigma05[k2p]*(v[k2p+1]-v[k2p-1]);
                OneMinusT[k2p] += filtertemp*expsigma05[kp ]*(v[kp +1]-v[kp -1]);
            }

            // border corection
            OneMinusT[k ]+= (2*atan((v[0      ]-v[k ])*omegasrcrec)+M_PI)
                            /M_PI*expsigma05[0]
                           -(2*atan((v[count-1]-v[k ])*omegasrcrec)-M_PI)
                            /M_PI*expsigma05[count-1];
            OneMinusT[kp]+= (2*atan((v[0      ]-v[kp])*omegasrcrec)+M_PI)
                            /M_PI*expsigma05[0]
                           -(2*atan((v[count-1]-v[kp])*omegasrcrec)-M_PI)
                            /M_PI*expsigma05[count-1];
        }
    }
    else
        // naiv convolution, slow
        for(int k = 1; k < count-1; k++){
            for(int k2 = 1; k2 < count-1; k2++)
                OneMinusT[k  ] += f_lorenz(v[k],omegasrc,v[k2],1)
                                 *expsigma05[k2 ]*(v[k2 +1]-v[k2 -1]);
            // border corection
            OneMinusT[k] +=((2*atan((v[0      ]-v[k])/omegasrc)+M_PI)/2./M_PI
                           -(2*atan((v[count-1]-v[k])/omegasrc)-M_PI)/2./M_PI)
                            *(expsigma05[0]+expsigma05[count-1]);
        }
    OneMinusT[0] = OneMinusT[count-1] = 0;
    delete expsigma05;
}

double Hc2_WHH_fun(double Hc2, void *params){
    double *p = (double*)params;
    double result = -log(1./p[0]);
    double h = 4./3.141592654/3.141592654*Hc2/p[1];
    /*for(int i = -100; i < 101; i++)
        result += 1./qAbs(2.*i+1)-1./(qAbs(2.*i+1.)+h/p[0]+(p[2]*h/p[0])*(p[2]*h/p[0])/(qAbs(2.*i+1.)+(h+p[3])/p[0]));
    return result;
    */
    int i = 0;
    double lastresult = result;
    result += 1*(1./qAbs(2.*i+1)-1./(qAbs(2.*i+1.)+h/p[0]+(p[2]*h/p[0])*(p[2]*h/p[0])/(qAbs(2.*i+1.)+(h+p[3])/p[0])));
    while(result != lastresult && i < 1e3){
        i++;
        lastresult = result;
        result += 1./qAbs( 2.*i+1)-1./(qAbs( 2.*i+1.)+h/p[0]+(p[2]*h/p[0])*(p[2]*h/p[0])/(qAbs( 2.*i+1.)+(h+p[3])/p[0]));
        result += 1./qAbs(-2.*i+1)-1./(qAbs(-2.*i+1.)+h/p[0]+(p[2]*h/p[0])*(p[2]*h/p[0])/(qAbs(-2.*i+1.)+(h+p[3])/p[0]));
    }
    return result;

}
double MoessCalc::Hc2_WHH(double T, double Tc, double dHc2dT, double alpha,
               double lambdaSO){
    double t = T/Tc;
    if(t < 1 && dHc2dT != 0){
        double para[4];
        para[0] = T/Tc;
        para[1] = dHc2dT;
        para[2] = alpha;
        para[3] = lambdaSO;

        int status;
        int iter = 0, max_iter = 100;
        const gsl_root_fsolver_type *Tgsl;
        gsl_root_fsolver *s;
        double r = 0, r_expected = sqrt (5.0);
        double x_lo = 0, x_hi = 1e10;
        gsl_function F;
        F.function = &Hc2_WHH_fun;
        F.params = (void*)&(para[0]);
        Tgsl = gsl_root_fsolver_brent;
        s = gsl_root_fsolver_alloc (Tgsl);
        gsl_root_fsolver_set (s, &F, x_lo, x_hi);
        gsl_root_fsolver_name (s);
        do{
            iter++;
            status = gsl_root_fsolver_iterate (s);
            r = gsl_root_fsolver_root (s);
            x_lo = gsl_root_fsolver_x_lower (s);
            x_hi = gsl_root_fsolver_x_upper (s);
            status = gsl_root_test_interval (x_lo, x_hi,
            0, 0.000000000001);
        }
        while (status == GSL_CONTINUE && iter < max_iter);
        gsl_root_fsolver_free (s);
        return r;
    }
    else
        if(dHc2dT == 0)
            return 1e100;
        else
            return 0;
}
double MoessCalc::errorfunction(double arg){
    //if(arg < 0)
    //    return 0;
    //else
        return gsl_sf_erf(arg);
}
double OrderParameterDistrIntegrand(double t, void *params){
    double Tn = ((double*)params)[0];
    double sigma = ((double*)params)[1];
    double alpha = ((double*)params)[2];
    double beta = ((double*)params)[3];
    double T = ((double*)params)[4];
    return 1/sqrt(2*M_PI*sigma*sigma)*exp(-0.5*(Tn-t)*(Tn-t)/(sigma*sigma))*pow(1-pow(T/t,alpha),beta);
}
double MoessCalc::OrderParameterDistr(double T, double Tn, double sigma,
                                      double alpha, double beta){
    double result;
    if(sigma > 0){
        gsl_integration_workspace * w = gsl_integration_workspace_alloc(1000);
        double alph[5],error;
        alph[0] = Tn;
        alph[1] = sigma;
        alph[2] = alpha;
        alph[3] = beta;
        alph[4] = T;
        gsl_function F;
        F.function = &OrderParameterDistrIntegrand;
        F.params = &alph;
        double pts[2];
        pts[0] = T;
        pts[1] = Tn+20*sigma;
        if(pts[0]<pts[1]){
            gsl_integration_qagp(&F,pts,2,0, 1e-7, 1000,  w, &result, &error);
            gsl_integration_workspace_free (w);
        }
        else
            result = 0;
    }
    else
        if(T<Tn)
            result = pow(1-pow(T/Tn,alpha),beta);
        else
            result = 0;
    return result;
}
void MoessCalc::get_FeAs_spectrum(double *I, int cc , double* v, double* p){
    double *Ibufbuf = new double[cc];
    init_array(I,cc,0.0);
    double pt[8] = {0,0, 1,0, 53,0, 0.13,0};
    double pi[5][3] = {{4.1566818182, 9,0.21},{5.2505454545,27,0.27},{3.60975,45,0.19},{2.5158863636,63,0.17},{1.7501818182,81,0.16}};
    for(int i = 0; i < 5; i++){
        pt[0] = pi[i][0]*pow(1-pow(qMin(1.,p[0]/p[4]),2.25),0.21);
        pt[1] = -(28.96+(300-p[0])*0.0183685614);
        pt[3] = pi[i][1];
        pt[5] = p[1]+dv_Debye(p[0],p[3],57);
        pt[7] = p[2]*pi[i][2]*Abs_Debye(p[0],p[3],56.93539);
        get_Powder_spectrum(Ibufbuf,cc,v,pt);
        for(int j = 0; j < cc; j++) I[j] += Ibufbuf[j];
    }
    delete Ibufbuf;
}
void MoessCalc::get_Fe2As_spectrum(double *I, int cc , double* v, double* p){
    double *Ibufbuf = new double[cc];
    init_array(I,cc,0.0);
    double T = p[0];
    double CS0 = p[1];
    double A0 = p[2];
    double TD = p[3];
    double Tn = p[4];
    double pt[8] = {0,0, 0,90,0,0, 0.13,0};
    pt[6] = p[5];
    // calulate Site 1
    if(T < Tn)  pt[0] = 12.88*pow(1-pow(T/Tn,2.57),0.33);
    else        pt[0] = 0;
    pt[1] = 2*(0.01101*T - 1.90577);
    pt[5] = CS0 + 0.498 + dv_Debye(T,TD,57);
    pt[7] = A0*0.5*Abs_Debye(T,TD,57);
    get_Powder_spectrum(Ibufbuf,cc,v,pt);
    for(int j = 0; j < cc; j++) I[j] += Ibufbuf[j];
    // calulate Site 2
    if(T < Tn)  pt[0] = 16.37*pow(1-pow(T/Tn,1.8),0.32);
    else        pt[0] = 0;
    pt[1] = 2*(-0.00729*T - 16.08634);
    pt[5] = CS0 + 0.70182 + dv_Debye(T,TD,57);
    pt[7] = A0*0.5*Abs_Debye(T,TD,57);
    get_Powder_spectrum(Ibufbuf,cc,v,pt);
    for(int j = 0; j < cc; j++) I[j] += Ibufbuf[j];
    // free
    delete Ibufbuf;
}
double BJ(double J, double y){
    if(y == 0)
        return 0;
    else
        if(y*(1+0.5/J) > 100)
            return 1;
        else
            if(y*(1+0.5/J) < 0.1){
                double J2 = J*J;
                double J3 = J2*J;
                double J4 = J3*J;
                double J5 = J4*J;
                double J6 = J5*J;
                double J7 = J6*J;
                double J8 = J7*J;
                double J9 = J8*J;
                return ((J+1)*y)/(3*J)
                      -((2*J3+4*J2+3*J+1)*y*y*y)/(90*J3)
                      +((16*J5+48*J4+60*J3+40*J2+15*J+3)*y*y*y*y*y)/(7560*J5)
                      +(-(16*J7+64*J6+112*J5+112*J4+70*J3+28*J2+7*J+1)*y*y*y*y*y*y*y)/(75600*J7)
                      +((256*J9+1280*J8+2880*J7+3840*J6+3360*J5+2016*J4+840*J3+240*J2+45*J+5)*y*y*y*y*y*y*y*y*y)/(11975040*J9);
            }
            else
                return ((2.*J+1.)/(2.*J)*coth((2.*J+1.)/(2.*J)*y)-1./(2.*J)*coth(y/(2.*J)));
}

double Mmf_fun(double y, void *params){
    double *p = (double*)params;
    double B = p[0];
    double Ms = p[1];
    double J = qMax(0.5,p[2]);
    double T = p[3];
    double lambda = p[4];
    double g = p[5];
    //double check1 = (y*T/(g*0.6717132486*J*lambda)-B/lambda)-Ms*BJ(J,y);
    //double check2 = Ms*BJ(J,y);
    //double check3 = (y*T/(g*0.6717132486*J*lambda)-B/lambda);
    return (y*T/(g*0.6717132486*J*lambda)-B/lambda)-Ms*BJ(J,y);
}

double MoessCalc::Mmf(double B, double Ms, double J, double T, double Tc, double g){
    double para[6],y;
    double lambda = Tc*3./(g*0.6717132486*(J+1)*Ms);
    J = qMax(0.5,J);
    para[0] = B;
    para[1] = Ms;
    para[2] = J;
    para[3] = T;
    para[4] = lambda;
    para[5] = g;
    //double check1 = Tc/(g*0.6717132486*J*lambda) - (J+1)/3/J*Ms;
    //double check2 = T/(g*0.6717132486*J*lambda) - (J+1)/3/J*Ms;
    //return Ms*BJ(J,T);
    if(g*J != 0){
        if(T == 0)
            return Ms;
        if(T>=Tc && B==0)
            return 0;
        double y;
        double y_lo = 1e-40;
        double y_hi = g*0.6717132486*J*(B+lambda*Ms)/T*1.01;

        if(lambda != 0 &&
                Mmf_fun(y_lo,(void*)&(para[0])) < 0 &&
                Mmf_fun(y_hi,(void*)&(para[0])) > 0){
            int status;
            int iter = 0, max_iter = 100;
            const gsl_root_fsolver_type *Tgsl;
            gsl_root_fsolver *s;
            gsl_function F;
            F.function = &Mmf_fun;
            F.params = (void*)&(para[0]);
            Tgsl = gsl_root_fsolver_brent;
            s = gsl_root_fsolver_alloc (Tgsl);
            gsl_root_fsolver_set (s, &F, y_lo, y_hi);
            gsl_root_fsolver_name(s);
            do{
                iter++;
                status = gsl_root_fsolver_iterate (s);
                y = gsl_root_fsolver_root (s);
                y_lo = gsl_root_fsolver_x_lower (s);
                y_hi = gsl_root_fsolver_x_upper (s);
                status = gsl_root_test_interval (y_lo, y_hi,
                0, 0.000000000001);
            }
            while (status == GSL_CONTINUE && iter < max_iter);
            gsl_root_fsolver_free (s);
        }
        else
            y = g*0.6717132486*J*B/T;
        return Ms*BJ(J,y);
    }
    else
        return 0;
}
double MoessCalc::Bezier_y(double x, double x1, double y1, double x2, double y2,
                           double x3, double y3, double x4, double y4){
    double X[4],Y[4];
    X[0] = x1;
    X[1] = x2;
    X[2] = x3;
    X[3] = x4;
    Y[0] = y1;
    Y[1] = y2;
    Y[2] = y3;
    Y[3] = y4;
    return Bezier_y(X,Y,4,x);
}

double MoessCalc::Bezier_y(double *X, double *Y, int N, double x){
    double l = 0.;
    double r = 1.;
    double xl = X[0];
    double xr = X[N-1];
    double xt = X[0],yt;
    if(xl>xr){
        xl = X[N-1];
        xr = X[0];
        l = 1;
        r = 0;
    }
    double dx = xr-xl;
    if(dx != 0 && x >= xl && xr >= x ){
        do{
            double t = (l+r)/2.;
            Bezier(X,Y,N,t,&xt,&yt);
            if(xt > x)
                r = t;
            else
                l = t;
        }while(qAbs((xt-x)/dx) > 1e-8);
    }
    return yt;
}
void MoessCalc::Bezier(double *X, double *Y, int N, double t, double* x, double* y){
    *x = 0;
    *y = 0;
    for(int i = 0; i < N; i++){
        double Bi = gsl_ran_binomial_pdf(uint(i),t,uint(N-1));
        *x += Bi * X[i];
        *y += Bi * Y[i];
    }
}
void MoessCalc::theta_phi(double mt, double mp, double Vzzt, double Vzzp,
                          double Vxxt, double Vxxp, double *theta, double *phi){
    mt *= M_PI/180.;
    mp *= M_PI/180.;
    Vzzt *= M_PI/180.;
    Vzzp *= M_PI/180.;
    Vxxt *= M_PI/180.;
    Vxxp *= M_PI/180.;

    double xz = sin(Vzzt)*cos(Vzzp);
    double yz = sin(Vzzt)*sin(Vzzp);
    double zz = cos(Vzzt);
    double xx = sin(Vxxt)*cos(Vxxp);
    double yx = sin(Vxxt)*sin(Vxxp);
    double zx = cos(Vxxt);
    double xm = sin(mt)*cos(mp);
    double ym = sin(mt)*sin(mp);
    double zm = cos(mt);

    *theta = RadDeg*acos(xm*xz+ym*yz+zm*zz);
    *phi = RadDeg*acos(qMin(1.,
                       qMax(-1.,(xm*xx+ym*yx+zm*zx)/sin(*theta * DegRad))));
}
void MoessCalc::get_skyrmion(double *I, int cc , double* v, double* p){
    init_array(I,cc,0);                 // reset spectrum
    if(p[5]==0) return;                 // no absorption at this site
    double* It = new double[cc];

    double ps[8];
    ps[0] = p[0];                       // B
    ps[1] = p[1];                       // Vzz
    ps[2] = p[2];                       // eta
    double thetaV  = p[3];              // theta Vzz
    ps[5] = p[4];                       // CS
    ps[6] = p[5];                       // omega
    ps[7] = p[6];                       // A
    double K = p[7];                    // wave vector
    double xi = p[8];                   // coherence length
    int maxEZ = p[9];                   // supercell expansion
    int ca = p[10];                     // crystal lattice parameter a
    double dFe = 0.2257439553*ca;
    int N = maxEZ*maxEZ*3-2;
    double norm = 1./N;
    double coherence = 0, coherent, x,y,d,alpha, thetaz;

    // coherent moments
    for(int h = 0; h < maxEZ; h++){
        for(int k = 0; k < maxEZ; k++){
            for(int ialpha = 0; ialpha < 3; ialpha++){
                if(!(h == 0 & k == 0 & ialpha > 0)){
                    alpha = 2*M_PI*ialpha/3.;
                    x = (- h + 0.5 * k) * ca + cos(alpha) * dFe;
                    y = sin(M_PI/3)* k  * ca + sin(alpha) * dFe;
                    d = sqrt(x*x+y*y);
                    thetaz = K*d;
                    coherent = exp(-d/xi);
                    ps[7] = p[6] * norm*coherent;
                    coherence += coherent*norm;

                    theta_phi(thetaz*RadDeg,atan2(y,x)*RadDeg,
                              thetaV*RadDeg,alpha*RadDeg,
                              90.,alpha * RadDeg + 90.,
                              &(ps[3]),&(ps[4]));

                    get_Powder_spectrum(It,cc,v,ps);

                    sum(1.,I,It,I,cc);
                }
            }
        }
    }

    // incoherent moments, powder averaging
    for(double theta = 2.5; theta < 90; theta += 5)
        for(double phi = 5; phi < 90; phi += 10){
            double rho = (1.-coherence)*0.0096931972*sin(theta*DegRad);
            ps[3] = theta;
            ps[4] = phi;
            ps[7] = p[6] * rho;

            get_Powder_spectrum(It,cc,v,ps);
            sum(1.,I,It,I,cc);
    }
    delete It;
}

void MoessCalc::skyrmion_theta(double *theta, double thetaV, double K, double xi,
                               double dtheta, int maxEZ,  double ca){
    double Theta[270];
    init_array(Theta,270,0);
    init_array(theta,90,0);
    thetaV *= M_PI/180.;
    double dFe = 0.2257439553*ca;
    int N = maxEZ*maxEZ*3-2;
    double norm = 1./N;
    double coherence = 0, coherent, x,y,d,thetaz,theta0,alpha;
    for(int h = 0; h < maxEZ; h++){
        for(int k = 0; k < maxEZ; k++){
            for(int ialpha = 0; ialpha < 3; ialpha++){
                if(!(h == 0 & k == 0 & ialpha > 0)){
                    alpha = 2*M_PI*ialpha/3.;
                    x = (- h + 0.5 * k) * ca + cos(alpha) * dFe;
                    y = sin(M_PI/3)* k  * ca + sin(alpha) * dFe;
                    d = sqrt(x*x+y*y);
                    thetaz = K*d;
                    coherent = exp(-d/xi);
                    coherence += coherent*norm;
                    theta0 = acos( sin(thetaz)*x/d*sin(thetaV)*cos(alpha)
                                  +sin(thetaz)*y/d*sin(thetaV)*sin(alpha)
                                  +cos(thetaz)    *cos(thetaV));
                    theta0 = 90.-qAbs(90.-theta0/M_PI*180.);
                    theta0 += 89.5;
                    for(int i = 0; i < 270; i++)
                        Theta[i] += 1./sqrt(2*M_PI*dtheta*dtheta)
                                  * exp(-0.5*(i-theta0)*(i-theta0)/dtheta/dtheta)
                                  * coherent * norm;
                }
            }
        }
    }
    for(int i = 0; i < 90; i++){
        theta[i] += Theta[ 89-i];
        theta[i] += Theta[ 90+i];
        theta[i] += Theta[269-i];
        theta[i] += (1-coherence)*sin((i+0.5)/180.*M_PI)*M_PI/180.;
    }
}

void MoessCalc::correlation_theta(double *theta, double thetaV, double K, double xi,
                               double dtheta, int maxEZ,  double ca){
    double Theta[270];
    init_array(Theta,270,0);
    init_array(theta,90,0);
    double dFe = 0.2257439553*ca;
    double coherence = 0, coherent,d,thetaz,theta0,buf;
    for(int h = 0; h < maxEZ; h++){
        d = dFe + h*ca;
        thetaz = K*d*RadDeg;
        coherent = exp(-d/xi);
        // phi - averaging
        for(double Phi = 0.5; Phi < 180; Phi++){
            theta_phi(thetaz,Phi,thetaV,0,0,0,&theta0,&buf);
            theta0 = 90.-qAbs(90.-theta0);
            theta0 += 89.5;
            for(int i = 0; i < 270; i++)
                Theta[i] += 1./sqrt(2*M_PI*dtheta*dtheta)
                          * exp(-0.5*(i-theta0)*(i-theta0)/dtheta/dtheta)
                          * coherent;
        }
    }

    for(int i = 0; i < 90; i++){
        theta[i] += Theta[ 89-i];
        theta[i] += Theta[ 90+i];
        theta[i] += Theta[269-i];
        theta[i] += (1-coherence)*sin((i+0.5)/180.*M_PI)*M_PI/180.;
    }
    // normalize
    scale(theta,90,1./sum(theta,90));
}

CosSmearing::CosSmearing(double r_src, double r_smpl, double r_det, double z_smlp,
            double z_det){
    r0 = r1 = r2 = z1 = z2 = -1;
    steps = 100;
    Rho = new double[steps];
    costheta = new double[steps];
    sintheta = new double[steps];
    update(r_src, r_smpl, r_det, z_smlp, z_det);
}

CosSmearing::~CosSmearing(){
    delete Rho;
    delete costheta;
    delete sintheta;
}

void CosSmearing::update(double r_src, double r_smpl, double r_det, double z_smlp,
       double z_det){
    // recalculate if something changed
    if(r0 != r_src || r1 != r_smpl || r2 != r_det ||
       z1 != z_smlp || z2 != z_det){
        r0 = r_src;
        r1 = r_smpl;
        r2 = r_det;
        z1 = z_smlp;
        z2 = z_det;
        // set the histogram border angle
        if(z2 > z1)
            // transmission geometry, limited by either detector or sample
            one_minus_cosmin = qMin(1 -(z2-z1)/sqrt((z2-z1)*(z2-z1)+(r0+r1)*(r0+r1)),
                               qMin(1 - z1/sqrt(z1*z1+(r0+r1)*(r0+r1)),
                                    1 - z2/sqrt(z2*z2+(r0+r2)*(r0+r2))));
        else
            // backscatter geometry, limited by sample
            one_minus_cosmin =      1 - z1/sqrt(z1*z1+(r0+r1)*(r0+r1));
        // guarantee that Rho[steps-1]=0
        one_minus_cosmin *= 1.001;
        // precalulate help numbers
        for(int i = 0; i < steps; i++){
            costheta[i] = 1 - one_minus_cosmin * (i)/(steps-1);
            sintheta[i] = sqrt(1-costheta[i]*costheta[i]);
            Rho[i] = 0;
        }
        // test all gamma rays emitted by sample
        for(double r = 0; r < r0*(1+0.5/steps); r += r0/steps)
            for(int i = 0; i < steps; i++){
                for(double phi = 0; phi < 2*M_PI*(1+0.5/steps); phi += 2.*M_PI/steps)
                    if(in_smpl(r,i,phi) && in_det(r,i,phi))
                        Rho[i] += r;
            }
        // normalize
        double Sum = (sum(&(Rho[1]),steps-2)+0.5*(Rho[0]+Rho[steps-1]))
                     *one_minus_cosmin/(steps-1);
        if(Sum == 0)
            Rho[0] = 2.*(steps-1)/one_minus_cosmin;
        else
            scale(Rho,steps,1./Sum);
    }
}
bool CosSmearing::in_smpl(double r,int thetai,double phi){
    double x = r - cos(phi)*z1/costheta[thetai]*sintheta[thetai];
    double y = sin(phi)*z1/costheta[thetai]*sintheta[thetai];

    if(sqrt(x*x+y*y) <= r1)
        return true;
    else
        return false;
}

bool CosSmearing::in_det(double r,int thetai,double phi){
    double x = r - cos(phi)*z2/costheta[thetai]*sintheta[thetai];
    double y = sin(phi)*z2/costheta[thetai]*sintheta[thetai];

    if(sqrt(x*x+y*y) <= r2)
        return true;
    else
        return false;
}
double CosSmearing::rho(double dv, double v0){
    double i = ((dv/v0)/one_minus_cosmin)*(steps-1);
    return rho(i)/v0;
}
double CosSmearing::rho(double i){
    if(i >= steps)
        return 0;
    if(i < 0)
        i = 0;
    int J = int(floor(i));
    double j = 1 - (i-floor(i));
    return (Rho[J]*j+Rho[J+1]*(1-j));
}
void CosSmearing::smear(double *SHI,double *v, int count, double I0){
    double* I = new double[count];
    double* dv = new double[count];
    int i;
    for(i = 0; i < count; i++){
        I[i] = -SHI[i] + I0;
        SHI[i] = 0;
        dv[i] = (v[qMin(i+1,count-1)]-v[qMax(i-1,0)])*0.25;
    }
    dv[0] *= 2;
    dv[count-1] *= 2;

    // sorted velocity data is assumed!
    for(i = 0; i < count; i++){
        if(v[i] == 0)
            SHI[i] += I[i];
        else{
            int j = i,dj = 0;
            if(i==234)
                dj = 5;
            if(v[i] < 0) dj = -1;
            if(v[i] >= 0) dj = +1;
            double Pv = 1;
            while(j >= 0 && j < count && Pv > 0){
                Pv = P(qAbs(v[j]-v[i])-dv[j],qAbs(v[j]-v[i])+dv[j],qAbs(v[i]));
                if(dv[j] > 0)
                    SHI[j] += Pv/(2*dv[j])*I[i]*2*dv[i];
                j += dj;
            }
        }
    }
    for(i = 0; i < count; i++)
        SHI[i] = I0 - SHI[i];
}
double CosSmearing::P(double v01, double v02, double v0){
    double v1 = ((v01/v0)/one_minus_cosmin)*(steps-1);
    double v2 = ((v02/v0)/one_minus_cosmin)*(steps-1);
    if(v1 < 0        ) v1 = 0;
    if(v2 < 0        ) v2 = 0;
    if(v1 > (steps-1)) v1 = steps-1;
    if(v2 > (steps-1)) v2 = steps-1;
    int v1int = int(ceil(v1));
    int v2int = int(floor(v2));

    double result = 0;
    // sum up all intervalls in between
    for(int i = v1int; i < v2int; i++)
        result += 0.5 * (Rho[i] + Rho[i+1]);
    if(v1int <= v2int){
        // add start
        if(v1int > 0)
            result += (Rho[v1int] +rho(v1)) * (v1int-v1) * 0.5;
        // add end
        if(v2int < steps-1)
            result += (Rho[v2int] +rho(v2)) * (v2-v2int) * 0.5;
    }
    else
        // intermediate only
        if(v1 >= 0 && v2 <= steps-1)
            result += (rho(v1) +rho(v2)) * (v2-v1) * 0.5;

    // noramlize
    result *= one_minus_cosmin/(steps-1);
    return result;
}

